// Control.cpp Copyright 1999 Steve Ludtke
// 1/14/99   This is where most of the logic for 'boxer' resides
#include "Control.h"

#define Inherited ControlData

#include <math.h>
#include <qprogressdialog.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qfiledlg.h>
#include <qapp.h>
#include <qutil.h>
#include <qkeycode.h>
#include "OneLiner.h"
#include "AutoboxD.h"


Control::Control(QWidget* parent,const char* name):Inherited( parent, name ) {
	QPopupMenu *m1;

	initMetaObject();
	setCaption( "Boxer" );
	
	mgrp->setExclusive(TRUE);

	// FILE MENU
	m1=new QPopupMenu(NULL,"File");
	menu->insertItem(QString("File"),m1,100);
	m1->insertItem(QString("Read Micrograph"),this,SLOT(Mreadimage()),0);
	m1->insertItem(QString("Save Micrograph"),this,SLOT(Msaveimage()),0);
	m1->insertSeparator();
	m1->insertItem(QString("Exit"),this,SLOT(Mquit()),ALT+Key_X);

	// BOXES MENU
	m1=new QPopupMenu(NULL,"Boxes");
	menu->insertItem(QString("Boxes"),m1);
	m1->insertItem(QString("Save Boxed Particles"),this,SLOT(Msaveclips()),0);
	m1->insertSeparator();
	m1->insertItem(QString("Read Box DB"),this,SLOT(Mreadbdb()),0);
	m1->insertItem(QString("Save Box DB"),this,SLOT(Msavebdb()),0);
	m1->insertItem(QString("Adjust Boxes"),this,SLOT(adjustBoxes()),0);
	m1->insertItem(QString("Focal Pair Autoalign"),this,SLOT(Mfocalauto()),0);
	m1->insertSeparator();
	m1->insertItem(QString("Autobox"),this,SLOT(autoBox1()),0);
	m1->insertItem(QString("Clear Boxes"),this,SLOT(delAll()),0);
	m1->insertItem(QString("Zero -> Noise"),this,SLOT(Mzerotonoise()),0);
	m1->insertItem(QString("Resize boxes"),this,SLOT(Mnewboxsize()),0);
	m1->insertItem(QString("Autobox from model"),this,SLOT(autoBoxM()),0);
	m1->insertItem(QString("Check for drift"),this,SLOT(Mcheckdrift()),0);

	// PROCESS MENU
	m1=new QPopupMenu(NULL,"PROCESS");
	menu->insertItem(QString("Process"),m1);
	m1->insertItem(QString("Median Filter (3x3)"),this,SLOT(Mmedian3()),0);
	m1->insertItem(QString("Median Filter (5x5)"),this,SLOT(Mmedian5()),0);
	m1->insertItem(QString("Median Filter (7x7)"),this,SLOT(Mmedian7()),0);
	m1->insertItem(QString("Median Filter (9x9)"),this,SLOT(Mmedian9()),0);

	menu->insertSeparator();

	// HELP MENU
	m1=new QPopupMenu(NULL,"Help");
	menu->insertItem(QString("Help"),m1);
	m1->insertItem("Help",this,SLOT(Mhelp()),0);
	m1->insertItem(QString("About boxer"),this,SLOT(Mabout()),0);
	

	imview=new Image;
	imview->resize(500,500);
	imview->setScale(1.0);
	imview->show();
	
	clipview=new ImageMx();
	clipview->setMode(-1);
	clipview->resize(520,300);
	clipview->viewport()->setPalette(QPalette(black));
	clipview->show();

// This restores the window geometries
	FILE *in = in=DFopenr("boxerwin");
	if (in) {
		int x,y,w,h;
#ifndef SGI		
		if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) move(x,y);
		if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) clipview->setGeometry(x,y,w,h);
		if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) imview->setGeometry(x,y,w,h);
#endif
		fclose(in);
	}

	ptcl.setAutoDelete(TRUE);
	abptcl.setAutoDelete(TRUE);
	abvec.setAutoDelete(TRUE);

	scale=1.0;
	apix=4.6667;
	boxsize=100;
	data=NULL;

	curshape=-1;
	
	connect(measure,SIGNAL(clicked()),this,SLOT(setMeasure()));
	connect(select,SIGNAL(clicked()),this,SLOT(setSelect()));
	connect(zap,SIGNAL(clicked()),this,SLOT(setDelete()));
	connect(draw,SIGNAL(clicked()),this,SLOT(setDraw()));

	connect(clipview,SIGNAL(mousePress(QMouseEvent *,int ,int , int)),this,
		SLOT(clipClick(QMouseEvent *,int ,int , int)));

	measure->toggle();

	connect(Size,SIGNAL(textChanged(const QString&)),this,SLOT(setSize(const QString&)));
	connect(APix,SIGNAL(textChanged(const QString&)),this,SLOT(setAPix(const QString&)));
	connect(scalesel,SIGNAL(textChanged(const QString&)),this,SLOT(setScale(const QString&)));

//	connect(brite,SIGNAL(valueChanged(int)),this,SLOT(setBC(int)));
//	connect(cont,SIGNAL(valueChanged(int)),this,SLOT(setBC(int)));

	connect(imview,SIGNAL(mousePress(QMouseEvent *)),this,SLOT(imDown(QMouseEvent *)));
	connect(imview,SIGNAL(mouseMove(QMouseEvent *)),this,SLOT(imMove(QMouseEvent *)));
	connect(imview,SIGNAL(mouseRelease(QMouseEvent *)),this,SLOT(imUp(QMouseEvent *)));
	connect(imview,SIGNAL(resized(int,int)),SLOT(imResized(int,int)));
	connect(imview,SIGNAL(moved(int,int)),SLOT(imMoved(int,int)));

	connect(pan,SIGNAL(newValue(float,float)),SLOT(panned(float,float)));
}


Control::~Control() {
delete data;
delete imview;
delete clipview;
}

void Control::Mhelp() {
	char s[512];
	sprintf(s,"netscape -remote 'openFile(%s/doc/progs/boxer.html,new-window)'",
		EMDIR());
	run(s);
}

void Control::panned(float x,float y) {
imview->setOrigin(x,y);
}

void Control::setMeasure() {
mode=MEASURE;
}

void Control::setSelect() {
mode=SELECT;
}

void Control::setDelete() {
mode=DELETE;
}

void Control::setDraw() {
mode=DRAW;
}

void Control::setSize(const QString &t) {
boxsize=t.toInt();
}

void Control::setAPix(const QString &t) {
apix=t.toFloat();
}

void Control::setScale(const QString &t) {
scale=t.toFloat();
imview->setScale(scale);
imResized(imview->width(),imview->height());
}

void Control::setBC(int value) {
/*float b=(brite->value()-500.0)/500.0;
float c=cont->value()/1000.0;
Image *im;

imview->setBrite(b);
imview->setCont(c);

clipview->setBC(b,c);*/

}

void Control::Mabout() { 
	QMessageBox::about(this,"About Boxer",
 "boxer\nCopyright 1999 Steve Ludtke\n\n\
 Boxer is designed to cut out particle\n\
 images from whole micrographs. It has\n\
 features for both manual and automatic\n\
 particle selection.\n");
}

void Control::readimage(char *fsp,int num,int denom,int bsize) { 
int x0,y0,w,h;

if (num<0) {
	QApplication::setOverrideCursor( waitCursor );

	printf("Reading Image: %s\n",fsp);

	if (data) delete data;
	data=new EMData;
	data->setProgCB(doQprog);
	data->readImage(fsp,0);
	if (data->Mean()>1.0||data->Sigma()>10.0) data->normalize();
	imview->setData(data);
	imResized(imview->width(),imview->height());


	QApplication::setOverrideCursor( arrowCursor );
	return;
}

if (num>=denom) { printf("num must be less than denom\n"); exit(1); }

if (sqrt((float)denom)*sqrt((float)denom)!=(float)denom) { printf("denom must be 4,9,16,25,etc.\n"); exit(1); }
denom=(int)floor(sqrt((float)denom));

data=new EMData;
data->setProgCB(doQprog);
if (data->readMRC(fsp,1)!=0) {
	printf("The 'area' argument only works with MRC images\n"); exit(1); 
}

QApplication::setOverrideCursor( waitCursor );

w=data->xSize()/denom;
h=data->ySize()/denom;
x0=(num%denom)*w;
y0=(num/denom)*h;
w+=bsize;
h+=bsize;
if (x0+w>=data->xSize()) w=data->xSize()-x0-1;
if (y0+h>=data->ySize()) h=data->ySize()-y0-1;

delete data;

data=new EMData;
data->setProgCB(doQprog);
printf("Reading area %d,%d %d,%d\n",x0,y0,w,h);
data->readMRCArea(fsp,x0,y0,w,h);
data->normalize();
imview->setData(data);
imResized(imview->width(),imview->height());
QApplication::setOverrideCursor( arrowCursor );

}

// Checks each particle for drift
void Control::Mcheckdrift() { 
int i,j,k;
int x0,y0,xx,yy,nx,ny;
Shape *ss;
EMData *av,*f;
EMData rf[16];
float *d,a,r,x,y,v0,v1,aa[16];
QList<EMData> avs;

avs.setAutoDelete(FALSE);
nx=ptcl.at(0)->xSize()+2;
ny=ptcl.at(1)->ySize();

// We generate 16 references in 90 degrees. The other 90 degrees is
// just a sign flip.
for (k=0; k<16; k++) {
	rf[k].setSize(nx,ny,1);	// cos
	rf[k].setComplex(1);
	rf[k].setRI(0);
	d=rf[k].getData();

	for (i=0; i<nx; i+=2) {
		for (j=0; j<ny; j++) {
			a=atan2((float)j-ny/2.0,(float)i/2.0);
			r=hypot((float)j-ny/2.0,(float)i/2.0);
			if (r>=ny/2 || i==0) d[i+j*nx]=0;
			else {
				d[i+j*nx]=cos(a*2.0+k*PI/32.0+PI/64.0)
							*exp(-(r*r*4.0)/(ny*ny))*tanh(r/4.0);
				aa[k]=k*PI/32.0+1.0/64.0;
			}

			d[i+j*nx+1]=0;
		}
	}
	rf[k].doneData();
}

//r1.normalize();
//r2.normalize();
//printf("%f %f %f\n",r1.dot(&r3,1),r2.dot(&r3,1),r2.dot(&r2,1));


for (i=0; i<ptcl.count(); i++) {
	ss=imview->shapeAt(i);
	x0=ss->x0;
	y0=ss->y0;
	av=ptcl.at(i)->doFFT()->copy(0,0);
	for (j=0; j<ptcl.count(); j++) {
		if (j==i) continue;
		ss=imview->shapeAt(j);
		xx=ss->x0;
		yy=ss->y0;

		if (hypot((float)xx-x0,(float)yy-y0)>256) continue;
		f=ptcl.at(j)->doFFT();
		av->addIncoherent(f);
	}

	v0=1.0e10;
	v1=0;
	a=0;
	for (k=0; k<16; k++) {
		x=av->dot(&rf[k]);
		if (fabs(x)<v0) v0=fabs(x);
		if (fabs(x)>v1) { 
			v1=fabs(x); 
			if (x>0) a=aa[k]; else a=aa[k]+PI/2.0; 
		}
	}
	r=10000.0*(v1-v0)/(ny*ny);

	printf("%d.\t%4.2f\t%4.2f\t%4.2f (%1.2f)\n",i,v0,v1,r,a*180.0/PI);

	ss=imview->shapeAt(i);
	ss->type=Shape::LINE;
	ss->x0=(int)floor(x0-50*r*cos(a));
	ss->y0=(int)floor(y0-50*r*sin(a));
	ss->x1=(int)floor(x0+50*r*cos(a));
	ss->y1=(int)floor(y0+50*r*sin(a));

//	av->ap2ri();
//	avs.append(av);
	delete av;
}

/*
ptcl.clear();
char s[50];
for (i=0; i<avs.count(); i++) {
	sprintf(s,"AA.%d.mrc",i);
	avs.at(i)->writeMRC(s);
	ptcl.append(avs.at(i)); 
}
printf("%d new particles\n",ptcl.count());
*/
}

void Control::Mmedian3() { 
doMedian(1);
}

void Control::Mmedian5() { 
doMedian(2);
}

void Control::Mmedian7() { 
doMedian(3);
}

void Control::Mmedian9() { 
doMedian(4);
}

void Control::doMedian(int v) {
QApplication::setOverrideCursor( waitCursor );
data->realFilter(20,v);
printf("Filter %d\n",v);
QApplication::setOverrideCursor( arrowCursor );
imview->imageChanged();
}

void Control::Mreadimage() { 
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
QFile file(s);

QApplication::setOverrideCursor( waitCursor );

if (data) delete data;
data=new EMData;
data->setProgCB(doQprog);
data->readImage(s.data(),0);
if (data->Mean()>1.0||data->Sigma()>10.0) data->normalize();
imview->setData(data);
imResized(imview->width(),imview->height());

QApplication::setOverrideCursor( arrowCursor );

}

void Control::Msaveimage() {
QString s( QFileDialog::getSaveFileName(0,"*.mrc") ),t;
if ( s.isNull() ) return;
int lx,ly;
EMData *d;
int fno=1;

QApplication::setOverrideCursor( waitCursor );

data->writeMRC(s.data());

QApplication::setOverrideCursor( arrowCursor );

}

void Control::Msaveclips() {
QString s( QFileDialog::getSaveFileName(0,"*.hed") ),t;
if ( s.isNull() ) return;
int lx,ly;
EMData *d;
int fno=1;

QApplication::setOverrideCursor( waitCursor );

d=ptcl.first();
lx=d->xSize();
ly=d->ySize();

for (d=ptcl.first(); d!=NULL; d=ptcl.next()) {
	if (d->xSize()!=lx || d->ySize()!=ly) {
		t.sprintf(".%d",fno++);
		if (s[s.length()-4]=='.') s.insert(s.length()-4,t);
		else s.append(t);
	}
	d->writeIMAGIC(s.data(),-1);
	lx=d->xSize();
	ly=d->ySize();
}

QApplication::setOverrideCursor( arrowCursor );

}

void Control::Mreadbdb() {
int i,j=0,x0,y0,x1,y1;
char t[20];
EMData *d;
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
FILE *in=fopen(s.data(),"r");
if (!in) return;

for (i=0; fscanf(in," %d %d %d %d",&x0,&y0,&x1,&y1)==4; i++) {
	if (x0>data->xSize() ||x1<0 || y0>data->ySize() || y1<0) continue;
	j=newClip(x0,y0,x1,y1);
	d=data->clip(x0,y0,x1,y1);
	d->ufloat=d->Sigma();
	d->normalize();
	ptcl.remove(j);
	ptcl.insert(j,d);
	clipview->addData(d);
}

boxsize=x1; 
sprintf(t,"%d",x1); 
Size->setText(t);

fclose(in);
boxname=s;
redisplayBoxes();
}

void Control::Msavebdb() {
int i;
Shape *ss;
QString s( QFileDialog::getSaveFileName(0,"*.box") ),t;
if ( s.isNull() ) return;
FILE *out=fopen(s.data(),"w");
if (!out) return;

for (i=0; i<imview->nShapes(); i++) {
	ss=imview->shapeAt(i);
	fprintf(out,"%d\t%d\t%d\t%d\n",ss->x0,ss->y0,ss->x1,ss->y1);
}

fclose(out);

}

void Control::Mnewboxsize() {
QString r;
Shape *ss;
int ns;
r=OneLiner::getOneLine("Enter the new box size","0");
ns=r.toInt();
if (ns<=3) return;

for (int i=0; i<imview->nShapes(); i++) {
	ss=imview->shapeAt(i);
	ss->x0+=(ss->x1-ns)/2;
	ss->y0+=(ss->y1-ns)/2;
	ss->x1=ns;
	ss->y1=ns;
	boxMoved(i);
}

redisplayBoxes();
}

void Control::closeEvent( QCloseEvent *e ) {
Mquit();
}	


void Control::Mquit() { 
// This saves the window geometries
	FILE *out = DFopenw("boxerwin");
	if (out) {
		fprintf(out,"%d %d %d %d\n",x(),y(),width(),height());
		fprintf(out,"%d %d %d %d\n",clipview->x(),clipview->y(),clipview->width(),clipview->height());
		fprintf(out,"%d %d %d %d\n",imview->x(),imview->y(),imview->width(),imview->height());
		fclose(out);	
	}


	exit(0); 
}

void Control::Mzerotonoise() {
float *d,s;
int i,n;

n=data->xSize()*data->ySize();
s=data->Sigma()*.9;
d=data->getData();
for (i=0; i<n; i++) {
	if (d[i]==0) d[i]=grand(0,s);
}
data->doneData();
}

void Control::clipClick(QMouseEvent *e,int i,int x,int y) {
Shape *s;

imview->select.type=Shape::NONE;
switch (mode) {
case SELECT:
	s=imview->shapeAt(i);
	if (!s) return;
	imview->setOrigin((s->x0+s->x1/2)*scale-imview->width()/2,
		(s->y0+s->y1/2)*scale-imview->height()/2);
	imview->hilight(i);
	break;
case DELETE:
	delClip(i);
	redisplayBoxes();
	break;
}

}

void Control::delClip(int i) {
if (i>=ptcl.count()) return;
imview->delShape(i);
clipview->removeData(i);
ptcl.remove(i);
imview->imageChanged();
}

void Control::delAll() {
imview->clearShapes();
clipview->clearData();
ptcl.clear();
imview->imageChanged();
}

int Control::newClip(int x0,int y0,int x1, int y1) {
Shape *s;
int i;
EMData *d;

	s=new Shape;
	s->type=Shape::DOTBOX;
	s->x0=x0;
	s->y0=y0;
	s->x1=x1;
	s->y1=y1;
	s->color=red;
	s->text="";
	i=imview->addShape(s);
	d=data->clip(x0,y0,x1,y1);
	d->ufloat=d->Sigma();
	ptcl.append(d);
	imview->imageChanged();
//	clipview->addData(ptcl.at(i));

return i;
}

void Control::imMoved(int x, int y) {
pan->setValue(x,y);
}

void Control::imResized(int w, int h) {
float xh,yh;

if (data->xSize()>data->ySize()) 
	pan->resize(100,100*data->ySize()/data->xSize());
else pan->resize(100*data->xSize()/data->ySize(),100);
pan->setXRange(0,data->xSize()*scale-w);
pan->setYRange(0,data->ySize()*scale-h);
if (data->xSize()*scale<w) xh=1.0;
else xh=w/(data->xSize()*scale);
if (data->ySize()*scale<h) yh=1.0;
else yh=h/(data->ySize()*scale);

pan->setHandleSize(xh,yh);

}

void Control::imDown(QMouseEvent *e) {
int n=imview->nShapes();
int i;
Shape *s;

if (curshape!=-1) printf("Down before up!\n");

mx0=e->x();
my0=e->y();

imview->select.type=Shape::NONE;

switch (mode) {
case MEASURE:
	s=new Shape;
	s->type=Shape::LINE;
	s->x1=s->x0=imview->pxtodx(e->x());
	s->y1=s->y0=imview->pytody(e->y());
	s->text="";
	s->color=red;
	curshape=imview->addShape(s);
	break;
case SELECT:
	// see if click was inside existing box
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) {
			curshape=i;
			sx0=s->x0;
			sy0=s->y0;
			break;
		}
	}
	if (i!=n) {
		imview->hilight(curshape);
		break;
	}
	// if not, make a new one
	curshape=newClip(imview->pxtodx(e->x())-boxsize/2,imview->pytody(e->y())-boxsize/2,
		boxsize,boxsize);
	sx0=imview->pxtodx(e->x())-boxsize/2;
	sy0=imview->pytody(e->y())-boxsize/2;
	imview->hilight(curshape);
//printf("select %d %d   %d %d\n",mx0,my0,sx0,sy0);
	break;
case DELETE:
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) break;
	}
	if (i==n) break;
	delClip(i);
	redisplayBoxes();
	break;
case DRAW:
	int j,k,nx,ny,xo,yo,x0,x1,y0,y1;
	float *d;
	nx=data->xSize();
	ny=data->ySize();
	
	xo=imview->pxtodx(e->x());
	yo=imview->pytody(e->y());

	x0=xo-boxsize/2;
	x1=xo+boxsize/2;
	y0=yo-boxsize/2;
	y1=yo+boxsize/2;
	
	if (x0<0) x0=0;
	if (y0<0) y0=0;
	if (x1>nx-1) x1=nx-1;
	if (y1>ny-1) y1=ny-1;

	d=data->getDataRO();
	for (j=y0; j<y1; j++) {
		for (i=x0; i<x1; i++) {
			if ((e->state()&ControlButton) || hypot((float)i-xo,(float)j-yo)<boxsize/2 ) d[i+j*nx]=0;
		}
	}
	data->doneData();
	imview->imageChanged();
	break;
}

}

void Control::imMove(QMouseEvent *e) {
Shape *s;

if (curshape<0 && mode!=DRAW) return;

switch (mode) {
case MEASURE:
	s=imview->shapeAt(curshape);
	s->x1=imview->pxtodx(e->x());
	s->y1=imview->pytody(e->y());
	s->text.sprintf("%d pix %1.2f A",(int)hypot(s->x1-s->x0,s->y1-s->y0),hypot(s->x1-s->x0,s->y1-s->y0)*apix);
	imview->imageChanged();
	break;
case SELECT:
	s=imview->shapeAt(curshape);
	s->x0=(int)floor((e->x()-mx0)/imview->lscale+sx0);
	s->y0=(int)floor(-(e->y()-my0)/imview->lscale+sy0);
	imview->imageChanged();
	imview->hilight(curshape);
	break;
case DELETE:

	return;
case DRAW:
	int i,j,k,nx,ny,xo,yo,x0,x1,y0,y1;
	float *d;
	nx=data->xSize();
	ny=data->ySize();
	
	xo=imview->pxtodx(e->x());
	yo=imview->pytody(e->y());

	x0=xo-boxsize/2;
	x1=xo+boxsize/2;
	y0=yo-boxsize/2;
	y1=yo+boxsize/2;
	
	if (x0<0) x0=0;
	if (y0<0) y0=0;
	if (x1>nx-1) x1=nx-1;
	if (y1>ny-1) y1=ny-1;

	d=data->getDataRO();
	for (j=y0; j<y1; j++) {
		for (i=x0; i<x1; i++) {
			if ((e->state()&ControlButton) || hypot((float)i-xo,(float)j-yo)<boxsize/2 ) d[i+j*nx]=0;
		}
	}
	data->doneData();
	imview->imageChanged();
	break;
}

}

void Control::imUp(QMouseEvent *e) {
Shape *s;
EMData *d;

if (curshape<0) return;

switch (mode) {
case MEASURE:
	imview->delShape(curshape);
	curshape=-1;
	break;
case SELECT:
	s=imview->shapeAt(curshape);
	s->x0=(int)floor((e->x()-mx0)/imview->lscale+sx0);
	s->y0=(int)floor(-(e->y()-my0)/imview->lscale+sy0);
	imview->imageChanged();
	boxMoved(curshape);
	redisplayBoxes(curshape);
	curshape=-1;
	break;
case DELETE:
	return;
}


}

void Control::boxMoved(int n) {
	Shape *s;
	EMData *d;

	s=imview->shapeAt(n);
	d=data->clip(s->x0,s->y0,s->x1,s->y1);
	d->ufloat=d->Sigma();
	d->normalize();
	clipview->replaceData(n,d);
	clipview->makeVisible(n);
	ptcl.remove(n);
	ptcl.insert(n,d);
}

void Control::redisplayBoxes(int n) {
clipview->viewport()->update();

}

#define AUTOCLIP 1024
void Control::Mfocalauto() {
int r=QMessageBox::information(NULL,"Important note","This option will take box positions from one image in\
 a focal pair\nand adjust the box positions to fit the second image. The machine MUST have enough\n\
 memory to hold BOTH micrographs at once. To use this option, you should have already boxed\n\
and saved the particles from the first image. When you close this window, a\n\
file dialog will appear. Select the OTHER image (the one you haven't boxed). Boxer will\n\
adjust the box positions, and the new image will be displayed on the screen.");
 
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

int nx=data->xSize(),ny=data->ySize();
QProgressDialog progress( "Realigning boxes", 0, ((nx-1)/AUTOCLIP+1)*((ny-1)/AUTOCLIP+1)+1, this, 0 ,TRUE );
progress.setProgress( 0 );

EMData *pair=new EMData;
EMData *clip, *clipp, *clipc, *t;
if (pair->readImage(s,0)) return;
pair->normalize();
int nx2=pair->xSize(),ny2=pair->ySize();
printf("Read image '%s'\n",(const char *)s);

int i,j,k,pr=1;
for (i=0; i<imview->nShapes(); i++) imview->shapeAt(i)->user=0;

float da,bda,bdx,bdy,pk;
for (i=0; i<nx; i+=AUTOCLIP) {
	for (j=0; j<ny; j+=AUTOCLIP) {
		progress.setProgress(pr++);
		Olist best(3);
		if (i+AUTOCLIP>nx) i=nx-AUTOCLIP;
		if (j+AUTOCLIP>ny) j=ny-AUTOCLIP;
		t=data->clip(i,j,AUTOCLIP,AUTOCLIP);
		t->medianShrink(2);
		t->filter(3.0,AUTOCLIP/8,1);
		clip=t->clip(AUTOCLIP/16,AUTOCLIP/16,AUTOCLIP*3/8 ,AUTOCLIP*3/8);
		clip->applyMask(-1,4);
		clipc=clip->copy();
		delete t;

		t=pair->clip(i,j,AUTOCLIP,AUTOCLIP);
		t->medianShrink(2);
		t->filter(3.0,AUTOCLIP/8,1);
		clipp=t->clip(AUTOCLIP/16,AUTOCLIP/16,AUTOCLIP*3/8 ,AUTOCLIP*3/8);
		clipp->applyMask(-1,4);
		clipp->normalize();

		for (da=-10.0*PI/180.0; da<10.0*PI/180.0; da+=2.0*PI/180.0) {
			clipc->setRAlign(da);
			clipc->setTAlign(0,0,0);
			clipc->rotateAndTranslate();
			clipc->normalize();
			clipc->transAlign(clipp,0,1,nx/4,&pk);
			best.insert(pk,clipc->Dx(),clipc->Dy(),da);
		}
		bda=best.da[0];
		for (da=bda-2.0*PI/180.0; da<bda+2.0*PI/180.0; da+=.5*PI/180.0) {
			clipc->setRAlign(da);
			clipc->setTAlign(0,0,0);
			clipc->rotateAndTranslate();
			clipc->normalize();
			clipc->transAlign(clipp,0,1,nx/4,&pk);
			best.insert(pk,clipc->Dx()*cos(da)+clipc->Dy()*sin(da),
				-clipc->Dx()*sin(da)+clipc->Dy()*cos(da),da);
		}
		bda=best.da[0];
		bdx=best.dx[0]*2;
		bdy=best.dy[0]*2;
		
		printf("(%d,%d)\t%1.1f (%1.0f,%1.0f)\n",i,j,bda*180.0/PI,best.dx[0],best.dy[0]);

		float mx0=cos(bda);
		float mx1=sin(bda);
		Shape *sh;
		float x,y;
		for (k=0; k<imview->nShapes(); k++) {
			sh=imview->shapeAt(k);
			if (sh->user) continue;
			x=sh->x0-i-AUTOCLIP/2+sh->x1/2+bdx;
			y=sh->y0-j-AUTOCLIP/2+sh->y1/2+bdy;
			if (x<-AUTOCLIP/2||x>AUTOCLIP/2 || y<-AUTOCLIP/2||y>AUTOCLIP/2) continue;
			sh->x0=(x*mx0-y*mx1)+i+AUTOCLIP/2-sh->x1/2;
			sh->y0=(x*mx1+y*mx0)+j+AUTOCLIP/2-sh->y1/2;
			sh->user=1;
//printf("%1.0f,%1.0f -> %d,%d\n",x+i+AUTOCLIP/2-sh->x1/2,y+j+AUTOCLIP/2-sh->y1/2,sh->x0,sh->y0);
			if (sh->x0<0 || sh->y0<0 || sh->x0+sh->x1>nx || sh->y0+sh->y1>ny) {
				delClip(k);
				k--;
			}
		}
		
		// for testing only
/*		clipc->setRAlign(bda);
		clipc->setTAlign(bdx/2,bdy/2,0);
		clipc->rotateAndTranslate();
		clipc->normalize();
		clipc->writeIMAGIC("tst.hed",-1);
		clipp->writeIMAGIC("tst.hed",-1);
		EMData *e=clipc->calcCCF(clipp);
		e->writeIMAGIC("tst.hed",-1);*/

		delete clipc;
		delete clip;
		delete clipp;
	}
}

delete data;
data=pair;

imview->setData(data);
imview->imageChanged();
imResized(imview->width(),imview->height());

for (k=0; k<imview->nShapes(); k++) boxMoved(k);

redisplayBoxes();
}

struct sRect { int x,y,x2,y2; };

// This was cut from another program, so it's sort of bizzare
void Control::adjustBoxes() {
int i,j,n=imview->nShapes();
sRect *rect;
float dx=0,dy=0,nm=0,ma=0,f,xAve=0, yAve=0, xNewAve=0.,yNewAve=0.;
int x0,y0;

rect=(sRect *)malloc(sizeof(sRect)*imview->nShapes());

// First read original (untranslated) box database file
//QString s( QFileDialog::getOpenFileName("Original box database file") );
QString s(boxname);
if ( s.isNull() ) return;
FILE *in=fopen(s.data(),"r");
if (!in) return;

for (i=0; fscanf(in," %d %d %d %d",&x0,&y0,&j,&j)==4; i++) {
	rect[i].x=x0;
	rect[i].y=y0;

	if (i>=n) { printf("You MUST use the original box database file\n"); exit(1); }
}
if (i<n) { printf(" %d != %d\n",i,n); return; }

fclose(in);

// Then set the adjusted box positions
for (i=0; i<n; i++) {
	rect[i].x2=imview->shapeAt(i)->x0;
	rect[i].y2=imview->shapeAt(i)->y0;
	if (rect[i].x==rect[i].x2 && rect[i].y==rect[i].y2) { rect[i].x2=rect[i].y2=-1; }
}

// now do a translational shift
nm=0;
for (i=0; i<n; i++) {
	if (rect[i].x2>=0 || rect[i].y2>=0) {
		xAve +=(float)rect[i].x;     // average positions before
		yAve +=(float)rect[i].y;
		xNewAve += (float)rect[i].x2; // ave positions after
		yNewAve += (float)rect[i].y2;
		nm+=1.0;
	}
}
xAve =   xAve/nm;
yAve =   yAve/nm;
xNewAve = xNewAve/nm;
yNewAve = yNewAve/nm;

float OOXX=0,  OOXY=0,  OOYX=0,  OOYY=0;  // old old matrix
float NOXX=0,  NOXY=0,  NOYX=0,  NOYY=0;  // new new matrix
float M11, M12, M21, M22, deltaX, deltaY;
float x1, y1, x2, y2;


if (nm==0) { printf("None of the boxes moved. Nothing to do.\n"); return; }

if (nm==1) { 
	//printf("Only doing translations");
	M11 =1; M12 =0; M21 =0; M22 = 1;
}

if (nm==2) { 
	//printf("Only doing translations, rotation, rescale");
	for (i=0; i<n; i++) {
		if (rect[i].x2>=0 || rect[i].y2>=0) {
			x1  = (rect[i].x  - xAve);
			x2  = (rect[i].x2 - xNewAve);
			y1  = (rect[i].y  - yAve);
			y2  = (rect[i].y2 - yNewAve);
			continue;
		}
	}
	M11 = (x1*x2+y1*y2)/(x1*x1+y1*y1) ; 
	M12 = (y1*x2-y2*x1)/(x1*x1+y1*y1) ; 
	M21 = -M12; 
	M22 =  M11;
}

// Change in coordinate is NewAve - Ave + (M-1)(Old_i -Ave)

if (nm>=3) { 
	//printf("Doing Full Linear Transformation and Translation");
	for (i=0; i<n; i++) {
		if (rect[i].x2>=0 || rect[i].y2>=0) {
			OOXX +=  (rect[i].x  -xAve   )*(rect[i].x -xAve); 
			OOXY +=  (rect[i].x  -xAve   )*(rect[i].y -yAve); 
			OOYX +=  (rect[i].y  -yAve   )*(rect[i].x -xAve); 
			OOYY +=  (rect[i].y  -yAve   )*(rect[i].y -yAve); 
			NOXX +=  (rect[i].x2 -xNewAve)*(rect[i].x -xAve); 
			NOXY +=  (rect[i].x2 -xNewAve)*(rect[i].y -yAve); 
			NOYX +=  (rect[i].y2 -yNewAve)*(rect[i].x -xAve); 
			NOYY +=  (rect[i].y2 -yNewAve)*(rect[i].y -yAve); 
			continue;
		}
		M11 = ( NOXX*OOYY-NOXY* OOYX) /(OOXX*OOYY - OOXY*OOYX) ; 
		M12 = (-NOXX*OOXY+NOXY* OOXX) /(OOXX*OOYY - OOXY*OOYX)  ; 
		M21 = ( NOYX*OOYY-NOYY* OOYX) /(OOXX*OOYY  - OOXY*OOYX); 
		M22 = (-NOYX*OOXY+NOYY* OOXX) /(OOXX*OOYY  - OOXY*OOYX); 
	}
}

	for (i=0; i<n; i++) {
		deltaX = rect[i].x - xAve;
		deltaY = rect[i].y - yAve;
		rect[i].x += xNewAve  -xAve + (M11-1)*deltaX +  M12*deltaY;
		rect[i].y += yNewAve  -yAve +  M21 *deltaX   + (M22-1)*deltaY;
		rect[i].x2 = -1;
		rect[i].y2 = -1;
	}

// now write the shifted boxes back to the original array
for (i=j=0; i<n; i++,j++) {
	imview->shapeAt(i)->x0=rect[j].x;
	imview->shapeAt(i)->y0=rect[j].y;
//printf("%d %d   %d %d\n",rect[j].x,rect[j].y,rect[j].x+imview->shapeAt(i)->x1,rect[j].y+imview->shapeAt(i)->y1);
	if (rect[j].x<0 || rect[j].y<0 || rect[j].x+imview->shapeAt(i)->x1>data->xSize() 
			|| rect[j].y+imview->shapeAt(i)->y1>data->ySize() ) {
		delClip(i);
		i--;
		n--;
	}
	else boxMoved(i);
}
imview->imageChanged();

redisplayBoxes();
free(rect);
}

// this defines how large an area to autobox at once (*2)
#define CLIP 512

void Control::autoBox1() {
EMData *d1,*d2;
AutoboxD *ab;
int px,py;
Shape *s;

if (ptcl.count()<1) {
	QMessageBox::information(this,"Usage","Before selecting Autobox, manually pick\na few representative particles representing various views.",
		"Ok");
	return;
}

QApplication::setOverrideCursor( waitCursor );

abptcl.clear();
abvec.clear();
absig=0;
for (d1=ptcl.first(); d1; d1=ptcl.next()) {

	absig+=d1->ufloat;

	abvec.append(new RVEC(d1));
	d1->radialAverage();
	d1->applyMask(-1,4);
	d2=d1->copy();
	d2->setParent(NULL);
	d2->medianShrink(2);
	abptcl.append(d2);
}
absig/=abptcl.count();

abmm=data->Max()-data->Min();

s=imview->shapeAt(0);
px=s->x0+s->x1/2-512;
py=s->y0+s->y1/2-512;
if (px<0) px=0;
if (py<0) py=0;
if (px>data->xSize()-1024) px=data->xSize()-1025;
if (py>data->ySize()-1024) py=data->ySize()-1025;
d1=ptclCorr(px,py);

//d1->writeMRC("tstb.mrc");
ab=new AutoboxD();
ab->setup(px,py,d1->Min(),d1->Max(),d1,this);
ab->show();
QApplication::setOverrideCursor( arrowCursor );
}

void Control::autoBoxM() {
EMData *d1,*d2;
AutoboxD *ab;
int px,py,i;
Shape *s;
List *refs;


QString f( QFileDialog::getOpenFileName() );
if ( f.isNull() ) return;
refs=readImages(f.data(),-1,-1);

QApplication::setOverrideCursor( waitCursor );
if (refs->count()<1) return;

abptcl.clear();
abvec.clear();
absig=0;
for (i=0; i<refs->count(); i++) {
	d1=(EMData *)refs->objectAt(i);
	d1->applyMask(-1,4);
	
	absig+=d1->ufloat;

	d2=d1->copy(0,0);
	d2->medianShrink(2);
	abptcl.append(d2);
	delete d1;
}
delete refs;

absig/=abptcl.count();
abmm=data->Max()-data->Min();

s=imview->shapeAt(0);
if (s) {
	px=s->x0+s->x1/2-512;
	py=s->y0+s->y1/2-512;
	if (px<0) px=0;
	if (py<0) py=0;
	if (px>data->xSize()-1024) px=data->xSize()-1025;
	if (py>data->ySize()-1024) py=data->ySize()-1025;
}
else {
	px=py=0;
}
d1=ptclCorr(px,py);

//d1->writeMRC("tstb.mrc");
ab=new AutoboxD();
ab->setup(px,py,d1->Min(),d1->Max(),d1,this);
ab->show();
QApplication::setOverrideCursor( arrowCursor );
}

void Control::autoBox2(float *thresh) {
EMData *d1,*d2,*da;
int i,j,nx,ny;

nx=(data->xSize()-boxsize*2)/(CLIP*2-boxsize)+1;
ny=(data->ySize()-boxsize*2)/(CLIP*2-boxsize)+1;

delAll();

QProgressDialog progress( "Autoboxing", "Abort", nx*ny, this, 0 ,TRUE );
progress.setProgress( 0 );

for (i=0; i<ny; i++) {
	for (j=0; j<nx; j++) {
		progress.setProgress( i*nx+j );
		d1=ptclCorr(j*(CLIP*2-boxsize),i*(CLIP*2-boxsize));
		boxesFrom(d1,j*(CLIP*2-boxsize),i*(CLIP*2-boxsize),0,0,thresh);
		delete d1;
		if ( progress.wasCancelled() ) return;
	}
}

}

void Control::boxesFrom(EMData *map,int x0,int y0,int sx,int sy,float *thresh) {
int i,j,k,l,c,n=0,mn;
float *d,*d2,da1,da2,bd;
EMData *dt,*dtc;

if (!map) return;
clipview->setUpdatesEnabled(FALSE);

//if (x0==0 && y0==0) delAll();

// sx and sy allows overlaping areas to be dealt with
if (sx<boxsize/4) sx=boxsize/4;
if (sy<boxsize/4) sy=boxsize/4;

mn=4*CLIP*CLIP*4/(boxsize*boxsize);

d=map->getData();
for (i=sx; i<CLIP-boxsize/4; i++) {
	for (j=sy; j<CLIP-boxsize/4; j++) {
		if (d[i+j*CLIP]>thresh[0]) {
			if (n>mn ||i*2+x0>data->xSize()-boxsize || j*2+y0>data->ySize()-boxsize) continue;
			dt=data->clip(i*2+x0-boxsize/2,j*2+y0-boxsize/2,boxsize,boxsize);

			// here we check for particles with large 'drawn' areas and eliminate them
			d2=dt->getDataRO();	
			c=0;
			for (l=0; l<boxsize*boxsize; l++) {
				if (d2[l]==0) c++;
			}
			dt->doneData();
			if (c>boxsize*2) {
				delete dt;
				continue;
			}			

			// use the second & third thresholds as a sigma variation tolerance
			dtc=dt->clip(boxsize/4,boxsize/4,boxsize/2,boxsize/2);
			da1=SQR((1.0-thresh[1])/2.0);
			da2=SQR((1.0-thresh[2])/2.0)*absig;
//			printf("%f %f %f\n",dtc->Max()-dtc->Min(),abmm,da1);
			if ((dtc->Max()-dtc->Min())>da1*abmm || dtc->Sigma()<absig-da2) {
				delete dt;
				delete dtc;
				continue;
			}
			delete dtc;

			// now we deal with the final threshold. This one checks to make
			// sure that the partical has reasonable azimuthal variations
			// the first threshold already dealt with radial variations
			RVEC rv(dt),*v2;
			bd=0;
			for (v2=abvec.first(); v2; v2=abvec.next()) bd=maxf(bd,v2->dot(rv));
			if (bd<thresh[3]) {
				delete dt;
				continue;
			}

			// ok, we've got a good particle			
			k=newClip(i*2+x0-boxsize/2,j*2+y0-boxsize/2,boxsize,boxsize);
			dt->normalize();
			ptcl.remove(k);
			ptcl.insert(k,dt);
			clipview->addData(dt);
			n++;
		}
	}
}
map->doneData();
clipview->setUpdatesEnabled(TRUE);
redisplayBoxes();
}



EMData *Control::ptclCorr(int xo,int yo) { 
EMData *dc,*dd,*ccf,*ref,*dt;
int i,j,ii,jj,k,x1,y1;
float *dp,*ddp,t,m,s;

// a little padding to get rid of edge effects
dd=data->clip(xo-128,yo-128,CLIP*2+256,CLIP*2+256);
dd->medianShrink(2);

// Now we fill in those padded zero regions with decent values
dp=dd->getDataRO();

dc=dd->clip(64,64,CLIP,6);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=0; j<64; j++) {
	for (i=0; i<CLIP+128; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dc=dd->clip(64,64+CLIP-6,CLIP,6);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=CLIP+64; j<CLIP+128; j++) {
	for (i=0; i<CLIP+128; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dc=dd->clip(64,64,6,CLIP);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=64; j<64+CLIP; j++) {
	for (i=0; i<64; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dc=dd->clip(64+CLIP-6,64,6,CLIP);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=64; j<64+CLIP; j++) {
	for (i=CLIP+64; i<CLIP+128; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dd->doneData();
dd->update();

//dd->writeIMAGIC("xxx.hed",-1);

dd->filter(3,10000,1);		// low pass filter to clean up gradients
dc=dd->clip(64,64,CLIP,CLIP);
delete dd;

dd=dc->copy(0,0);
dd->zero();

for (i=0; i<abptcl.count(); i++) {
//	ptcl.at(i)->writeIMAGIC("ref.hed",-1);
	ref=abptcl.at(i)->clip(-(CLIP-boxsize/2)/2,-(CLIP-boxsize/2)/2,CLIP,CLIP);
	ref->normalize();
	ref->toCorner();
	ccf=dc->calcCCF(ref);

	dp=ccf->getDataRO();
	ddp=dd->getData();
	for (j=0; j<CLIP*CLIP; j++) {
		if (dp[j]>ddp[j]) ddp[j]=dp[j];
	}
	ccf->doneData();
	dd->doneData();
	
	delete ccf;
	delete ref;
}

//dc->writeMRC("tsta.mrc");
//dd->writeMRC("tstb.mrc");

// This does a low pass filter so are peaks are guarenteed to be separated from each other
//dd->filter(0,CLIP/(boxsize*0.5554),5);	// This is a 'perfect' 1/2 box filter
//dd->filter(0,CLIP*8.0/(boxsize*.5554),5);
dd->filter(0,CLIP/6.0,5);
t=dd->Max();
dp=dd->getData();

// first, we locate peaks
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>dp[k-1] && dp[k]>dp[k+1] && dp[k]>dp[k+CLIP] && dp[k]>dp[k-CLIP] && dp[k]>0 &&
			dp[k]>dp[k-1-CLIP] && dp[k]>dp[k+1-CLIP] && dp[k]>dp[k-1+CLIP] && dp[k]>dp[k+1+CLIP]) {
				dp[k]+=t;
		}
	}
}

// Now we eliminate boxes which are too close to each other
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>t) {
			x1=i+boxsize/4-1;
			y1=j+boxsize/4-1;
			for (ii=i-boxsize/4; ii<x1; ii++) {
				for (jj=j-boxsize/4; jj<y1; jj++) {
					if (dp[ii+jj*CLIP]>dp[k]) { dp[k]=0; break; }
				}
			}

		}
	}
}


/*
dd->doneData();
dd->writeMRC("tstc.mrc");
dp=dd->getData();
*/

// now we eliminate everything but the peaks
for (i=1; i<CLIP-1; i++) {
	for (j=1; j<CLIP-1; j++) {
		k=i+j*CLIP;
		if (dp[k]<=t) dp[k]=0; 
		else {
			dp[k]-=t;
		}
	}
}

dd->doneData();

delete dc;
//delete dd;
return dd;
}


/*
#define CLIP 512		// 1/2 clip size


void Control::autoBox2() { 
EMData *d1,*d2,*d3,*d4,*da,*dc;
float *y,*y2,*y3,*y4,m,mf2,r,a,rm;
int nx,i,j,mf;

// calculate average fft of 'background'
d1=ptcl.first();
nx=d1->xSize();
d1->edgeNormalize();
d1->applyMask(-1,4);
d1->normalize();
da=d1->doFFT()->copy();

while (d1=ptcl.next()) {
	d1->edgeNormalize();
	d1->applyMask(-1,4);
	d1->normalize();
	d2=d1->doFFT();
	da->addIncoherent(d2);
}

delAll();

y=malloc(26*sizeof(float));
y2=malloc(26*sizeof(float));
abft->calcRadDist(25,3.0,(nx-3.0)/35.0,y);
da->calcRadDist(25,3.0,(nx-3.0)/35.0,y2);

//abft->writeMRC("tsta.mrc");
//da->writeMRC("tstb.mrc");

delete da;
delete abft;
abft=NULL;

// find the frequency with the most information
mf=0;
m=-1.0e20;
for (i=0; i<25; i++) {
	if (y[i]-y2[i]>m) { m=y[i]-y2[i]; mf=i; }
}

//save_data(3.0,(nx-3.0)/35.0, y, 25, "t1");
//save_data(3.0,(nx-3.0)/35.0, y2, 25, "t2");

free(y);
free(y2);

mf2=(mf+3)*(nx-3.0)/35.0;
printf("Max freq=%d %f %f pix\n",mf,mf2,nx/mf2);
mf2=nx/mf2;

dc=data->clip(0,0,1024,1024);

da=new EMData;
da->setSize(CLIP*2,CLIP*2,1);
rm=SQR(.85*nx/2.0);

// generate cos*cos fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=cos(i*2.0*PI/mf2)*cos(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst1.mrc");
da->toCorner();
d1=dc->calcCCF(da);

// generate sin*sin fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=sin(i*2.0*PI/mf2)*sin(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst2.mrc");
da->toCorner();
d2=dc->calcCCF(da);

// generate sin*cos fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=sin(i*2.0*PI/mf2)*cos(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst3.mrc");
da->toCorner();
d3=dc->calcCCF(da);

// generate cos*sin fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=cos(i*2.0*PI/mf2)*sin(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst4.mrc");
da->toCorner();
d4=dc->calcCCF(da);


d1->ri2ap();
d2->ri2ap();

y=d1->getData();
y2=d2->getDataRO();
y3=d3->getDataRO();
y4=d4->getDataRO();
for (i=0; i<1024*1024; i++) {
	y[i]=sqrt(y[i]*y[i]+y2[i]*y2[i]+y3[i]*y3[i]+y4[i]*y4[i]);
}
d1->doneData();
d2->doneData();
d3->doneData();
d4->doneData();

d1->writeMRC("tsta.mrc");
dc->writeMRC("tstb.mrc");

delete d1;
delete d2;
delete d3;
delete d4;
delete dc;
delete da;
}

*/
