// Control.cpp  Copyright 1998 Steve Ludtke (4/98)
// This is the core object for the 'ctfit' package
// It primarily handles user interface operations. Most of the 
// real work is handed off to CtfData

#include "Control.h"
#include <qfiledlg.h>
#include <qapp.h>
#include <qfont.h>
#include <qcolor.h>
#include <qmsgbox.h> 
#include <qprogressdialog.h>
#include <qmessagebox.h>
#include <math.h>
#include "OneLiner.h"
#include "ColorListItem.h"

#define Inherited ControlData

extern QString dfdir;

QString simpdate(QDate &d) {
	QString ret;
	if (!d.isValid()) return("MM/DD/YY");
	ret=QString::number(d.month())+"/"+QString::number(d.day())+"/"+
		QString::number(d.year()>1999?d.year()-2000:d.year()-1900);
	return ret;
}

QDate simpdate(QString &dt) {
int y,m,d;
QDate ret=QDate::currentDate();

if (sscanf((const char *)dt,"%d/%d/%d",&m,&d,&y)!=3) return ret;
if (y<80) y+=2000;
if (y<100) y+=1900;
ret.setYMD(y,m,d);

return ret;
}


Control::Control(QWidget* parent,const char* name):Inherited( parent, name ) {
QPopupMenu *m1,*m2;

initMetaObject();

data=new QList<CtfData>;
current=new CtfData;
data->append(current);

plotwin=new PlotCtf(NULL,"CTF Plot");
plotwin->setData(data);
plotwin->show();
plotwin->newplots();

imgwin=new Image;
imgwin->resize(300,300);
imgwin->show();

// This restores the window geometries
FILE *in = in=DFopenr("ctfitwin");
if (in) {
	int x,y,w,h;

#ifndef SGI
	if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) move(x,y);
	if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) plotwin->setGeometry(x,y,w,h);
	if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) imgwin->setGeometry(x,y,w,h);
#endif
	fclose(in);
}

ignore=lastdata=0;
curgroup=-1;
mem.setAutoDelete(TRUE);

// FILE MENU
m1=new QPopupMenu(NULL,"File");
menu->insertItem("File",m1,100);
//m1->insertItem("Open Whole Micrograph",this,SLOT(Mopenmicro()),ALT+Key_O);
//m1->insertItem("Split focal pair",this,SLOT(Msplitmicro()),0);
m1->insertItem("Open Single Image(s)",this,SLOT(Mopen1clip()),0);
m1->insertItem("Open Particle Set",this,SLOT(Mopenclips()),0);
m1->insertItem("Split Particle Set",this,SLOT(MopenclipsS()),0);
m1->insertItem("Open Set (oversample)",this,SLOT(Mopenclipsos()),0);
m1->insertItem("Open Real FFT",this,SLOT(MopenRealFFT()),0);
m1->insertItem("Struc. Fac. -> Image",this,SLOT(Mopenstruc()),0);
m1->insertItem("Save FFT to MRC",this,SLOT(Msavefft()));
m1->insertItem("Save Plot as Spreadsheet",this,SLOT(Msavedata()));
m1->insertSeparator();
m1->insertItem("Print",this,SLOT(Mprint()),ALT+Key_P);
m1->insertSeparator();
m1->insertItem("Exit",this,SLOT(Mquit()),ALT+Key_X);

// PROCESS MENU
m1=new QPopupMenu(NULL,"Process");
menu->insertItem("Process",m1);
m1->insertItem("Phase Correct",this,SLOT(Mfixset()),0);
m1->insertItem("Set CTF Parms",this,SLOT(Msetparm()),0);
m1->insertItem("Focal Pair Merge",this,SLOT(Mpair()),0);
m2=new QPopupMenu(NULL,"Fit");
m1->insertItem("Fit",m2);
m2->insertItem("Generic Fit",this,SLOT(doFitQ()),0);
m2->insertItem("Fit with x-ray",this,SLOT(doFitX()),0);
m2=new QPopupMenu(NULL,"Simulate");
m1->insertItem("Simulate",m2);
m2->insertItem("Apply CTF",this,SLOT(Mapply()),0);
m2->insertItem("Apply CTF 3D",this,SLOT(Mapply3d()),0);
m2->insertItem("Simple CTF Cor 2D/3D",this,SLOT(Munapply()),0);
m2->insertItem("Add noise",this,SLOT(Mnoise()),0);
m2->insertItem("Set Struc Fac. 3D",this,SLOT(MsetSF()),0);
m2=new QPopupMenu(NULL,"Advanced");
m1->insertItem("Advanced",m2);
m2->insertItem("Edit User Fn",this,SLOT(Mseteqn()),0);
m2->insertItem("Save Average Profile",this,SLOT(Msaveavg()),0);
m2->insertItem("Wiener Filter",this,SLOT(MWFilter()),0);

menu->insertSeparator();

// HELP MENU
m1=new QPopupMenu(NULL,"Help");
menu->insertItem("Help",m1);
m1->insertItem("Help",this,SLOT(Mhelp()),0);
m1->insertItem("About CTFIT",this,SLOT(Mabout()),0);

df->setMin(-5.0);
df->setMax(-.01);
envel->setMax(20.0);
amp->setMax(5.0);
ampc->setMax(100.0);
expa->setMax(10.0);
expw->setMax(80.0);
gaussa->setMax(10.0);
gaussw->setMax(25.0);
stigang->setMin(0);
stigang->setMax(360.0);
stigdf->setMin(-3.0);
stigdf->setMax(0);

//microscope->insertItem("");
//specimen->insertItem("");
//scanner->insertItem("");

#ifdef CHIU
microscope->insertItem("JEOL 1200 (chiu)");
microscope->insertItem("JEOL 4000 (chiu)");
microscope->insertItem("JEOL 3000 (chiu)");
scanner->insertItem("Zeiss");
scanner->insertItem("PDS");
scanner->insertItem("CCD (4000)");
scanner->insertItem("CCD (3000)");
specimen->insertItem("Default");
#else
microscope->insertItem("Default");
scanner->insertItem("Default");
specimen->insertItem("Default");
#endif

initMem();
envbflag=0;

QObject::connect(df,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(envel,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(amp,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(ampc,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(exps4,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(expa,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(expw,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(gaussa,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(gaussw,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(stigang,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(stigdf,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
//QObject::connect(ptclsize,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));

QObject::connect(sfnone,SIGNAL(clicked()),this,SLOT(setsfnone()));
QObject::connect(sfsph,SIGNAL(clicked()),this,SLOT(setsfsph()));
QObject::connect(sffile,SIGNAL(clicked()),this,SLOT(setsffile()));

QObject::connect(voltage,SIGNAL(returnPressed()),this,SLOT(vchange()));
QObject::connect(Cs,SIGNAL(returnPressed()),this,SLOT(vchange()));
QObject::connect(mag,SIGNAL(returnPressed()),this,SLOT(vchange()));
QObject::connect(apix,SIGNAL(returnPressed()),this,SLOT(apixchange()));
QObject::connect(newapix,SIGNAL(clicked()),this,SLOT(magchange()));

QObject::connect(ebtog,SIGNAL(clicked()),this,SLOT(ebTog()));

QObject::connect(datanewsim,SIGNAL(clicked()),this,SLOT(dataNewsim()));
QObject::connect(datadel,SIGNAL(clicked()),this,SLOT(dataDel()));
QObject::connect(datalist,SIGNAL(highlighted(int)),this,SLOT(dataHi(int)));
QObject::connect(datalist,SIGNAL(selected(int)),this,SLOT(dataSel(int)));
QObject::connect(dataname,SIGNAL(returnPressed()),this,SLOT(newDataName()));

QObject::connect(Memset,SIGNAL(clicked()),this,SLOT(memSet()));
QObject::connect(memdel,SIGNAL(clicked()),this,SLOT(memDel()));
QObject::connect(memrcl,SIGNAL(clicked()),this,SLOT(memRcl()));
QObject::connect(memnew,SIGNAL(clicked()),this,SLOT(memNew()));
QObject::connect(memlist,SIGNAL(highlighted(int)),this,SLOT(memHi(int)));
QObject::connect(memlist,SIGNAL(selected(int)),this,SLOT(memRcl(int)));
QObject::connect(memname,SIGNAL(returnPressed()),this,SLOT(newMemName()));

//QObject::connect(dofit,SIGNAL(clicked()),this,SLOT(doFit()));
//QObject::connect(dofitq,SIGNAL(clicked()),this,SLOT(doFitQ()));

QObject::connect(imgwin,SIGNAL(sChange(float,float,float)),this,SLOT(newS(float,float,float)));

QObject::connect(group,SIGNAL(activated(const QString &)),this,SLOT(memGroup(const QString &)));
QObject::connect(microscope,SIGNAL(activated(const QString &)),this,SLOT(newMicroscope(const QString &)));
QObject::connect(specimen,SIGNAL(activated(const QString &)),this,SLOT(newSpecimen(const QString &)));
QObject::connect(scanner,SIGNAL(activated(const QString &)),this,SLOT(newScanner(const QString &)));

QObject::connect(mag,SIGNAL(returnPressed()),this,SLOT(dbchange()));
QObject::connect(comment,SIGNAL(returnPressed()),this,SLOT(dbchange()));
QObject::connect(dose,SIGNAL(returnPressed()),this,SLOT(dbchange()));
QObject::connect(date,SIGNAL(returnPressed()),this,SLOT(dbchange()));
QObject::connect(film,SIGNAL(returnPressed()),this,SLOT(dbchange()));
QObject::connect(nptcl,SIGNAL(returnPressed()),this,SLOT(dbchange()));

updateDList();

//printf("Df font: %s %d\n",QFont::defaultFont().family(),QFont::defaultFont().pointSize());
}

Control::~Control() {

//delete plotwin;
delete imgwin;
saveMem();
//printf("Memory saved\n");
}

// If the parameters that aren't used for calculations change...
void Control::dbchange() {
current->mag=mag->text().toFloat();
current->comment=comment->text();
current->dose=dose->text().toFloat();
current->date=simpdate(date->text());
current->filmno=film->text();
current->nptcl=nptcl->text().toInt();
}

void Control::newMicroscope(const QString &t) {
QString s;

s=t.stripWhiteSpace();
if (s.isEmpty() || s.isNull() || s=="(null)") return;

#ifdef CHIU
QString v,cs;
if (t=="JEOL 1200 (chiu)") { v="100"; cs="5.6"; }  
else if (t=="JEOL 4000 (chiu)") { v="400"; cs="4.1"; }
else if (t=="JEOL 3000 (chiu)") { v="300"; cs="1"; }
if (!v.isEmpty()) { 
	voltage->setText(v);
	Cs->setText(cs);
}
#endif

for (int i=0; i<microscope->count(); i++) {
	if (microscope->text(i)==s) { microscope->setCurrentItem(i); current->microscope=s; return; }
}
microscope->insertItem(s);
microscope->setCurrentItem(microscope->count()-1);
current->microscope=s;
}

void Control::newSpecimen(const QString &t) {
QString s;

s=t.stripWhiteSpace();
if (s.isEmpty() || s.isNull() || s=="(null)") return;

for (int i=0; i<specimen->count(); i++) {
	if (specimen->text(i)==s) { specimen->setCurrentItem(i); current->specimen=s; return; }
}
specimen->insertItem(s);
specimen->setCurrentItem(specimen->count()-1);
current->specimen=s;
}

void Control::newScanner(const QString &t) {
QString s;

s=t.stripWhiteSpace();
if (s.isEmpty() || s.isNull() || s=="(null)") return;

for (int i=0; i<scanner->count(); i++) {
	if (scanner->text(i)==s) { scanner->setCurrentItem(i); current->scanner=s; return; }
}
scanner->insertItem(s);
scanner->setCurrentItem(scanner->count()-1);
current->scanner=s;
}

void Control::ebTog() {

if (envbflag) {
	envbflag=0;
	ebtog->setText("Envelope:");
	envel->setMax(20.0);
	envel->setValue(sqrt(envel->Value())*2.0/PI);
	return;
}
else {
	envbflag=1;
	ebtog->setText("B:");
	envel->setMax(1000.0);
	envel->setValue(SQR(envel->Value()*PI/2.0));
	return;
}

}

void Control::updateDList() {
int i,j;
char s[80];

//printf("Update\n");
plotwin->setCurrent(current);
plotwin->newplots();

datalist->setAutoUpdate(FALSE);
datalist->setMultiSelection(TRUE);
datalist->clear();
for (i=0; i<data->count(); i++) {
//	sprintf(s,"%c%1.78s",data->at(i)->flags()&CDT_PLOT?'*':' ',
//		data->at(i)->getText());
//	datalist->insertItem(s);

	QString ss=data->at(i)->getText();
	j=ss.findRev('/',-1);
	if (j>0) j=ss.findRev('/',j-1);
	if (j>0) ss=ss.mid(j+1);
	sprintf(s,"%1.78s",(const char *)ss);
	if (data->at(i)->flags()&CDT_PLOT) 
		datalist->insertItem(new ColorListItem(s,data->at(i)->getColor()));
	else datalist->insertItem(new ColorListItem(s,white));
	if (data->at(i)==current) datalist->setCurrentItem(i);
}
datalist->setMultiSelection(FALSE);
datalist->setAutoUpdate(TRUE);
datalist->update();

}


void Control::Mopenmicro() { 
warn("Not implimented yet!");
return;
/*QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
QFile file(s);

EMData *d1,*d2;
CtfData *cd;

QApplication::setOverrideCursor( waitCursor );

d1=new EMData;
d1->readImage(s.data(),0);

if (d1->isComplex()) { d2=d1; d1=d2->doIFT(); d2->gimmeFFT(); }
else { d1->normalize(); d2=d1->doFFT(); d1->gimmeFFT(); }

cd=new CtfData(d1->ySize()>400?400:d1->ySize());
data->append(cd);

cd->copy(current);
current=cd;

cd->setRealImage(d1);
cd->setFFTImage(d2);

cd->setText(s.data());

updateDList();

QApplication::setOverrideCursor( arrowCursor );
*/
}

void Control::Msplitmicro() { 
EMData *d1,*d2,*c1,*c2,*c3;
int x,y,w,h,dx,dy;

warn("Not implimented yet!");
return;
QString s1( QFileDialog::getOpenFileName() );
if ( s1.isNull() ) return;
QString s2( QFileDialog::getOpenFileName() );
if ( s2.isNull() ) return;

d1=new EMData;
d2=new EMData;
if (d1->readImage(s1,0)) { printf("Image 1 read failed\n"); return; }
if (d2->readImage(s2,0)) { printf("Image 2 read failed\n"); return; }

printf("Image 1 is %dx%d\n",d1->xSize(),d1->ySize());
printf("Image 2 is %dx%d\n",d2->xSize(),d2->ySize());

w=d1->xSize()<d2->xSize()?d1->xSize():d2->xSize();
h=d1->ySize()<d2->ySize()?d1->ySize():d2->ySize();

c1=d1->clip(w/2-512,h/2-512,1024,1024);
c2=d2->clip(w/2-512,h/2-512,1024,1024);

c3=c1->RTFAlign(c2,NULL);

c1->writeMRC("x1.mrc");
c2->writeMRC("x2.mrc");
c3->writeMRC("x3.mrc");

dx=(int)c3->Dx();
dy=(int)c3->Dy();

delete d1;
delete d2;
}

// This will read in a set of single image files
void Control::Mopen1clip() { 
EMData *d1,*d2,*d3;
CtfData *cd;
int i,n,nx,ny;
char *ss;

QStringList s(QFileDialog::getOpenFileNames(NULL,dfdir));
n=s.count();
printf("%d\n",n);
for (i=0; i<n; i++) printf("%s\n",(const char *)s[i]);
if ( n==0 ) return;

QApplication::setOverrideCursor( waitCursor );


QProgressDialog progress( "Reading Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

d1=new EMData;
d1->readImage(s[0],0);
nx=d1->xSize();
ny=d1->ySize();

if (nx!=ny && !d1->isComplex() && (nx<1024 || ny<1024)) {
	printf("'%s' is not square and is less than 1024 pixels. %d != %d\n",(const char *)s[0],nx,ny);
	return;
}


ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());

// if input is complex, we can only handle 1 file
if (d1->isComplex()) {
	d3=d1;
	d1=NULL;
	cd=new CtfData(d3->ySize()>250?250:d3->ySize());
	data->append(cd);

	cd->copy(current);
	current=cd;
	newData();

	if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
	free(ss);

	cd->setRealImage(NULL);
	cd->setFFTImage(d3);

	cd->setText(s[0]);
	cd->setFile(s[0]);

	updateDList();

	QApplication::setOverrideCursor( arrowCursor );
	return;
}

if (nx!=ny) {
	printf("Large rectangular image, clipping 1k x 1k area from center\n");
	d3=d1->clip((nx-1024)/2,(ny-1024)/2,1024,1024);
	delete d1;
	d1=d3;
}
d1->applyMask(-1,6);
d1->normalize();
d3=d1->doFFT();
d1->gimmeFFT();
d3->multConst(d3->ySize());
d3->ri2ap();

for (i=1; i<n; i++) {
	d1->readImage(s[i],0);
	if (d1->xSize()!=nx || d1->ySize()!=ny) {
		printf("'%s' is the wrong size %d != %d || %d != %d\n",(const char *)s[i],d1->xSize(),nx,d1->ySize(),ny);
		continue;
	}
	d1->applyMask(-1,6);
	d1->normalize();
	d2=d1->doFFT();
	d1->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
d3->multConst(1.0/sqrt((float)n));
//d3->normalize();
//d3->ri2ap();
//d3->writeMRC("xxx.img");

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
data->append(cd);

cd->copy(current);
current=cd;
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

cd->setText(s[0]);
cd->setFile(s[0]);

updateDList();

QApplication::setOverrideCursor( arrowCursor );

}

void Control::MopenRealFFT() {
QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

EMData *d1,*d2;
CtfData *cd;
float *da1,*da2;
int i;

QApplication::setOverrideCursor( waitCursor );

d2=new EMData;
d2->readImage(s,0);

d1=new EMData;
d1->setComplex(1);
d1->setSize(d2->xSize()*2,d2->ySize(),1);

da1=d1->getData();
da2=d2->getDataRO();
for (i=0; i<d2->xSize()*d2->ySize(); i++) da1[i*2]=da2[i];
d1->doneData();
d2->doneData();

delete d2;

if (d1->isComplex()) { d2=d1; d1=d2->doIFT(); d2->gimmeFFT(); }
else { d1->normalize(); d2=d1->doFFT(); d1->gimmeFFT(); d2->multConst(d2->ySize()); }

d2->ri2ap();

cd=new CtfData(d1->ySize()>250?250:d1->ySize());
data->append(cd);

cd->copy(current);
current=cd;
newData();

cd->setRealImage(d1);
cd->setFFTImage(d2);

cd->setText(s);
cd->setFile(s);

updateDList();

QApplication::setOverrideCursor( arrowCursor );

}

void Control::Mopenstruc() { 
QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

EMData *d1,*d2;
CtfData *cd;
FILE *in;
char t[121];
int i,n=0,r0;
float *x,*y,ds;

QApplication::setOverrideCursor( waitCursor );

in=fopen(s,"r");
while (fgets(t,120,in)!=NULL) {
	if (sscanf(t," %f %f",&ds,&ds)==2) n++;
}
printf("Radial function has %d elements\n",n);
if (n==0) { warn("No data read\n"); return; }

rewind(in);
x=(float*)malloc(n*sizeof(float));
y=(float*)malloc(n*sizeof(float));
for (i=0; i<n; i++) {
	fgets(t,120,in);
	if (sscanf(t," %f %f",x+i,y+i)!=2) i--;
}
fclose (in);

ds=x[1]-x[0];
if (ds!=x[2]-x[1]) warn("Unevenly sampled function\nResult Invalid!");
r0=(int)floor(x[0]/ds+.5);
for (i=0; i<n; i++) x[i]=r0+i;

printf("Image size %d\n",2*(r0+n));

d1=new EMData;
d1->setSize((r0+n)*2+2,(r0+n)*2,1);
d1->setComplex(1);
d1->one();

d1->applyRadFn(n,r0,1,y,1);

free(x);
free(y);

d1->ri2ap();

cd=new CtfData((r0+n)>250?250:(r0+n));
data->append(cd);

cd->copy(current);
current=cd;
newData();

cd->setRealImage(NULL);
cd->setFFTImage(d1);
cd->setImParm(400.0,4.1,1.0/((r0+n)*ds*2.0),.25);

cd->setText(s);
cd->setFile(s);

cd->setFlags(CDT_STRUC|CDT_HASFFT);
cd->setCol(11,"sqrt(x10)*amp");
updateDList();

QApplication::setOverrideCursor( arrowCursor );

}


void Control::Mopenclips() { 
EMData *d1,*d2,*d3;
CtfData *cd;
int n;
char *ss;

QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

QApplication::setOverrideCursor( waitCursor );

n=fileCount(s);
if (n==0) return;

QProgressDialog progress( "Reading Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

d1=new EMData;
d1->readImage(s,0);
ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());
d1->applyMask(-1,6);
d1->normalize();
d3=d1->doFFT();
d1->gimmeFFT();
d3->multConst(d3->ySize());
d3->ri2ap();

for (int i=1; i<n; i++) {
	d1->readImage(s,i);
	d1->applyMask(-1,6);
	d1->normalize();
	d2=d1->doFFT();
	d1->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
d3->multConst(1.0/sqrt((float)n));
//d3->normalize();
//d3->ri2ap();
//d3->writeMRC("xxx.img");

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
data->append(cd);

cd->copy(current);
current=cd;
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

cd->setText(s);
cd->setFile(s);
cd->nptcl=n;
cd->filmno=s.mid(s.findRev('/',-1)+1);
cd->filmno.truncate(s.findRev('.',-1)-1);

updateDList();

QApplication::setOverrideCursor( arrowCursor );

}

void Control::MopenclipsS() { 
EMData *d1,*d2,*d3;
CtfData *cd;
int n,m,j;
char *ss;

QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

m=QMessageBox::information( this, "Ctfit","Select how many particles you'd like\n"
	"In each group?","50", "100", "1/9 total",2,0 );

QApplication::setOverrideCursor( waitCursor );

n=fileCount(s);
if (n==0) return;

if (m==0) m=50;
else if (m==1) m=100;
else m=n/9+1;

QProgressDialog progress( "Reading Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

for (j=0; j<n/m; j++) {
d1=new EMData;
d1->readImage(s,j*m);
ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());
d1->applyMask(-1,6);
d1->normalize();
d3=d1->doFFT();
d1->gimmeFFT();
d3->multConst(d3->ySize());

for (int i=j*m+1; i<(j+1)*m; i++) {
	d1->readImage(s,i);
	d1->applyMask(-1,6);
	d1->normalize();
	d2=d1->doFFT();
	d1->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
d3->multConst(1.0/sqrt((float)m));

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
data->append(cd);

cd->copy(current);
current=cd;
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

QString t;
t.sprintf("(%d-%d)",j*m,(j+1)*m-1);
if (s.contains('/')) t.prepend(s+(s.length()-s.findRev('/')));
else t.prepend(s);

cd->setFile(s);
cd->setText(t);
printf("%s read\n",(const char *)t);
}

updateDList();

QApplication::setOverrideCursor( arrowCursor );

}

// open a clip set with oversampling
void Control::Mopenclipsos() { 
EMData *d1,*d2,*d3,*dc;
CtfData *cd;
int n,os,sz;
char *ss;

QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);


n=fileCount(s);
if (n==0) return;

os=QMessageBox::information(this,"Query","How much oversampling would you like ?",
	"2x","3x","4x");
os+=2;

QApplication::setOverrideCursor( waitCursor );

QProgressDialog progress( "Transforming Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

d1=new EMData;
d1->readImage(s,0);
ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());
d1->applyMask(-1,6);
d1->normalize();
sz=d1->ySize();
dc=d1->clip(-sz*(os-1)/2,-sz*(os-1)/2,sz*os,sz*os);
//dc->normalize();
d3=dc->doFFT();
dc->gimmeFFT();
delete dc;
d3->multConst(d3->ySize());

for (int i=1; i<n; i++) {
	d1->readImage(s,i);
	d1->applyMask(-1,6);
	d1->normalize();
	dc=d1->clip(-sz*(os-1)/2,-sz*(os-1)/2,sz*os,sz*os);
//	dc->writeIMAGIC("tst.img",-1);
//	dc->normalize();
	d2=dc->doFFT();
	dc->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	delete dc;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
// This insures that the normalization is always the same
// The 'os' is to compensate for the empty padded area with no energy
// The 1.12... is sqrt(4/PI) to compensate for the circular mask
//d3->multConst(os*1.128379/sqrt((float)n));	
d3->multConst(os/sqrt((float)n));	


//d3->normalize();
//d3->ri2ap();
//d3->writeMRC("xxx.img");

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
data->append(cd);

cd->copy(current);
current=cd;
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

cd->setText(s);
cd->setFile(s);

updateDList();

QApplication::setOverrideCursor( arrowCursor );

}

void Control::Msavefft() {
if (!current->getFFTImage()) {
	warn("This entry doesn't have a FFT image!");
	return;
}
QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

current->getFFTImage()->writeMRC(s);

}

void Control::Msaveavg() {
QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

CtfData *cur;
DataSet *ds = current->getData();
Column *col = ds->getColumn(10);

int i,n = ds->rows();
float *d = (float *)calloc(sizeof(float),n);
float *d2;

for (cur=data->first(); cur; cur=data->next()) {
	if (!(cur->flags()&CDT_PLOT)) continue;
	ds=cur->getData();
	if (ds->rows()!=n) continue;
	col=ds->getColumn(14);
	d2=col->getData();
	for (i=0; i<n; i++) d[i]+=d2[i];
}

col=ds->getColumn(0);
save_data(ds->dataAt(0,0),ds->dataAt(0,1)-ds->dataAt(0,0),d,n,(const char *)s);
free(d);
}

void Control::Msavedata() {
QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

current->getData()->writeFile((const char *)s);

}

void Control::Mprint() { plotwin->print(); }

void Control::Mquit() { 
// This restores the window geometries
FILE *out = DFopenw("ctfitwin");
if (out) {
	fprintf(out,"%d %d %d %d\n",x(),y(),width(),height());
	fprintf(out,"%d %d %d %d\n",plotwin->x(),plotwin->y(),plotwin->width(),plotwin->height());
	fprintf(out,"%d %d %d %d\n",imgwin->x(),imgwin->y(),imgwin->width(),imgwin->height());
	fclose(out);	
}

saveMem(); 
exit(0); 
}

void Control::Mopenclipscf() { warn("Open carbon film clips not supported yet!"); }

void Control::MWFilter() {

if (current) current->WFilter();

}
void Control::Mfixset() { 

if (current) current->fixPhase();

}

void Control::Msetparm() { 

if (current) current->setParm();

}

void Control::Mpair() { warn("Focal pair merging not implimented yet!"); }

void Control::Mapply() { 
if (current) current->applyCtf();
}

void Control::Mapply3d() { 
if  (!current) return;

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

current->applyCtfTo(s,1);
}

void Control::MsetSF() { 
if  (!current) return;

QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

current->setSF(s);
}

void Control::Munapply() { 
if  (!current) return;

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

current->applyCtfTo(s,-1);
}

void Control::Mnoise() { 
if (current) current->applyCtf(1);
}

void Control::Mseteqn() {
DataSet *ds = current->getData();
Column *col = ds->getColumn(14);

QString neq = OneLiner::getOneLine("Enter a new equation using:\n\
x0=s, x1=gamma, x3=ctf amp, x5=noise inten,\n\
x10=img pow spec, x11=calc SF, x13=SF from file\n\
also Cs,lambda,defocus,B,amp,ampcont,exps4,noise(1,3,2,4)\n",
	 col->getEqn());

if (!neq.isEmpty()) {
	CtfData *cur;
	for (cur=data->first(); cur; cur=data->next()) {
		if (!(cur->flags()&CDT_PLOT)) continue;
		ds=cur->getData();
		col=ds->getColumn(14);
		col->setEqn((const char *)neq,current->steps(),ds); 
	}
}

}

void Control::Mhelp() {
	char s[512];
	sprintf(s,"netscape -remote 'openFile(%s/doc/progs/ctfit.html,new-window)'",
		EMDIR());
	run(s);
}

void Control::Mabout() { 
	QMessageBox::about(this,"About CTFIT",
 "CTFIT\nCopyright 1998 Steve Ludtke\n\n\
 Ctfit is designed to simulate transmission\n\
 electron microscope CTFs as well as determine\n\
 CTF parameters from micrographs. It is then\n\
 capable of doing phase correction of images\n\
 and storing parameters for external CTF\n\
 correction software.");
}

void Control::warn(char *msg) {
	QMessageBox::warning(this,"CTFIT Warning",msg,"Ok");
	return;
}

void Control::closeEvent(QCloseEvent *e) {
	e->accept();
	Mquit();
} 

// when recalculation is required
void Control::parmchange(float value) {
float B;

B=envel->Value();
if (!envbflag) B=pow(B*PI/2.0,2.0);

current->setImParm(atof(voltage->text()),atof(Cs->text()),atof(apix->text()),.5/atof(apix->text()));	
current->setSigParm(df->Value(),B, amp->Value(),ampc->Value()/100.0,exps4->Value());
current->setNoiseParm(expa->Value(),expw->Value(),gaussa->Value(),gaussw->Value());
current->setStigParm(stigang->Value(),stigdf->Value());
current->setStrucParm(ptclsize->text().toFloat());
current->updateEqn();

imgwin->setRad(current->getZeroR(),current->getZeroStigR(),current->stigang);

//plotwin->newplots();
//plotwin->replot();
}

// when V or Cs change
void Control::vchange() {
parmchange(0);
}

// when A/pix changes
void Control::apixchange() {
parmchange(0);
}

// when mag or scan change
void Control::magchange() {
float Mag,Scan;
char Apix[20];
QString str;

str=OneLiner::getOneLine("Enter scan step (um)","14",NULL);

if (sscanf((const char *)str," %f",&Scan)!=1) return;

sprintf(Apix,"%1.3f",Scan*10.0/current->mag);
apix->setText(Apix);

parmchange(0);
}

void Control::dataNewsim() {
CtfData *d;

d=new CtfData();
data->append(d);

d->copy(current);
current=d;

updateDList();
}

void Control::dataDel() {

data->remove(current);
updateDList();
}

void Control::newDataName() {

if (!current) return;
current->setText(dataname->text());
updateDList();
}

// When a new data set is created this makes sure the interface is consistent
// the new data set should be set to 'current'
void Control::newData() {
char s[20];

if (current==NULL) return;

df->setValueQ(current->defocus);
if (envbflag) envel->setValueQ(current->B);
else envel->setValueQ(sqrt(current->B)*2.0/PI);
amp->setValueQ(current->amp);
ampc->setValueQ(current->ampcont*100.0);
exps4->setValueQ(current->exps4);
expa->setValueQ(current->noise1);
expw->setValueQ(current->noise2);
gaussa->setValueQ(current->noise3);
gaussw->setValueQ(current->noise4);
stigang->setValueQ(current->stigang);
stigdf->setValueQ(current->stigdf);

sprintf(s,"%d",(int)floor(current->strucsize+.5));
ptclsize->setText(s);

sprintf(s,"%d",(int)floor(current->V+.5));
voltage->setText(s);

sprintf(s,"%3.1f",current->Cs);
Cs->setText(s);

sprintf(s,"%1.3f",current->sstep);
apix->setText(s);

sprintf(s,"%1.1f",1.0/current->smax);
// maxres->setText(s);

mag->setText(QString::number(current->mag,'f',1));
comment->setText(current->comment);
dose->setText(QString::number(current->dose,'f',1));
date->setText(simpdate(current->date));
film->setText(current->filmno);
nptcl->setText(QString::number(current->nptcl));
newMicroscope(current->microscope);
newSpecimen(current->specimen);
newScanner(current->scanner);

//plotwin->setCurrent(current);
//plotwin->newplots();

//imgwin->setData(current->getFFTImage());

//dataname->setText(current->getText());
}

void Control::dataHi(int index) {
char s[20];

if (ignore) { ignore--; return; }
//printf("datahi %d %d\n",index,lastdata);

lastdata=data->find(current);
if (lastdata<0) lastdata=0;

if (index>=0) current=data->at(index);
if (current==NULL) return;

df->setValueQ(current->defocus);
if (envbflag) envel->setValueQ(current->B);
else envel->setValueQ(sqrt(current->B)*2.0/PI);
amp->setValueQ(current->amp);
ampc->setValueQ(current->ampcont*100.0);
exps4->setValueQ(current->exps4);
expa->setValueQ(current->noise1);
expw->setValueQ(current->noise2);
gaussa->setValueQ(current->noise3);
gaussw->setValueQ(current->noise4);
stigang->setValueQ(current->stigang);
stigdf->setValueQ(current->stigdf);

sprintf(s,"%d",(int)floor(current->strucsize+.5));
ptclsize->setText(s);

sprintf(s,"%d",(int)floor(current->V+.5));
voltage->setText(s);

sprintf(s,"%3.1f",current->Cs);
Cs->setText(s);

sprintf(s,"%1.3f",current->sstep);
apix->setText(s);

mag->setText(QString::number(current->mag,'f',1));
comment->setText(current->comment);
film->setText(current->filmno);
nptcl->setText(QString::number(current->nptcl));
newMicroscope(current->microscope);
newSpecimen(current->specimen);
newScanner(current->scanner);
dose->setText(QString::number(current->dose,'f',1));
date->setText(simpdate(current->date));

plotwin->setCurrent(current);
plotwin->newplots();

imgwin->setData(current->getFFTImage());

dataname->setText(current->getText());
}

void Control::doFit() {
current->doFit();
dataHi(-1);
}

void fitQ(QList<CtfData> *);

void Control::doFitX() {

}

void Control::doFitQ() {
QList<CtfData> dsel;

for (int i=0; i<data->count(); i++) {
	if (data->at(i)->flags()&CDT_PLOT && data->at(i)->flags()&CDT_HASFFT) dsel.append(data->at(i));
}

fitQ(&dsel);

dataHi(-1);
}

void Control::newS(float s,float s2,float ang) {
if (!current) return; 
current->setSZero(s,s2,ang);
df->setValueQ(current->defocus);
stigdf->setValueQ(current->stigdf);
stigang->setValueQ(ang);
current->updateEqn();
}

void Control::dataSel(int index) {
int i;

//printf("datasel %d %d\n",index,lastdata);

current=data->at(index);
i=current->flags();
i^=CDT_PLOT;
current->setFlags(i);
//if (!(i&CDT_PLOT)) {  current=data->at(lastdata); }

ignore=1;
updateDList();
}

void Control::initMem() {
int i;
FILE *in;
CtfParm *p;
char *d = DFdir();
QString dd(d);

// This will put an old format database in the new format
QString old(dd+"/ctfitmem");
p=new CtfParm;
if (in=fopen((const char *)old,"r")) {
	QString old2(dd+"/ctfit.Old");
	FILE *out=fopen((const char *)old2,"w");
	while (p->oldread(in)) p->write(out);
	fclose(in);
	fclose(out);
	old2=dd+"/ctfitmem.oldver";
	rename((const char *)old,(const char *)old2);
}

QDir dir(dd,"ctfit.*");
free(d);

QStringList l = dir.entryList();
QStringList::Iterator it;

group->clear();
group->insertItem("Misc",-1);

for (it = l.begin(); it != l.end(); it++) {
	// this makes the microscope,specimin,scanner list complete
	CtfParm p;
	FILE *in=DFopenr((const char *)(*it));
	if (in) {
		int c=0;
		while (p.read(in)) {
			newMicroscope(p.microscope);
			newSpecimen(p.specimen);
			newScanner(p.scanner);
			c++;
		}
		fclose(in);
		printf("'%s' with %d\n",(const char *)(*it),c);
		if (!c) continue;
	}
	else printf("Cannot open '%s'\n",(const char *)(*it));
	
	// Add the group to the list if new
	if (*it=="ctfit.Misc") continue;
	QString n=(*it).mid(6);
	group->insertItem(n,-1);
}


group->setCurrentItem(0);
memGroup("Misc");
memRcl(0);
return;
}

void Control::memGroup(const QString &s) {
FILE *in;
CtfParm *p;
QString s2 = "ctfit."+s;
int g,i;

for (i=0; i<group->count(); i++) {
	if (group->text(i)==s) { g=i; group->setCurrentItem(g); break; }
}
if (i==group->count()) {
	group->insertItem(s);
	group->setCurrentItem((g=scanner->count()-1));
}

printf("Group %d -> %d\n",curgroup,g);

saveMem();
mem.clear();
curgroup=g;

in=DFopenr((const char *)s2);
if (!in) {
	if (s=="Misc") mem.append(new CtfParm);
	updateMList();
	return;
}

while (p=new CtfParm, p->read(in)) {
	mem.append(p);
//	newMicroscope(p->microscope);
//	newSpecimen(p->specimen);
//	newScanner(p->scanner);
}
delete p;
curmem=0;
updateMList();

newData();

//memRcl();
fclose(in);
}

void Control::saveMem() {
FILE *out;
if (curgroup<0) return;
QString s = "ctfit."+group->text(curgroup);

if (s=="ctfit.") return;
printf("Savemem '%s'\n",(const char *)s);

out=DFopenw((const char *)s);
if (!out) return;

for (int i=0; i<mem.count(); i++) mem.at(i)->write(out);

fclose(out);
}

void Control::setsfnone() {

current->setStrucMode(0,QString());
parmchange(0);
}

void Control::setsfsph() {
current->setStrucMode(1,QString());
parmchange(0);
}

void Control::setsffile() {
QString fsp(QFileDialog::getOpenFileName (current->strucfile));
if (fsp.isNull()) return;

current->setStrucMode(-1,fsp);
parmchange(0);
}

void Control::updateMList() {
int i;
char s[80];

memlist->setAutoUpdate(FALSE);
memlist->clear();
for (i=0; i<mem.count(); i++) {
	memlist->insertItem(mem.at(i)->name);
}
if (curmem>=0) memlist->setCurrentItem(curmem);
memlist->setAutoUpdate(TRUE);
memlist->update();

}


void Control::memSet() {
if (!current) return;
CtfParm *p = mem.at(curmem>=0?curmem:0);

p->defocus=current->defocus;
p->B=current->B;
p->amp=current->amp;
p->ampcont=current->ampcont;
p->exps4=current->exps4;
p->noise1=current->noise1;
p->noise2=current->noise2;
p->noise3=current->noise3;
p->noise4=current->noise4;
p->stigang=current->stigang;
p->stigdf=current->stigdf;
p->strucsize=current->strucsize;
p->structype=current->structype;
p->strucfile=current->strucfile;
p->V=current->V;
p->Cs=current->Cs;
p->sstep=current->sstep;
p->mag=current->mag;

p->dose=dose->text().toFloat();
p->date=simpdate(date->text());
p->comment=comment->text();
p->filmno=film->text();
p->nptcl=nptcl->text().toInt();
p->microscope=microscope->currentText();
p->specimen=specimen->currentText();
p->scanner=scanner->currentText();
}

void Control::memDel() {
if (curmem>=0) mem.remove(curmem);
updateMList();
}

void Control::memRcl() { memRcl(0); }

void Control::memRcl(int n) {
CtfParm *parm;

parm=mem.at(curmem);
if (!parm) return;
current->microscope=parm->microscope;
current->setImParm(parm->V,parm->Cs,parm->sstep,1.0/(parm->sstep*2));	
current->setSigParm(parm->defocus,parm->B,parm->amp,parm->ampcont,parm->exps4);
current->setNoiseParm(parm->noise1,parm->noise2,parm->noise3,parm->noise4);
current->setStrucParm(parm->strucsize);
current->setStrucMode(parm->structype,parm->strucfile);
current->setStigParm(parm->stigang,parm->stigdf);
current->mag=parm->mag;
current->comment=parm->comment;
current->dose=parm->dose;
current->date=parm->date;
current->scanner=parm->scanner;
current->specimen=parm->specimen;
current->filmno=parm->filmno;
current->nptcl=parm->nptcl;

current->updateEqn();
dataHi(-1);
}

void Control::memNew() {
curmem=mem.count();
mem.append(new CtfParm);
memSet();
updateMList();
}

void Control::memHi(int n) {
if (n<0) n=0;
curmem=n;
memname->setText(mem.at(n)->name);
return;
}

void Control::newMemName() {
if (curmem<0) return;
mem.at(curmem)->name=memname->text();
updateMList();
}

CtfParm::CtfParm() {
name="Default";
comment="";
microscope="";
scanner="";
specimen="";

defocus=-1.5;
B=400.0;
amp=1.0;
ampcont=.1;
exps4=0;
noise1=0;
noise2=0;
noise3=0;
noise4=0;
stigang=0;
stigdf=0;
strucsize=300.0;
structype=0;
V=100.0;
Cs=5.6;
sstep=4.66667;
}

int CtfParm::oldread(FILE *in) {
char c[40];
float smax;

fgets(c,39,in);
c[39]=0;
c[strlen(c)-1]=0;
name=c;
if (fscanf(in," %f %f %f %f %f %f %f %f %f %f %f %f %d %f %f %f %f",&defocus,&B,
        &amp,&ampcont,&exps4,&noise1,&noise2,&noise3,&noise4,
        &stigang,&stigdf,&strucsize,&structype,&V,&Cs,&sstep,&smax)!=17) return 0;
fgets(c,4,in);

mag=140.0/sstep;	//assume default of 14um pixel
comment="";
microscope="";
scanner="";
specimen="";
filmno=name;
dose=0;
date=QDate::currentDate();

return 1;
}

int CtfParm::read(FILE *in) {
char c[512];
int m,d,y,i;

fgets(c,39,in);
c[39]=0;
c[strlen(c)-1]=0;
name=c;
fgets(c,511,in);
if ((i=sscanf(c," %f %f %f %f %f %f %f %f %f %f %f %f %d %f %f %f %f %d %f %d %d %d",&defocus,&B,
	&amp,&ampcont,&exps4,&noise1,&noise2,&noise3,&noise4,
	&stigang,&stigdf,&strucsize,&structype,&V,&Cs,&sstep,&mag,&nptcl,&dose,&y,&m,&d))<18) return 0;
//printf("%d %d %d %d\n",i,y,m,d);
if (i>=22) date.setYMD(y,m,d); else date=QDate::currentDate();
if (i<19) dose=0;
fgets(c,128,in);
comment=c;
comment=comment.stripWhiteSpace();
fgets(c,128,in);
microscope=c;
microscope=microscope.stripWhiteSpace();
fgets(c,128,in);
scanner=c;
scanner=scanner.stripWhiteSpace();
fgets(c,128,in);
specimen=c;
specimen=specimen.stripWhiteSpace();
fgets(c,128,in);
filmno=c;
filmno=filmno.stripWhiteSpace();

return 1;
}

void CtfParm::write(FILE *out) {
int i;

fputs((const char *)name,out);
fprintf(out,"\n%f %f %f %f %f %f %f %f %f %f %f %f %d %f %f %f %f %d %1.2f %d %d %d\n",defocus,B,
	amp,ampcont,exps4,noise1,noise2,noise3,noise4,
	stigang,stigdf,strucsize,structype,V,Cs,sstep,mag,nptcl,dose,date.year(),date.month(),date.day());

if ((i=comment.find('\n')) || (i=comment.find('\r'))) comment.truncate(i);
if ((i=microscope.find('\n')) || (i=microscope.find('\r'))) microscope.truncate(i);
if ((i=scanner.find('\n')) || (i=scanner.find('\r'))) scanner.truncate(i);
if ((i=specimen.find('\n')) || (i=specimen.find('\r'))) specimen.truncate(i);
if ((i=filmno.find('\n')) || (i=filmno.find('\r'))) filmno.truncate(i);
comment.truncate(127);
microscope.truncate(127);
scanner.truncate(127);
specimen.truncate(127);
filmno.truncate(10);

fprintf(out,"%s\n%s\n%s\n%s\n%s\n",(const char *)comment,(const char *)microscope,
	(const char *)scanner,(const char *)specimen,(const char *)filmno);
}

