#ifndef Control_included
#define Control_included

#include "ControlData.h"
#include "PlotCtf.h"
#include "Image.h"
#include "qdatetime.h"
#include "CtfData.h"
#include "qkeycode.h"
#include "qlist.h"
#include <qstring.h>

struct CtfParm { 
	CtfParm();
	int read(FILE *in);
	int oldread(FILE *in);
	void write(FILE *out);

	QString name;
	float defocus;		// defocus in microns (- is underfocus)
	float B;			// envelope function width (Pi/2 * Wg)^2
	float amp;			// ctf amplitude
	float ampcont;		// % amplitude contrast (0 to 1, displayed as 0 to 100)
	float exps4;		// % of exp(s^4) content (vs gaussian)
	float noise1;		// noise exponential decay amplitude
	float noise2;		// width
	float noise3;		// noise gaussian amplitude
	float noise4;		// noide gaussian width
	float stigang;		// astigmatism major axis angle (0 is x)
	float stigdf;		// astigmatism defocus difference
	float strucsize;	// object size for structure factor approximation
	int structype;		// structure factor type (0 disabled)
	QString strucfile;	// structure factor filename
	float V;			// voltage in kv
	float Cs;			// spherical abberation of scope in mm	
	float sstep;		// pixel size in angstroms
	float mag;
	int nptcl;
	float dose;
	QDate date;

	QString comment;
	QString microscope;
	QString scanner;
	QString specimen;
	QString filmno;
};

class Control : public ControlData
{
    Q_OBJECT

public:

    Control
    (
        QWidget* parent = NULL,
        const char* name = NULL
    );

    virtual ~Control();


	PlotCtf *plotwin;
	Image *imgwin;
	QList<CtfData> *data;
	CtfData *current;

	QList<CtfParm> mem;
	int curmem;
	int nmem;
	int curgroup;
	int lastdata,ignore;

	short envbflag;

	void updateDList();
	void updateCList();
	void newData();

	virtual void closeEvent(QCloseEvent *e); 

public slots:
	void Mopenmicro();
	void Msplitmicro();
	void Mopen1clip();
	void MopenRealFFT();
	void Mopenclips();
	void MopenclipsS();
	void Mopenclipsos();
	void Mopenclipscf();
	void Mopenstruc();
	void Msavefft();
	void Msavedata();
	void MsetSF();
	void Mprint();
	void Mquit();
	void MWFilter();
	void Mfixset();
	void Msetparm();
	void Mpair();
	void Mapply();
	void Mapply3d();
	void Munapply();
	void Mnoise();
	void Mhelp();
	void Mabout();
	void Mseteqn(); 
	void Msaveavg();
	
	void newMicroscope(const QString &s);
	void newSpecimen(const QString &s);
	void newScanner(const QString &s);

	void parmchange(float value);
	void vchange();
	void dbchange();
	void apixchange();
	void magchange();

	void doFit();
	void doFitQ();
	void doFitX();

	void dataNewsim();
	void dataDel();
	void dataHi(int index);
	void dataSel(int index);
	void newDataName();

	void setsfnone();
	void setsfsph();
	void setsffile();

	void ebTog();

	void newS(float s,float s2,float ang);
	
	void initMem();
	void saveMem();
	void updateMList();
	void memGroup(const QString &s);
	void memSet();
	void memDel();
	void memRcl(int n);
	void memRcl();
	void memNew();
	void memHi(int n);
	void newMemName();
	
	void warn(char *msg);
};
#endif // Control_included
