#include "CtfData.h"
#include "fit.h"
#include <qmsgbox.h>
#include <qfiledialog.h> 
#include <math.h>
#include "CtfDatab.C"

CtfData::CtfData(int ctst) {
int i;

//printf("%d\n",ctst);
if (ctst>1000||ctst<10) ctst=250;

CTFSTEPS=ctst;

text="New";

file="/";

initMetaObject();                           // initialize meta object
defocus=0;
B=200.0;
amp=1.0;
ampcont=0.1;
exps4=0;
noise1=0;
noise2=1.0;
noise3=0;
noise4=1.0;
stigang=0;
stigdf=0;
strucsize=0;
structype=0;
V=400.0;
Cs=4.1;
sstep=4.66667;
smax=1.0/8.0;
ctfmap=NULL;

Flags=CDT_HASEQ|CDT_PLOT;

data=new DataSet;
realimage=fftimage=NULL;
cmgamma=new Equation;
cmctf=new Equation;

data->setRC(15,CTFSTEPS);
// s
data->getColumn(0)->setRange(smax/(float)CTFSTEPS,smax,CTFSTEPS);

// gamma
data->getColumn(1)->setEqn("-2*pi*(Cs*1.0e7/4*lambda^3*x0^4+defocus*5000*lambda*x0^2)",
	CTFSTEPS,data);
eqs1=(char *)malloc(60);
strcpy(eqs1,"-2*pi*(Cs*1.0e7/4*lambda^3*S^4+defocus*5000*lambda*S^2)");
cmgamma->parseEqn(eqs1);

// gamma, stigmatic
data->getColumn(2)->setEqn("-2*pi*(Cs*1.0e7/4*lambda^3*x0^4+(defocus+stigdf)*5000*lambda*x0^2)",
	CTFSTEPS,data);

// CTF (not squared)
data->getColumn(3)->setEqn(
	"amp*exp(-(exps4*B^.5*x0+B*(1-exps4)*x0^2))*(sqrt(1-ampcont^2)*sin(x1)+ampcont*cos(x1))",
	CTFSTEPS,data);
eqs2=(char *)malloc(100);
strcpy(eqs2,"amp*exp(-(exps4*B^.5*S+B*(1-exps4)*S^2))*(sqrt(1-ampcont^2)*sin(gamma)+ampcont*cos(gamma))");
cmctf->parseEqn(eqs2);

/*data->getColumn(3)->setEqn(
	"amp*exp(-B*(exps4*B*x0^4+(1-exps4)*x0^2))*(sqrt(1-ampcont^2)*sin(x1)+ampcont*cos(x1))",
	CTFSTEPS,data);*/

// CTF, stigmatic
data->getColumn(4)->setEqn(
	"amp*exp(-(exps4*B^.5*x0+B*(1-exps4)*x0^2))*(sqrt(1-ampcont^2)*sin(x2)+ampcont*cos(x2))",
	CTFSTEPS,data);

/*data->getColumn(4)->setEqn(
	"amp*exp(-B*(exps4*B*x0^4+(1-exps4)*x0^2))*(sqrt(1-ampcont^2)*sin(x1)+ampcont*cos(x1))",
	CTFSTEPS,data);*/

// noise (intensity)
/* data->getColumn(5)->setEqn(
	"(noise3*exp(-((pi/2*noise4*x0)^2))+noise1*exp(-x0*noise2))^2",CTFSTEPS,
	data); */

/*data->getColumn(5)->setEqn(
	"(noise3*exp(-((pi/2*noise4*x0)^2))+noise1*exp(-x0*noise2))",CTFSTEPS,
	data);*/

data->getColumn(5)->setEqn(
	"(noise3*exp(-((pi/2*noise4*x0)^2)-x0*noise2-sqrt(fabs(x0))*noise1))",CTFSTEPS,
	data);


// overall CTF
data->getColumn(6)->setEqn("x13*x3^2+x5",CTFSTEPS,data);

// stigmatic overall CTF
data->getColumn(7)->setEqn("x13*x4^2+x5",CTFSTEPS,data);

// SNR
data->getColumn(8)->setEqn("x3^2/x5",CTFSTEPS,data);
//data->getColumn(8)->setEqn("x10/x5-1",CTFSTEPS,data);

// Ptcl reqd vs resolution
//data->getColumn(9)->setEqn("2*x0*strucsize/x8",CTFSTEPS,data);
data->getColumn(9)->setEqn("2*x0*x5/(amp^2*exp(-(2*B*x0^2)))",CTFSTEPS,data);

// Actual data
data->getColumn(10)->setData((float *)calloc(1,CTFSTEPS*sizeof(float)),CTFSTEPS);

// calculated structure factor
//data->getColumn(11)->setEqn("sqrt(exp(-B*S^2)*(x10-x5)/x3^2)",CTFSTEPS,data);
data->getColumn(11)->setEqn("sqrt((x10-x5)/x3^2)",CTFSTEPS,data);

// stigmatic axis data
data->getColumn(12)->setData((float *)calloc(1,CTFSTEPS*sizeof(float)),CTFSTEPS);

// structure factor from file
data->getColumn(13)->setRange(1.0,1.0,CTFSTEPS);

// User function
data->getColumn(14)->setEqn("x5",CTFSTEPS,data);		// default is noise without signal

updateEqn();

for (i=0; i<6; i++) {
	plot[i]=new LSPlot;
	plot[i]->setData(data);
	plot[i]->setCols(0,6,1,1);
}

plot[0]->setLabel("New");
}

CtfData::~CtfData() {
int i;

if (realimage) delete realimage;
if (fftimage) delete fftimage;
if (ctfmap) delete ctfmap;
free(eqs1);
free(eqs2);

delete data;
for (i=0; i<4; i++) delete plot[i];
}

void CtfData::setText(QString &s) { 
text=s;
data->title=s;  //commented out by prb

if (s.findRev('/')>=0) plot[0]->setLabel(s.mid(s.findRev('/')+1));
else plot[0]->setLabel(s);
}


void CtfData::setImParm(float V0,float Cs0,float sstep0,float smax0) {
int ui=0;

if (V0!=V||sstep0!=sstep||smax0!=smax) ui=1;

V=V0;
Cs=Cs0;
sstep=sstep0;
smax=smax0;
data->getColumn(0)->setRange(smax/(float)CTFSTEPS,smax,CTFSTEPS);
if (ui) updateImage();
}

void CtfData::setSigParm(float defocus0,float envwid0, float amp0, float ampcont0, float exps40) {
defocus=defocus0;
B=envwid0;
amp=amp0;
ampcont=ampcont0;
exps4=exps40;

}

void CtfData::setNoiseParm(float noise10,float noise20,float noise30,float noise40) {
noise1=noise10; 
noise2=noise20; 
noise3=noise30; 
noise4=noise40;
}

void CtfData::setSZero(float r,float r2,float ang) {
float s,s2;

if (!fftimage) return;
s=r/(sstep*fftimage->ySize());
s2=r2/(sstep*fftimage->ySize());
//defocus=(asin(ampcont)/(PI*2.0)-.5-Cs*.25*pow(lambda,3.0)*pow(s,4.0))/(5000.0*lambda*s*s);
defocus=-(2.0*PI+pow(lambda,3.0)*PI*pow(s,4.0)*Cs*1.0e7-2.0*asin(ampcont))/(20000.0*lambda*PI*s*s);
stigdf=-defocus-(2.0*PI+pow(lambda,3.0)*PI*pow(s2,4.0)*Cs*1.0e7-2.0*asin(ampcont))/(20000.0*lambda*PI*s2*s2);
stigang=ang;
updateEqn();
}

float CtfData::getZeroR() { 
	float s;
	if (!fftimage) return(0);
	s=1.0/(sstep*fftimage->ySize());
	return zero/s;
}

float CtfData::getZeroStigR() { 
	float s;
	if (!fftimage) return(0);
	s=1.0/(sstep*fftimage->ySize());
	return zerostig/s;
}

void CtfData::setParmStr(const char *s) {
sscanf(s+2,"%f %f %f %f %f %f %f %f %f %f %f",
	&defocus,&B,&amp,&ampcont,&noise1,&noise2,&noise3,&noise4,&V,&Cs,&sstep);

updateEqn();
}

void CtfData::setStigParm(float stigang0, float stigdf0) {
int i=0;

if (stigang!=stigang0 && stigdf0!=0) i=1;
if ((stigdf0>=-.01 && stigdf<-.01) || (stigdf>=-.01 && stigdf0<-.01)) i=1;
stigang=stigang0;
stigdf=stigdf0;
if (i) updateImage();
}

void CtfData::setStrucMode(int mode,QString name) {
structype=mode;
strucfile=name;

if (structype==-1) {
	FILE *in = fopen(strucfile.data(),"r");
	if (in) {
		float s,*d,fs0,fd0,fs1=-1.0,fd1;
		int i,j;
		
		d=(float *)malloc(CTFSTEPS*sizeof(float));
	
		for (i=0; i<CTFSTEPS; i++) {
			s=data->dataAt(0,i);
			// read until we have data bracketing the correct s value
			while (fs1<s) {
				fs0=fs1; 
				fd0=fd1; 
				if (fscanf(in," %f %f",&fs1,&fd1)!=2) { fs1=-1.0; break; }
			}
			// an exact match
			if (fs1==s) d[i]=fd1;
			else if (fs0==s) d[i]=fd0;
				// no data in the file in the right range
			else if (fs0<0 || fs1<0) { 
				d[i]=1.0; 
				printf("WARNING: structure factor undefined at s=%1.4f\n",s); 
			}
			// linear interpolation
			else {
				d[i]=(fd1*(s-fs0) +fd0*(fs1-s))/(fs1-fs0);
//				printf("%f\t%f\n",s,d[i]);
			}
		}
		data->getColumn(13)->setData(d,CTFSTEPS);
		
	}
	else printf("Warning, SF read failed\n");

}

if (structype==1) data->getColumn(13)->setEqn("10^-6*(sin(x0*strucsize)/x0^3 - strucsize*cos(x0*strucsize)/x0^2)",CTFSTEPS,data);
else if (structype==0) data->getColumn(13)->setRange(1.0,1.0,CTFSTEPS);

}

void CtfData::setStrucParm(float strucsize0) {
strucsize=strucsize0;
}

void CtfData::setRealImage(EMData *image) {
realimage=image;
if (image) Flags|=CDT_HASREAL; else Flags&=~CDT_HASREAL;

}

void CtfData::setFFTImage(EMData *image) {
fftimage=image;
if (image) Flags|=CDT_HASFFT; else Flags&=~CDT_HASFFT;
updateImage();
}


void CtfData::WFilter(){
float *fix;
float s0,ds, SNR,noiseInt,lessthan1Factor;
int i,ny,ny2;
EMData *r,*f,*r2,*rs;
QString nm;
char n[80];

if (!(Flags&CDT_HASFFT)) { warn("This image doesn't have a FFT."); return; }


// Find index corresponding to the first zero


float deltaf = defocus*(1.e-6);
float epsilon = asin((ampcont));
float lam = lambda*1.e-10;
float C_s = Cs*1.e-3;

float FirstZero =  sqrt(lam)* sqrt(       \
	(sqrt( deltaf*deltaf - 2*lam *C_s* ( 1 - epsilon) )  \
	- deltaf )/(2 *( 1 - epsilon) )); 
float spacing = (data->dataAt(0,2)) - (data->dataAt(0,1));
int IFirstZero = 1./(FirstZero*1.e10*spacing);
float FirstZeroAmp = data->dataAt(13,IFirstZero);
 printf("deltaf =%2.2e, FirstZero (Ang)=%2.2e,  spacing=%2.2e, IfirstZero=%d , FirstZeroAmp=%2.2e\n",
 		deltaf,FirstZero*1.e10, spacing,IFirstZero,FirstZeroAmp);

fix=(float *)malloc(CTFSTEPS*sizeof(float));

for (i=0; i<CTFSTEPS; i++) {
//                       %%    NEED TO CHANGE THIS PART (fix)
// if (data->dataAt(3,i)<0) fix[i]=-1.0; else fix[i]=1.0; 
 //   if (data->dataAt(10,i)>0) {
		SNR = (data->dataAt(3,i)) * (data ->dataAt(13,i));
		noiseInt = data->dataAt(5,i);
		SNR= SNR*SNR/noiseInt;
		lessthan1Factor = 2* sqrt(SNR)/(1+SNR);	
		fix[i] =  (data ->dataAt(13,i)) * lessthan1Factor / (2*sqrt(noiseInt)) ;
		if ( data->dataAt(3,i) <0) fix[i] *= -1;
//	} else fix[i]=0; 
//	if (i < IFirstZero) fix[i] *= FirstZeroAmp /(data->dataAt(13,i) )      ;
 printf("i=%d fix[i]=%f fac<1 =%2.2e SNR=%2.2e , SF=%2.2e, 1/noiseInt=%2.2e, 1/m =%2.2e \n",
    i,fix[i],lessthan1Factor,SNR,data -> dataAt(13,i) , 1/noiseInt, 1/(data->dataAt(3,i)));
 }


s0=data->dataAt(0,0)*sstep*fftimage->ySize();
ds=(data->dataAt(0,1)-data->dataAt(0,0))*sstep*fftimage->ySize();

save_data(s0,ds,fix,CTFSTEPS,"phases");

r=new EMData;
if (file.findRev('.')>0) nm=file.left(file.findRev('.'));
else nm=file;
nm+=".WF";

sprintf(n,"!-%1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g",
	defocus,B,amp,ampcont,noise1,noise2,noise3,noise4,V,Cs,sstep);
n[79]=0;

updateEqn(11);	// generates a 'sign map' for the current simulation
ny=ctfmap->ySize();

r->readImage(getText(),0);
ny2=r->ySize();
printf("ny =%d  ny2=%d \n",ny ,ny2);
if (ny!=ny2) { warn("Do NOT use oversampling when performing CTF correction."); return; }

for (i=0; r->readImage(getText(),i)==0; i++) {
	ny2=r->ySize();
	if (ny!=ny2) rs=r->clip(-(ny-ny2)/2,-(ny-ny2)/2,ny,ny);
	else rs=r->copy();
	rs->normalize();
	f=rs->doFFT();
	rs->gimmeFFT();
	delete rs;
	f->mult(ctfmap);
	r2=f->doIFT();
	f->gimmeFFT();
	r2->setName(n);
	if (ny!=ny2) {
		rs=r2->clip((ny-ny2)/2,(ny-ny2)/2,ny2,ny2);
		rs->writeIMAGIC(nm,i);
		delete rs;
	}
	else r2->writeIMAGIC(nm,i);
	delete f;
	delete r2;
	printf("%d done\r",i);
}

free(fix); 
}


void CtfData::fixPhase() {
float *fix;
float s0,ds;
int i,ny,ny2;
EMData *r,*f,*r2,*rs;
QString nm;
char n[80];

if (!(Flags&CDT_HASFFT)) { warn("This image doesn't have a FFT."); return; }


fix=(float *)malloc(CTFSTEPS*sizeof(float));
for (i=0; i<CTFSTEPS; i++) { if (data->dataAt(3,i)<0) fix[i]=-1.0; else fix[i]=1.0; }

s0=data->dataAt(0,0)*sstep*fftimage->ySize();
ds=(data->dataAt(0,1)-data->dataAt(0,0))*sstep*fftimage->ySize();

save_data(s0,ds,fix,CTFSTEPS,"phases");

r=new EMData;
if (file.findRev('.')>0) nm=file.left(file.findRev('.'));
else nm=file;
nm+=".fix";

sprintf(n,"!-%1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g",
	defocus,B,amp,ampcont,noise1,noise2,noise3,noise4,V,Cs,sstep);
n[79]=0;

updateEqn(1);	// generates a 'sign map' for the current simulation
ny=ctfmap->ySize();

r->readImage(getText(),0);
ny2=r->ySize();
if (ny!=ny2) { warn("Do NOT use oversampling when performing CTF correction."); return; }

for (i=0; r->readImage(getText(),i)==0; i++) {
	ny2=r->ySize();
	if (ny!=ny2) rs=r->clip(-(ny-ny2)/2,-(ny-ny2)/2,ny,ny);
	else rs=r->copy();
	rs->normalize();
	f=rs->doFFT();
	rs->gimmeFFT();
	delete rs;
	f->mult(ctfmap);
	r2=f->doIFT();
	f->gimmeFFT();
	r2->setName(n);
	if (ny!=ny2) {
		rs=r2->clip((ny-ny2)/2,(ny-ny2)/2,ny2,ny2);
		rs->writeIMAGIC(nm,i);
		delete rs;
	}
	else r2->writeIMAGIC(nm,i);
	delete f;
	delete r2;
	printf("%d done\n",i);
}

free(fix); 
}

void CtfData::setParm() {
int i;
EMData *r;
char n[80];

if (!(Flags&CDT_HASFFT)) { warn("This image doesn't have a FFT."); return; }

r=new EMData;
sprintf(n,"!-%1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g",
	defocus,B,amp,ampcont,noise1,noise2,noise3,noise4,V,Cs,sstep);
n[79]=0;
	

for (i=0; r->readImage(getText(),i)==0; i++) {
	r->setName(n);
	r->writeIMAGIC(file,i);
	printf("%d done\n",i);
}

delete r;
}

void CtfData::setSF(const char *name) {
EMData *d;

updateEqn(5,fftimage->ySize());
fftimage->mult(ctfmap);
d=fftimage->doIFT();
QString s(name);
d->writeMRC(name);
delete d;
}

void CtfData::applyCtfTo(const char *name,int forward) {
EMData *d,*df;
int i,n;

d=new EMData;
d->readImage(name,0,1);
if (d->zSize()>1) {
	delete d;		// just to make sure no prob with nodata flag
	d=new EMData;
	d->readImage(name,0);
	df=d->doFFT();
	d->gimmeFFT();
	delete d;
	if (forward==-1) updateEqn(4,d->ySize());
	else updateEqn(3,d->ySize());
	df->mult(ctfmap);
	d=df->doIFT();
	QString s(name);
	if (forward==1) s.insert(s.length()-4,".ctf");
	else s.insert(s.length()-4,".ctfc");
	d->writeMRC(s.data());
	delete df;
	return;
}

if (forward!=-1) { printf("applyCtfTo(), forward!=-1. Unsupported.\n"); return; }
QString s(name);
s.insert(s.length()-4,".ctfc");
n=fileCount(name);
delete d;

for (i=0; i<n; i++) {
	d=new EMData;
	d->readImage(name,i);
	df=d->doFFT();
	d->gimmeFFT();
	delete d;	
	updateEqn(6,d->ySize());
	df->mult(ctfmap);
	d=df->doIFT();
	d->writeIMAGIC(s.data(),-1);
	delete df;
}

}

void CtfData::applyCtf(int noiseonly) {
float *ctf,*noise;
float s0,ds;
int i,ny,ny2;
EMData *r,*rs,*f,*r2;
char n[120];
QString nm;

if (!(Flags&CDT_HASFFT)) { warn("This image doesn't have a FFT."); return; }

ctf=data->getColumn(3)->getData();
noise=data->getColumn(5)->getData();

s0=data->dataAt(0,0)*sstep*fftimage->ySize();
ds=(data->dataAt(0,1)-data->dataAt(0,0))*sstep*fftimage->ySize();

r=new EMData;

if (file.findRev('.')>0) nm=file.left(file.findRev('.'));
else nm=file;
nm+=".fix";

sprintf(n,"!-%1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g",
	defocus,B,amp,ampcont,noise1,noise2,noise3,noise4,V,Cs,sstep);
n[79]=0;

printf("ctf %d %f %f\n",CTFSTEPS,s0,ds);
updateEqn(2);
ny=ctfmap->ySize();

for (i=0; r->readImage(getText(),i)==0; i++) {
	ny2=r->ySize();
	if (ny!=ny2) rs=r->clip(-(ny-ny2)/2,-(ny-ny2)/2,ny,ny);
	else rs=r->copy();
	rs->normalize();
	f=rs->doFFT();
	rs->gimmeFFT();
	delete rs;
	if (!noiseonly) f->mult(ctfmap);
	f->addRadNoise(CTFSTEPS,s0,ds,noise,0);
	r2=f->doIFT();
	f->gimmeFFT();
	r2->setName(n);
	if (ny!=ny2) {
		rs=r2->clip((ny-ny2)/2,(ny-ny2)/2,ny2,ny2);
		rs->writeIMAGIC(nm,i);
		delete rs;
	}
	else r2->writeIMAGIC(nm,i);
	delete f;
	delete r2;
	printf("%d done\n",i);
}

}


void CtfData::updateImage() {
float *dat;
int i;
float s0,ds;
if (!fftimage) return;

s0=data->dataAt(0,0)*sstep*fftimage->ySize();
ds=(data->dataAt(0,1)-data->dataAt(0,0))*sstep*fftimage->ySize();
printf("updateimage %1.2f %1.2f\n",s0,ds);
//printf("%f %f\n",s0,ds);
dat=data->getColumn(10)->getData();
if (stigdf>-.01) fftimage->calcRadDist(CTFSTEPS,s0,ds,dat);
else fftimage->calcRadDist(CTFSTEPS,s0,ds,dat,stigang*PI/180.0,10.0*PI/180.0);
//for (i=0; i<CTFSTEPS; i++) dat[i]*=dat[i];
//dat[0]=0;
data->getColumn(10)->setData();

dat=data->getColumn(12)->getData();
if (stigdf==0) fftimage->calcRadDist(CTFSTEPS,s0,ds,dat);
else fftimage->calcRadDist(CTFSTEPS,s0,ds,dat,(stigang-90.0)*PI/180.0,10.0*PI/180.0);
//for (i=0; i<CTFSTEPS; i++) dat[i]*=dat[i];
//dat[0]=0;
data->getColumn(12)->setData();

}

void CtfData::warn(const char *msg) {
	QMessageBox::warning(NULL,"CTFIT Warning",msg,"Ok");
	return;
}

void CtfData::copy(CtfData *src) {

setStrucParm(src->strucsize);
setStrucMode(src->structype,src->strucfile);
setStigParm(src->stigang,src->stigdf);
setNoiseParm(src->noise1,src->noise2,src->noise3,src->noise4);
setSigParm(src->defocus,src->B,src->amp,src->ampcont,src->exps4);
setImParm(src->V,src->Cs,src->sstep,src->smax);

updateEqn();
}

// makemap  1->phase flip map, 2->2d Fourier apply CTF, 6-> 2d Fourier fix CTF
// 3->3d Fourier apply CTP, 4->3d Fourier fix CTF, 5->set x-ray SF to 3d
void CtfData::updateEqn(int makemap,int size) {

if (!data) return;
if (ampcont<0||ampcont>1.0) { ampcont=0.0; printf("Warning, ampcont out of range\n"); }
if (V<0) V=100.0;

lambda=12.2639/sqrt(V*1000.0+.97845*V*V);
zero=sqrt(-(defocus*10000.0+sqrt(defocus*defocus*1.0e8-2.0*lambda*Cs*1.0e7+
	2.0*lambda*Cs*1.0e7/PI*asin(ampcont)))/(lambda*lambda*Cs*1.0e7));

zerostig=sqrt(-((defocus+stigdf)*10000.0+sqrt(SQR(defocus+stigdf)*1.0e8-2.0*lambda*Cs*1.0e7+
	2.0*lambda*Cs*1.0e7/PI*asin(ampcont)))/(lambda*lambda*Cs*1.0e7));
	
data->setVar("lambda",lambda);
data->setVar("Cs",Cs);
data->setVar("stigdf",stigdf);
data->setVar("defocus",defocus);
data->setVar("exps4",exps4);
data->setVar("amp",amp);
data->setVar("ampcont",ampcont);
data->setVar("B",B);
data->setVar("noise4",noise4);
data->setVar("noise1",noise1);
data->setVar("noise2",noise2);
data->setVar("noise3",noise3);
data->setVar("strucsize",strucsize);

if (!fftimage && makemap<3) return;	// no point in going on if there isn't any data

// make a sign map which can be multiplied by a fourier image to correct phase flips
if (makemap==1) {
	int i,j,ny;
	float s,ds,*d;
	
	if (ctfmap) delete ctfmap;
	ctfmap=fftimage->copyHead();
	ctfmap->setRI(1);

	cmgamma->setVar("Cs",Cs);
	cmgamma->setVar("lambda",lambda);
	cmgamma->setVar("defocus",defocus);	

	cmctf->setVar("B",B);
	cmctf->setVar("exps4",exps4);
	cmctf->setVar("ampcont",ampcont);
	cmctf->setVar("amp",amp);

	ds=1.0/(sstep*ctfmap->ySize());
	
	ny=ctfmap->ySize();
	d=ctfmap->getData();

	for (j=0; j<ny; j++) {
		for (i=0; i<ny+2; i+=2) {
			s=hypot((float)i/2.0,(float)j-ny/2.0)*ds;
			cmgamma->setVar("S",s);
			cmctf->setVar("gamma",cmgamma->eval());
			cmctf->setVar("S",s);
			if (cmctf->eval()>0) { *d=1.0; d++; *d=1.0; d++; } 
			else { *d=-1.0; d++; *d=-1.0; d++; }
		}
	}
	
	ctfmap->doneData();
	
	ctfmap->writeMRC("signmap.mrc");
}


// make a Wiener map which can be multiplied by a fourier image to correct phase flips
if (makemap==11) {
	int i,j,ny;
	float s,ds,*d;

	int is;
	float SNR,noiseInt,lessthan1Factor;
	float deltaf = defocus*(1.e-6);
	float epsilon = asin((ampcont));
	float lam = lambda*1.e-10;
	float C_s = Cs*1.e-3;

	float FirstZero =  sqrt(lam)* sqrt(       \
		(sqrt( deltaf*deltaf - 2*lam *C_s* ( 1 - epsilon) )  \
		- deltaf )/(2 *( 1 - epsilon) )); 
	
	float spacing = (data->dataAt(0,2)) - (data->dataAt(0,1));
	int IFirstZero = 1./(FirstZero*1.e10*spacing);
	float FirstZeroAmp = data->dataAt(13,IFirstZero);
	float fixtmp, xrayStuff, CTFstuff;
	
	if (ctfmap) delete ctfmap;
	ctfmap=fftimage->copyHead();
	ctfmap->setRI(1);

	cmgamma->setVar("Cs",Cs);
	cmgamma->setVar("lambda",lambda);
	cmgamma->setVar("defocus",defocus);	

	cmctf->setVar("B",B);
	cmctf->setVar("exps4",exps4);
	cmctf->setVar("ampcont",ampcont);
	cmctf->setVar("amp",amp);

	ds=1.0/(sstep*ctfmap->ySize());
	
	ny=ctfmap->ySize();
	d=ctfmap->getData();

	int iquest=4;
	for (j=0; j<ny; j++) {
		for (i=0; i<ny+2; i+=2) {
//			s=hypot((float)i/2.0,(float)j-ny/2.0)*ds;
			s=hypot((float)i/2.0,(float)j-ny/2.0)*2.;
			cmgamma->setVar("S",s);
			cmctf->setVar("gamma",cmgamma->eval());
			cmctf->setVar("S",s);
			is = s ;
//			if (cmctf->eval()>0) { *d=1.0; d++; *d=1.0; d++; }  
//			else { *d=-1.0; d++; *d=-1.0; d++; } USED to be for  makemap=1
			fixtmp =0;
			if (is<ny-1) {
//	   			if (data->dataAt(10,is)>0) {
					xrayStuff = (s-is)*(data ->dataAt(13,is)) + (is+1-s)*(data ->dataAt(13,is+1));
					CTFstuff = (s-is)* (data->dataAt(3,is))+ (is+1-s)*(data->dataAt(3,is+1));
					SNR =  CTFstuff* xrayStuff;
					noiseInt = data->dataAt(5,is);
					SNR= SNR*SNR/noiseInt;
					lessthan1Factor = 2.* sqrt(SNR)/(1.+SNR);	
//					lessthan1Factor = 2.* 1/(sqrt(1.+SNR));	for SF data
					fixtmp =  xrayStuff * lessthan1Factor / (2.*sqrt(noiseInt)) ;
					if ( CTFstuff <0) fixtmp *= -1;
				if (j==ny/2) { printf("%d,%d,%d, %f \n", i, j, is, fixtmp);}
			}
//			if (is < IFirstZero) fixtmp *= (sqrt(FirstZeroAmp /(data->dataAt(13,is)
//			)));
			*d = fixtmp; d++; *d = fixtmp; d++;
		}
	}
	
	ctfmap->doneData();
	
	ctfmap->writeMRC("signmap.mrc");
}


// make a map which can be multiplied by a fourier image to apply a CTF
if (makemap==2 || makemap==6) {
	int i,j,ny;
	float s,ds,*d,e;
	
	if (ctfmap) delete ctfmap;
	ctfmap=fftimage->copyHead();
	ctfmap->setRI(1);

	cmgamma->setVar("Cs",Cs);
	cmgamma->setVar("lambda",lambda);
	cmgamma->setVar("defocus",defocus);	

	cmctf->setVar("B",B);
	cmctf->setVar("exps4",exps4);
	cmctf->setVar("ampcont",ampcont);
	cmctf->setVar("amp",amp);

	ds=1.0/(sstep*ctfmap->ySize());
	
	ny=ctfmap->ySize();
	d=ctfmap->getData();

	for (j=0; j<ny; j++) {
		for (i=0; i<ny+2; i+=2) {
			s=hypot((float)i/2.0,(float)j-ny/2.0)*ds;
			cmgamma->setVar("S",s);
			cmctf->setVar("gamma",cmgamma->eval());
			cmctf->setVar("S",s);
			e=cmctf->eval();
			if (makemap==6) {
				if (fabs(e)<.04) e=25; else e=1.0/e;
			}
			*d=e;
			d++;
			*d=e;
			d++;
		}
	}
	
	ctfmap->doneData();
	
	ctfmap->writeMRC("ctfmap.mrc");
}

// this makes a 3d amp mult map for modifying 3d volumes
// 3 adds a CTF, 4 does a simple CTF correction
if (makemap==3 || makemap==4) {
	int i,j,k,f=0,ny;
	float s,ds,*d,e;
	
	if (ctfmap) delete ctfmap;
	ctfmap=new EMData;
	ctfmap->setSize(size+2,size,size);
	ctfmap->setRI(1);

	cmgamma->setVar("Cs",Cs);
	cmgamma->setVar("lambda",lambda);
	cmgamma->setVar("defocus",defocus);	

	cmctf->setVar("B",B);
	cmctf->setVar("exps4",exps4);
	cmctf->setVar("ampcont",ampcont);
	cmctf->setVar("amp",amp);

	ds=1.0/(sstep*ctfmap->ySize());
	
	ny=ctfmap->ySize();
	d=ctfmap->getData();

	for (k=0; k<ny; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<ny+2; i+=2) {
				s=sqrt(i*i/4.0+(j-ny/2)*(j-ny/2)+(k-ny/2)*(k-ny/2))*ds;
				cmgamma->setVar("S",s);
				cmctf->setVar("gamma",cmgamma->eval());
				cmctf->setVar("S",s);
				e=cmctf->eval();
				// 1/ctf, but stop at 1st zero
				if (makemap==4) {
/*					if (fabs(e)>.05 && !f) f=1;
					if (fabs(e)<.05 && f) f=2; 
					if (f==2) e=20.0; else e=1.0/e; */
					if (fabs(e)<.04) e=25; else e=1.0/e;
				}
				*d=e;
				d++;
				*d=e;
				d++;
			}
		}
	}
	
	ctfmap->doneData();
	
//	ctfmap->writeMRC("ctfmap.mrc");

}

// This will make a 3d amp mult map which allows an x-ray SF to be
// applied to a 3d data set
if (makemap==5) {
	int i,j,k,r,ny;
	float s,ds,*d,e,l=1.0;
	
	if (ctfmap) delete ctfmap;
	ctfmap=new EMData;
	ctfmap->setSize(size+2,size,size);
	ctfmap->setRI(1);

	cmgamma->setVar("Cs",Cs);
	cmgamma->setVar("lambda",lambda);
	cmgamma->setVar("defocus",defocus);	

	cmctf->setVar("B",B);
	cmctf->setVar("exps4",exps4);
	cmctf->setVar("ampcont",ampcont);
	cmctf->setVar("amp",amp);

	ds=1.0/(sstep*ctfmap->ySize());
	
	ny=ctfmap->ySize();
	d=ctfmap->getData();

	for (k=0; k<ny; k++) {
		for (j=0; j<ny; j++) {
			for (i=0; i<ny+2; i+=2) {
				s=sqrt(i*i/4.0+(j-ny/2)*(j-ny/2)+(k-ny/2)*(k-ny/2))*ds/(smax/CTFSTEPS)-1.0;
				r=fabs(s);	// r is the element number in the data array
				s-=r;		// s is now remainder for interpolation
				
				if (data->dataAt(10,r+1)<=0||data->dataAt(10,r+1)<=0||data->dataAt(13,r+1)==1.0) e=l;
				else {
					e=s*data->dataAt(13,r+1)/data->dataAt(10,r+1)+
						(1.0-s)*data->dataAt(13,r)/data->dataAt(10,r);
					if (e<0) e=0; else e=sqrt(e);
					l=e;
				}
				
				*d=e;
				d++;
				*d=e;
				d++;
			}
		}
	}
	
	ctfmap->doneData();
	
//	ctfmap->writeMRC("ctfmap.mrc");

}

}
