#include "TriView.h"
#include <qfile.h>
#include <qfiledialog.h>
#include <qcolor.h>

TriView::TriView(QWidget* parent,const char* name):QWidget( parent, name ) {
initMetaObject();

qp=new QPainter();
qpm=new QPixmap(size());

setBackgroundMode(NoBackground);

points.setAutoDelete(TRUE);
x0=y0=0;
x1=y1=.00001;
vmin=1.0e30;
vmax=-1.0e30;
selpoint=-1;
needupd=1;
colormode=1;
}


TriView::~TriView() {

}

void TriView::clear() {
points.clear();
x0=y0=0;
x1=y1=.00001;
vmin=1.0e30;
vmax=-1.0e30;
selpoint=-1;
needupd=1;
}

void TriView::resizeEvent ( QResizeEvent *qe ) {
qp->begin(this);
qp->eraseRect(qp->window());
qp->end();

int s=size().width()<size().height()?size().width():size().height();
qpm->resize(s,s);
needupd=1;
}

void TriView::selPointMap(int m) { 

selpoint=-1;

for (int i=0; i<points.count(); i++) {
	TriPoint *p=points.at(i);
	if (p->map==m) { selpoint=i; break; }
}

needupd=1; 
}

void TriView::paintEvent( QPaintEvent *pe ) {
int sgv;

if (vmin==vmax && points.count()>1) vmin-=.1;

if (needupd) {
	QPointArray tri(3);
	QPointArray tri2(3);
	QPointArray tri3(3);
	tri.setPoint(0,0,size().height()-1);
	tri.setPoint(1,size().width()-1,size().height()-1);
	tri.setPoint(2,size().width()/2,0);

	tri2.setPoint(0,0,size().height()-2);
	tri2.setPoint(1,0,0);
	tri2.setPoint(2,size().width()/2-1,0);

	tri3.setPoint(0,size().width()-1,size().height()-2);
	tri3.setPoint(1,size().width()-1,0);
	tri3.setPoint(2,size().width()/2+1,0);

	qp->begin( qpm);
	qp->setPen(blue);
	qp->setBrush(black);
	qp->drawPolygon(tri);
	
	if (points.count()>0) {
		for (int i=0; i<points.count(); i++) {
			TriPoint *p=points.at(i);
			if (vmax<=vmin) vmax=vmin+.001;
			int gv=(int)(floor(240.0*(p->val-vmin)/(vmax-vmin))+15);
			if (gv>255) gv=255;
			if (gv<0) gv=0;
			QColor dcol(gv,gv,gv);
			if (colormode==1 && gv>240) dcol.setRgb(gv,(255-gv)*16,(255-gv)*16);
			if (i==selpoint) continue;
			qp->setPen(dcol);
			qp->setBrush(dcol);
			qp->drawEllipse(xofaa(p->x,p->y)-2,yofa(p->y)-2,5,5);
//			printf("%f %f -> %1.0f %1.0f\n",p->x,p->y,xofaa(p->x,p->y),yofa(p->y));
		}
		if (selpoint>0) {
			TriPoint *p=points.at(selpoint);
			qp->setPen(blue);
			qp->setBrush(blue);
			qp->drawEllipse(xofaa(p->x,p->y)-2,yofa(p->y)-2,5,5);
		}
	}

	QColor bcol(192,192,192);
	qp->setPen(bcol);
	qp->setBrush(bcol);
	qp->drawPolygon(tri2);
	qp->drawPolygon(tri3);
	
	qp->setPen(blue);
	qp->setBrush(NoBrush);
	qp->drawPolygon(tri);
	
	qp->end();
	needupd=0;
}
bitBlt( this, 0,0, qpm );
}

void TriView::saveEPS() {
QString fsp = QFileDialog::getSaveFileName(getenv("PWD"));
FILE *out;

if (fsp.isNull() || !(out=fopen(fsp.ascii(),"w"))) return;

fprintf(out,"%%!PS-Adobe-3.0 EPSF-3.0\n%%%%Creator: EMAN\n%%%%BoundingBox: 10 10 260 230\n");
fprintf(out,"10 dict begin /cir { setgray 4 0 360 arc gsave 0 setgray stroke grestore fill } def\n");
fprintf(out,"1 setgray 10 10 260 10 135 226 10 10 moveto lineto lineto lineto fill\n");
fprintf(out,"1.5 setlinewidth 0 setgray\n");
fprintf(out,"10 10 260 10 135 226 10 10 moveto lineto lineto lineto stroke\n");
fprintf(out,"gsave 10 10 260 10 135 226 10 10 moveto lineto lineto lineto clip\n");

fprintf(out,".2 setlinewidth\n");
if (points.count()>0) {
	for (int i=0; i<points.count(); i++) {
		TriPoint *p=points.at(i);
		if (vmax<=vmin) vmax=vmin+.001;
		float gv=(p->val-vmin)/(vmax-vmin);
		if (gv>255) gv=255;
		if (gv<0) gv=0;
		fprintf(out,"%1.2f %1.2f %0.3f cir\n",
			10.0+250.0/2+(p->x-(x0+x1)/2.0)/(x1-x0)*(250.0-18)*(p->y)/(y1-y0),
			10.0+(216.0-16)*(y1-p->y)/(y1-y0)+6,gv);
	}
}

fprintf(out,"grestore end\n");
fclose(out);
}

float TriView::xofaa(float az,float alt) {
return size().width()/2+(az-(x0+x1)/2.0)/(x1-x0)*(size().width()-6)*(alt)/(y1-y0);
}

float TriView::yofa(float alt) {
return (size().height()-6)*(alt)/(y1-y0)+3;
}

float TriView::altofy(float y) {
return (y-3)*(y1-y0)/(size().height()-6.0)+y0;
}

float TriView::azofxy(float x,float y) {
if (y<4) y=4;
return (x-size().width()/2)/(size().width()*(y-3)/(size().height()-6))*(x1-x0)+(x1+x0)/2.0;
}

void TriView::mouseDoubleClickEvent(QMouseEvent *e) { };

void TriView::mouseEnter(int) { };

void TriView::mouseLeave(int) { };

void TriView::keyPressEvent(QKeyEvent *) { };

void TriView::keyReleaseEvent(QKeyEvent *) { };

void TriView::mousePressEvent(QMouseEvent *e) {
int i,x,y,bn=0;
float alt,az,r,br=1.0e30;

if (x1<=x0 || y1<=y0 || points.count()==0) return;

x=e->x();
y=e->y();

alt=altofy(y);
az=azofxy(x,y);

for (i=0; i<points.count(); i++) {
	r=hypot(xofaa(points.at(i)->x,points.at(i)->y)-x,yofa(points.at(i)->y)-y);
	if (r<br) { br=r; bn=i; }
}
selpoint=bn;

emit(select(az,alt,bn,points.at(bn)->val,points.at(bn)->map));

needupd=1;
update();
}

void TriView::mouseReleaseEvent(QMouseEvent *e) {
int *a,b;

}

void TriView::mouseMoveEvent(QMouseEvent *e) {

}

void TriView::addPoint(float x,float y,float v,int m) {
TriPoint *tp;

if (y==0) x=0;

tp=new TriPoint;
tp->x=x;
tp->y=y;
tp->val=v;
tp->map=m;
points.append(tp);

if (v>vmax) vmax=v;
if (v<vmin) vmin=v;

if (x<x0) x0=x;
if (y<y0) y0=y;
if (x>x1) x1=x;
if (y>y1) y1=y;

needupd=1;
}
