// EMData.C - Steve Ludtke  6/97
// This is the heart of the EMAN project. It does just about all of the
// hard stuff. It reads and writes EM images in various formats, does FFTs,
// filters, etc...
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "List.h"
#include "mrc.h"
#include "imagic.h"
#include "emim.h"
#include "util.h"
#include "EMData.h"

// defined from 0 to 5 (pixels) in .001 steps
extern float HYPERGEOM[];

// linear interpolation of HYPERGEOM
float hyperg(float v) {
float r;
int a;

//if (v<0||v>=4.999) { printf("Waring: hypergeom %f outside range\n",v); return(0); }
if (v<0 || v>4.998) return 0;

r=v/.001;
a=(int)floor(r);
r-=a;

//printf("%d %f\n",a,HYPERGEOM[a]*(1.0-r)+HYPERGEOM[a+1]*r);

return -(HYPERGEOM[a]*(1.0-r)+HYPERGEOM[a+1]*r);

}

// This is used to define the width of the gaussian for 5x5x5 interpolation
// in 3d slices
#define I5G	(10.4/(PI*PI))	// used for 5x5x5 interpolation
#define I4G	(8.8/(PI*PI))	// used for 4 interpolation
#define I3G	(6.4/(PI*PI))	// used for 3 and 5x5x5 interpolation
#define I2G	(4.0/(PI*PI))	// 2 interpolation


//#define DEBUG

// subtract 2 angles (radians)
inline float asub(float x,float y) {
float r;
r=fabs(x-y);
if (r>PI) r=2.0*PI-r;
return r;
}

// subtract 2 angles (radians)
inline float asub2(float x,float y) {
float r;

r=fmod(fabs(x-y),PI);
if (r>PI/2.0) r=PI-r;
return r;
}

// uses dx,dy,euler.alt() to rotate 'parent' into this image. If parent is
// NULL, just rotates/translates the current image
// scaling factor is optional, and not persistent
void EMData::rotateAndTranslate(float scale,float dxc,float dyc,float dzc) {
float *ddata,*sdata;
float x,y,z,yy,x2,y2,z2,mx[9];
float t,u,v,p[4],em,fl;
int k0,k1,k2,k3;
int i,j,k,ii,jj;


if (scale!=0) scale=1.0/scale; else scale=1.0;

if (parent) {
	sdata=parent->getDataRO();
	ddata=getData();
}
else {
	sdata=getData();
	ddata=(float *)Smalloc(nx*ny*nz*sizeof(float),1);
}

//if (flags&EMDATA_FLIP) fl=-1.0; else fl=1.0;
fl=1.0;

if (nz==1) {
//em=edgeMean();
em=0;
mx[0]=scale*cos(euler.alt());
mx[1]=scale*sin(euler.alt());
for (j=0,y=-ny/2.0+dxc; j<ny; j++,y+=1.0) {
	for (i=0,x=-nx/2.0+dyc; i<nx; i++,x+=1.0) {
		// rotate coordinates
		x2=(mx[0]*x+mx[1]*y*fl)+nx/2.0-dxc-dx;
		y2=(-mx[1]*x+mx[0]*y*fl)+ny/2.0-dyc-dy;

		if (x2<0||x2>nx-1.0||y2<0||y2>ny-1.0) {
//			ddata[i+j*nx]=em;
//			ddata[i+j*nx]=grand(Mean(),Sigma());
//			ddata[i+j*nx]=Mean();
			ddata[i+j*nx]=0;
			continue;
		}

		ii=floor2(x2);
		jj=floor2(y2);
		k0=ii+jj*nx;
		k1=k0+1;
		k2=k0+nx+1;
		k3=k0+nx;
		
		if (ii==nx-1) { k1--; k2--; }
		if (jj==ny-1) { k2-=nx; k3-=nx; }
		
		// no interpolation
		//ddata[i+j*nx]=sdata[ii+jj*nx];

		// bilinear interpolation of raw data
		t=(x2-(float)ii);
		u=(y2-(float)jj);

		p[0]=sdata[k0];
		p[1]=sdata[k1];
		p[3]=sdata[k3];
		p[2]=sdata[k2];
		ddata[i+j*nx]=(1.0-t)*(1.0-u)*p[0]+t*(1.0-u)*p[1]+(1.0-t)*u*p[3]+t*u*p[2];
	}
}
}
else {
	float x3,y3,z3, x4,y4,z4;
	int l,xy,mr;
	
	mx[0]=scale*(cos(euler.phi())*cos(euler.az())-cos(euler.alt())*sin(euler.az())*sin(euler.phi()));
	mx[1]=-scale*(sin(euler.phi())*cos(euler.az())+cos(euler.alt())*sin(euler.az())*cos(euler.phi()));
	mx[2]=scale*sin(euler.alt())*sin(euler.az());
	mx[3]=scale*(cos(euler.phi())*sin(euler.az())+cos(euler.alt())*cos(euler.az())*sin(euler.phi()));
	mx[4]=scale*(-sin(euler.phi())*sin(euler.az())+cos(euler.alt())*cos(euler.az())*cos(euler.phi()));
	mx[5]=-scale*sin(euler.alt())*cos(euler.az());
	mx[6]=scale*sin(euler.alt())*sin(euler.phi());
	mx[7]=scale*sin(euler.alt())*cos(euler.phi());
	mx[8]=scale*cos(euler.alt());

	x4=(mx[0]*(-nx/2.0+dxc)+mx[1]*(-ny/2.0+dyc)+mx[2]*(-nz/2.0+dzc))+nx/2.0-dxc-dx;
	y4=(mx[3]*(-nx/2.0+dxc)+mx[4]*(-ny/2.0+dyc)+mx[5]*(-nz/2.0+dzc))+ny/2.0-dyc-dy;
	z4=(mx[6]*(-nx/2.0+dxc)+mx[7]*(-ny/2.0+dyc)+mx[8]*(-nz/2.0+dzc))+nz/2.0-dzc-dz;

	xy=nx*ny;
	if (nx<ny && nx<nz) mr=(nx-2)*(nx-2)/4;
	else if (ny<nz) mr=(ny-2)*(ny-2)/4;
	else mr=(nz-2)*(nz-2)/4;
	
	for (k=-nz/2,l=0; k<nz/2; k++,x4+=mx[2],y4+=mx[5],z4+=mx[8]) {	
		x3=x4;
		y3=y4;
		z3=z4;
		for (j=-ny/2; j<ny/2; j++,x3+=mx[1],y3+=mx[4],z3+=mx[7]) {
			x2=x3;
			y2=y3;
			z2=z3;
			for (i=-nx/2; i<nx/2; i++,l++,x2+=mx[0],y2+=mx[3],z2+=mx[6]) {
			
				if (x2<0||y2<0||z2<0|| x2>=nx-1||y2>=ny-1||z2>=nz-1) {
//				if (i*i+j*j+k*k>=mr) {
					ddata[l]=0;
					continue;
				}

				x=floor2(x2);
				y=floor2(y2);
				z=floor2(z2);

				t=x2-x;
				u=y2-y;
				v=z2-z;

				// real part
				ii=(int)(x+y*nx+z*xy);

				ddata[l]=trilin(sdata[ii],sdata[ii+1],sdata[ii+nx],sdata[ii+nx+1],
					sdata[ii+nx*ny],sdata[ii+xy+1],sdata[ii+xy+nx],sdata[ii+xy+nx+1],
					t,u,v);
			}
		}
	}

/*	for (k=0,z=-nz/2.0+dzc; k<nz; k++,z+=1.0) {
		for (j=0,y=-ny/2.0+dyc; j<ny; j++,y+=1.0) {
			for (i=0,x=-nx/2.0+dxc; i<nx; i++,x+=1.0) {
				x2=(mx[0]*x+mx[1]*y+mx[2]*z)+nx/2.0-dxc-dx;
				y2=(mx[3]*x+mx[4]*y+mx[5]*z)+ny/2.0-dyc-dy;
				z2=(mx[6]*x+mx[7]*y+mx[8]*z)+nz/2.0-dzc-dz;

				if (x2<0||y2<0||z2<0|| x2>=nx-1||y2>=ny-1||z2>=nz-1) {
					ddata[i+j*nx+k*nx*ny]=0;
					continue;
				}

				t=x2-floor(x2);
				u=y2-floor(y2);
				v=z2-floor(z2);

				// real part
				ii=(int)(floor(x2)+floor(y2)*nx+floor(z2)*nx*ny);

				ddata[i+j*nx+k*nx*ny]=trilin(sdata[ii],sdata[ii+1],sdata[ii+nx],sdata[ii+nx+1],
					sdata[ii+nx*ny],sdata[ii+nx*ny+1],sdata[ii+nx*ny+nx],sdata[ii+nx*ny+nx+1],
					t,u,v);
			}
		}
	}*/

}

if (parent) parent->doneData();
else {
	free(rdata);
	rdata=ddata;
	dx=dy=dz=0;
	setRAlign(0,0,0);
//	flags&=~EMDATA_FLIP;
}
doneData();

update();
return;
}

// faster version of rotateAndTranslate above, but lower quality
void EMData::rotateAndTranslateFast(float scale) {
float *ddata,*sdata;
float x,y,z,yy,x2,y2,z2,mx[9];
float dxx,dxy,dyy,dyx;
float t,u,v,p[4],em,fl;
int k0,k1,k2,k3;
int i,j,k,ii,jj;


if (scale!=0) scale=1.0/scale; else scale=1.0;

if (parent) {
	sdata=parent->getDataRO();
	ddata=getData();
}
else {
	sdata=getData();
	ddata=(float *)Smalloc(nx*ny*nz*sizeof(float),1);
}

if (nz==1) {
mx[0]=scale*cos(euler.alt());
mx[1]=scale*sin(euler.alt());
dxx=mx[0];
dyx=-mx[1];
dxy=mx[1];
dyy=mx[0];
for (j=k=0, x=-mx[1]*ny/2.0+nx/2.0-dx,y=-mx[0]*ny/2.0+ny/2.0-dy;   
	j<ny; j++,x+=dxy,y+=dyy) {
	for (i=0,x2=x-mx[0]*nx/2.0,y2=y+mx[1]*nx/2.0; i<nx; i++,k++,x2+=dxx,y2+=dyx) {

		if (x2<0||x2>nx-2.0||y2<0||y2>ny-2.0) {
//			ddata[k]=Mean();
//			ddata[k]=grand(Mean(),Sigma());
			ddata[k]=0;
			continue;
		}

		ii=(int)floor(x2+.5);
		jj=(int)floor(y2+.5);
		
		// no interpolation
		ddata[k]=sdata[ii+jj*nx];
	}
}
}
// not fast yet
else {
	float x3,y3,z3, x4,y4,z4;
	int l,xy,mr;
	
	mx[0]=scale*(cos(euler.phi())*cos(euler.az())-cos(euler.alt())*sin(euler.az())*sin(euler.phi()));
	mx[1]=-scale*(sin(euler.phi())*cos(euler.az())+cos(euler.alt())*sin(euler.az())*cos(euler.phi()));
	mx[2]=scale*sin(euler.alt())*sin(euler.az());
	mx[3]=scale*(cos(euler.phi())*sin(euler.az())+cos(euler.alt())*cos(euler.az())*sin(euler.phi()));
	mx[4]=scale*(-sin(euler.phi())*sin(euler.az())+cos(euler.alt())*cos(euler.az())*cos(euler.phi()));
	mx[5]=-scale*sin(euler.alt())*cos(euler.az());
	mx[6]=scale*sin(euler.alt())*sin(euler.phi());
	mx[7]=scale*sin(euler.alt())*cos(euler.phi());
	mx[8]=scale*cos(euler.alt());

	x4=(mx[0]*(-nx/2.0)+mx[1]*(-ny/2.0)+mx[2]*(-nz/2.0))+nx/2.0-dx;
	y4=(mx[3]*(-nx/2.0)+mx[4]*(-ny/2.0)+mx[5]*(-nz/2.0))+ny/2.0-dy;
	z4=(mx[6]*(-nx/2.0)+mx[7]*(-ny/2.0)+mx[8]*(-nz/2.0))+nz/2.0-dz;

	xy=nx*ny;
	if (nx<ny && nx<nz) mr=(nx-2)*(nx-2)/4;
	else if (ny<nz) mr=(ny-2)*(ny-2)/4;
	else mr=(nz-2)*(nz-2)/4;
	
	for (k=-nz/2,l=0; k<nz/2; k++,x4+=mx[2],y4+=mx[5],z4+=mx[8]) {	
		x3=x4;
		y3=y4;
		z3=z4;
		for (j=-ny/2; j<ny/2; j++,x3+=mx[1],y3+=mx[4],z3+=mx[7]) {
			x2=x3;
			y2=y3;
			z2=z3;
			for (i=-nx/2; i<nx/2; i++,l++,x2+=mx[0],y2+=mx[3],z2+=mx[6]) {
			
//				if (x2<0||y2<0||z2<0|| x2>=nx-1||y2>=ny-1||z2>=nz-1) {
				if (i*i+j*j+k*k>=mr) {
					ddata[l]=0;
					continue;
				}

				x=floor(x2+.5);
				y=floor(y2+.5);
				z=floor(z2+.5);

				// real part
				ii=(int)(x+y*nx+z*xy);

				ddata[l]=sdata[ii];
			}
		}
	}
}

if (parent) parent->doneData();
else {
	free(rdata);
	rdata=ddata;
	dx=dy=dz=0;
	setRAlign(0,0,0);
//	flags&=~EMDATA_FLIP;
}
doneData();

update();
return;
}

// Complex only, 
void EMData::applyRadFn(int n,float x0,float dx,float *y,int interp) {
int i, j, m, k=0, l;
float r,f;

if (!isComplex()) return;
//writeMRC("a.img");
ap2ri();
getData();

// 2d
if (nz==1) {
	for (j=0; j<ny; j++) {
	    for (i=0; i<nx; i+=2, k+=2) {
//	r=hypot((float)i/2.0,(float)j-ny/2.0-1.0);
		r=hypot((float)i/2.0,(float)j-ny/2.0);
		r=(r-x0)/dx;
		if (interp) l=(int)floor(r); else l=(int)floor(r+1);
		r-=l;
		if (l>=n-2) f=y[n-1];
		else {
			if (interp) f=(y[l]*(1.0-r)+y[l+1]*r);
			else f=y[l];
		}
		rdata[k]*=f;
		rdata[k+1]*=f;
		}
	}
}
else {
	for (m=k=0; m<nz; m++) {
		for (j=0; j<ny; j++) {
		    for (i=0; i<nx; i+=2, k+=2) {
//	r=hypot((float)i/2.0,(float)j-ny/2.0-1.0);
			r=sqrt((float)(i*i/4.0)+(float)((j-ny/2.0)*(j-ny/2.0))+(float)((m-nz/2.0)*(m-nz/2.0)));
			r=(r-x0)/dx;
			if (interp) l=(int)floor(r); else l=(int)floor(r+1);
			r-=l;
			if (l>=n-2) f=y[n-1];
			else {
				if (interp) f=(y[l]*(1.0-r)+y[l+1]*r);
				else f=y[l];
			}
			rdata[k]*=f;
			rdata[k+1]*=f;
			}
		}
	}

}

doneData();
update();
//writeMRC("b.img");
}

// the passed noise is a mean noise INTENSITY. The noise distribution is a gaussian. This
// fact is not based on any real data
void EMData::addRadNoise(int n,float x0,float dx,float *y,int interp) {
int i, j, m, k=0, l;
float r,f,a;

if (!isComplex()) return;
//writeMRC("a.img");
ap2ri();
getData();

if (nz==1) {
	for (j=k=0; j<ny; j++) {
	    for (i=0; i<nx; i+=2, k+=2) {
//	r=hypot((float)i/2.0,(float)j-ny/2.0-1.0);
			r=hypot((float)i/2.0,(float)j-ny/2.0);
			r=(r-x0)/dx;
			if (interp) l=(int)floor(r); else l=(int)floor(r+.5);
			r-=l;
			if (l>=n-2) f=y[n-1];
			else if (l<0) l=0;
			else {
				if (interp) f=(y[l]*(1.0-r)+y[l+1]*r);
				else f=y[l];
			}
			f=grand(sqrt(f),sqrt(f)/3.0);
			a=frand(0,2.0*PI);
			if (i==0) f*=sqrt(2.0);
			rdata[k]+=f*cos(a);
			rdata[k+1]+=f*sin(a);
		}
	}
}
else { printf("3d addRadNoise unimplimented\n"); exit(1); }
doneData();

}

// subtract 'obj' from 'this'
void EMData::subtract(EMData *obj) {
int i,j,c;
float *dest,*src;

if (obj->isComplex()) obj->ap2ri();
if (obj->xSize()!=nx || obj->ySize()!=ny || obj->zSize()!=nz) return;

dest=getData();
src=obj->getDataRO();

for (j=0; j<nx*ny*nz; j++) dest[j]-=src[j];

obj->doneData();
doneData();

update();
}

// Does a vertical flip of a data set. This flips around the vertical 'phase origin'.
// ie - the bottom row/plane is not mirrored
void EMData::vFlip() {
int i,j,k;
float t;

getData();

/*
if (nz==1) {
	for (i=0; i<(ny+1)>>1; i++) {
		for (j=0; j<nx; j++) {
			t=rdata[i*nx+j];
			rdata[i*nx+j]=rdata[(ny-i-1)*nx+j];
			rdata[(ny-i-1)*nx+j]=t;
		}
	}
}
else {
	for (i=0; i<(nz+1)>>1; i++) {
		for (j=0; j<ny; j++) {
			for (k=0; k<nx; k++) {
				t=rdata[i*nx*ny+j*nx+k];
				rdata[i*nx*ny+j*nx+k]=rdata[(nz-i-1)*nx*ny+j*nx+k];
				rdata[(nz-i-1)*nx*ny+j*nx+k]=t;
			}
		}
	}
}
*/

if (nz==1) {
	for (i=1; i<ny/2; i++) {
		for (j=0; j<nx; j++) {
			t=rdata[i*nx+j];
			rdata[i*nx+j]=rdata[(ny-i)*nx+j];
			rdata[(ny-i)*nx+j]=t;
		}
	}
	// bottom is a copy of the row above the bottom (hopefully avoids fourier artifacts)
	for (j=0; j<nx; j++) {
		rdata[j]=rdata[nx+j];
	}
}
else {
	for (i=1; i<nz/2; i++) {
		for (j=0; j<ny; j++) {
			for (k=0; k<nx; k++) {
				t=rdata[i*nx*ny+j*nx+k];
				rdata[i*nx*ny+j*nx+k]=rdata[(nz-i)*nx*ny+j*nx+k];
				rdata[(nz-i)*nx*ny+j*nx+k]=t;
			}
		}
	}
	for (j=0; j<ny; j++) {
		for (k=0; k<nx; k++) {
			rdata[j*nx+k]=rdata[nx*ny+j*nx+k];
		}
	}
}

doneData();
}

// in place DWT, basis/level defined elsewhere
void EMData::doDWT(int basis, int level) {	

}

// inverse wavelet x-form
void EMData::doIWT(int basis, int level) {

}

// in place nonlinear threshold wavelet filter
// thresh is with respect to the 'ideal' threshold
void EMData::DWTFilt(int basis, int level, float thresh) {

}

// applies a circular mask to the data
// type=0 is a step cutoff to the mean value
// type=1 fills in with flatband random noise
// type=2 is a tanh cutoff to 0
// type=3 is a gaussian falloff, r is the 1/e width
// type=4 is a step cutoff to 0
// type=5 is an INNER step cutoff to 0 (inside r -> 0)
// type=6 is a cutoff to the circular edgemean value
// type=7 generates a 2 pixel ring of '1', replacing the current contents
// type=107 is an exp(x^4) to fix hypergeometric 3d interpolation
// type=105/6 is an inverse gaussian to fix gaussian 3d interpolation (modes 5-6)
// type=104 (mode 4)
// type=103 (mode 3)
// type=102 is an inverse gaussian to fix gaussian 3d interpolation (mode 2)
// type=101 is actually a null function
// type=20 is a forward gaussian to 'unfix' gaussian 3d interpolation (modes 3-5)
// type=-1 takes the fabs() of the data
// real only at the moment
void EMData::applyMask(int r,int type) {
int x,y,z,i;
float k,rr,m;
float x1,y1,z1;
if (flags&EMDATA_COMPLEX) return;
if (type==5 &&r<=0) return;
if (r<=0) r=nx/2;

getData();
if (nz==1) {
	switch (type) {
	case 0:
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				x1=fabs(x-nx/2.0);
				y1=fabs(y-ny/2.0);
				if (x1>r || y1>r || hypot((float)x1,(float)y1)>r ) rdata[i]=Mean();
			}
		}
		break;
	case 1:
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				x1=fabs(x-nx/2.0);
				y1=fabs(y-ny/2.0);
				if (x1>r || y1>r || hypot((float)x1,(float)y1)>r ) rdata[i]=grand(Mean(),Sigma());
			}
		}
		break;
	case 2:
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=hypot(x-nx/2,y-ny/2);
				rdata[i]*=(tanh(((float)r-rr)/2.0)/2.0+.5);
			}
		}
		break;
	case 3:
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=hypot(x-nx/2,y-ny/2);
				rdata[i]*=exp(-rr*rr/(r*r));
			}
		}
		break;
	case 4:
		r=r*r;
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				if (hypot2(x-nx/2.0,y-ny/2.0)>r ) rdata[i]=0;
			}
		}
		break;
	case 5:
		r=r*r;
		if (r<1) break;
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				if (hypot2(x-nx/2.0,y-ny/2.0)<r ) rdata[i]=0;
			}
		}
		break;
	case 6:
		rr=k=0;
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				x1=fabs(x-nx/2.0);
				y1=fabs(y-ny/2.0);
				if (hypot((float)x1,(float)y1)<=r+1 &&
					hypot((float)x1,(float)y1)>=r-1) { rr+=rdata[i]; k+=1.0; }
			}
		}
		rr/=k;

		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				x1=fabs(x-nx/2.0);
				y1=fabs(y-ny/2.0);
				if (x1>r || y1>r || hypot((float)x1,(float)y1)>r ) rdata[i]=rr;
			}
		}
		break;		
	case 7:
		x1=(r-1)*(r-1);
		y1=(r+1)*(r+1);
		if (r<1) break;
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=hypot2(x-nx/2.0,y-ny/2.0);
				if (rr>=x1 && rr<=y1) rdata[i]=1;
			}
		}
		break;
	case -1:
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rdata[i]=fabs(rdata[i]);
			}
		}
		break;
	case 10:
	case 11:
	case 12:
		printf("ERROR!!!  This program uses an old interpolation method!\n"); exit(1);
	case 20:
		k=I5G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=hypot(x-nx/2.0,y-ny/2.0);
				rdata[i]*=exp(-rr*rr*k);
			}
		}
		break;
	case 107:
		m=ny*ny/4.0;
//		k=3.24*16.0/(float)(ny*ny*ny*ny);		// corrects for precalculated exp(-x^4)
		k=1.80*16.0/(float)(ny*ny*ny*ny);		// corrects for precalculated exp(-x^4)
										// kernel is hypergeometric
/*		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=SQR(x-nx/2.0)+SQR(y-ny/2.0);
				if (rr>m) rdata[i]=0;
				else rdata[i]/=exp(-rr*rr*k);
			}
		}*/
		break;
	case 106:
	case 105:
		k=I5G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=SQR(x-nx/2.0)+SQR(y-ny/2.0);
				rdata[i]/=exp(-rr*k);
			}
		}
		break;
	case 104:
		k=I4G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=SQR(x-nx/2.0)+SQR(y-ny/2.0);
				rdata[i]/=exp(-rr*k);
			}
		}
		break;
	case 103:
		k=I3G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=SQR(x-nx/2.0)+SQR(y-ny/2.0);
				rdata[i]/=exp(-rr*k);
			}
		}
		break;
	case 102:
		k=I2G*PI*PI/(float)(ny*ny);		// make sure we use I2G used in 3d slicing
		for (y=i=0; y<ny; y++) {
			for (x=0; x<nx; x++,i++) {
				rr=hypot(x-nx/2.0,y-ny/2.0);
				rdata[i]/=exp(-rr*rr*k);
			}
		}
		break;
	case 101:
		break;
	}
}
else {
	switch (type) {
	case 0:
		r=r*r;
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					if (hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0)<r) continue;
					rdata[i]=Mean();
				}
			}
		}
		break;
	case 1:
		r=r*r;
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					if (hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0)<r) continue;
					rdata[i]=grand(Mean(),Sigma());
				}
			}
		}
		break;
	case 2:
		k=16.0/(float)(ny*ny);
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=sqrt(hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0));
					rdata[i]*=(tanh(((float)r-rr)/2.0)/2.0+.5);
				}
			}
		}
		break;
	case 3:
		r=r*r;
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0);
					rdata[i]*=exp(-rr/r);
				}
			}
		}
		break;
	case 4:
		r=r*r;
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					if (hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0)<r) continue;
					rdata[i]=0;
				}
			}
		}
		break;
	case 5:
		if (r<1) break;
		r=r*r;
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					if (hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0)>r) continue;
					rdata[i]=0;
				}
			}
		}
		break;
	case 6:
		rr=k=0;
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					x1=sqrt(hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0));
					if (x1<=r+1 && x1>=r-1) { rr+=rdata[i]; k+=1.0; }
				}
			}
		}
		rr/=k;

		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					if (hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0)<r*r) continue;
					rdata[i]=rr;
				}
			}
		}

		break;		
	case 7:
		x1=(r-1)*(r-1);
		y1=(r+1)*(r+1);
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0);
					if (rr>=x1 && rr<=y1) rdata[i]=1.0;
				}
			}
		}
		break;
	case 10:
	case 11:
	case 12:
		printf("ERROR!!!  This program uses an old interpolation method!\n"); exit(1);
	case 20:
		k=I5G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=sqrt(hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0));
					rdata[i]*=exp(-rr*rr*k);
				}
			}
		}
		break;
	case 107:
		m=ny*ny/4.0;
		k=3.24*16.0/(float)(ny*ny*ny*ny);		// corrects for precalculated exp(-x^4)
//		k=1.8*16.0/(float)(ny*ny*ny*ny);		// corrects for precalculated exp(-x^4)
										// kernel is hypergeometric
/*		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0);
					if (rr>m) rdata[i]=0;
					else rdata[i]/=exp(-rr*rr*k);
				}
			}
		}*/
		break;
	case 106:
	case 105:
		k=I5G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=sqrt(hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0));
					rdata[i]/=exp(-rr*rr*k);
				}
			}
		}
		break;
	case 104:
		k=I4G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=sqrt(hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0));
					rdata[i]/=exp(-rr*rr*k);
				}
			}
		}
		break;
	case 103:
		k=I3G*PI*PI/(float)(ny*ny);		// make sure we use I5G used in 3d slicing
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=sqrt(hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0));
					rdata[i]/=exp(-rr*rr*k);
				}
			}
		}
		break;
	case 102:
		k=I2G*PI*PI/(float)(ny*ny);		// make sure we use I2G used in 3d slicing
		for (z=i=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x++,i++) {
					rr=sqrt(hypot3(x-nx/2.0,y-ny/2.0,z-nz/2.0));
					rdata[i]/=exp(-rr*rr*k);
				}
			}
		}
		break;
	case 101:
		break;
	}
}

doneData();

update();
}

// calculates radial distribution, works for real
// and complex images. Note that this returns the mean INTENSITY
// for complex images !!!
// For 3D data, only complex form currently supported
// x must be defined on input and y must be allocated.
void EMData::calcRadDist(int n,float x0,float dx,float *d) {
int x,y,z;
int i,c;
float r,h;
float *yc;

getData();

yc=(float *)malloc(n*sizeof(float));
for (i=0; i<n; i++) { 
	d[i]=0; 
	yc[i]=0;			// for empty bins
}

if (nz==1) {
	if (flags&EMDATA_COMPLEX) {
		if (flags&EMDATA_RI) {
			for (y=c=0; y<ny; y++) {
				for (x=0; x<nx; x+=2,c+=2) {
					r=(hypot(x/2.0,y-ny/2.0)-x0)/dx;
					
					i=(int)floor(r);						// lower bin to put data in
					r-=i;							// float remainder, how much to put in each bin
					if (i<0 || i>=n) continue;		// beyond the range we're calculating
					if (i==0) { d[0]+=rdata[c]*(1.0-r); yc[0]+=(1.0-r); }
					else if (i==n-1) { d[n-1]+=rdata[c]*(r); yc[n-1]+=(r); }
					else {
						h=SQR(rdata[c])+SQR(rdata[c+1]);
						d[i]+=h*(1.0-r); 
						yc[i]+=(1.0-r); 
						d[i+1]+=h*r; 
						yc[i+1]+=(r); 
					}
				}
			}
		}
		else {
			for (y=c=0; y<ny; y++) {
				for (x=0; x<nx; x+=2,c+=2) {
					r=(hypot(x/2.0,y-ny/2.0)-x0)/dx;
					
					i=(int)floor(r);						// lower bin to put data in
					r-=i;							// float remainder, how much to put in each bin
					if (i<0 || i>=n) continue;		// beyond the range we're calculating
					if (i==0) { d[0]+=rdata[c]*(1.0-r); yc[0]+=(1.0-r); }
					else if (i==n-1) { d[n-1]+=rdata[c]*(r); yc[n-1]+=(r); }
					else {
						h=rdata[c]*rdata[c];
						d[i]+=h*(1.0-r); 
						yc[i]+=(1.0-r); 
						d[i+1]+=h*r; 
						yc[i+1]+=(r); 
					}
				}
			}
		}
	}
	else {
		for (y=c=0; y<ny; y++) {
			for (x=0; x<nx; x++,c++) {
				r=(hypot(x-(nx)/2.0,y-(ny)/2.0)-x0)/dx;
				
				i=(int)floor(r);						// lower bin to put data in
				r-=i;							// float remainder, how much to put in each bin
				if (i<0 || i>=n) continue;		// beyond the range we're calculating
				if (i==0) { d[0]+=rdata[c]*(1.0-r); yc[0]+=(1.0-r); }
				else if (i==n-1) { d[n-1]+=rdata[c]*(r); yc[n-1]+=(r); }
				else { 
					d[i]+=rdata[c]*(1.0-r); 
					yc[i]+=(1.0-r); 
					d[i+1]+=rdata[c]*(r); 
					yc[i+1]+=(r); 
				}
			}
		}
	}
}
else {
	if (flags&EMDATA_COMPLEX) {
//		if (!flags&EMDATA_RI) ap2ri();
		for (z=c=0; z<nz; z++) {
			for (y=0; y<ny; y++) {
				for (x=0; x<nx; x+=2,c+=2) {
					r=(sqrt(SQR(x/2.0)+SQR(y-ny/2.0)+SQR(z-nz/2.0))-x0)/dx;
				
					i=(int)floor(r);						// lower bin to put data in
					r-=i;							// float remainder, how much to put in each bin
					if (i<0 || i>=n) continue;		// beyond the range we're calculating
					if (i==0) { d[0]+=rdata[c]*(1.0-r); yc[0]+=(1.0-r); }
					else if (i==n-1) { d[n-1]+=rdata[c]*(r); yc[n-1]+=(r); }
					else {
						h=SQR(rdata[c])+SQR(rdata[c+1]);
						d[i]+=h*(1.0-r); 
						yc[i]+=(1.0-r); 
						d[i+1]+=h*r; 
						yc[i+1]+=(r); 
					}
				}
			}
		}
	}
}

for (i=0; i<n; i++) {
	if (yc[i]!=0) d[i]/=yc[i];
}

// linear interpolation for 'holes'
// does NOT deal with 2 sequential holes
for (i=1; i<n-1; i++) {
	if (yc[i]==0) d[i]=(d[i-1]+d[i+1])/2.0;
}

doneData();
free(yc);
}

// Same as above, but only for a range of angles. amin,amax are in degrees from x axis
void EMData::calcRadDist(int n,float x0,float dx,float *d,float acen,float awid)  {
int x,y;
int i,c;
float r,h,a;
float *yc,*yc2,*d2;

getDataRO();

yc=(float *)malloc(n*sizeof(float));
yc2=(float *)malloc(n*sizeof(float));
d2=(float *)malloc(n*sizeof(float));

for (i=0; i<n; i++) { 
	d[i]=0; 
	yc[i]=0;
}

if (nz==1) {
	if (flags&EMDATA_COMPLEX) {
		if (flags&EMDATA_RI) {
			for (y=c=0; y<ny; y++) {
				for (x=0; x<nx; x+=2,c+=2) {
					if (y==ny/2.0 && x==0) a=0;
					else a=atan2(y-ny/2.0,x/2.0);
					if (fabs(asub2(a,acen))>awid) continue;
					r=(hypot(x/2.0,y-ny/2.0)-x0)/dx;
					
					i=(int)floor(r);				// lower bin to put data in
					r-=i;							// float remainder, how much to put in each bin
					if (i<0 || i>=n) continue;		// beyond the range we're calculating
					if (i==0) { d[0]+=rdata[c]*(1.0-r); yc[0]+=(1.0-r); }
					else if (i==n-1) { d[n-1]+=rdata[c]*(r); yc[n-1]+=(r); }
					else {
						h=hypot(rdata[c],rdata[c+1]);
						d[i]+=h*(1.0-r); 
						yc[i]+=(1.0-r); 
						d[i+1]+=h*r; 
						yc[i+1]+=(r); 
					}
				}
			}
		}
		else {
			for (y=c=0; y<ny; y++) {
				for (x=0; x<nx; x+=2,c+=2) {
					if (y==ny/2.0 && x==0) a=0;
					else a=atan2(y-ny/2.0,x/2.0);
					if (fabs(asub2(a,acen))>awid) continue;
					r=(hypot(x/2.0,y-ny/2.0)-x0)/dx;
					
					i=(int)floor(r);						// lower bin to put data in
					r-=i;							// float remainder, how much to put in each bin
					if (i<0 || i>=n) continue;		// beyond the range we're calculating
					if (i==0) { d[0]+=rdata[c]*(1.0-r); yc[0]+=(1.0-r); }
					else if (i==n-1) { d[n-1]+=rdata[c]*(r); yc[n-1]+=(r); }
					else {
						h=rdata[c];
						d[i]+=h*(1.0-r); 
						yc[i]+=(1.0-r); 
						d[i+1]+=h*r; 
						yc[i+1]+=(r); 
					}
				}
			}
		}
	}
	else {
		for (y=c=0; y<ny; y++) {
			for (x=0; x<nx; x++,c++) {
				if (y==ny/2.0 && x==nx/2.0) a=0;
				else a=atan2(y-ny/2.0,x-nx/2.0);
				if (fabs(asub2(a,acen))>awid) continue;
				r=(hypot(x-(nx-1)/2.0,y-(ny-1)/2.0)-x0)/dx;
				
				i=(int)floor(r);						// lower bin to put data in
				r-=i;							// float remainder, how much to put in each bin
				if (i<0 || i>=n) continue;		// beyond the range we're calculating
				if (i==0) { d[0]+=rdata[c]*(1.0-r); yc[0]+=(1.0-r); }
				else if (i==n-1) { d[n-1]+=rdata[c]*(r); yc[n-1]+=(r); }
				else { 
					d[i]+=rdata[c]*(1.0-r); 
					yc[i]+=(1.0-r); 
					d[i+1]+=rdata[c]*(r); 
					yc[i+1]+=(r); 
				}
			}
		}
	}
}
else {

}

for (i=0; i<n; i++) { d2[i]=d[i]; yc2[i]=yc[i]; }
for (i=1; i<n; i++) { d2[i-1]+=d[i]*.57; yc2[i-1]+=yc[i]*.57; }
for (i=2; i<n; i++) { d2[i-2]+=d[i]*.105; yc2[i-2]+=yc[i]*.105; }
for (i=0; i<n-1; i++) { d2[i+1]+=d[i]*.57; yc2[i+1]+=yc[i]*.57; }
for (i=0; i<n-2; i++) { d2[i+2]+=d[i]*.105; yc2[i+2]+=yc[i]*.105; }

for (i=0; i<n; i++) { if (yc2[i]!=0) d[i]=d2[i]/yc2[i]; else d[i]=0; }

/*
for (i=0; i<n; i++) { if (yc[i]!=0) d[i]/=yc[i]; } 

for (i=1; i<n-1; i++) {
	if (yc[i]==0) d[i]=(d[i-1]+d[i+1])/2.0;
}
*/

doneData();
free(yc);
free(yc2);
free(d2);
}

// works for real or complex images, 2d only for now
void EMData::calcAzDist(int n,float a0,float da,float *d,float rmin,float rmax) {
int x,y;
int i,c;
float r,h,a,w;
float *yc;

getData();



yc=(float *)malloc(n*sizeof(float));
for (i=0; i<n; i++) { 
	d[i]=0; 
	yc[i]=.00001;			// for empty bins
}

if (nz==1) {
	if (flags&EMDATA_COMPLEX) {
		if (flags&EMDATA_RI) {
			for (y=c=0; y<ny; y++) {
				for (x=0; x<nx; x+=2,c+=2) {
					r=hypot(x/2.0,y-ny/2.0);
					if (r<rmin||r>rmax) continue;
					if (y==ny/2.0 && x==0) a=0;
					else a=(atan2(y-ny/2.0,x/2.0)-a0)/da;
					
					i=(int)floor(a);						// lower bin to put data in
					a-=i;							// float remainder, how much to put in each bin
					if (i<0 || i>=n) continue;		// beyond the range we're calculating
					if (i==0) { d[0]+=rdata[c]*(1.0-a); yc[0]+=(1.0-a); }
					else if (i==n-1) { d[n-1]+=rdata[c]*(a); yc[n-1]+=(a); }
					else {
						h=hypot(rdata[c],rdata[c+1]);
						d[i]+=h*(1.0-a); 
						yc[i]+=(1.0-a); 
						d[i+1]+=h*a; 
						yc[i+1]+=(a); 
					}
				}
			}
		}
		else {
			for (y=c=0; y<ny; y++) {
				for (x=0; x<nx; x+=2,c+=2) {
					r=hypot(y-ny/2.0,x/2.0);
					if (y==ny/2.0 && x==0) a=0;
					else a=(atan2(y-ny/2.0,x/2.0)-a0)/da;
					
					i=(int)floor(a);				// lower bin to put data in
					a-=i;							// float remainder, how much to put in each bin
					if (i<0 || i>=n) continue;		// beyond the range we're calculating
					if (i==0) { d[0]+=rdata[c]*(1.0-a); yc[0]+=(1.0-a); }
					else if (i==n-1) { d[n-1]+=rdata[c]*(a); yc[n-1]+=(a); }
					else {
						h=rdata[c];
						d[i]+=h*(1.0-a); 
						yc[i]+=(1.0-a); 
						d[i+1]+=h*a;
						yc[i+1]+=(a); 
					}
				}
			}
		}
	}
	else {
		for (y=c=0; y<ny; y++) {
			for (x=0; x<nx; x++,c++) {
				r=hypot(y-(ny-1)/2.0,x-(nx-1)/2.0);
				if (r<rmin||r>rmax) continue;
				if (y-(ny-1)/2.0==0 && x-(nx-1)/2.0==0) a=0;
				else a=atan2(y-(ny-1)/2.0,x-(nx-1)/2.0);
				if (a<0) a+=PI*2.0;
				a=(a-a0)/da;
				
				i=(int)floor(a);				// lower bin to put data in
				a-=i;							// float remainder, how much to put in each bin
				if (i<0 || i>=n) continue;		// beyond the range we're calculating
				if (i==0) { d[0]+=rdata[c]*(1.0-a); yc[0]+=(1.0-a); }
				else if (i==n-1) { d[n-1]+=rdata[c]*(a); yc[n-1]+=(a); }
				else { 
					d[i]+=rdata[c]*(1.0-a); 
					yc[i]+=(1.0-a); 
					d[i+1]+=rdata[c]*(a); 
					yc[i+1]+=(a); 
				}
			}
		}
	}
}
else {
	error("Cannot calcazdist 3d images",ERR_CRIT);
}

for (i=0; i<n; i++) d[i]/=yc[i];

doneData();
free(yc);
}

// makes image circularly symmetric (real only)
void EMData::radialAverage() {
float *dist;
int i,x,y,c;
float r;

if (flags&EMDATA_COMPLEX) return;

dist=(float *)malloc(nx/2*sizeof(float));
calcRadDist(nx/2,0,1,dist);	// calculate the radial distribution

getData();

for (y=c=0; y<ny; y++) {
	for (x=0; x<nx; x++,c++) {
		r=hypot(x-(nx-1)/2.0,y-(ny-1)/2.0);

		i=(int)floor(r);						// lower bin to put data in
		r-=i;
		if (i>=0&&i<nx/2-1) rdata[c]=dist[i]*(1.0-r)+dist[i+1]*r;
		else if (i<0) rdata[c]=dist[0];
		else rdata[c]=dist[nx/2-1];
	}
}

doneData();
update();

free(dist);
}

// subtracts the rotationally averaged image to leave only the asymmetric part
void EMData::radialSubtract() {
float *dist;
int i,x,y,c;
float r;

if (flags&EMDATA_COMPLEX) return;

dist=(float *)malloc(nx/2*sizeof(float));
calcRadDist(nx/2,0,1,dist);	// calculate the radial distribution

getData();

for (y=c=0; y<ny; y++) {
	for (x=0; x<nx; x++,c++) {
		r=hypot(x-nx/2,y-ny/2);

		i=(int)floor(r);						// lower bin to put data in
		r-=i;
		if (i>=0&&i<nx/2-1) rdata[c]-=dist[i]*(1.0-r)+dist[i+1]*r;
		else if (i<0) rdata[c]=0;
		else rdata[c]=0;
	}
}

doneData();
update();

free(dist);
}


// linear comparison of 2 data sets, better match -> larger result
// 'this' should be noise-free and 'data' should be noisy
float EMData::lcmp(EMData *data) {
float ret=0,*d1,*d2;
int i;

if (nx!=data->xSize() || ny!=data->ySize() || nz!=data->zSize()) return 0;

d1=getDataRO();
d2=data->getDataRO();

//for (i=0; i<nx*ny*nz; i++,d1++,d2++) ret+=fabs(*d1 - *d2);
//ret/=(float)(nx*ny*nz);

// how about a probability measure (slow)
for (i=0; i<nx*ny*nz; i++,d1++,d2++) ret-=SQR(*d1 - *d2);
//for (i=0; i<nx*ny*nz; i++,d1++,d2++) ret*=exp(-SQR(*d1 - *d2));
//ret=sqrt(ret/(nx*ny*nz))/2.0;

doneData();
data->doneData();
return ret;
}

// dot product of 2 images (same size)
// complex does not check r/i vs a/p
float EMData::dot(EMData *data,int evenonly) {
float ret=0,*d1,*d2,*de;
int i,n;

if (nx!=data->xSize() || ny!=data->ySize() || nz!=data->zSize()) return 0;

d1=getDataRO();
d2=data->getDataRO();

de=d1+nx*ny*nz;
n=nx*ny*nz;

if (evenonly) {
	for (i=0; i<n; i+=2,d1+=2,d2+=2) ret+=*d1 * *d2;
} else {
	while (d1!=de) ret+=*(d1++) * *(d2++);
}
doneData();
data->doneData();
return ret;
}

// variety of real space filters
// type 0 is a threshold filter, x<val1 -> 0
// type 1 is just like type 0 with a final normalize
// type 2 is similar, but sets x<val1 -> 0 and x>val -> 1
// type 3 x>val1 -> x, 0<x<val1->0, x<0 -> val2, then normalize
// type 10 takes the sqrt of each pixel
// type 20 is a median filter with a +-val1 size, val1=1 -> 3x3 median
void EMData::realFilter(int type, float val1, float val2,float val3) {
float *d,*d2;
int i,j,k,n;

switch (type) {
case 0:
	d=getData();
	for (i=0; i<nx*ny*nz; i++,d++) if (*d<val1) *d=0;
	doneData();
	break;
case 1:
	d=getData();
	for (i=0; i<nx*ny*nz; i++,d++) if (*d<val1) *d=0;
	doneData();
	normalize();
	break;
case 2:
	d=getData();
	for (i=0; i<nx*ny*nz; i++,d++) if (*d<val1) *d=0; else *d=1.0;
	doneData();
//	normalize();
	break;
case 3:
	d=getData();
	for (i=0; i<nx*ny*nz; i++,d++) {
		if (*d<val1) {
			if (*d>0) *d=0; 
			else *d=val2;
		}
	}
	doneData();
	normalize();
	break;
case 10:
	d=getData();
	for (i=0; i<nx*ny*nz; i++,d++) {
		*d=sqrt(*d<0?0:*d);
	}
	doneData();
	normalize();
	break;
case 20:
	{
	int i2,j2,k2;
	EMData *dat=copy(0,0);
	n=round(val1);
	floatStat stat((n*2+1)*(n*2+1)*(n*2+1));	// maximum size
	normalize();
	d=getData();
	d2=dat->getDataRO();
	if (nz>1) {
		for (k=n; k<nz-n; k++) {
			if (updProgress && updProgress(k-n,nz-2*n,"Filtering")) break;
			for (j=n; j<ny-n; j++) {
				for (i=n; i<nx-n; i++) {
					stat.clear();
					
					for (i2=i-n; i2<=i+n; i2++) {
						for (j2=j-n; j2<=j+n; j2++) {	
							for (k2=k-n; k2<=k+n; k2++) {
								stat.qadd(d2[i2+j2*nx+k2*nx*ny]);
							}
						}
					}
				
					d[i+j*nx+k*nx*ny]=stat.median();
				}
			}
		}
	}
	else {
		for (j=n; j<ny-n; j++) {
			if (updProgress && updProgress(j-n,ny-2*n,"Filtering")) break;
			for (i=n; i<nx-n; i++) {
				stat.clear();
					
				for (i2=i-n; i2<=i+n; i2++) {
					for (j2=j-n; j2<=j+n; j2++) {	
							stat.qadd(d2[i2+j2*nx]);
						}
					}
				
				d[i+j*nx]=stat.median();
			}
		}
	}
	if (updProgress) updProgress(0,0,NULL);

	dat->doneData();
	doneData();
	delete dat;
	break;
	}
default:
	printf("UNKNOWN REAL FILTER!\n");
	break;
}

return;
}

// fourier filters an image, real or imaginary
// note, this routine reallocates the data pointer if a real image is passed
// type 0 -> sharp cutoff
// type 1 -> gaussian lowpass (tanh highpass)
// type 2 -> hyperbolic tangent
// type 3 -> butterworth highpass
// type 4 -> undo tanh highpass, lowpass ignored
// type 5 -> gaussian tanh mix
// type 99 -> strange low freq, attempt to roughly fix bad struc fac
void EMData::filter(float highpass, float lowpass,int type) {
float *y,*t;
float x;
int i;
EMData *tmp,*tmp2;

y=(float *)malloc(sizeof(float)*ny*4);  // 8 samples per pixel to handle non integral r

switch (type) {
case 0:
	for (i=0,x=0; i<ny*4; i++,x+=.125) {
		if (x>=highpass && x<=lowpass) y[i]=1.0; else y[i]=0;
	}
	break;
case 1:
	if (lowpass>0) {
		for (i=0,x=0; i<ny*4; i++,x+=.125) {
			y[i]=(tanh(x-highpass)/2.0+.5)*exp(-x*x/(lowpass*lowpass));
//		y[i]=1./(1.  + (highpass/1.5/ (x+.001)) * (highpass/1.5/ (x+.001)) );
//			y[i] *= exp(-x*x/(lowpass*lowpass));
		}
	}
	// negative lowpass denotes sharpening rather than blurring
	else {
		for (i=0,x=0; i<ny*4; i++,x+=.125) {
			y[i]=(tanh(x-highpass)/2.0+.5)*exp(x*x/(lowpass*lowpass));
		}
	}	
	break;	
case 2:
	for (i=0,x=0; i<ny*4; i++,x+=.125) {
		y[i]=(tanh(x-highpass)/2.0+.5)*(tanh(lowpass-x)/2.0+.5);
	}
	break;	
case 3:// Butterworth High Pass
	for (i=0,x=0; i<ny*4; i++,x+=.125) {
		y[i]=1./(1.  + (highpass/1.5/ (x+.001)) * (highpass/1.5/ (x+.001)) );
	}
	break;	
case 4:
	for (i=0,x=0; i<ny*4; i++,x+=.125) {
		y[i]=1.0/(tanh(x-highpass)/2.0+.5);
		y[0]=0;
	}
	break;	
case 5:
	for (i=0,x=0; i<ny*4; i++,x+=.125) {
		y[i]=exp(-x*x/(lowpass*lowpass*4))*(tanh(lowpass-x)/2.0+.5);
	}
	break;
case 99:
	for (i=0,x=0; i<ny*4; i++,x+=.125) {
		y[i]=(tanh(x-highpass)*.45+.55);	
		y[0]=0;
	}
	break;	
}

if (isComplex()) {
	applyRadFn(ny*4,0,.125,y);
	doneData();
	free(y);
	return;
}

//save_data(0,.5, y, nx, "xxx");

toCorner();
tmp=doFFT();	// transform
gimmeFFT();		// make sure we have control of transform
tmp->applyRadFn(ny*4,0,.125,y);		// filter
tmp2=tmp->doIFT();				// invert transform
tmp2->toCorner();
tmp->gimmeFFT();				// take control of the inverse

// now swap the data pointers for the result and the original, this
// is dangerous, but a lot faster than doing a copy  :^)
getData();
tmp2->getData();
t=rdata;
rdata=tmp2->rdata;
tmp2->rdata=t;
tmp2->doneData();

delete tmp;
delete tmp2;

doneData();
update();
free(y);
}

// Uses functional parameters to calculate ctf amplitude as a function of s
// type 0 -> ctf amp vs s
// type 1 -> noise inten vs s
// type 2 -> fabs(ctf amp) vs s
// type 3 -> SNR (relative, no struc fac)
// type 4 -> SNR (absolute, requires SF data)
//			pass S vs log10(SF intensity)
float *EMData::ctfCurve(int type, int nsf,float *S,float *SF) {
float *r,ds;
float *p=ctf;
float lambda,gamma,s;
int i;

if (!flags&EMDATA_HASCTF) return NULL;

r=(float *)malloc(ny/2*sizeof(float));

ds=1.0/(p[10]*ny);

lambda=12.2639/sqrt(p[8]*1000.0+.97845*p[8]*p[8]);

switch (type) {
case 0:
	for (i=0,s=0; i<ny/2; i++,s+=ds) {
		gamma=-2*PI*(p[9]*2.5e6*lambda*lambda*lambda*s*s*s*s+p[0]*5000.0*lambda*s*s);
		r[i]=p[2]*exp(-(p[1]*s*s))*(sqrt(1.0-p[3]*p[3])*sin(gamma)+p[3]*cos(gamma));
	}
	break;
case 2:
	for (i=0,s=0; i<ny/2; i++,s+=ds) {
		gamma=-2*PI*(p[9]*2.5e6*lambda*lambda*lambda*s*s*s*s+p[0]*5000.0*lambda*s*s);
		 r[i]=fabs(p[2]*exp(-(p[1]*s*s))*(sqrt(1.0-p[3]*p[3])*sin(gamma)+p[3]*cos(gamma)));
	}
	break;
case 1:
	for (i=0,s=0; i<ny/2; i++,s+=ds) {
		r[i]=(p[6]*exp(-(SQR(PI/2*p[7]*s))-s*p[5]-sqrt(fabs(s))*p[4]));
	}
	break;
case 3:
	for (i=0,s=0; i<ny/2; i++,s+=ds) {
		gamma=-2*PI*(p[9]*2.5e6*lambda*lambda*lambda*s*s*s*s+p[0]*5000.0*lambda*s*s);
		r[i]=p[2]*exp(-(p[1]*s*s))*(sqrt(1.0-p[3]*p[3])*sin(gamma)+p[3]*cos(gamma));
		r[i]=r[i]*r[i]/(p[6]*exp(-(SQR(PI/2*p[7]*s))-s*p[5]-sqrt(fabs(s))*p[4]));
	}
	break;
case 4:
	if (nsf<4||!S||!SF) return NULL;

	for (i=0,s=0; i<ny/2; i++,s+=ds) {
		gamma=-2*PI*(p[9]*2.5e6*lambda*lambda*lambda*s*s*s*s+p[0]*5000.0*lambda*s*s);
		r[i]=p[2]*exp(-(p[1]*s*s))*(sqrt(1.0-p[3]*p[3])*sin(gamma)+p[3]*cos(gamma));
		r[i]=r[i]*r[i]/(p[6]*exp(-(SQR(PI/2*p[7]*s))-s*p[5]-sqrt(fabs(s))*p[4]));
		if (s) r[i]*=pow(10.0,interpolate(s,nsf,S,SF));
	}

	break;
}

//save_data(0,ds,r,ny/2,"ctf.dat");

return r;
}

// Subtract average noise from intensity. Negative results allowed
void EMData::subNoise() {
float *y,r,f;
int i,j,k,l;
if (!isComplex()) { printf("subnoise on  FFT's only!\n"); return; }
if (!hasCTF()) { /*printf("No CTF to subtract\n");*/ return; }
ri2ap();

y=ctfCurve(1);
//save_data(0,1,y,ny/2,"ctf.dat");

getData();
for (j=k=0; j<ny; j++) {
    for (i=0; i<nx; i+=2, k+=2) {
		r=hypot((float)i/2.0,(float)j-ny/2.0);
		l=(int)floor(r+1);
		r-=l;
		if (l>=ny/2-2) f=y[ny/2-1];
		else f=y[l];
		rdata[k]=rdata[k]*rdata[k]-f;
		rdata[k+1]=0;
	}
}
doneData();
update();
free(y);
}

// averages the images in the list, result in 'this'
void EMData::makeAverage(List *in,EMData *sig) {
int i,j,nx2,ny2,nz2;
EMData *obj;
float *dest,*d2,*src;
float c;

if (in->count()==0) return;

// Start by copying the first object in the list and getting the size to
// make sure they're all the correct size
obj=(EMData *)in->objectAt(0);
nx2=obj->xSize();
ny2=obj->ySize();
nz2=obj->zSize();
setSize(nx2,ny2,nz2);
if (sig) sig->setSize(nx2,ny2,nz2);

dest=getData();
if (sig) d2=sig->getData(); else d2=NULL;
src=obj->getDataRO();
memcpy(dest,src,nx*ny*nz*sizeof(float));
if (d2) {
	memcpy(d2,src,nx*ny*nz*sizeof(float));
	for (j=0; j<nx*ny*nz; j++) d2[j]*=d2[j];
}
obj->doneData();

for (i=1,c=1.0; i<in->count(); i++) {
	obj=(EMData *)in->objectAt(i);
	
	// skip records that are different sizes
	if (obj->xSize()!=nx || obj->ySize()!=ny || obj->zSize()!=nz) continue;

	// add each image
	src=obj->getDataRO();
	for (j=0; j<nx*ny*nz; j++) dest[j]+=src[j];
	if (d2) {
		for (j=0; j<nx*ny*nz; j++) d2[j]+=src[j]*src[j];
	}
	obj->doneData();
	c+=1.0;
}

if (d2) {
	for (j=0; j<nx*ny*nz; j++) d2[j]=sqrt(d2[j]/c-SQR(dest[j]/c));
}

printf("%1.0f in ave\n",c);
// normalize
for (j=0; j<nx*ny*nz; j++) dest[j]/=c;
doneData();
if (d2) sig->doneData();
update();
}

// averages the images in the list, result in 'this'
void EMData::makeAverageIter(List *in) {
int i,j,k,l,nx2,ny2,nz2;
EMData *obj,*sig;
float *dest,*d2,*src;
float c,c2,sigma;

if (in->count()==0) return;

sig=new EMData;

// Start by copying the first object in the list and getting the size to
// make sure they're all the correct size
obj=(EMData *)in->objectAt(0);
nx2=obj->xSize();
ny2=obj->ySize();
nz2=obj->zSize();
setSize(nx2,ny2,nz2);
sig->setSize(nx2,ny2,nz2);

dest=getData();
d2=sig->getData();
src=obj->getDataRO();
memcpy(dest,src,nx*ny*nz*sizeof(float));
memcpy(d2,src,nx*ny*nz*sizeof(float));
for (j=0; j<nx*ny*nz; j++) d2[j]*=d2[j];

obj->doneData();

for (i=1,c=1.0; i<in->count(); i++) {
	obj=(EMData *)in->objectAt(i);
	
	// skip records that are different sizes
	if (obj->xSize()!=nx || obj->ySize()!=ny || obj->zSize()!=nz) continue;

	// add each image
	src=obj->getDataRO();
	for (j=0; j<nx*ny*nz; j++) dest[j]+=src[j];
	for (j=0; j<nx*ny*nz; j++) d2[j]+=src[j]*src[j];
		
	obj->doneData();
	c+=1.0;
}

// This one is std dev
for (j=0; j<nx*ny*nz; j++) d2[j]=sqrt(d2[j]/c-SQR(dest[j]/c))/sqrt(c);

// or std dev of mean
//for (j=0; j<nx*ny*nz; j++) d2[j]=sqrt(d2[j]/c-SQR(dest[j]/c));

printf("%1.0f in ave\n",c);
// normalize
for (j=0; j<nx*ny*nz; j++) dest[j]/=c;
doneData();
sig->doneData();
update();
sig->update();
writeIMAGIC("iter.hed",-1);

// now do the smoothing iteration
sigma=sig->Mean();
src=sig->getData();
dest=getData();
d2=(float *)malloc(nx*ny*sizeof(float));
memcpy(d2,dest,nx*ny*sizeof(float));
memcpy(src,dest,nx*ny*sizeof(float));

printf("Iter sigma=%f\n",sigma);

for (k=0; k<1000; k++) {
	for (i=1; i<nx-1; i++) {
		for (j=1; j<ny-1; j++) {
			l=i+j*nx;
			c=(d2[l-1]+d2[l+1]+d2[l-nx]+d2[l+nx])/4.0-d2[l];
			c2=fabs(dest[l]-src[l])/sigma;
			dest[l]+=c*erfc(c2)/100.0;
//if (i==nx/2) printf("%f %f %f\n",c,c2,erfc(c2));
		}
	}
	memcpy(d2,dest,nx*ny*sizeof(float));
}
free(d2);
doneData();
sig->doneData();
delete sig;
update();

writeIMAGIC("iter.hed",-1);

}

// averages the images in the list, result in 'this'
// Also does CTF correction if possible
// 2D ONLY!!!!
// filtr is in Angstroms!
void EMData::makeAverageCTFC(List *in,float filtr) {
int i,j,x,y,l,nx2,ny2,nz2;
EMData *obj,*res;
float *dest,*s2,**src;
float **Ctf,ctf0,f,r,ds;
char *s;

if (in->count()==0) return;

obj=((EMData *)in->objectAt(0));
if (!obj->hasCTF()) return;
ds=1.0/((obj->getCTF())[10]*ny);

// get data pointers and calculate individual CTF curves
Ctf=(float **)malloc(sizeof(float *)*in->count());
src=(float **)malloc(sizeof(float *)*in->count());
for (i=0; i<in->count(); i++) {
	obj=((EMData *)in->objectAt(i))->doFFT();
	obj->ap2ri();
	src[i]=obj->getDataRO();
	Ctf[i]=obj->ctfCurve();
	if (Ctf[i]==NULL) { printf("Error: Unexpected CTF correction problem\n"); exit(0); }
}


// Start by copying the first object in the list and getting the size to
// make sure they're all the correct size
obj=((EMData *)in->objectAt(0))->doFFT();
nx2=obj->xSize();
ny2=obj->ySize();
nz2=obj->zSize();
if (nz2!=1) { printf("Only 2d currently supported\n"); return; }
setSize(nx2-2,ny2,nz2);

ds=1.0/((((EMData *)in->objectAt(0))->getCTF())[10]*ny);
if (filtr==0) filtr=22.0;	// 22A default filter
//filtr=sqrt(2.0)/(ds*filtr*M_PI);		// filtr is now the radius in pixels
filtr=1.0/(filtr*ds);		// we use gaussian half-width in fourier space
printf("Filter = %f %f\n",filtr,ds);

// Normalize CTF curves
for (i=0; i<ny/2; i++) {
	ctf0=0;
	for (j=0; j<in->count(); j++) ctf0+=Ctf[j][i]*Ctf[j][i];
//printf("%f\n",ctf0);
	if (ctf0<=0.0001) { /*printf("Warning CTF==0\n");*/ ctf0=.1; }
	for (j=0; j<in->count(); j++) Ctf[j][i]=exp(-i*i/((float)filtr*filtr*4.0))*(tanh(filtr-i)/2.0+.5)*fabs(Ctf[j][i])/ctf0;
}
//save_data(0,1.0,Ctf[0],ny/2,"ctf.dat");

res=obj->copyHead();
res->ap2ri();

dest=res->getData();

for (y=j=0; y<ny2; y++) {
	for (x=0; x<nx2/2; x++,j+=2) {
		r=sqrt((float)(x*x)+(float)((y-ny2/2.0)*(y-ny2/2.0)));
		l=(int)floor(r);
		r-=l;
		dest[j]=0;
		dest[j+1]=0;
		for (i=0; i<in->count(); i++) {
			if (l>ny/2-2) f=0;
			else f=(Ctf[i][l]*(1.0-r)+Ctf[i][l+1]*r);

			dest[j]  +=src[i][j]  *f;
			dest[j+1]+=src[i][j+1]*f;
		}
	}
}

res->doneData();

dest=getData();
obj=res->doIFT();
s2=obj->getData();
memcpy(dest,s2,sizeof(float)*(nx2-2)*ny);
doneData();
obj->doneData();

delete res;

for (i=0; i<in->count(); i++) {
	obj=((EMData *)in->objectAt(i))->doFFT();
	obj->doneData();
	free(Ctf[i]);
}
free(src);
free(Ctf);
update();
}

// Median of the images in the list, result in 'this'
void EMData::makeMedian(List *in) {
int i,j,k,c,nx2,ny2,nz2,ni;
EMData *obj;
float v,*dest,**src,*srt;

if (in->count()==0) return;

// Start by copying the first object in the list and getting the size to
// make sure they're all the correct size
obj=(EMData *)in->objectAt(0);
nx2=obj->xSize();
ny2=obj->ySize();
nz2=obj->zSize();
setSize(nx2,ny2,nz2);
dest=getData();

ni=in->count();
srt=(float *)malloc(sizeof(float)*ni);
src=(float **)malloc(sizeof(float *)*ni);
for(i=0; i<ni; i++) src[i]=((EMData *)in->objectAt(i))->getDataRO();

for (i=0; i<nx*ny*nz; i++) {
	for (j=0; j<ni; j++) srt[j]=src[j][i];
	
	// not much data, just shell sort
	for (j=0; j<ni; j++) {
		for (k=j+1; k<ni; k++) {
			if (srt[j]<srt[k]) { v=srt[j]; srt[j]=srt[k]; srt[k]=v; }
		}
	}
	
	if (ni<3) dest[i]=srt[ni/2]; 
	else dest[i]=(srt[ni/2]+srt[ni/2+1]+srt[ni/2-1])/3.0;
}

doneData();
for (i=0; i<ni; i++) ((EMData *)in->objectAt(i))->doneData();
free(srt);
free(src);
update();
}


// adds 'obj' to 'this' incoherently
void EMData::addIncoherent(EMData *obj) {
int j;
float *dest,*src;

if (obj->xSize()!=nx || obj->ySize()!=ny || obj->zSize()!=nz) return;
if (!obj->isComplex() || !isComplex()) return;
ri2ap();
obj->ri2ap();

dest=getData();
src=obj->getDataRO();

for (j=0; j<nx*ny*nz; j+=2) {
	dest[j]=hypot(src[j],dest[j]);
	dest[j+1]=0;
}

obj->doneData();
doneData();

update();
}
// adds 'obj' to 'this'
void EMData::add(EMData *obj) {
int i,j,c;
float *dest,*src;

if (obj->isComplex()) obj->ap2ri();
if (obj->xSize()!=nx || obj->ySize()!=ny || obj->zSize()!=nz) return;

dest=getData();
src=obj->getDataRO();

for (j=0; j<nx*ny*nz; j++) dest[j]+=src[j];

obj->doneData();
doneData();

update();
}

// mult 'obj' by 'this'
void EMData::mult(EMData *obj) {
int i,j,c;
float *dest,*src;

if (isComplex()) ap2ri();
if (obj->isComplex()) obj->ap2ri();
if (obj->xSize()!=nx || obj->ySize()!=ny || obj->zSize()!=nz) { printf("multiply sizes different\n"); return; }

dest=getData();
if (obj!=this) src=obj->getDataRO(); else src=dest;

for (j=0; j<nx*ny*nz; j++) dest[j]*=src[j];

if (obj!=this) obj->doneData();
doneData();

update();
}

void EMData::multConst(float c) {
int i;

if (isComplex()) ap2ri();

getData();
for (i=0; i<nx*ny*nz; i++) rdata[i]*=c;
doneData();
update();
}


// This moves the center of the image to the lower left corner
// ie - phase origin shift from the center of the image to the
// phase origin used by the fft routines
// 2d and 3d only
void EMData::toCorner() {
int i,j,k;
float t;

getData();


if (nz==1) {
	for (j=0; j<ny/2; j++) {
		for (i=0; i<nx/2; i++) {
			t=rdata[i+j*nx];
			rdata[i+j*nx]=rdata[i+nx/2+(j+ny/2)*nx];
			rdata[i+nx/2+(j+ny/2)*nx]=t;
		}
	}

	for (j=0; j<ny/2; j++) {
		for (i=0; i<nx/2; i++) {
			t=rdata[i+nx/2+j*nx];
			rdata[i+nx/2+j*nx]=rdata[i+(j+ny/2)*nx];
			rdata[i+(j+ny/2)*nx]=t;
		}
	}
}
else {
	for (k=0; k<nz/2; k++) {
		for (j=0; j<ny/2; j++) {
			for (i=0; i<nx/2; i++) {
				t=rdata[i+j*nx+k*nx*ny];
				rdata[i+j*nx+k*nx*ny]=rdata[i+nx/2+(j+ny/2)*nx+(k+nz/2)*nx*ny];
				rdata[i+nx/2+(j+ny/2)*nx+(k+nz/2)*nx*ny]=t;
			}
		}
	}

	for (k=0; k<nz/2; k++) {
		for (j=0; j<ny/2; j++) {
			for (i=0; i<nx/2; i++) {
				t=rdata[i+nx/2+j*nx+k*nx*ny];
				rdata[i+nx/2+j*nx+k*nx*ny]=rdata[i+(j+ny/2)*nx+(k+nz/2)*nx*ny];
				rdata[i+(j+ny/2)*nx+(k+nz/2)*nx*ny]=t;
			}
		}
	}

	for (k=0; k<nz/2; k++) {
		for (j=0; j<ny/2; j++) {
			for (i=0; i<nx/2; i++) {
				t=rdata[i+nx/2+(j+ny/2)*nx+k*nx*ny];
				rdata[i+nx/2+(j+ny/2)*nx+k*nx*ny]=rdata[i+j*nx+(k+nz/2)*nx*ny];
				rdata[i+j*nx+(k+nz/2)*nx*ny]=t;
			}
		}
	}

	for (k=0; k<nz/2; k++) {
		for (j=0; j<ny/2; j++) {
			for (i=0; i<nx/2; i++) {
				t=rdata[i+(j+ny/2)*nx+k*nx*ny];
				rdata[i+(j+ny/2)*nx+k*nx*ny]=rdata[i+nx/2+j*nx+(k+nz/2)*nx*ny];
				rdata[i+nx/2+j*nx+(k+nz/2)*nx*ny]=t;
			}
		}
	}

}

doneData();
}

// calculates common lines of 2 data sets (assumes center is already correct)
// mode 0 is dot product		(1 best match)
// mode 1 is weighted phase residual, default (0 best match, PI bad)
// mode 2 is amplitude only dot product (doesn't require a good center)
// NOTE : for this to work well, you should edgenormalize your images, then pad them by 2x
//		before calling this routine
#define CCLSTEPS	180			// 1/2 the resolution of the map
void EMData::commonLines(EMData *d1,EMData *d2,int mode) {
float *im1,*im2;
int rmax;
int i,j,k,l,xp,yp,nx,ny,jmax;
float da=PI/CCLSTEPS;
float a,a1,a2,p1,p2,r,x,y,s1,s2;
float *r1,*r2;

if (!d1->isComplex()) d1=d1->doFFT();
if (!d2->isComplex()) d2=d2->doFFT();
d1->ap2ri();
d2->ap2ri();

if (d1->xSize()!=d2->xSize() || d1->ySize()!=d2->ySize()) return;
nx=d2->xSize();
ny=d2->ySize();

//rmax=ny/2-1;
rmax=ny/4-1;	// only use the inner 1/2 of fourier space
im1=(float *)calloc(CCLSTEPS*rmax*2,sizeof(float));
im2=(float *)calloc(CCLSTEPS*rmax*2,sizeof(float));

setSize(CCLSTEPS*2,CCLSTEPS*2,1);	// Final map is twice as big with complex conjugates
//setSize(rmax*2,CCLSTEPS,1);

//d1->writeMRC("xx.img");

r1=d1->getDataRO();
r2=d2->getDataRO();
getData();

// this generates radial lines
for (i=0,a=-PI/2.0+da/2.0; i<CCLSTEPS*2; i+=2,a+=da) {
	s1=s2=0;
	for (j=0,r=3.0; r<rmax-3.0; j+=2,r+=1.0) {
		x=r*cos(a);
		y=r*sin(a);
		if (x<0) { x=-x; y=-y; printf("CCL ERROR %d,%f !\n",i,-x); }
		k=(int)(floor(x)*2+floor(y+ny/2)*nx);

// real
		im1[i*rmax+j]=bilin(r1[k],r1[k+2],r1[k+2+nx],r1[k+nx],x-floor(x),y-floor(y));
		im2[i*rmax+j]=bilin(r2[k],r2[k+2],r2[k+2+nx],r2[k+nx],x-floor(x),y-floor(y));

// imaginary
		k++;
		im1[i*rmax+j+1]=bilin(r1[k],r1[k+2],r1[k+2+nx],r1[k+nx],x-floor(x),y-floor(y));
		im2[i*rmax+j+1]=bilin(r2[k],r2[k+2],r2[k+2+nx],r2[k+nx],x-floor(x),y-floor(y));

// complex conjugate for quad II & III
//		if (a>PI/2.0) { im1[i*rmax+j+1]*=-1.0; im2[i*rmax+j+1]*=-1.0; }

// normalization (sum of amplitudes^2)
		s1+=hypot2(im1[i*rmax+j],im1[i*rmax+j+1]);
		s2+=hypot2(im2[i*rmax+j],im2[i*rmax+j+1]);
	}
	jmax=j-1;
	
// normalize each line (sum of amplitudes^2 =1 )
	s1=sqrt(s1);
	s2=sqrt(s2);
	for (j=0,r=1; r<rmax; j+=2,r+=1.0) {
		im1[i*rmax+j]/=s1;
		im1[i*rmax+j+1]/=s1;
		im2[i*rmax+j]/=s2;
		im2[i*rmax+j+1]/=s2; 
	}
}


// map of line-line dot products
// each quadrant done individually for effeciency
// really should combine these into a single set of loops
switch (mode) {
case 0:
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+j*CCLSTEPS*2;
			rdata[l]=0;
//		for (k=4; k<rmax*2-4; k++) {
			for (k=0; k<jmax; k++) {
				rdata[l]+=im1[i*rmax*2+k]*im2[j*rmax*2+k];
			}
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
//		l=i+(CCLSTEPS*2-j-1)*CCLSTEPS*2;
			l=i+CCLSTEPS+j*CCLSTEPS*2;
			rdata[l]=0;
			for (k=0; k<jmax; k+=2) {
				rdata[l]+=im1[i*rmax*2+k]*im2[j*rmax*2+k];
				rdata[l]+=-im1[i*rmax*2+k+1]*im2[j*rmax*2+k+1];
			}
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
//		l=(CCLSTEPS*2-i-1)+j*CCLSTEPS*2;
			l=i+(j+CCLSTEPS)*CCLSTEPS*2;
			rdata[l]=0;
			for (k=0; k<jmax; k+=2) {
				rdata[l]+=im1[i*rmax*2+k]*im2[j*rmax*2+k];
				rdata[l]+=-im1[i*rmax*2+k+1]*im2[j*rmax*2+k+1];
			}
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+CCLSTEPS+(j+CCLSTEPS)*CCLSTEPS*2;
			rdata[l]=0;
			for (k=0; k<jmax; k+=2) {
				rdata[l]+=im1[i*rmax*2+k]*im2[j*rmax*2+k];
				rdata[l]+=im1[i*rmax*2+k+1]*im2[j*rmax*2+k+1];
			}
		}
	}
	break;
case 1:
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+j*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				p1=atan2(im1[i*rmax*2+k+1],im1[i*rmax*2+k]);
				p2=atan2(im2[j*rmax*2+k+1],im2[j*rmax*2+k]);
				rdata[l]+=asub(p1,p2)*a1*a2;
				a+=a1*a2;
			}
			rdata[l]/=a*PI/180.0;
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+CCLSTEPS+j*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				p1=atan2(im1[i*rmax*2+k+1],im1[i*rmax*2+k]);
				p2=atan2(im2[j*rmax*2+k+1],im2[j*rmax*2+k]);
				rdata[l]+=asub(-p1,p2)*a1*a2;
				a+=a1*a2;
			}
			rdata[l]/=a*PI/180.0;
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+(j+CCLSTEPS)*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				p1=atan2(im1[i*rmax*2+k+1],im1[i*rmax*2+k]);
				p2=atan2(im2[j*rmax*2+k+1],im2[j*rmax*2+k]);
				rdata[l]+=asub(-p1,p2)*a1*a2;
				a+=a1*a2;
			}
			rdata[l]/=a*PI/180.0;
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+CCLSTEPS+(j+CCLSTEPS)*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				p1=atan2(im1[i*rmax*2+k+1],im1[i*rmax*2+k]);
				p2=atan2(im2[j*rmax*2+k+1],im2[j*rmax*2+k]);
				rdata[l]+=asub(p1,p2)*a1*a2;
				a+=a1*a2;
			}
			rdata[l]/=a*PI/180.0;
		}
	}
	break;
case 2:
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+j*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				rdata[l]+=a1*a2;
			}
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+CCLSTEPS+j*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				rdata[l]+=a1*a2;
			}
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+(j+CCLSTEPS)*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				rdata[l]+=a1*a2;
			}
		}
	}
	for (i=0; i<CCLSTEPS; i++) {
		for (j=0; j<CCLSTEPS; j++) {
			l=i+CCLSTEPS+(j+CCLSTEPS)*CCLSTEPS*2;
			rdata[l]=a=0;
			for (k=0; k<jmax; k+=2) {
				a1=hypot(im1[i*rmax*2+k],im1[i*rmax*2+k+1]);
				a2=hypot(im2[j*rmax*2+k],im2[j*rmax*2+k+1]);
				rdata[l]+=a1*a2;
			}
		}
	}
	break;
}	
	
//memcpy(rdata,im1,rmax*CCLSTEPS*2*4);

free(im1);
free(im2);
d1->doneData();
d2->doneData();
doneData();
update();
}

// This will use fftslice or make a real-space projection of a 3d model
// mode=-1 is real-space with trilinear interpolation
// 'toCorner()' MUST be applied before fourier projection
// it must NOT be applied before real-space projection
// real space projection will cause the current data set to be rotated
EMData *EMData::project3d(float alt,float az, float phi, int mode, float thr)
{
EMData *ret,*tmp,*f;
float *dat,*din,mx[9],rm,x,y,z;
int i,j,k,ii,x2,y2;
float *norm;

if (mode>0) {
	if (!isComplex()) f=doFFT(); else f=this;
	f->setup4Slice(0);
	tmp=f->fftSlice(alt,az,phi,mode);
	ret=tmp->doIFT();
	tmp->gimmeFFT();
	ret->toCorner();
	ret->applyMask(-1,100+mode);
	ret->setRAlign(alt,az,phi);
	delete tmp;
	ret->update();
	return ret;
}
// slow accurate real space projection
if (mode==-1) {
	if (!parent) {
		parent=copy();
		parent->setParent(NULL);
//		error("Warning: parent allocated\n",ERR_WARN);
	}
	setRAlign(alt,az,phi);
	setTAlign(0,0,0);
	rotateAndTranslate();
	
	ret=new EMData;
	ret->setSize(nx,ny,1);
	dat=ret->getData();
	din=getDataRO();
	for (i=0; i<nx; i++) {
		for (j=0; j<ny; j++) {
			dat[i+j*nx]=0;
			for (k=0; k<nz; k++) {
				dat[i+j*nx]+=din[i+j*nx+k*nx*ny];
			}
			dat[i+j*nx]/=(nz/12.0);
		}
	}
	doneData();
	ret->doneData();
	ret->update();
	return ret;
}
// quick and dirty real space projection
if (mode==-2) {
	if (!parent) {
		parent=copy();
		parent->setParent(NULL);
//		error("Warning: parent allocated\n",ERR_WARN);
	}
	setRAlign(alt,az,phi);
	setTAlign(0,0,0);
	rotateAndTranslateFast();
	
	ret=new EMData;
	ret->setSize(nx,ny,1);
	dat=ret->getData();
	din=getDataRO();
	for (i=0; i<nx; i++) {
		for (j=0; j<ny; j++) {
			dat[i+j*nx]=0;
			for (k=0; k<nz; k++) {
				dat[i+j*nx]+=din[i+j*nx+k*nx*ny];
			}
			dat[i+j*nx]/=(nz/12.0);
		}
	}
	doneData();
	ret->doneData();
	ret->update();
	return ret;
}

// quick and dirty surface representation
if (mode==-3) {
	int l;
	float slx,sly,v0[3],v1[3],v2[3];
	
	if (!parent) {
		parent=copy();
		parent->setParent(NULL);
//		error("Warning: parent allocated\n",ERR_WARN);
	}
	setRAlign(alt,az,phi);
	setTAlign(0,0,0);
	rotateAndTranslate();
	
	ret=new EMData;
	ret->setSize(nx,ny,1);
	dat=ret->getData();
	din=getDataRO();
	for (i=0; i<nx; i++) {
		for (j=0; j<ny; j++) {
			for (k=nz-2; k>0; k--) {
				l=i+j*nx+k*nx*ny;
				if (din[l]>thr) break;
			}
			if (k==0) dat[i+j*nx]=-1000;
			else dat[i+j*nx]=nz-k-(thr-din[l+nx*ny])/(din[l]-din[l+nx*ny]);
		}
	}
	
	for (j=0; j<ny; j++) {
		for (i=0; i<nx; i++) {
			k=i+j*nx;
			slx=dat[k+1]+dat[k+nx+1]-dat[k]-dat[k+nx];
			sly=dat[k+nx]+dat[k+nx+1]-dat[k]-dat[k+1];
			if (fabs(slx)>50 || fabs(sly)>50 ||dat[k+1]==-1000|| j==ny-1 || i==nx-1) { dat[k]=2.0; continue; }
			v0[0]=1.0/hypot(1.0,slx); v0[1]=0; v0[2]=slx/hypot(1.0,slx);
			v1[0]=0; v1[1]=1.0/hypot(1.0,sly); v1[2]=sly/hypot(1.0,sly);
			v2[0]=v0[1]*v1[2]-v0[2]*v1[1];
			v2[1]=v0[2]*v1[0]-v0[0]*v1[2];
			v2[2]=v0[0]*v1[1]-v0[1]*v1[0];
			dat[k]=-v2[0]-v2[1]+v2[2];
		}
	}

	doneData();
	ret->doneData();
	ret->update();
	return ret;
}

return NULL;
}

// Note that once this has been allocated, it is used
// globally and never freed.
float *gimx = NULL;

// initialize global gaussian interpolation matrix
void initGIMX() {
int i,j,k;
float r;

gimx=(float*)malloc(sizeof(float)*1000000);
for(k=0; k<100; k++) {
	for(j=0; j<100; j++) {
		for(i=0; i<100; i++) {
			r=hypot3(i/39.0,j/39.0,k/39.0);
			gimx[i+j*100+k*10000]=exp(-r/I5G);
		}
	}
}

}

// This will cut a plane out of a complex volume and return a
// trilinear interpolated image. This is used to generate
// projections of a real space volume
EMData *EMData::fftSlice(float alt,float az,float phi,int mode)
{
float xx,yy,zz;
float t,u,v;
EMData *ret;
float *dat,mx[6],cc,r;
int i,j,k,ii,jj,kk,i0,j0,k0,x,y;

if (!isComplex() || nz!=ny || nx!=ny+2) { error("Cannot fftSlice this image",ERR_ERROR); return NULL; }
if (mode==5 && !gimx) initGIMX();

ap2ri();			// make sure we're in real/imaginary space
getDataRO();

ret=new EMData();
ret->setSize(nx,ny,1);
ret->setComplex(1);
ret->setRI(1);
dat=ret->getData();

mx[0]=(cos(phi)*cos(az)-cos(alt)*sin(az)*sin(phi));
mx[1]=-(sin(phi)*cos(az)+cos(alt)*sin(az)*cos(phi));
mx[2]=(cos(phi)*sin(az)+cos(alt)*cos(az)*sin(phi));
mx[3]=(-sin(phi)*sin(az)+cos(alt)*cos(az)*cos(phi));
mx[4]=sin(alt)*sin(phi);
mx[5]=sin(alt)*cos(phi);

for (y=0; y<ny; y++) {
	for (x=0; x<nx/2; x++) {
		ii=x*2+y*nx;
		if (hypot(x,y-ny/2)>=ny/2-1) { dat[ii]=0; dat[ii+1]=0; continue; }
	
		xx=x*mx[0]+(y-ny/2)*mx[1];
		yy=x*mx[2]+(y-ny/2)*mx[3];
		zz=x*mx[4]+(y-ny/2)*mx[5];

		// complex conjugate
		if (xx<0) { xx=-xx; yy=-yy; zz=-zz; cc=-1; } else cc=1;
		yy+=ny/2;
		zz+=nz/2;

		// gaussian interpolation, must be corrected in real space
		interpFT3d(xx,yy,zz,dat+ii,mode);
		dat[ii+1]*=cc;
	}
}

doneData();

ret->doneData();
ret->update();

return ret;
}


// This will interpolate a point in a 3d complex image with various
// quality levels.
// NOTE: no error checking is done because this routine is called a LOT
void EMData::interpFT3d(float x,float y,float z,float *dat,int mode) {
int x0,y0,z0;
int mx0,my0,mz0,mmx,mmy,mmz;
float dx,dy,dz,r,n,g;
int i,j,k,ii;

if (!supp) { error("WARNING: no setup4Slice done",ERR_WARN); setup4Slice(); }
getDataRO();

dat[0]=dat[1]=0;

switch (mode) {
case 1:
		x0=2*(int)floor(x+.5);
		y0=(int)floor(y+.5);
		z0=(int)floor(z+.5);

		dat[0]=rdata[x0+y0*nx+z0*nx*ny];
		dat[1]=rdata[x0+y0*nx+z0*nx*ny+1];
		break;
case 2:
		x0=(int)floor(x);
		y0=(int)floor(y);
		z0=(int)floor(z);

		dx=x-x0;
		dy=y-y0;
		dz=z-z0;

		if (x0>nx-2 || y0>ny-1 || z0>nz-1) break;

		i=(int)(x0*2+y0*nx+z0*nx*ny);

		n=agauss(1.0,dx,dy,dz,I2G)+agauss(1.0,1.0-dx,dy,dz,I2G)+
			agauss(1.0,dx,1.0-dy,dz,I2G)+agauss(1.0,1.0-dx,1.0-dy,dz,I2G)+
			agauss(1.0,dx,dy,1.0-dz,I2G)+agauss(1.0,1.0-dx,dy,1.0-dz,I2G)+
			agauss(1.0,dx,1.0-dy,1.0-dz,I2G)+agauss(1.0,1.0-dx,1.0-dy,1.0-dz,I2G);

		dat[0]=agauss(rdata[i],dx,dy,dz,I2G)+agauss(rdata[i+2],1.0-dx,dy,dz,I2G)+
			agauss(rdata[i+nx],dx,1.0-dy,dz,I2G)+agauss(rdata[i+nx+2],1.0-dx,1.0-dy,dz,I2G)+
			agauss(rdata[i+nx*ny],dx,dy,1.0-dz,I2G)+agauss(rdata[i+2+nx*ny],1.0-dx,dy,1.0-dz,I2G)+
			agauss(rdata[i+nx+nx*ny],dx,1.0-dy,1.0-dz,I2G)+agauss(rdata[i+2+nx+nx*ny],1.0-dx,1.0-dy,1.0-dz,I2G)/n;

		i++;

		dat[1]=agauss(rdata[i],dx,dy,dz,I2G)+agauss(rdata[i+2],1.0-dx,dy,dz,I2G)+
			agauss(rdata[i+nx],dx,1.0-dy,dz,I2G)+agauss(rdata[i+nx+2],1.0-dx,1.0-dy,dz,I2G)+
			agauss(rdata[i+nx*ny],dx,dy,1.0-dz,I2G)+agauss(rdata[i+2+nx*ny],1.0-dx,dy,1.0-dz,I2G)+
			agauss(rdata[i+nx+nx*ny],dx,1.0-dy,1.0-dz,I2G)+agauss(rdata[i+2+nx+nx*ny],1.0-dx,1.0-dy,1.0-dz,I2G)/n;

/*		dat[0]=trilin(rdata[i],rdata[i+2],rdata[i+nx],rdata[i+nx+2],
			rdata[i+nx*ny],rdata[i+nx*ny+2],rdata[i+nx*ny+nx],rdata[i+nx*ny+nx+2],
			dx,dy,dz);

		dat[1]=trilin(rdata[i+1],rdata[i+2+1],rdata[i+nx+1],rdata[i+nx+2+1],
			rdata[i+nx*ny+1],rdata[i+nx*ny+3],rdata[i+nx*ny+nx+1],rdata[i+nx*ny+nx+3],
			dx,dy,dz);*/
		break;
case 3:
		x0=2*(int)floor(x+.5);
		y0=(int)floor(y+.5);
		z0=(int)floor(z+.5);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		// if too close to x origin, use supplemental array
		n=0;
		if (x0==0) {
			x0+=4;
			for (k=z0-1; k<=z0+1; k++) {
				for (j=y0-1; j<=y0+1; j++) {
					for (i=x0-2; i<=x0+2; i+=2) {
						r=hypot3((float)i/2-x-2.0,(float)j-y,(float)k-z);
						
						g=exp(-r/I3G);
						n+=g;
						dat[0]+=supp[i+j*12+k*12*ny]*g;
						dat[1]+=supp[i+j*12+k*12*ny+1]*g;
					}
				}
			}
		}
		else {
			for (k=z0-1; k<=z0+1; k++) {
				for (j=y0-1; j<=y0+1; j++) {
					for (i=x0-2; i<=x0+2; i+=2) {
						r=hypot3((float)i/2-x,(float)j-y,(float)k-z);

						g=exp(-r/I3G);
						n+=g;
						dat[0]+=rdata[i+j*nx+k*nx*ny]*g;
						dat[1]+=rdata[i+j*nx+k*nx*ny+1]*g;
					}
				}
			}
		}
		dat[0]/=n;
		dat[1]/=n;
//printf("%f %f %f %f %f,%f\n",x,y,z,n,dat[0],dat[1]);
		break;
case 4:			// better check this one sometime
		x0=2*(int)floor(x);
		y0=(int)floor(y);
		z0=(int)floor(z);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		// if too close to x origin, use supplemental array
		n=0;
		if (x0==0) {
			x0+=4;
			for (k=z0-1; k<=z0+2; k++) {
				for (j=y0-1; j<=y0+2; j++) {
					for (i=x0-2; i<=x0+4; i+=2) {
						r=hypot3((float)i/2-x-2.0,(float)j-y,(float)k-z);
						
						g=exp(-r/I4G);
						n+=g;
						dat[0]+=supp[i+j*12+k*12*ny]*g;
						dat[1]+=supp[i+j*12+k*12*ny+1]*g;
					}
				}
			}
		}
		else {
			for (k=z0-1; k<=z0+2; k++) {
				for (j=y0-1; j<=y0+2; j++) {
					for (i=x0-2; i<=x0+4; i+=2) {
						r=hypot3((float)i/2-x,(float)j-y,(float)k-z);

						g=exp(-r/I4G);
						n+=g;
						dat[0]+=rdata[i+j*nx+k*nx*ny]*g;
						dat[1]+=rdata[i+j*nx+k*nx*ny+1]*g;
					}
				}
			}
		}
		dat[0]/=n;
		dat[1]/=n;
//printf("%f %f %f %f %f,%f\n",x,y,z,n,dat[0],dat[1]);
		break;
case 5:
		x0=(int)floor(x+.5);
		y0=(int)floor(y+.5);
		z0=(int)floor(z+.5);

		mx0=-(int)floor((x-x0)*39.0+.5)-78;
		my0=-(int)floor((y-y0)*39.0+.5)-78;
		mz0=-(int)floor((z-z0)*39.0+.5)-78;
		x0*=2;
		
		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		// if too close to x origin, use supplemental array
		n=0;
		if (x0<4) {
			x0+=4;
			for (k=z0-2,mmz=mz0; k<=z0+2; k++,mmz+=39) {
				for (j=y0-2,mmy=my0; j<=y0+2; j++,mmy+=39) {
					for (i=x0-4,mmx=mx0; i<=x0+4; i+=2,mmx+=39) {
						g=gimx[abs(mmx)+abs(mmy)*100+abs(mmz)*10000];

						n+=g;
						dat[0]+=supp[i+j*12+k*12*ny]*g;
						dat[1]+=supp[i+j*12+k*12*ny+1]*g;
					}
				}
			}
		}
		else {
			for (k=z0-2,mmz=mz0; k<=z0+2; k++,mmz+=39) {
				for (j=y0-2,mmy=my0; j<=y0+2; j++,mmy+=39) {
					for (i=x0-4,mmx=mx0; i<=x0+4; i+=2,mmx+=39) {
						ii=i+j*nx+k*nx*ny;
						g=gimx[abs(mmx)+abs(mmy)*100+abs(mmz)*10000];

						n+=g;
						dat[0]+=rdata[ii]*g;
						dat[1]+=rdata[ii+1]*g;
					}
				}
			}
		}
		dat[0]/=n;
		dat[1]/=n;
//printf("%f %f %f %f %f,%f\n",x,y,z,n,dat[0],dat[1]);
		break;
case 6:	
		x0=2*(int)floor(x+.5);
		y0=(int)floor(y+.5);
		z0=(int)floor(z+.5);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		// if too close to x origin, use supplemental array
		n=0;
		if (x0<4) {
			x0+=4;
			for (k=z0-2; k<=z0+2; k++) {
				for (j=y0-2; j<=y0+2; j++) {
					for (i=x0-4; i<=x0+4; i+=2) {
						r=hypot3((float)i/2-x-2.0,(float)j-y,(float)k-z);
						
						g=exp(-r/I5G);
						n+=g;
						dat[0]+=supp[i+j*12+k*12*ny]*g;
						dat[1]+=supp[i+j*12+k*12*ny+1]*g;
					}
				}
			}
		}
		else {
			for (k=z0-2; k<=z0+2; k++) {
				for (j=y0-2; j<=y0+2; j++) {
					for (i=x0-4; i<=x0+4; i+=2) {
						r=hypot3((float)i/2-x,(float)j-y,(float)k-z);

						g=exp(-r/I5G);
						n+=g;
						dat[0]+=rdata[i+j*nx+k*nx*ny]*g;
						dat[1]+=rdata[i+j*nx+k*nx*ny+1]*g;
					}
					
				}
			}
		}
		dat[0]/=n;
		dat[1]/=n;
//printf("%f %f %f %f %f,%f\n",x,y,z,n,dat[0],dat[1]);
		break;
// hypergeometric kernel, FT of exp(-x^4 * 1.8)
case 7:
		x0=2*(int)floor(x+.5);
		y0=(int)floor(y+.5);
		z0=(int)floor(z+.5);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		// if too close to x origin, use supplemental array
		n=0;
		if (x0<4) {
			x0+=4;
			for (k=z0-2; k<=z0+2; k++) {
				for (j=y0-2; j<=y0+2; j++) {
					for (i=x0-4; i<=x0+4; i+=2) {
						r=sqrt(hypot3((float)i/2-x-2.0,(float)j-y,(float)k-z));
						
						g=hyperg(r);
						n+=g;
						dat[0]+=supp[i+j*12+k*12*ny]*g;
						dat[1]+=supp[i+j*12+k*12*ny+1]*g;
					}
				}
			}
		}
		else {
			for (k=z0-2; k<=z0+2; k++) {
				for (j=y0-2; j<=y0+2; j++) {
					for (i=x0-4; i<=x0+4; i+=2) {
						r=sqrt(hypot3((float)i/2-x,(float)j-y,(float)k-z));

						g=hyperg(r);
						n+=g;
						dat[0]+=rdata[i+j*nx+k*nx*ny]*g;
						dat[1]+=rdata[i+j*nx+k*nx*ny+1]*g;
					}
					
				}
			}
		}
		dat[0]/=n;
		dat[1]/=n;
//printf("%f %f %f %f %f,%f\n",x,y,z,n,dat[0],dat[1]);
		break;
}

doneData();
}

// this initializes a new image to receive 'insertSlice' operations
// for fourier volume inversion. If the image has a parent, it will also
// be affected. WARNING: will use ~6*size^3 bytes of memory.
void EMData::setup4IS(int size)
{
int i;
float f;

setSize(size+2,size,size);
setComplex(1);
setRI(1);
zero();

// initialize with small random-phase number so phase residuals don't look good when empty
getData();
for (i=0; i<nx*ny*nz; i+=2) {
	f=frand(0,2.0*PI);
	rdata[i]=1.0e-10*sin(f);
	rdata[i+1]=1.0e-10*cos(f);
}
doneData();

if (!parent) parent=new EMData();
parent->setSize(size+2,size,size);
parent->zero();
}


// When interpolating in fourier space there is a little problem when we get
// close to x=0, since f(-x,-y,-z)=f(x,y,z)*
// So this makes a little array that allows for up to +-2 point interpolation
// all the way to the origin in x
void EMData::setup4Slice(int redo) {
int x,y,z,yy,zz;
int i;

if (!isComplex() || nz==1) return; 
if (supp) {
	if (redo) free(supp);
	else return;
}
supp=(float *)calloc(12,ny*nz*sizeof(float));
getDataRO();

for (z=0; z<nz; z++) {
	for (y=0; y<ny; y++) {
		for (x=0; x<8; x++) {
			supp[(x+4)+y*12+z*12*ny]=rdata[x+y*nx+z*nx*ny];
		}
	}
}

for (z=1,zz=nz-1; z<nz; z++,zz--) {
	for (y=1,yy=ny-1; y<ny; y++,yy--) {
		supp[  y*12+z*12*ny]= rdata[4+yy*nx+zz*nx*ny];
		supp[1+y*12+z*12*ny]=-rdata[5+yy*nx+zz*nx*ny];
		supp[2+y*12+z*12*ny]= rdata[2+yy*nx+zz*nx*ny];
		supp[3+y*12+z*12*ny]=-rdata[3+yy*nx+zz*nx*ny];
	}
}

//for (i=0; i<12; i++) printf("%f ",supp[i+(ny/2-1)*12+z*6*ny]);
//printf("\n");

doneData();
}

// caclulates normalization and phase residual for a slice in an already existing volume
// phase residual is calculated using only the inner 1/2 of the fourier sphere
float EMData::normSlice(EMData *slice,float alt,float az,float phi,float *phaseres)
{
int x0,y0,z0;
float xx,yy,zz,pr,prn,p1,p2;
EMData *ret;
float *dat,*norm,mx[6],cc;
float dt[2],d,r,rn,gg,rad;
int y,x,i;

if (!isComplex() || !slice->isComplex() || !parent) return 0;

slice->ap2ri();
getDataRO();
norm=parent->getData();
dat=slice->getData();

mx[0]=(cos(phi)*cos(az)-cos(alt)*sin(az)*sin(phi));
mx[1]=-(sin(phi)*cos(az)+cos(alt)*sin(az)*cos(phi));
mx[2]=(cos(phi)*sin(az)+cos(alt)*cos(az)*sin(phi));
mx[3]=(-sin(phi)*sin(az)+cos(alt)*cos(az)*cos(phi));
mx[4]=sin(alt)*sin(phi);
mx[5]=sin(alt)*cos(phi);


r=rn=0;		// normalization
pr=prn=0;	// phase residual
// find points that are already inserted and 
for (y=0; y<ny; y++) {
	for (x=0; x<nx/2; x++) {
		rad=hypot(x,y-ny/2);
		if (rad>=ny/2-1) continue;

		xx=x*mx[0]+(y-ny/2)*mx[1];
		yy=x*mx[2]+(y-ny/2)*mx[3];
		zz=x*mx[4]+(y-ny/2)*mx[5];
		// complex conjugate
		if (xx<0) { xx=-xx; yy=-yy; zz=-zz; cc=-1.0; } else cc=1.0;
		yy+=ny/2;
		zz+=nz/2;


		x0=2*(int)floor(xx+.5);
		y0=(int)floor(yy+.5);
		z0=(int)floor(zz+.5);

//		d=hypot3((float)x0/2-xx,(float)y0-yy,(float)z0-zz);
//		gg=exp(-d/I5G);
		
		i=x0+y0*nx+z0*nx*ny;
//		if (fabs(norm[i])<.8) continue;
		if (rdata[i]==0 || rdata[i+1]==0) continue;
/*		dt[1]=hypot(rdata[i],rdata[i+1]);
		dt[0]=dat[x*2+y*nx]/rdata[i]+cc*dat[x*2+1+y*nx]/rdata[i+1];
		r+=norm[i]*norm[i]*dt[1]*dt[0];
		rn+=norm[i]*dt[1];*/
		dt[0]=hypot(rdata[i],rdata[i+1]);
		dt[1]=hypot(dat[x*2+y*nx],dat[x*2+1+y*nx]);
		r+=norm[i]*dt[1];
		rn+=dt[0];

/*		if (rad>3.0 && norm[i]>0) {
			pr+=fabs(dt[0]/norm[i]-dt[1])*norm[i];
			prn+=norm[i];
		} */

		if (rdata[i+1]==0 && rdata[i]==0) p1=0;
		else p1=atan2(cc*rdata[i+1],rdata[i]);
		if (dat[x*2+1+y*nx]==0 && dat[x*2+y*nx]==0) p2=0;
		else p2=atan2(dat[x*2+1+y*nx],dat[x*2+y*nx]);

		// phase residual using only inner 1/2 of fourier space
//		if (rad<ny/4 && rad>3.0) {
		if (rad>3.0) {
			pr+=asub(p1,p2)*dt[0]*dt[1]*norm[i];
			prn+=dt[0]*dt[1]*norm[i];
		}
	}
}

if (phaseres) *phaseres= (prn==0)?0:pr/prn;

// ok now normalize
if (rn!=0) {
	r=r/rn;
//	for (i=0; i<nx*ny; i++) dat[i]/=r; 
//	printf("Slice norm %f (%f) \n",r,rn);
}
else r=1.0;

doneData();
parent->doneData();
slice->doneData();
slice->update();

return r;
}


// this inserts a complex slice into an image that has been setup4IS()
// NOTE:  ONLY MODE 5 & 6 DO A COMPLETE JOB (the others are close, just a little issue near the x origin)
void EMData::insertSlice(EMData *slice,float alt,float az,float phi,int mode, float weight)
{
int x0,y0,z0;
int mx0,my0,mz0,mmx,mmy,mmz;
float xx,yy,zz;
float t,u,v,d,r,gg;
EMData *ret;
float *dat,*norm,mx[6],cc;
int i,j,k,l,ll,ii,x,y,off[8],rl;
float dt[2],g[8];

if (!isComplex() || !slice->isComplex() || !parent) return;

// This is a big matrix for speedy interpolation. Hopefully
// it's accurate enough...
// really should take advantage of symmetry here...
if (mode==5 && !gimx) initGIMX();

if (mode==2) {
	off[0]=0;
	off[1]=2;
	off[2]=nx;
	off[3]=nx+2;
	off[4]=nx*ny;
	off[5]=nx*ny+2;
	off[6]=nx*ny+nx;
	off[7]=nx*ny+nx+2;
}

getDataRO();
norm=parent->getData();
dat=slice->getData();

mx[0]=(cos(phi)*cos(az)-cos(alt)*sin(az)*sin(phi));
mx[1]=-(sin(phi)*cos(az)+cos(alt)*sin(az)*cos(phi));
mx[2]=(cos(phi)*sin(az)+cos(alt)*cos(az)*sin(phi));
mx[3]=(-sin(phi)*sin(az)+cos(alt)*cos(az)*cos(phi));
mx[4]=sin(alt)*sin(phi);
mx[5]=sin(alt)*cos(phi);

rl=(ny/2-1)*(ny/2-1);

for (y=0; y<ny; y++) {
	for (x=0; x<nx/2; x++) {
		if ((x*x+SQR(y-ny/2))>=rl) continue;

		xx=x*mx[0]+(y-ny/2)*mx[1];
		yy=x*mx[2]+(y-ny/2)*mx[3];
		zz=x*mx[4]+(y-ny/2)*mx[5];
		// complex conjugate
		if (xx<0) { xx=-xx; yy=-yy; zz=-zz; cc=-1.0; } else cc=1.0;
		yy+=ny/2;
		zz+=nz/2;

		dt[0]=dat[x*2+y*nx];
		dt[1]=cc*dat[x*2+1+y*nx];

switch (mode) {
	case 1:
		x0=2*(int)floor(xx+.5);
		y0=(int)floor(yy+.5);
		z0=(int)floor(zz+.5);
		rdata[x0+y0*nx+z0*nx*ny]+=weight*dt[0];
		rdata[x0+y0*nx+z0*nx*ny+1]+=weight*dt[1];
		norm[x0+y0*nx+z0*nx*ny]+=weight;
		break;
	case 2:
		x0=(int)floor(xx);
		y0=(int)floor(yy);
		z0=(int)floor(zz);

		dx=xx-x0;
		dy=yy-y0;
		dz=zz-z0;

		if (x0>nx-2 || y0>ny-1 || z0>nz-1) break;

		i=(int)(x0*2+y0*nx+z0*nx*ny);

		g[0]=agauss(1.0,dx,dy,dz,I2G);
		g[1]=agauss(1.0,1.0-dx,dy,dz,I2G);
		g[2]=agauss(1.0,dx,1.0-dy,dz,I2G);
		g[3]=agauss(1.0,1.0-dx,1.0-dy,dz,I2G);
		g[4]=agauss(1.0,dx,dy,1.0-dz,I2G);
		g[5]=agauss(1.0,1.0-dx,dy,1.0-dz,I2G);
		g[6]=agauss(1.0,dx,1.0-dy,1.0-dz,I2G);
		g[7]=agauss(1.0,1.0-dx,1.0-dy,1.0-dz,I2G);

		for (j=0; j<8; j++) {
			k=i+off[j];
			rdata[k]+=weight*dt[0]*g[j];
			rdata[k+1]+=weight*dt[1]*g[j];
			norm[k]+=weight*g[j];
		}

		break;
	case 3:
		x0=2*(int)floor(xx+.5);
		y0=(int)floor(yy+.5);
		z0=(int)floor(zz+.5);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		// if too close to x origin, use supplemental array
		if (x0==0) l=x0; else l=x0-2;
		for (k=z0-1; k<=z0+1; k++) {
			for (j=y0-1; j<=y0+1; j++) {
				for (i=l; i<=x0+2; i+=2) {
					r=hypot3((float)i/2-xx,(float)j-yy,(float)k-zz);
					gg=exp(-r/I3G);

					rdata[i+j*nx+k*nx*ny]+=weight*gg*dt[0];
					rdata[i+j*nx+k*nx*ny+1]+=weight*gg*dt[1];
					norm[i+j*nx+k*nx*ny]+=weight*gg;
				}
			}
		}
		break;
	case 4:			// better check this one sometime
		x0=2*(int)floor(xx);
		y0=(int)floor(yy);
		z0=(int)floor(zz);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		if (x0==0) l=x0; else l=x0-2;
		for (k=z0-1; k<=z0+2; k++) {
			for (j=y0-1; j<=y0+2; j++) {
				for (i=l; i<=x0+4; i+=2) {
					r=hypot3((float)i/2-xx,(float)j-yy,(float)k-zz);
					gg=exp(-r/I4G);

					rdata[i+j*nx+k*nx*ny]+=weight*gg*dt[0];
					rdata[i+j*nx+k*nx*ny+1]+=weight*gg*dt[1];
					norm[i+j*nx+k*nx*ny]+=weight*gg;
				}
			}
		}
		break;
	case 5:
	
		x0=(int)floor(xx+.5);
		y0=(int)floor(yy+.5);
		z0=(int)floor(zz+.5);

		mx0=-(int)floor((xx-x0)*39.0+.5)-78;
		my0=-(int)floor((yy-y0)*39.0+.5)-78;
		mz0=-(int)floor((zz-z0)*39.0+.5)-78;
		x0*=2;
//printf("%3d %3d %3d   %f %f %f\n",mx0,my0,mz0,xx,yy,zz);
		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		if (x0==0) { l=0; mx0+=78; }
		else if (x0==2) { l=0; mx0+=39; }
		else l=x0-4;
		for (k=z0-2,mmz=mz0; k<=z0+2; k++,mmz+=39) {
			for (j=y0-2,mmy=my0; j<=y0+2; j++,mmy+=39) {
				for (i=l,mmx=mx0; i<=x0+4; i+=2,mmx+=39) {
					ii=i+j*nx+k*nx*ny;
					gg=weight*gimx[abs(mmx)+abs(mmy)*100+abs(mmz)*10000];

//					r=hypot3((float)i/2-xx,(float)j-yy,(float)k-zz);
//	if (mx0%39!=0)				printf("%d %d %d\t%f %f\t%f\n",mmx,mmy,mmz,gg,weight*exp(-r/I5G),gg-weight*exp(-r/I5G));
					
					rdata[ii]+=gg*dt[0];
					rdata[ii+1]+=gg*dt[1];
					norm[ii]+=gg;
				}
			}
		}
		
		// when we're too close to the origin we need to deal with the mirrored points
		if (x0<=2) {
			xx=-xx;
			yy=-(yy-ny/2)+ny/2;
			zz=-(zz-nz/2)+nz/2;
			x0=(int)floor(xx+.5);
			y0=(int)floor(yy+.5);
			z0=(int)floor(zz+.5);
			mx0=-(int)floor((xx-x0)*39.0+.5);
			x0*=2.0;

			if (y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

			for (k=z0-2,mmz=mz0; k<=z0+2; k++,mmz+=39) {
				for (j=y0-2,mmy=my0; j<=y0+2; j++,mmy+=39) {
					for (i=0,mmx=mx0; i<=x0+4; i+=2,mmx+=39) {
						ii=i+j*nx+k*nx*ny;
						gg=weight*gimx[abs(mmx)+abs(mmy)*100+abs(mmz)*10000];
	
						rdata[ii]+=gg*dt[0];
						rdata[ii+1]-=gg*dt[1];	// note the -, complex conj.
						norm[ii]+=gg;
					}
				}
			}
		}
		break;
// mode 6 is now mode 5 without the fast interpolation
	case 6:
	
		x0=2*(int)floor(xx+.5);
		y0=(int)floor(yy+.5);
		z0=(int)floor(zz+.5);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		if (x0<=2) l=0; 
		else l=x0-4;
		for (k=z0-2; k<=z0+2; k++) {
			for (j=y0-2; j<=y0+2; j++) {
				for (i=l; i<=x0+4; i+=2) {
					ii=i+j*nx+k*nx*ny;
					r=hypot3((float)i/2-xx,(float)j-yy,(float)k-zz);
					gg=weight*exp(-r/I5G);
					
					rdata[ii]+=gg*dt[0];
					rdata[ii+1]+=gg*dt[1];
					norm[ii]+=gg;
				}
			}
		}
		
		// when we're too close to the origin we need to deal with the mirrored points
		if (x0<=2) {
			xx=-xx;
			yy=-(yy-ny/2)+ny/2;
			zz=-(zz-nz/2)+nz/2;
			x0=2*(int)floor(xx+.5);
			y0=(int)floor(yy+.5);
			z0=(int)floor(zz+.5);

			if (y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

			for (k=z0-2; k<=z0+2; k++) {
				for (j=y0-2; j<=y0+2; j++) {
					for (i=0; i<=x0+4; i+=2) {
						ii=i+j*nx+k*nx*ny;
						r=hypot3((float)i/2-xx,(float)j-yy,(float)k-zz);
						gg=weight*exp(-r/I5G);
	
						rdata[ii]+=gg*dt[0];
						rdata[ii+1]-=gg*dt[1];	// note the -, complex conj.
						norm[ii]+=gg;
					}
				}
			}
		}
		break;
// mode 7 is 5x5x5 with a hypergeometric correction factor
	case 7:
	
		x0=2*(int)floor(xx+.5);
		y0=(int)floor(yy+.5);
		z0=(int)floor(zz+.5);

		// there is a 2 pixel zero zone around the edge of fourier space
		if (x0>=nx-4 || y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

		if (x0<=2) l=0; 
		else l=x0-4;
		for (k=z0-2; k<=z0+2; k++) {
			for (j=y0-2; j<=y0+2; j++) {
				for (i=l; i<=x0+4; i+=2) {
					ii=i+j*nx+k*nx*ny;
					r=sqrt(hypot3((float)i/2-xx,(float)j-yy,(float)k-zz));
					gg=weight*hyperg(r);
					
					rdata[ii]+=gg*dt[0];
					rdata[ii+1]+=gg*dt[1];
					norm[ii]+=gg;
				}
			}
		}
		
		// when we're too close to the origin we need to deal with the mirrored points
		if (x0<=2) {
			xx=-xx;
			yy=-(yy-ny/2)+ny/2;
			zz=-(zz-nz/2)+nz/2;
			x0=2*(int)floor(xx+.5);
			y0=(int)floor(yy+.5);
			z0=(int)floor(zz+.5);

			if (y0>ny-3 || z0>nz-3 || y0<2 || z0<2) break;

			for (k=z0-2; k<=z0+2; k++) {
				for (j=y0-2; j<=y0+2; j++) {
					for (i=0; i<=x0+4; i+=2) {
						ii=i+j*nx+k*nx*ny;
						r=sqrt(hypot3((float)i/2-xx,(float)j-yy,(float)k-zz));
						gg=weight*hyperg(r);
	
						rdata[ii]+=gg*dt[0];
						rdata[ii+1]-=gg*dt[1];	// note the -, complex conj.
						norm[ii]+=gg;
					}
				}
			}
		}
		break;
	}

	}
}

doneData();
parent->doneData();
slice->doneData();
slice->update();

return;
}

// once all of the slices have been added with insertSlice(), this routine
// does a normalization do an IFT to get the reconstructed volume
void EMData::doneSlice()
{
int i;
float *norm;
float d;
EMData *ret;

getData();
norm=parent->getData();

for (i=0; i<nx*ny*nz; i+=2) {
	d=norm[i];
	if (d==0) rdata[i]=rdata[i+1]=0;
	else {
		rdata[i]/=d;
		rdata[i+1]/=d;
	}
}

doneData();
parent->doneData();

}

// does an IFT and corrects for trilinear sampling
EMData *EMData::iftSlice()
{
float x,y,z,r;
int i;
EMData *ret;
float *rd;

ret=doIFT();
/*ret->toCorner();

rd=ret->getData();

i=0;
for (z=-.5; z<=.5; z+=1.0/(nz-1.0)) {
	for (y=-.5; y<.5; y+=1.0/(ny-1.0)) {
		for (x=-.5; x<.5; x+=1.0/(nx-3.0),i++) { 
			rd[i]/=((1.0-cos(2.0*PI*x))*(1.0-cos(2.0*PI*y))*(1.0-cos(2.0*PI*z))/
				(4*PI*PI*x*x*y*y*z*z));
		}
	}
}

ret->doneData();
ret->toCorner();*/
return ret;
}
