#include <stdio.h>
#include <math.h>
#include "Euler.h"
#include <string.h>
#define PI 3.1415926
//       %% Euler.C         %%                    Philip R. Baldwin 11/5/98
//            This is a C++ class, 
//               for performing operations on Euler angles and rotations.
//             The angles are kept internally in EMAN format (altitude,
//              azimuth, phi) or (theta, psi, phi) following Goldstein's
//                 mechanics book p.147.
//             The rotation angles may be accessed in various electrom
//               microscopy formats  and intraconverted.  
//             In addition symmetry operations are created for various
//                  common crystal types: thus from an initial angle
//                  assignment, symmetry equivalent angles may be created.
//             The private variables may be viewed from Euler.h
//                   they are       
//						char *symmetry_;  int symcounter_;
//                      int typ_; float alt_ , az_ , phi_ ;  
//                      Euler * symoperation_; int MaxSymEl_;
//
//               Many of the functions are overloaded, 
//                 since rotations may be defined using quaternions, points
//                  on S^3,  three by three matrices, spin- and spin-axis
//                  and IMAGIC format.


//       Arguments in Radians!!!!!!!!!!!!!!!!!!!!!!

//      THE CONSTRUCTORS
Euler::Euler(){ alt_=0.; az_=0.; phi_=0.;} // already rectified

	
Euler::Euler(float alt, float az, float phi){// EMAN by default
	setAngle(alt,az,phi);
}

Euler::Euler(float a1, float a2, float a3, int typ) { 
	setAngle(a1,a2,a3,typ);
} // end EMAN, IMAGIC, SPIDER constructor

Euler::Euler(float e1, float e2, float e3, float e0, int typ) {
	setAngle(e1,e2,e3,e0,typ); 
}           // end quaternion, and spin constructor	

Euler::Euler(float *m){
	setAngle(m);
}

void Euler:: rectify(){
	if (sqrt(alt_*alt_) <= .001) { az_ += phi_; phi_=0;}
	while (phi_ < 0) phi_ += 2.*PI; while (phi_ >= 2.*PI) phi_ += -2.*PI ;
	while (az_ < 0) az_ += 2.*PI; while (az_  >= 2.*PI) az_ += -2.*PI  ;
}

/* *****************************************************************************  */
//		               FUNCTIONS
//   ********   1. Set Type  EMAN, IMAGIC, QUATernion, SPIN
//     

void Euler::setTyp(int typ) { typ_=typ ;}


//          2.    setAngle  overloaded( 
//
	
void Euler::setAngle(float a1, float a2, float a3, int typ){ 
	alt_ =a1 ; az_ = a2 ; phi_ = a3;
	if (typ == IMAGIC) { az_ +=PI; phi_ +=PI;} // then a1= beta, a2=gamma, a3=alpha
	if (typ == SPIDER) { az_ += -PI/2.; phi_ += PI/2.;}    // then a1=theta, a2=phi, a3=psi
    	if (typ !=EMAN && typ !=IMAGIC && typ != SPIDER) {	 
			printf("unknown declaration of type for Euler constructor /n");} 
			rectify();
}           // end EMAN and IMAGIC setAngle
			
void  Euler::setAngle(float e1, float e2, float e3, float e0, int typ) {
	typ_ = typ ;
	if (typ !=SPIN && typ!=QUAT && typ != SGIROT) {	 
		printf("unknown declaration of type for Euler constructor /n");}
//   if not already, put in S4 language
	if (typ == SPIN || typ==SGIROT) { 
		float normal = sqrt(e1*e1+e2*e2+e3*e3);
		float sinQo2 = sin(e0/2.);		e0 = cos(e0/2.);
		e1 *= sinQo2/normal; e2 *= sinQo2/normal; e3 *= sinQo2/normal; }

		
	alt_ = acos((e0*e0 - e1*e1 - e2*e2 + e3*e3)/(e0*e0+e1*e1+e2*e2+e3*e3)); 
	az_  = atan2( e2*e0 - e1*e3 , e1*e0 + e2*e3); 
	phi_ = atan2(-e2*e0 - e1*e3 , e1*e0 - e2*e3); 
	rectify();
	if (typ==SGIROT) inverse();
}           // end quaternion, and spin setAngle

void Euler::setAngle(float *m){
	typ_ = MATRIX;
	alt_ = acos( m[8]    );
	az_  = atan2(m[6],-m[7]);
	phi_ = atan2(m[2], m[5]);
	if (m[8] > 0.999) {
		alt_=0;
		az_ = atan2(m[1],m[4]);
		phi_ =0;}
	if (m[8] < -0.9999) {
		alt_ = PI;
		az_ = atan2(m[1],-m[4]);
		phi_ =0;
	}		 		 
	rectify();
}
// ********** %% 2.3 Increment an Euler %%
void Euler::increment(float delAlt, float delAz, float delPhi){
	alt_ += delAlt; az_ += delAz; phi_ += delPhi; rectify();}
	

//    ******** %% 2.5  Take an inverse of an Euler %%
Euler Euler::inverse() {
        Euler ereturn(alt_,3.*PI - phi_,3.*PI- az_ );
       ereturn.rectify();  
        return ereturn;
}

//    ******** %% 3.  Multiplying Eulers (from Left) %%

void Euler::fromLeft(Euler leftEuler) {
//	find ac = bc* ab   = leftEuler * currentEuler
	float * mreturn =matmulmat(leftEuler.mat(), (*this).mat());
	Euler ereturn( mreturn );
	alt_ = ereturn.alt(); az_= ereturn.az(); phi_ = ereturn.phi();
	free(mreturn);
}  // end applying Euler fromLeft




void Euler::fromLeft(float alt, float az, float phi) {
//	find ac = bc* ab   = leftEuler * currentEuler
	Euler ereturn(alt,az,phi);
	float * mreturn =matmulmat(ereturn.mat(), (*this).mat());
	ereturn.setAngle( mreturn );
	alt_ = ereturn.alt(); az_= ereturn.az(); phi_ = ereturn.phi();
	free(mreturn);
}  // end applying Euler fromLeft



Euler Euler::EulerMult(Euler bc) {
	Euler ac(alt_, az_ , phi_);  // actually this starts ac at ab
	 ac.fromLeft(bc);		   // but now it rotates ac to
	 return ac;         // the value of interest
} // end Euler multiply


Euler Euler::operator *(Euler ab) {
	 ab.fromLeft(*this); 
	 return ab;                           
} // end Euler multiply

Euler Euler::operator /(Euler ab) {
	return ((*this) * (ab.inverse()));
} // end Euler divide

int Euler::operator !=(Euler ab) {
	int iquestalt = (ab.alt() != (*this).alt());
	int iquestaz = (ab.az() !=   (*this).az());
	int iquestphi = (ab.phi() != (*this).phi());
	return iquestalt+iquestaz+iquestphi;
}

//    ***     %% 4.  SYMMETRY   SYMMETRY SYMMERY  %% !!!!!!!!!!!!!!!!!!!!!!!!!
// **********     ***********

int Euler::getMaxSymEl(){
	return MaxSymEl_;}	


// NOTE: D symmetry implimented same as C symmetry. 2nd 2-fold must be handled externally
void Euler::setSym(char *symmetry) { // sets the symmetry and defines symmetry operations
	symcounter_ = 0;			  //      only C(N) and icosahedral symmetries implimented
	symmetry_ = symmetry ;
	int i;
	float piover = PI/180.;
	
	if (!symmetry) symmetry_="i";
	if (tolower(symmetry_[0])=='c' || tolower(symmetry_[0])=='d') MaxSymEl_=atoi(symmetry_+1);	
	else if (strncmp(symmetry_,"icos",4)==0 || strncmp(symmetry_,"ICOS",4)==0)
		MaxSymEl_=60;
	else if (tolower(symmetry_[0])=='i') MaxSymEl_=1;

	symoperation_=(Euler *)malloc(sizeof(Euler)*MaxSymEl_);

	if(tolower(symmetry_[0])=='c' || tolower(symmetry_[0])=='d') {
		for (i=0; i<MaxSymEl_; i++) 
			symoperation_[i].setAngle(0,PI*2.0*i/(float)MaxSymEl_,0);
	}
	else if(strncmp(symmetry_,"icos",3) ==0 || strncmp(symmetry_,"ICOS",4)==0) {
		float icos[180]={
	      0.,0.  ,   0., 0.,0. ,  288., 0.,0. ,  216., 0.,0. ,  144., 0.,0. ,   72., \
	      0.,63.4349 , 36., 0.,63.4349 , 324., 0.,63.4349 , 252., 0.,63.4349 , 180., 0.,63.4349 , 108., \
	      72.,63.4349 , 36., 72.,63.4349 , 324., 72.,63.4349 , 252., 72.,63.4349 , 180., 72.,63.4349 , 108.,\
	     144.,63.4349,36.,144.,63.4349 ,324.,144.,63.4349 , 252.,144.,63.4349 , 180., 144.,63.4349, 108., \
	     216.,63.4349 , 36., 216.,63.4349,324., 216.,63.4349 ,252.,216.,63.4349 ,180., 216.,63.4349 ,108., \
	     288.,63.4349 , 36., 288.,63.4349,324., 288.,63.4349,252., 288.,63.4349,180.,288.,63.4349 ,108.,\
	      36.,116.5651 ,0., 36.,116.5651,288., 36.,116.5651 ,216., 36.,116.5651 ,144., 36.,116.5651 ,72.,\
	     108.,116.5651 ,0.,108.,116.5651 ,288., 108.,116.5651 ,216.,108.,116.5651 ,144., 108.,116.5651 ,72.,\
	     180.,116.5651 ,0.,180.,116.5651 ,288., 180.,116.5651 ,216., 180.,116.5651 ,144., 180.,116.5651 ,72.,\
	     252.,116.5651 ,0., 252.,116.5651 ,288., 252.,116.5651 ,216., 252.,116.5651 ,144., 252.,116.5651 ,72.,\
	     324.,116.5651 ,0.,324.,116.5651 ,288., 324.,116.5651 ,216., 324.,116.5651 ,144., 324.,116.5651 ,72.,\
	     0.,180.,0., 0.,180.,288., 0.,180.,216., 0.,180.,144., 0.,180.,72.  };
//		
		for (i=0; i<60; i++) 
			symoperation_[i].setAngle(icos[3*i+1]*piover, icos[3*i+2]*piover,icos[3*i]*piover);  		
	} // end case icos
	else symoperation_[0].setAngle(0,0,0);

} // end setSym

void Euler::resetSym(){
// This program initializes the symmetry element arrays
// The correct symmetry already needs to be identified
	symcounter_ = 0; 
	setSym(symmetry_);
	return;	
}

Euler Euler::FirstSym(){

	resetSym();
	return NextSym();
}

Euler Euler::NextSym() {
	symcounter_ ++;
	Euler bc(alt_,az_,phi_);
	Euler invalid(-.1,-.1,-.1);
//	Euler symoperation;
	if (symcounter_ > MaxSymEl_) { 
		return invalid;
		} else {	
		return ( bc*symoperation_[symcounter_-1] ) ;
	} 

}  // end NextSym

int Euler::valid() {
	if (alt() < 0) return 0;
	return 1;
}
	

//                     INLINE FUNCTIONS
//inline float Euler::operator[](int b) { return angle[b];}

/* *****************************************************************************  */
// %% we need as many ways as possible to READ THE DATA %%

//        Internal and EMAN formulation of the rotation
float  Euler::alt() { return alt_ ;}
float  Euler::az() {return az_  ;}
float  Euler::phi() { return phi_ ;}

float  Euler::alpha() { return ((phi_ +PI) );}    // IMAGIC format Angles
float  Euler::beta() { return alt_ ; }
float  Euler::gamma() { return ((az_ + PI));}

float  Euler::psi() { return ((phi_ + PI/2.) );}    // SPIDER format Angles
float  Euler::theta() { return alt_ ; }
float  Euler::phiSpider() { return ((az_ - PI/2.));}

float  Euler::e0() { return (  cos((phi_ + az_)/2.) * cos(alt_/2.)  );}  // Quaternion format
float  Euler::e1() { return (  cos((phi_ - az_)/2.) * sin(alt_/2.)  );}
float  Euler::e2() { return ( -sin((phi_ - az_)/2.) * sin(alt_/2.)  );}
float  Euler::e3() { return ( -sin((phi_ + az_)/2.) * cos(alt_/2.)  );}

// Spin-axis format
float  Euler::Q() { return ( 2.* acos( cos((phi_ + az_)/2.) * cos(alt_/2.))  );}
float  Euler::n1() { float e0 =cos((phi_ + az_)/2.) * cos(alt_/2.)  ;
					return (  cos((phi_ - az_)/2.) * sin(alt_/2.) / (sqrt(1-e0*e0)) );}
float  Euler::n2() { float e0 = cos((phi_ + az_)/2.) * cos(alt_/2.)  ;
					return ( -sin((phi_ - az_)/2.) * sin(alt_/2.) / (sqrt(1-e0*e0))  );}
float  Euler::n3() { float e0 = cos((phi_ + az_)/2.) * cos(alt_/2.)  ;
					return ( -sin((phi_ + az_)/2.) * cos(alt_/2.)/ (sqrt(1-e0*e0))   );}


// SGI format
float  Euler::Qsgi() { return ( 2.* acos( cos((phi_ + az_)/2.) * cos(alt_/2.))  );}
float  Euler::n1sgi() { float e0 =cos((phi_ + az_)/2.) * cos(alt_/2.)  ;
					return (  -cos((phi_ - az_)/2.) * sin(alt_/2.) / (sqrt(1-e0*e0)) );}
float  Euler::n2sgi() { float e0 = cos((phi_ + az_)/2.) * cos(alt_/2.)  ;
					return ( -sin((phi_ - az_)/2.) * sin(alt_/2.) / (sqrt(1-e0*e0))  );}
float  Euler::n3sgi() { float e0 = cos((phi_ + az_)/2.) * cos(alt_/2.)  ;
					return ( sin((phi_ + az_)/2.) * cos(alt_/2.)/ (sqrt(1-e0*e0))   );}

// Matrix format for a rotation
float *Euler::mat() { 
	m_[0]=  cos(phi_) * cos(az_) - sin(phi_) * cos(alt_)*sin(az_);
	m_[1]=  cos(phi_) * sin(az_) + sin(phi_) * cos(alt_)*cos(az_);
	m_[2]=  sin(phi_) * sin(alt_);
	m_[3]= -sin(phi_) * cos(az_) - cos(phi_) * cos(alt_)*sin(az_);
	m_[4]= -sin(phi_) * sin(az_) + cos(phi_) * cos(alt_)*cos(az_);
	m_[5]=  cos(phi_) * sin(alt_);
	m_[6]=  sin(alt_) * sin(az_) ;
	m_[7]= -sin(alt_) * cos(az_) ;
	m_[8]=  cos(alt_);
	return m_;
}		




float * matmulmat(float *mleft, float *mright) { 
	float *mreturn;
	int i,j,k;	
	mreturn = (float *) malloc(9*sizeof(float));
	for (i=0; i <3; i++){ 
		for (j=0; j<3; j++){ 
			mreturn[j+3*i]=0;
			for (k=0;k<3;k++){
				mreturn[j+3*i] += mleft[k+3*i] * mright[j+3*k];
	}}} // end k j i loops	
	return mreturn;
} // end matmulmat

