#ifndef EULERINC
#define EULERINC

#include <stdlib.h>
#include <math.h>
#include <ctype.h>
// #include "util.h"

// int MAXEL[] = { 1 , 2 ,5,6,7 };

class Euler{

public:

// that means SPIN axis variables: closely related to QUATernionic variables
enum { EMAN,IMAGIC,SPIN,QUAT,MATRIX,SGIROT,SPIDER };

//      CONSTRUCTORS
// Euler(float a1, float a2, float a3, float a4=0, int typ_=1);  // An all-purpose one?
Euler();
Euler(float a1, float a2, float a3);// EMAN by default
Euler(float a1, float a2, float a3, int typ); // EMAN, IMAGIC or SPIDER
Euler(float e1, float e2, float e3, float e0, int typ); // QUATERNIONS, SPIN axis, or SGIROT 
// ~Euler();
Euler(float *m); // matrix form


//		FUNCTION DECLARATIONS
void rectify();
void setAngle(float a1, float a2, float a3, int typ = EMAN);  //  EMAN, IMAGIC or SPIDER setAngle			
void setAngle(float e1, float e2, float e3, float e0, int typ); // quaternion, spin, and SGIROT setAngle
void setAngle(float *m); // matrix setAngle
void setTyp(int i);

void increment(float delAlt, float delAz, float delPhi); // increment an Euler in place

Euler inverse();       // takes the inverse of an euler
void fromLeft(Euler leftEuler); // rotates a given rotation  on the left
void fromLeft(float alt, float az, float phi); // rotates a given rotation  on the left
void fromLeftold(Euler bc); // rotates a given rotation  on the left; don't use this
Euler EulerMult(Euler bc) ;  // returns the value of the given rotation above

Euler operator*(Euler ab );
Euler operator/(Euler ab) ;
int operator!=(Euler ab);

int getMaxSymEl(); // gets the order of the group
void setSym(char *symmetry);// initialize symcounter to zero, 
void resetSym();	// initializes symcounter_ without returning 1st element
Euler FirstSym();	// initializes symcounter_, symoperation_m MaxSymEl_										// 	unless told otherwise
Euler NextSym();
int valid();


// we need as many ways as possible to read the data
float alt();
float az();
float phi();

float alpha();  // IMAGIC lingo
float beta();
float gamma();

float psi();    // SPIDER lingo
float theta();
float phiSpider();

float e0();      // QUATernion lingo
float e1() ;
float e2() ;
float e3() ;

float Q();      // SPIN axis lingo
float n1() ;
float n2() ;
float n3() ;

float Qsgi();      //SGIROT lingo
float n1sgi() ;
float n2sgi() ;
float n3sgi() ;

float *mat(); // get out the 3 by 3 matrix; 

//float operator[];

private:
char *symmetry_;         // The symmetry of the protein to which an orientation is assigned
int symcounter_;         // Lables a symmetry related orientation wrt a primary orientation
int typ_;                  // The representation of the orientation that the user is employing
float alt_ , az_ , phi_ ;  // EMAN and internal variables
float m_[9];

// float alpha, beta, gamma; // IMAGIC variables
// float n1, n2, n3, Q;   // spin variables
// float e0, e1, e2, e3; // S4 or quaternionic variables
Euler * symoperation_;
int MaxSymEl_;


}; // ends declaration of class Euler

float *matmulmat(float *mleft, float *mright); // multiply 3 by 3 matrices

#endif
