#ifndef UTILDEF
#define UTILDEF

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "emver.h"

#define PI (3.141592654)

#define MAXPATH 256
#define PEAKMAX 2

//#define MSB_FIRST

#define ERR_EXIT	0
#define ERR_CRIT	1
#define ERR_ERROR	2
#define ERR_WARN	3
#define ERR_MSG		4
#define ERR_INFO	5
#define ERR_DEBUG	6

#ifndef MAXPATHLEN
#define MAXPATHLEN	1024
#endif

struct ANG {
float a1[PEAKMAX+1];
float a2[PEAKMAX+1];
float l[PEAKMAX+1];
};

inline float maxf(float f1,float f2,float f3=-1.0e-30) {
	if (f1>=f2 && f1>=f3) return f1;
	if (f2>=f1 && f2>=f3) return f2;
	return f3;
}

inline float minf(float f1,float f2,float f3=1.0e30) {
	if (f1<=f2 && f1<=f3) return f1;
	if (f2<=f1 && f2<=f3) return f2;
	return f3;
}

// faster than floor
inline int floor2(float x) {
	if (x<0) return(int)x-1;
	return (int)x;
}

// faster than floor(x+.5)
inline int round(float x) {
	if (x<0) return (int)(x-.5);
	return (int)(x+.5);
}

// hypot squared
inline float hypot2(float x, float y) { return x*x+y*y; }

// this returns hypot^2 for 3 args
inline float hypot3(float x,float y,float z) { return x*x+y*y+z*z; }

// used for parsing special delimited HTML files for generating
// custom sets of instructions
void addHTMLSection(char *sourcefile,FILE *out,char *secid);

struct euler { float alt,az,phi; };

void error(char *msg,int pri);
void progress(float pct);

int file_lock_wait(const char *name);
int file_lock(const char *name);
void file_unlock(const char *name);

// default file opening
FILE *DFopenr(const char *name);
FILE *DFopenw(const char *name);
FILE *DFopena(const char *name);
char *DFdir();

char *EMDIR();

// ref is a reference number returned by LOGbegin
#define LOG_MISC		0	
#define LOG_BEGIN		1	// program begin
#define LOG_END			2	// program end
#define LOG_INFILE		10	// input file for processing
#define LOG_OUTFILE		11	// output file for processing
#define LOG_REFFILE		12	// reference file, used with other files
#define LOG_EXTIN		20	// file accessed by a program in another directory
#define LOG_EXTOUT		21
#define LOG_EXTREF		22
#define LOG_ERROR		999
void LOG(int ref,char *file,int mode,char *text);
int  LOGbegin(int argc,char *argv[],int ppid=0);
void LOGerror(int ref,int err,char *text);

// All applications should call this before parsing the command line
void appinit(int argc,char *argv[]);

float frand(float min,float max);

// random number with gaussian distribution
float grand(float mean,float sigma);

// just like modulus, except forces negatives to be in the positive range
inline int cyc(int i,int n) {
	if (i<0) return(n-((-i)%n)-1);
	else return i%n;
}

// linear interpolation in an x/y array at x=xl
float interpolate(float xl,int n,float *x,float *y);

// bilinear interpolation p1=x0,y0; p2=x1,y0; p3=x1,y1; p4=x0,y1
inline float bilin(float p1, float p2, float p3, float p4, float t, float u) {
	return (1.0-t)*(1.0-u)*p1+t*(1.0-u)*p2+t*u*p3+(1.0-t)*u*p4;
}

// trilinear interpolation p1=x0,y0,z0; p2=x1,y0,z0; p3=x0,y1,z0; etc... (binary pattern)
inline float trilin(float p1,float p2,float p3,float p4,float p5,float p6,float p7,float p8,float t,float u, float v) {
	return (1.0-t)*(1.0-u)*(1.0-v)*p1+t*(1.0-u)*(1.0-v)*p2+
		(1.0-t)*u*(1.0-v)*p3+t*u*(1.0-v)*p4+
		(1.0-t)*(1.0-u)*v*p5+t*(1.0-u)*v*p6+
		(1.0-t)*u*v*p7+t*u*v*p8;
}

// this will interpolate a peak location. v3 is considered to be at x=0
// The other points are nearest neighbors
// returns 0 if not well defined
float p5peak(float v1, float v2,float v3, float v4,float v5);

inline float agauss(float a,float dx,float dy, float dz,float d) {
	return a*exp(-(dx*dx+dy*dy+dz*dz)/d);
}

inline float SQR(float x) { return x*x; }

// 'trackball' rotation of a set of euler angles
// rotates the euler angles about a vector perpendicular to dx,dy
// by an amount proportional to their length
void roteulertb(float *alt,float *az,float *phi, float dx,float dy);

void swapint(char *d,int n);
void swapshort(char *d,int n);
int npfa(int low);
int isvalidfft(int low);

void save_data(float *x, float *y, int n, const char *fsp);
void save_data(float x0,float dx, float *y, int n, const char *fsp);

// memory allocation routines
inline void *Smalloc(int size,int shared) {
return malloc(size);
}

inline void Sfree(void *ptr, int shared) {
free(ptr);
}

inline void *Srealloc(void *ptr, int size, int shared) {
return realloc(ptr,size);
}

// This is basically just a version of 'system'
void run(char *s);

// prototypes when I start using shared mem
/*
void *Smalloc(int size,int shared);
void Sfree(void *ptr, int shared);
void *Srealloc(void *ptr, int size, int shared);
*/

#endif
