#include "Image.h"
#include <qfile.h>
#include <qdragobject.h>
#include <qprinter.h>

Image::Image(QWidget* parent,const char* name):QWidget( parent, name ) {
initMetaObject();

resize(200,200);

qp=new QPainter();
qpm=new QPixmap(size());
qpd=NULL;

data=dcopy=NULL;

setBackgroundMode(NoBackground);

shapes.setAutoDelete(TRUE);
lscale=1.0;

mode=0;
cont=0;
brite=0;
needupd=1;
scale=0;
invert=0;
imc=NULL;
}

Image::~Image() {
if (imc) delete imc;
delete qp;
delete qpm;
}

void Image::resizeEvent ( QResizeEvent *qe ) {
qp->begin(this);
qp->eraseRect(qp->window());
qp->end();
if (!data) return;

qpm->resize(size().width(),size().height());
needupd=1;
emit(resized(size().width(),size().height()));
}

void Image::paintEvent( QPaintEvent *pe ) {
char *d;
float mm,b;
int i,sx,sy;
Shape *cur;



//sz=width()<height()?width():height();
sx=qpm->width();
sy=qpm->height();
if (data && scale==0) {
	lscale=(float)sx/(data->xSize()-.5);
	originx=originy=0;
}
else lscale=scale;

if (needupd||qpd) {
	if (!data) {
		qpm->fill();
		qp->begin(qpm);
		qp->setPen(black);
		qp->drawText(0,0,size().width(),size().height(),AlignCenter,mtstr.data());
		qp->end();
	}
	else if (data->zSize()==1) {
//		qpm->fill();

		mm=-data->Min()<data->Max()?data->Max():-data->Min();
		b=mm*brite;
	   mm*=(1.0-sqrt(cont));
		d=(char *)malloc(sx*sy+80);
		memset(d,127,sx*sy+80);

		sprintf(d,"P5\n%d %d\n255\n",sx,sy);
		i=strlen(d);
		data->renderAmp8((unsigned char *)d+i,originx/lscale,originy/lscale,sx,sy,sx,
			lscale,invert?254:1,invert?1:254,-mm-b,mm-b);

		qpm->loadFromData((uchar *)d,(uint)(i+sx*sy));

//pr->renderAmp8((unsigned char *)d+i,0,0,width(),height(),width(),
//	scale,1,254,-mm-b,mm-b);

/*QFile out("abc.pgm");
out.open(IO_WriteOnly);
out.writeBlock(d,(i+height()*width()));
out.close();*/

		free(d);
		needupd=0;
//printf("%d,%d  %d,%d\n",size().width(),size().height(),qpm->size().width(),qpm->size().height());

	}
	else if (dcopy) {
		EMData *pr;
		
		if (mode) {
			pr=dcopy->project3d(angle.alt(),angle.az(),angle.phi(),-3,(brite+1.0)*dcopy->Max()/2.0);
			mm=-pr->Min()<pr->Max()?pr->Max():-pr->Min();
//			b=mm*brite;
			mm*=(1.0-cont);
		} else {
			pr=dcopy->project3d(angle.alt(),angle.az(),angle.phi(),-2);
			mm=-pr->Min()<pr->Max()?pr->Max():-pr->Min();
			b=mm*brite;
			mm*=(1.0-cont);
		}

		d=(char *)malloc(sx*sy+80);
		memset(d,0,sx*sy+80);

		sprintf(d,"P5\n%d %d\n255\n",sx,sy);
		i=strlen(d);
		if (pr->isComplex()) pr->ri2ap();
		pr->renderAmp8((unsigned char *)d+i,originx/lscale,originy/lscale,sx,sy,sx,
			lscale,invert?254:1,invert?1:254,-mm-b,mm-b);

		qpm->loadFromData((uchar *)d,(uint)(i+sx*sy));

		free(d);
		needupd=0;
	}

	qp->begin(qpm);
	qp->setPen(red);

	shapes.append(&select);
	for (cur=shapes.first(); cur!=NULL; cur=shapes.next()) {
//printf("%d %d  %d %d\n",cur->x0,cur->y0,dxtopx(cur->x0),dytopy(cur->y0));
		qp->setPen(cur->color);
		switch(cur->type) {
		case Shape::LINE:
			qp->drawLine(dxtopx(cur->x0),dytopy(cur->y0),dxtopx(cur->x1),dytopy(cur->y1));
			if (!cur->text.isEmpty()) {
				qp->setBackgroundMode(OpaqueMode);
				qp->setBackgroundColor(white);
				qp->setPen(black);
				if (cur->y1>cur->y0) qp->drawText(dxtopx(cur->x0),dytopy(cur->y0)+12,cur->text);
				else qp->drawText(dxtopx(cur->x0),dytopy(cur->y0)-5,cur->text);
				qp->setPen(red);
			}
			break;
		case Shape::DOTBOX:
			qp->drawEllipse(dxtopx(cur->x0+cur->x1/2)-1,dytopy(cur->y0+cur->y1/2)-1,3,3);
		case Shape::BOX:
			qp->drawRect(dxtopx(cur->x0),dytopy(cur->y0),cur->x1*lscale,-cur->y1*lscale);
			if (!cur->text.isEmpty()) {
				qp->setBackgroundMode(OpaqueMode);
				qp->setBackgroundColor(white);
				qp->setPen(black);
				qp->drawText(dxtopx(cur->x0),dytopy(cur->y0)+12,cur->text);
			}
			break;
		case Shape::CIRCLE:
			qp->drawEllipse(dxtopx(cur->x0),dytopy(cur->y0),cur->x1*lscale,-cur->y1*lscale);
			break;
		case Shape::TEXT:
			if (!cur->text.isEmpty()) qp->drawText(dxtopx(cur->x0),dytopy(cur->y0),cur->text);
			break;
		}
	}
	shapes.last();
	shapes.take();
	qp->end();
}
bitBlt( this, 0,0, qpm );


/*
if (!qpd) qp->begin(this);
qp->setPen(red);

if (rad>0) {
	qp->translate(sz/2,height()-sz/2);
	qp->rotate(ang+90.0);
//	qp->drawEllipse(sz/2-rad,height()-(sz/2-rad2),rad*2,-rad2*2);
	qp->drawEllipse(-rad,-rad2,rad*2,rad2*2);
	qp->resetXForm();
}
qp->end();
*/

}

int Image::pxtodx(int px) {
return (int)((px+originx)/lscale);
}

int Image::pytody(int py) {
return (int)((height()-py+originy)/lscale);
}

int Image::dxtopx(int dx) {
return (int)(dx*lscale-originx);
}

int Image::dytopy(int dy) {
return (int)(-(dy*lscale-originy-height()));
}


void Image::print(QPaintDevice *dev) {
int x,y;

update();
}

void Image::setData(EMData *d) {
char s[140];

if (!d) return;
data=d; 
if (d && data->ufloat==0) data->ufloat=data->Max();

if (dcopy) { delete dcopy; dcopy=NULL; }
if (d && d->zSize()>1) dcopy=data->copy();

strncpy(s,d->Name(),100);
s[100]=0;
strcat(s," (middle mouse for controls)");
setCaption(s);
qpm->resize(size().width(),size().height());

needupd=1;
update();
}

void Image::mouseDoubleClickEvent(QMouseEvent *e) { };

void Image::mouseEnter(int) { };

void Image::mouseLeave(int) { };

void Image::keyPressEvent(QKeyEvent *) { };

void Image::keyReleaseEvent(QKeyEvent *) { };

void Image::mousePressEvent(QMouseEvent *e) {
int x,y;

if (!data) return;
if ((e->button()&LeftButton)) {
	if ((e->state()&ShiftButton) && qpm) {
/*		QImage *im = new QImage;
		char *d;
		float mm,b;
		int i,sx=qpm->width(),sy=qpm->height();

		mm=-data->Min()<data->Max()?data->Max():-data->Min();
		b=mm*brite;
		mm*=(1.0-cont);
		d=(char *)malloc(sx*sy+80);
		memset(d,127,sx*sy+80);

		sprintf(d,"P5\n%d %d\n255\n",sx,sy);
		i=strlen(d);
		data->renderAmp8((unsigned char *)d+i,originx/lscale,originy/lscale,sx,sy,sx,
			lscale,invert?254:1,invert?1:254,-mm-b,mm-b);

		im->loadFromData((uchar *)d,(uint)(i+sx*sy));

		free(d); */
		
		QImageDrag *idr = new QImageDrag(qpm->convertToImage(),this);
		if (idr) idr->dragCopy();
		else printf("Drag error\n");
	}
	else emit(mousePress(e));
}
/*if ((e->button()&MidButton)) { 
	invert^=1; 
	if (imc) imc->setup(brite,cont,invert,-1); 
	needupd=1; 
	update(); 
}*/
/*if ((e->button()&RightButton) && data && (e->state()&ShiftButton)) { 
	mode^=1; 
	needupd=1; 
	update(); 
}
else*/ 
if ((e->button()&MidButton) && data) {
	if (!imc) {	
		imc=new ImageMxCon; 
		connect(imc,SIGNAL(setBC(float,float)),SLOT(setBC(float,float)));
		connect(imc,SIGNAL(invToggle(int)),SLOT(setInvert(int)));
		connect(imc,SIGNAL(saveImage()),SLOT(saveImage()));
		connect(imc,SIGNAL(print()),SLOT(doPrint()));
	}
	imc->setup(brite,cont,invert,-1,-1);
	imc->show();
	imc->raise();
}

mdx=e->x();
mdy=e->y();

}

void Image::doPrint() {
QPrinter pr;
pr.setup();
QPainter pa(&pr);

pa.drawPixmap(0,0,*qpm);
pa.end();
}

void Image::saveImage() {
QString s1( QFileDialog::getSaveFileName(getenv("PWD"),"*.png") );
if ( s1.isNull() ) return;

qpm->save(s1.data(),"PNG");
}

void Image::setBC(float b,float c) { cont=c; brite=b; needupd=1; update();}

void Image::setInvert(int i) { invert=i; needupd=1; update(); }

void Image::notVisible() { if (imc) imc->hide(); }

void Image::mouseReleaseEvent(QMouseEvent *e) {
int *a,b;

if ((e->button()&LeftButton)) emit(mouseRelease(e));

mdx=e->x();
mdy=e->y();

}

void Image::mouseMoveEvent(QMouseEvent *e) {
int x,y,sz,nx;
float *d,az,r,a=0,c=0,scale;

if (!data) return;

x=e->x();
y=e->y();

if ((e->state()&LeftButton) && data) {
	if (!(e->state()&ShiftButton)) emit(mouseMove(e));
	return;
}
else if (e->state()&RightButton) {
	if (data->zSize()>1) {
		if (y-mdy==0 && x-mdx==0) return;
		angle.setAngle(angle.alt()-(y-mdy)/40.0,angle.az()-(x-mdx)/40.0,angle.phi());
		needupd=1;
		update();
		mdy=y;
		mdx=x;
		return;
	}
	else {
		setOrigin(originx-(x-mdx),originy+y-mdy);
		needupd=1;
		update();
		mdy=y;
		mdx=x;
	}
}
else {
}

}

void Image::hilight(int i) { 
	if (i>=shapes.count()) return;
	select.x0=(*shapes.at(i)).x0; 
	select.y0=(*shapes.at(i)).y0; 
	select.x1=(*shapes.at(i)).x1; 
	select.y1=(*shapes.at(i)).y1; 
	select.user=(*shapes.at(i)).user; 
	select.type=(*shapes.at(i)).type;
	select.color=green;
}

void Image::imageChanged() { 
needupd=1;
update();
}

int Image::addShape(Shape *s) {
shapes.append(s);
return shapes.count()-1;
}

void Image::delShape(int n) {
shapes.remove(n);
}

void Image::clearShapes() {
shapes.clear();
}

Shape *Image::shapeAt(int n) {
return shapes.at(n);
}

void Image::setOrigin(int x,int y) { 
if (!data) return;
originx=x; 
originy=y; 
if (originx<0) originx=0;
if (originy<0) originy=0;
if (originx>data->xSize()*scale-width() && data->xSize()*scale-width()>0) 
	originx= data->xSize()*scale-width();
if (originy>data->ySize()*scale-height() && data->ySize()*scale-height()>0) 
	originy= data->ySize()*scale-height();
//printf("%d %d %d %d\n",originx,originy,width()-data->xSize(), height()-data->ySize());
needupd=1; 
update(); 
emit(moved(x,y));
}
