#ifndef IMAGEH
#define IMAGEH

#include <stdio.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qfont.h> 
#include <qfontmet.h> 
#include <qdict.h>
#include <qintdict.h>
#include <qlist.h>
#include "EMData.h"
#include "Euler.h"
#include "ImageMxCon.h"
#include <qfiledialog.h>

struct Shape {
enum { NONE, LINE, BOX, DOTBOX, CIRCLE, TEXT };
int type;
int x0,y0,x1,y1;
int user;	// misc use for flags, etc.
QColor color;
QString text;

Shape() { text=""; type=NONE; }
~Shape() { }

};

class Image:public QWidget {
    Q_OBJECT

QPainter *qp;
QPaintDevice *qpd;
QPixmap *qpm;

EMData *data;
EMData *dcopy;
Euler angle;

ImageMxCon *imc;

QString mtstr;

int mdx,mdy;
int needupd;
int mode,invert;
float cont,brite;

int originx,originy;
float scale;

QList<Shape> shapes;

public:

float lscale;		// DO NOT MODIFY EXTERNALLY

Shape select;

Image( QWidget *parent=0, const char *name=0 );
~Image();

void setData(EMData *d);
EMData *getData();

void setNullString(char *str);

int addShape(Shape *s);
void delShape(int n);
void clearShapes();
Shape *shapeAt(int n);
int nShapes();
void hilight(int i);

int pxtodx(int);
int pytody(int);
int dxtopx(int);
int dytopy(int);

virtual void paintEvent( QPaintEvent *qp );
virtual void mousePressEvent(QMouseEvent *);
virtual void mouseReleaseEvent(QMouseEvent *);
virtual void mouseDoubleClickEvent(QMouseEvent *e);
virtual void mouseMoveEvent(QMouseEvent *);
virtual void mouseEnter(int);
virtual void mouseLeave(int);
virtual void keyPressEvent(QKeyEvent *);
virtual void keyReleaseEvent(QKeyEvent *);
virtual void resizeEvent ( QResizeEvent * );
virtual void print(QPaintDevice *dev);

signals:
	void	resized(int neww, int newh);
	void	mousePress(QMouseEvent *);
	void	mouseMove(QMouseEvent *);
	void	mouseRelease(QMouseEvent *);
	void	moved(int xo,int yo);
	void	changed();

public slots:
	void	imageChanged();
	void	setMode(int val);
	void	setCont(float val);
	void	setBrite(float val);
	void	setBC(float b, float c);
	void	setScale(float sca);
	void	setOrigin(int x,int y);
	void	setInvert(int);
	void 	saveImage();
	void	notVisible();
	void	doPrint();
};


inline EMData *Image::getData() { return data; }
inline void Image::setMode(int val) { mode=val; needupd=1; update(); }
inline void Image::setCont(float val) { cont=val; needupd=1; update(); }
inline void Image::setBrite(float val) { brite=val; needupd=1; update(); }
inline void Image::setNullString(char *str) { mtstr=str; update(); }
inline void Image::setScale(float sca) { scale=sca; needupd=1; update(); }
inline int Image::nShapes() { return shapes.count(); }
#endif
