// c1startup.C	Phil Baldwin

//N c1startup
//F This program uses cross-common lines to generate a 3D model from a set of projections
//1
//P <input file>	Source file with class averages
//P <max>
//P [fixed=<int gt 0>]
//P [output=<file of numbers>]
//U c1startup good.img
//D This program is used to generate and initial 3D model for refinement
//E in cases where there is no symmetry, or the symmetry specific routines
//E don't work very well. Generally you just specify the input file
//E containing a variety of good class averages (usually 10-20). The
//E model will be written to threed.0a.mrc.

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define RADIUS 2.5   // This is the exclusion limit for peak
#define PEAKMAX 2   // This is the exclusion limit for peak

void com2angs(ANG *ang,char *argv,float radius, int peaks, int numbertouse) ;

void triangles(float gamma, float alpha, float beta, float *tiltxBC, float *tiltxCA, float *tiltxAB) ;	

struct compareme { float v ; int iq;};
int compare(compareme *c1, compareme *c2) { 
	return (c1->v > c2->v)?-1:((c1->v < c2->v)?1:0)  ; }

inline float pabs( float x){ return sqrt(x*x);}

//                     c1startup.C            last revision 6/1/98; 1/27/99
//   
//
//
main(int argc, char *argv[]) {
int i,j,k;
int index, ifix=-1, iwrite=-1;
compareme *totalqual;
int PEAKS = PEAKMAX, max; 
float pi=3.1415926, piover = pi/180. ;
float AB,BA,CA, AC, BC,CB ;
float tiltxAB, tiltxBC, tiltxCA, sum;
float sum1, sum2 , sum3;
List *imageList;
FILE *eulersplt;
EMData *d3;
ANG *ang;

appinit(argc,argv);


if (argc<3) { printf("c1startup\nUsage: c1startup <input file><max> \
[fixed=<int gt 0>][output=<file of numbers>]\n"); 
printf("max is the maximum to search through \n"); exit(0); }
printf(" RADIUS= %f		\n",RADIUS);

for (i=3; i<argc; i++) {
	if (strncmp("fixed=",argv[i],6)==0) {ifix=atoi(&argv[i][6]);}
	else if (strncmp("output=",argv[i],7)==0) {iwrite=1; eulersplt=fopen(argv[i],"w");}
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}


int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);


imageList=readImages(argv[1],0,-1);
max=atoi(argv[2]);
int N=imageList->count();
int imax=max;

if (N<max) max=N;

for (i=0; i<max; i++) ((EMData*)imageList->objectAt(i))->toCorner();


// **************  %% 1. Read in Peaks;   
//     
// *********  Assign the az, phi variables
// *********    ang[ij].a1


ang=(ANG *)calloc(max*max,sizeof(ANG));

totalqual = (compareme *)calloc(max*max*max,sizeof(compareme));

printf("reading in peaks \n");

// gets phi, az  info from the emdata uses RADIUS, PEAKS, &argv[1]
//     writes ang[ij], rewrites argv[1]
//  s
//       argv[1] = "reordered.img";                    
com2angs(ang,argv[1],RADIUS,PEAKS,max);

printf("reading in peaks 2 \n");fflush(stdout);

// **************** 2.  Renormalize major peaks by minor    **** //

for (i=0   ; i < max ; i++) {
	for (j=i+1 ; j < max ; j++) { 
		ang[i + j*max].l[0] *= 1./ (ang[i+j*max]).l[1] ;
		ang[j + i*max].l[0] *= 1./ (ang[j+i*max]).l[1] ;
//		printf(" %f %f \n",ang[i+j*max].l[0], ang[j+i*max].l[0]);
}}

// *************** 3. Find 3 with best contrast   ****** //
// printf("step 3\n");fflush(stdout);

if (ifix >0){ imageList -> swap(0,ifix-1);
			imax=1;}

for (i=0 ; i < imax ; i++) {
	for (j=i+1 ; j < max  ;j++) { 
		for (k=j+1 ; k < max ; k++) {
		totalqual[k+j*max + i *max*max].v = ang[j+i*max].l[0]
				+ ang[k+i*max].l[0] + ang[k+j*max].l[0];
		totalqual[k+j*max + i*max*max].iq = k+j*max + i*max*max;

		AB  =  piover*ang[j + i*max].a1[0];
		BA  =  piover*ang[i + j*max].a1[0]; 
		CA  =  piover*ang[i + k*max].a1[0]; 
		AC  =  piover*ang[k + i*max].a1[0];
		BC  =  piover*ang[k + j*max].a1[0];
		CB  =  piover*ang[j + k*max].a1[0];


		sum1 = CA - CB + 4*PI;  while (sum1>2*PI) sum1 += -2.*PI;
		sum2 = AB - AC + 4*PI;  while (sum2>2*PI) sum2 += -2.*PI;
		sum3 = BC - BA + 4*PI;  while (sum3>2*PI) sum3 += -2.*PI;
		sum = sum1+sum2+sum3;   while (sum >2*PI) sum  += -2.*PI;
		if ( pabs(cos(sum1) - cos(sum2)*cos(sum3))   >pabs( sin(sum2)*sin(sum3) ) ||
			 pabs(cos(sum2) - cos(sum3)*cos(sum1))   >pabs( sin(sum3)*sin(sum1) ) ||
			 pabs(cos(sum3) - cos(sum1)*cos(sum2))   >pabs( sin(sum1)*sin(sum2) ) )
		totalqual[k+j*max + i *max*max].v = -1000;

}}}
qsort(totalqual,max*max*max,sizeof(compareme), (int (*)(const void *, const void *)) compare);


for (index=0 ; index < max*max ; index++) { 
		i = totalqual[index].iq / (max*max) ; // this is the value of the angle of A
		j= (totalqual[index].iq -i*max*max)/max ;
		k=  totalqual[index].iq -i*max*max-j*max ;
		printf("in list=%d  in order=%d i=%d j=%d k=%d average quality=%f  \n",
				index+1, totalqual[index].iq ,i+1,j+1,k+1, totalqual[index].v/3.); 
		AB  =  piover*ang[j + i*max].a1[0];
		BA  =  piover*ang[i + j*max].a1[0]; 
		CA  =  piover*ang[i + k*max].a1[0]; 
		AC  =  piover*ang[k + i*max].a1[0];
		BC  =  piover*ang[k + j*max].a1[0];
		CB  =  piover*ang[j + k*max].a1[0];

		sum1 = CA - CB + 4*PI;  while (sum1>2*PI) sum1 += -2.*PI;
		sum2 = AB - AC + 4*PI;  while (sum2>2*PI) sum2 += -2.*PI;
		sum3 = BC - BA + 4*PI;  while (sum3>2*PI) sum3 += -2.*PI;
		sum = sum1+sum2+sum3; // while (sum >2*PI) sum  += -2.*PI;

		printf("sum1=%f  sum2=%f  sum3=%f  sum=%f   \n",
						sum1/piover,sum2/piover, sum3/piover, sum/piover); 

		if ( pabs(cos(sum1) - cos(sum2)*cos(sum3))   >pabs( sin(sum2)*sin(sum3) ) )
				printf(" Busted!!!! on 1,  \t"); fflush(stdout);
		if ( pabs(cos(sum2) - cos(sum3)*cos(sum1))   >pabs( sin(sum3)*sin(sum1) ) ) 
				printf(" Busted!!!! on 2,  \t"); fflush(stdout);
		if ( pabs(cos(sum3) - cos(sum1)*cos(sum2))   >pabs( sin(sum1)*sin(sum2) ) )
				printf(" Busted!!!! on 3,  \t"); fflush(stdout);
		triangles(CA-CB,AB-AC,BC-BA,&tiltxAB,&tiltxBC,&tiltxCA);
        sum = tiltxAB*tiltxAB + tiltxBC*tiltxBC + tiltxCA*tiltxCA;
		if (sum < 3*360*360) index +=max*max;
		if (index==max-1){ printf("choose larger value for max and try again \n"); exit(0);}
		printf("\t\t%f %f %f %f\n",tiltxAB/piover,tiltxBC/piover,tiltxCA/piover,sum);
//         3a Check to make sure they don't give nonsense    
//
}

// ****** 4. Reorder                     **                //
imageList -> swap(0,i);
imageList -> swap(1,j);
imageList -> swap(2,k);

//                 Also write it out now!!!!!!!!!!!
	for (index=0; index < max ; index++) {
	 	d3=(EMData*)imageList->objectAt(index);
		if (index==0) d3->setRAlign(0,0,0);
		if (index==1) d3->setRAlign(tiltxAB,AB,BA);
		if (index==2) d3->setRAlign(-tiltxCA, AC,CA);
		d3->setNImg(1);
		d3->toCorner();
		d3->writeIMAGIC(argv[1],index);	
		}

printf("\n%d\t%f\t%f\t%f \n",1,0.,0.,0.);
printf("%d\t%f\t%f\t%f \n",2, tiltxAB/piover,AB/piover,BA/piover);
printf("%d\t%f\t%f\t%f \n",3,-tiltxCA/piover,AC/piover,CA/piover);

if (iwrite==1) {
	fprintf(eulersplt,"\n%d\t%f\t%f\t%f \n",1,0.,0.,0.);
	fprintf(eulersplt,"%d\t%f\t%f\t%f \n",2, tiltxAB/piover,AB/piover,BA/piover);
	fprintf(eulersplt,"%d\t%f\t%f\t%f \n",3,-tiltxCA/piover,AC/piover,CA/piover);
}
	
	LOG(Ref,argv[1],LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);


}
