// classalign2 ?/98  Steve Ludtke

//N classalign2
//F Aligns a set of images iteratively to generate a class-average.
//1
//P <input file>	Input file, usually a cls*img file
//P <iterations>	Number of iterative loops
//P [ref1]	Indicates that the first image in the file is a reference
//P [finalref]	Once the average has been generated, a final alignment to the reference is performed
//P [saveali]	Saves the individual aligned images in a separate file
//P [savecen]	Save the translationally (but not rotationally) aligned images
//P [savesteps]	Save the class average after each iterative step to monitor convergence
//P [badfile]	Save all excluded images in 'bad' files.
//P [stat]	Writes individual image statistics to 'class.stat'
//P [saverefs]	Write the reference image as well as the class average to the output file
//P [mask=<rad>]	Circular mask radius for the class average
//P [keep=<sigmamult>]	The threshold value for keeping images. Standard deviation multiplier	
//P [locfromfile]	Determine the output file image position from the input filename.
//P [median]	When CTF correction is NOT used, this produces better statistics.
//P [ctfc=<res>]	Enables CTF correction, thru resolution <res> in angstroms.
//P [imask=<rad>]	Inside mask radius.
//P [sigfilt]	Filters the class averages so only intense values are used in alignment
//P [even|odd]	Only use the even or odd images in the input file
//P [sigmaimg]	Generate a standard deviation image
//U classalign2 cls000.hed 8 ref1 finalref saverefs mask=32 keep=.8 locfromfile ctfc=15
//D This is one of the most important programs in EMAN. It is responsible for taking a set
//E of particles within a single class, that is, they are supposed to be in the same 
//E orientation, and performing a mutual alignement to generate a class-average. This 
//E program is largely responsible for the model independance of EMAN, ie - why EMAN can 
//E converge in so few iterations. This program is also where CTF amplitude correction occurs. 

//D There are quite a few options for this program. It is typically called from classalignall,
//E which has almost identical options. The few options that are present here, but not in 
//E classalignall, are intended for occasional use when checking robustness of the proceedures. 

//D There are 3 manditory options which are specified in all cases. First, is the
//E <input file>. This file contains the images which are to be used to generate a class
//E average. Optionally, the first image in this file may be a reference image with
//E assigned euler angles. The reference will be used only during the first iteration
//E of the alignment process, so the final class average is nearly independent of this
//E reference. It also provides a final rotational orientation for the class average. 

//D The number of iterations (with no prefixed text) and the
//E 'keep=<sigmamult>' values must also be specified. This program functions
//E by: first, aligning all of the images to a common reference, then
//E averaging the aligned images together to generate a new reference. This
//E reference is then used for another round of alignment, etc. The number
//E of iterations specifies how many times this process should repeat.
//E Each time, the images should generate a class average more consitent
//E with the data. In addition, after each loop, each aligned image is
//E compared with the class average from the previous iteration. If the
//E quality of the match is worse than a certain threshold value, the image
//E is discarded. Each iteration this threshold is made more stringent
//E until <sigmamult> is reached in the final iteration. The cutoff value
//E is determined by calculating the standard deviation of all of the match
//E qualities and multiplying by <sigmamult>. The smaller <sigmamult> is,
//E the more consistent the images in the final class average will be, but
//E as more images are excluded, the class average will also become more
//E noisy. Similarly, increasing the number of iterations will increase the
//E consistency of the final class average, but will also typically cause
//E more data to be discarded. There are no 'correct' values to use. There
//E is always a tradeoff between data homogeneity and noise level.  Typical
//E starting values are 8 iterations with <sigmamult>=1.2. <sigmamult> is
//E usually >= 1.0, but may be lower when data homogenity is very poor. More
//E iterations can always be used, but 8 is usually sufficient to converge
//E quite well.

//D When the program runs, it displays some useful information on the
//E screen. First you will see 1 or more rows of '.'s appear. Each '.'
//E represents an image being aligned. If a '*' appears instead of a '.',
//E that means that particular image was below the threshold and discarded.
//E Note that '*' images are only excluded from the average during a
//E particular cycle. They may be included again in later cycles if the
//E class average 'wanders' in their direction.

//D If your input file contains a reference image (which the other images
//E are expected to look at least vaguely similar to), then 'ref1' should be
//E specified. Usually 'finalref' is also specified. Normally the initial
//E reference is used only during the first iteration, however, during
//E several iterations the orientation of the class average in the plane
//E can drift. 'finalref' insures that the final class average is oriented
//E as closely as possible to the orientation of the initial reference. It
//E does not change the appearance of the class average, only its orientation.

//D 'saverefs' is also usually specified with 'ref1'. This causes the
//E reference images to be saved along with the class averages in the output
//E file.  This is used to compare each reference with the corresponding class
//E average. When the refinement cycle begins to converge, the reference
//E images should be nearly identical to the class averages, although the
//E class averages will always be noisier.

//D 'saveali', 'savesteps', 'badfile' and 'stat' are all used primarily when
//E something seems to be going wrong. They cause various output files to be
//E generated, which can then be used to see exactly what's happening. They
//E are rarely used in a normal refinement cycle.

//D The default output image from classalign2 is a class average. This means,
//E each pixel is equal to the average of that pixel's value in each of the
//E aligned images. However, very often the median value will have better
//E properties than the mean value (less affected by outliers). Median
//E images can be generated instead of averages by specifying the 'median'
//E option. Note that this option is incompatible with CTF correction (see
//E 'ctfc' below).

//D Usually, classalign2 will simply append each output class average to the
//E end of 'classes.hed'. However, if the program is being run in parallel,
//E it is usually desirable to preserve the ordering of the images in the
//E output file. 'locfromfile' will cause the output images to be put in
//E specific 'slots' in the output file, overwriting any image already in that
//E location in the file. This is used by default when classalignall is run.

//D 'sigfilt' is used when a refinement is failing to produce consistent
//E results in the early rounds of refinement. This applies a non-linear
//E noise reducing filter to the reference image to try to get better
//E initial alignment. It is generally only used when necessary, and then,
//E only for early refinement rounds. It should not be used when the final
//E few refinement iterations are performed.

//D Last, but not least, 'ctfc' is provided to enable CTF correction when
//E generating class averages. For this to work properly all of the images
//E in the input file must be pretreated by the ctfit program, so the
//E phases are flipped and the CTF parameters are stored in the individual
//E image headers. See the documentation for that program 
//E and <a href=../ctfc/ctfc.html> the CTF correction primer</a>
//E for a discussion of how CTF correction
//E is performed. When 'ctfc' is specified, you should also specify
//E 'filt=<angstr>'. If the final CTF corrected image is not low pass
//E filtered to some resolution, high frequency noise will dominate the
//E output image. <angstr> should be set to the final resolution goal of
//E your reconstruction in most cases. ctfit will allow you to determine an
//E appropriate value for this. If you do not specify <angstr>, a default
//E value of 22 angstroms will be used.

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i,j,k,n,ny,iter=2,r1=0,savesteps=0,badf=0,stat=0,saver=0,half=0,sigfilt=0,savecen=0;
int mask=-1,imask=-1,locffile=-1,median=0,saveali=0,dot=1,ctfc=0,precen=0,quiet=0,maxshift;
List *data,*dataf,*d2a,*d2ma;
EMData *d,*df,*d2,*avg,*oavg=NULL,*first=NULL,*firstf=NULL,*mask1,*tmp,*sigim=NULL;
EMData *reforig=NULL;
char s[1024];
float f,m,mean,sigma,mean2,sigma2,cut,keep=1.2,norm,norm2,filt=0;
float eul[3];
float *ctf;
FILE *out;

appinit(argc,argv);

if (argc<2) { printf("Align images in a class\nUsage: classalign <input file> \
[ref1] [finalref] [saveali] [savecen] [savesteps] [badfile] [stat] [saverefs] \
[iterations] [mask=<rad>] [keep=<sigmamult>] [filt=<angstr>] [locfromfile] [median] [dot] \
[precen] [ctfc] [imask=<rad>] [sigfilt] [even|odd] [sigmaimg]\n"); exit(0); }

for (i=2; i<argc; i++) {
	if (strncmp("ref1",argv[i],4)==0) r1=1;
	else if (strncmp("finalref",argv[i],8)==0) r1=2;
	else if (strncmp("savesteps",argv[i],9)==0) savesteps=1;
	else if (strncmp("badfile",argv[i],7)==0) badf=1;
	else if (strncmp("sigfilt",argv[i],7)==0) sigfilt=1;
	else if (strncmp("sigmaimg",argv[i],8)==0) sigim=new EMData;
	else if (strncmp("saverefs",argv[i],8)==0) saver=1;
	else if (strncmp("saveali",argv[i],7)==0) saveali=1;
	else if (strncmp("savecen",argv[i],7)==0) savecen=1;
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("stat",argv[i],4)==0) stat=1;
	else if (strncmp("ctfc=",argv[i],5)==0) { ctfc=1; filt=atof(&argv[i][5]); }
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("keep=",argv[i],5)==0) keep=atof(&argv[i][5]);
	else if (strncmp("locfromfile",argv[i],11)==0) locffile=1;
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("dot",argv[i],3)==0) dot=1;
	else if (strncmp("even",argv[i],4)==0) half=2;
	else if (strncmp("odd",argv[i],3)==0) half=1;
	else if (strncmp("quiet",argv[i],5)==0) quiet=1;
	else if (isdigit(argv[i][0])) iter=atoi(argv[i])-2;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}
if (r1==1) iter++;
if (iter<1) exit(1);
if (ctfc) median=0;

if (imask>0 && !precen) { printf("The imask option must be used in conjunction with precen\n"); exit(0); }


printf("Classalign: %s\n",argv[1]);

n=fileCount(argv[1]);
//if (half) n/=2;
if ((n<5 && !r1) || (n<2 && r1)) { printf("Not enough elements\n"); exit(0); }
if (n<10) iter=1;	// only do 2 passes if <10 data sets to average
if (n<3) iter=0;	// only 1 pass if a single image
data=readImages(argv[1],0,n);		// read all input images
if (data==NULL) error("Cannot read input",ERR_EXIT);
printf("Read %d images.\n",data->count());

if (half) {
	n=fileCount(argv[1]);
	if (r1) {
		if (half==1) {
			for (i=n-1; i>=1; i-=2) data->removeObjectAt(i);
		}
		else {
			for (i=n-2; i>=1; i-=2) data->removeObjectAt(i);
		}
	}
	else {
		if (half==1) {
			for (i=n-1; i>=0; i-=2) data->removeObjectAt(i);
		}
		else {
			for (i=n-2; i>=0; i-=2) data->removeObjectAt(i);
		}	
	}
	n=data->count();
	printf("Processing %d images\n",n);
}

int Ref=0;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

mask1=((EMData *)data->objectAt(0))->copyHead();
mask1->one();
mask1->applyMask(mask,4);
//mask1->writeMRC("mask.img");

d2a=new List;
d2ma=new List;

dataf=new List;
for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	d->normalize();
//if (i==0) d->writeIMAGIC("abc.hed",-1);
//	d->applyMask(-1,0);
	d2=d->copy();
	d2->vFlip();
	d2->setParent(NULL);
	dataf->addObject(d2);
//	d=d->copy();
}

d=(EMData *)data->objectAt(0);
ny=d->ySize();
if (mask<=0) maxshift=ny/4; else maxshift=ny/2-mask;

if (sigim) { sigim->setSize(ny,ny,1); sigim->zero(); }

// If we're starting with a reference, we don't need the first pass
if (r1) {
	avg=d=(EMData *)data->objectAt(0);
	first=avg->copy();
	first->setParent(NULL);
	data->removeObjectAt(0);
	dataf->removeObjectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
		norm2=PI/(4.0*d->xSize()*d->ySize());
	}
	else{
		norm=mask*mask*PI/1000.0;
		norm2=PI/(d->xSize()*d->ySize());
	}
	eul[0]=d->alt();  
	eul[1]=d->az();  
	eul[2]=d->phi();
	printf("Eulers %f,%f,%f\n",eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	n--;
}

// simple shell sort, fast enough
d=(EMData *)data->objectAt(0);
if (d->hasCTF() && ctfc) {
	for (i=0; i<n-1; i++) {
		d=(EMData *)data->objectAt(i);
		for (j=i+1; j<n; j++) {
			d2=(EMData *)data->objectAt(j);
			if (memcmp(d->getCTF(),d2->getCTF(),11*sizeof(float))<0) {
				data->swap(i,j);
				dataf->swap(i,j);
				d=(EMData *)data->objectAt(i);
			}
		}
	}
}

//if (r1!=1)
if (!r1) {
	// pass 1, align to the first data set
	d=(EMData *)data->objectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
		norm2=PI/(4.0*d->xSize()*d->ySize());
	}
	else{
		norm=mask*mask*PI/1000.0;
		norm2=PI/(d->xSize()*d->ySize());
	}
	avg=d->copy();
	for (i=1; i<n; i++) {
		d=(EMData *)data->objectAt(i);
		df=(EMData *)dataf->objectAt(i);
		if (precen) d2=d->RFAlign(avg,df,imask);
		else d2=d->RTFAlign(avg,df,dot,maxshift);
//		d2->mult(mask1);
		d2->applyMask(mask,6);
		avg->add(d2);
		delete d2;
		putchar('o'); fflush(stdout);
	}
	putchar('\n');
	if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	if (!precen) {
		avg->cmCenter();
		if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	}
}

if (n==1) iter=0;
cut=0;
for (j=0; j<iter; j++) {
// pass 2-?, align to the first average
	mean=sigma=m=0;
	if (reforig && reforig!=oavg) delete reforig;
	if (oavg) { delete oavg; oavg=NULL; }
	reforig=avg;
	reforig->normalize();
//	oavg->writeIMAGIC("aaa.hed",-1);
	for (i=0; i<n; i++) {
		d=(EMData *)data->objectAt(i);		// data to align
		df=(EMData *)dataf->objectAt(i);	// flipped version of data

		// if there's a CTF, modify the reference by the same CTF
		if(d->hasCTF() && ctfc) {
			// if several images in a row have the same ctf, don't recalculate
			if (!oavg || memcmp(d->getCTF(),oavg->getCTF(),11*sizeof(float))) {
				if (oavg) delete oavg;
				ctf=d->ctfCurve(2);
				d2=reforig->doFFT()->copy();
				d2->applyRadFn(d->ySize()/2,0,1,ctf,0);
				oavg=d2->doIFT();
				d2->gimmeFFT();
				oavg->normalize();
				oavg->setParent(NULL);
				oavg->setCTF(d->getCTF());
				delete d2;
				free(ctf);
//oavg->writeIMAGIC("aaa.hed",-1);
			}
		}
		else oavg=reforig;

		if (sigfilt) oavg->realFilter(1,oavg->Sigma());
		
		if (precen) d2=d->RFAlign(oavg,df,imask);
		else {
			d2=d->RTFAlign(oavg,df,dot,maxshift);
			if (j>iter-3) d2->refineAlign(oavg,1);
		}
		if (d->hasCTF() && ctfc) d2->setCTF(d->getCTF());
//		d2->mult(mask1);
		d2->applyMask(mask,6);
		if (dot) f=oavg->dot(d2);
		else f=(1.0-oavg->lcmp(d2))/norm2;
//printf("%f %f %f\n",oavg->lcmp(d2),cut,f);
//d2->writeIMAGIC("aaa.hed",-1);
//printf(" %f,%f\n",f,cut);

		d2ma->addObject(d2);
		mean+=f;
		sigma+=f*f;

		putchar('?'); fflush(stdout);


/*		if ((f>cut || keep>5 || (i>n-3 && m<2)) && finite(f)) {
			d2a->addObject(d2);
			putchar('.'); fflush(stdout);
			mean+=f;
			sigma+=f*f;
			m++;
		} else {
			putchar('*'); fflush(stdout);	
			delete d2;
		}*/
	}
	putchar('\r');

// calculate statistics
	mean/=(float)n;
	sigma=sqrt(sigma/(float)n-mean*mean);
	if (!finite(mean)) mean=1.0;
	if (!finite(sigma)) sigma=mean/4.0;
	if (sigma==0) exit(0);		// down to 1 image, just quit
	cut=mean-sigma*keep*(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0));

	mean2=sigma2=m=0;
// decide which images to keep
	for (i=0; i<n; i++) {
		d2=(EMData *)d2ma->objectAt(i);
		if (dot) f=oavg->dot(d2);
		else f=(1.0-oavg->lcmp(d2))/norm2;
		
		if ((f>cut || keep>5 || (i>n-3 && m<2)) && finite(f)) {
			putchar('.'); fflush(stdout);
			d2a->addObject(d2);			
			mean2+=f;
			sigma2+=f*f;
			m++;
		} else {
			putchar('*'); fflush(stdout);		
			delete d2;
		}
	}
	putchar('\n');
	d2ma->empty();

	mean2/=(float)m;
	sigma2=sqrt(sigma2/(float)m-mean2*mean2);
	if (!finite(mean2)) mean2=1.0;
	if (!finite(sigma2)) sigma2=0;

	if (savesteps) oavg->writeIMAGIC("steps.hed",-1);

	avg=new EMData;
	if (median) avg->makeMedian(d2a); 
	else if (ctfc) avg->makeAverageCTFC(d2a,filt);
	else avg->makeAverage(d2a);
	avg->normalize();


//	avg->mult(mask1);
	avg->applyMask(mask,6);
	
	for (i=0; i<d2a->count(); i++) { d2=(EMData *)d2a->objectAt(i); delete d2; }
	d2a->empty();
	

//	printf("Mean=%f\tSigma=%f\tCut=%f\twt=%f\n",mean/norm,sigma/norm,cut/norm,
//		(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0)));
//	printf("Mean=%f\tSigma=%f\n",mean2/norm,sigma2/norm);

	printf("Mean=%f\tSigma=%f\tCut=%f\twt=%f\n",mean2/norm,sigma2/norm,cut/norm,
		(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0)));

	// if we have an original reference, align the early averages to it
	// just to get the orientation correct
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");

	if (j<iter-2 && r1) {
		if (oavg) { delete oavg; reforig=NULL; }
		df=avg->copy();
		df->setParent(NULL);
		df->vFlip();
		if (precen) oavg=d->RFAlign(first,df,imask);
		else oavg=avg->RTFAlign(first,df,dot,maxshift);
//		printf("firstalign %p %5.2f %5.2f %5.2f\n",oavg,oavg->Dx(),oavg->Dy(),oavg->Daz()*180.0/PI);
		if (oavg) { 
			delete avg;
			delete df;
			oavg->setParent(NULL);
			avg=oavg;
			oavg=NULL;
			avg->setRAlign(0,0,0);
			avg->setTAlign(0,0,0);
			avg->update();
		}
	}
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");
	if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	if (!r1 && !precen) {
		avg->cmCenter();		// if we have an initial reference
								// don't spoil the centering
		if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	}
}

if (savesteps && !quiet) LOG(Ref,"steps.hed",LOG_OUTFILE,NULL);


// last pass, align to the final previous average
mean=sigma=m=0;
//oavg=avg;
//oavg->normalize();
if (reforig && reforig!=oavg) delete reforig;
if (oavg) { delete oavg; oavg=NULL; }
reforig=avg;
reforig->normalize();

if (stat) out=fopen("class.stat","a");

for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	df=(EMData *)dataf->objectAt(i);

	// if there's a CTF, modify the reference by the same CTF
	if(d->hasCTF() && ctfc) {
		// if several images in a row have the same ctf, don't recalculate
		if (!oavg || memcmp(d->getCTF(),oavg->getCTF(),11*sizeof(float))) {
			if (oavg) delete oavg;
			ctf=d->ctfCurve(2);
			d2=reforig->doFFT()->copy();
			d2->applyRadFn(d->ySize()/2,0,1,ctf,0);
			oavg=d2->doIFT();
			d2->gimmeFFT();
			oavg->normalize();
			oavg->setParent(NULL);
			oavg->setCTF(d->getCTF());
			delete d2;
			free(ctf);
		}
	}
	else oavg=reforig;
//oavg->writeIMAGIC("aaa.hed",-1);

	if (sigfilt) oavg->realFilter(1,oavg->Sigma());

	if (precen) d2=d->RFAlign(oavg,df,imask);
	else {
		d2=d->RTFAlign(oavg,df,dot,maxshift);
		d2->refineAlign(oavg,1);
	}
	if (d->hasCTF() && ctfc) d2->setCTF(d->getCTF());
//	d2->mult(mask1);
	d2->applyMask(mask,6);
//d2->writeIMAGIC("aaa.hed",-1);
	if (dot) f=oavg->dot(d2);
	else f=(1.0-oavg->lcmp(d2))/norm2;

	if (sigim) {
		tmp=d2->copy(0);
		tmp->normalizeTo(oavg);
		tmp->subtract(oavg);
		tmp->mult(tmp);
		sigim->add(tmp);
		delete tmp;
	}

	d2->setNImg(1);
	d2->setRAlign(eul[0],eul[1],eul[2]);
	d2ma->addObject(d2);
	mean+=f;
	sigma+=f*f;
	putchar('-'); fflush(stdout);

/*	if ((f>cut || keep>5) &&finite(f)) {
		d2->setNImg(1);
		d2->setRAlign(eul[0],eul[1],eul[2]);
		d2a->addObject(d2);
		mean+=f;
		sigma+=f*f;
		m++;
	}
	else {
		sprintf(s,"b%s",argv[1]);
		if (badf) d->writeIMAGIC(s,-1);
		delete d2;
	} 

	if (stat) fprintf(out,"%f\n",f/norm);
	printf("%6.1f %c  ",f/norm,f>cut?' ':'*');
	if (i%8==0) putchar('\n');
	fflush(stdout); */
}
putchar('\n');

if (sigim) {
	sigim->multConst(1.0/(float)n);
	sigim->realFilter(10,0,0,0);	// sqrt
	sigim->setRAlign(eul[0],eul[1],eul[2]);
	sigim->setNImg((int)n);
	sigim->writeIMAGIC("classsig.hed",-1);
}

mean/=(float)n;
sigma=sqrt(sigma/(float)n-mean*mean);
if (!finite(mean)) mean=1.0;
if (!finite(sigma)) sigma=mean/4.0;
if (sigma==0) exit(0);		// down to 1 image, just quit
cut=mean-sigma*keep*(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0));

mean2=sigma2=m=0;

// decide which images to keep
for (i=0; i<n; i++) {
	d2=(EMData *)d2ma->objectAt(i);
	if (dot) f=oavg->dot(d2);
	else f=(1.0-oavg->lcmp(d2))/norm2;

	if (stat) fprintf(out,"%f\n",f/norm);
	printf("%6.1f %c  ",f/norm,f>cut?' ':'*');
	if (i%8==0) putchar('\n');
	fflush(stdout);
	
	if ((f>cut || keep>5) &&finite(f)) {
		d2a->addObject(d2);			
		mean2+=f;
		sigma2+=f*f;
		m++;
	} else {
		sprintf(s,"b%s",argv[1]);
		if (badf) d2->writeIMAGIC(s,-1);
		delete d2;
	}
}
putchar('\n');
d2ma->empty();

mean2/=(float)m;
sigma2=sqrt(sigma2/(float)m-mean2*mean2);
if (!finite(mean2)) mean2=1.0;
if (!finite(sigma2)) sigma2=0;


//if (badf && !quiet) LOG(Ref,"nogood.hed",LOG_OUTFILE,NULL);

if (d2a->count()==0) { printf("ERROR: No images left!\n"); exit(0); }
avg=new EMData;
if (median) avg->makeMedian(d2a); 
else if (ctfc) avg->makeAverageCTFC(d2a,filt);
else avg->makeAverage(d2a);

sprintf(s,"a%s",argv[1]);
if (saveali && !quiet) LOG(Ref,s,LOG_OUTFILE,NULL);
if (savecen && !quiet) LOG(Ref,"start.cen.hed",LOG_OUTFILE,NULL);
for (i=0; i<d2a->count(); i++) { 
	d2=(EMData *)d2a->objectAt(i);
	if (saveali) {
		d2->writeIMAGIC(s,-1);
	}
	if (savecen) {
//		printf("%4.1f\t%4.1f\n",d2->Dx(),d2->Dy());
		d2->setRAlign(0,0,0);
		d2->setTAlign(floor(d2->Dx()+.5),floor(d2->Dy()+.5),0);
		d2->rotateAndTranslate();
		d2->writeIMAGIC("start.cen.hed",-1);
	}
	delete d2; 
}
d2a->empty();


//if (!r1) avg->cmCenter();
if (stat) fclose(out);

//mean/=(float)n;
//sigma=sqrt(sigma/(float)n-mean*mean);
printf("Mean=%f\tSigma=%f\n%0.0f/%d good images\n",mean/norm,sigma/norm,m,data->count());
printf("Mean=%f\tSigma=%f\n",mean2/norm,sigma2/norm);

if (locffile!=-1) {
	locffile=atoi(argv[1]+strlen(argv[1])-8);
	if (saver) locffile*=2;
}


if (r1==2) {
	avg->rotAlignTI(first,dot);
	f=avg->alt();
	f=f-PI*floor(f/PI+.5);
	printf("Final realign %4.2f  (%4.2f %4.2f %4.2f)\n",f*180.0/PI,eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	eul[2]-=f;
}

if (saver) {
	first->setNImg(-1);
	first->normalize();
	first->writeIMAGIC("classes.hed",locffile);
	if (locffile>=0) locffile++;
}

if (badf && r1) {
	sprintf(s,"b%s",argv[1]);
	first->writeIMAGIC(s,0);
}

//avg->mult(mask1);
avg->applyMask(mask,6);
avg->normalize();
avg->setRAlign(eul[0],eul[1],eul[2]);
avg->setNImg((int)m);
avg->writeIMAGIC("classes.hed",locffile);
if (!quiet) LOG(Ref,"classes.hed",LOG_OUTFILE,NULL);
if (savesteps) avg->writeIMAGIC("steps.hed",-1);
if (!quiet) LOG(Ref,NULL,LOG_END,NULL);
}


