// classes.C - Steve Ludtke  2/13/98
// This impliments k-mean classification, an iterative method
// which finds good spherical clusters for an n dimensional
// distribution

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define MINMEM	8		// minimum members in a class to keep it
#define WRITEAV			// this will write out all of the images in each class

struct DOT { float d; unsigned short i1,i2; };

inline int sgn(float x) { if (x==0) return 0; return x<0?-1:1; }

int dotcomp(const void *a, const void *b) { return sgn(*((float *)b)- *((float *)a)); }

main(int argc, char *argv[]) {
float **f,**cl;
float min,max,r,mean,sig,norm;
char s[121],fsp[121],*wtfile=NULL;
int n,n2,nc,bad;
int i,j,k,ii,c,*set,ff=0;
FILE *in,*out;
EMData *dat;

appinit(argc,argv);

if (argc<3) { printf("classes.c\nUsage: classes <footprints> <# classes> [list] [weight=<file>]\n"); exit(0); }

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"list",4)==0) ff=1;
	else if (strncmp(argv[i],"weight=",7)==0) wtfile=&argv[i][7];
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

in=fopen(argv[1],"r");
if (!in) { printf("Cannot open footprints\n"); exit(0); }
fread(s,120,1,in);
fread(&n,4,1,in);
fread(&n2,4,1,in);
f=(float **)malloc(n2*sizeof(float *));

for (i=0; i<n2; i++) {
	f[i]=(float *)malloc(n*sizeof(float));
	fread(f[i],n,sizeof(float),in);
}
fclose(in);

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);
LOG(Ref,"cls000.hed",LOG_OUTFILE,NULL);

if (wtfile) {
	in=fopen(wtfile,"r");

	for (i=0; i<n2; i++) {
		
		mean=sig=0;
		
		for (j=0; j<n; j++) {
			mean+=f[i][j];
			sig+=f[i][j]*f[i][j];
		}
		
		mean/=(float)n;
		sig=sig/(float)n-mean*mean;

		fscanf(in," %d %f",&j,&r);
		if (j!=i) printf("Warning, weights out of order %d!=%d!\n",i,j);
		norm=r/sig;

		for (k=0; k<n; k++) f[i][k]*=r;
	}
	
	fclose(in);
}

nc=atoi(argv[2]);
cl=(float **)malloc(n2*sizeof(float *));


// memory for class data points
for (i=0; i<n2; i++) {
	cl[i]=(float *)malloc(nc*sizeof(float));
}

// assign each class to a random point in the data set
// since the sets are usually randomly distributed, we just use the
// first data sets
for (i=0; i<nc; i++) {
	for (j=0; j<n2; j++) cl[j][i]=f[j][i];
}

// this holds the info on which class each image is in
set=(int *)malloc(n*sizeof(int));
memset(set,0,n*sizeof(int));

while (1) {
	// classify each image, put each in the class it's closest to
	bad=0;
	for (i=0; i<n; i++) {
		min=1.0e20;
		ii=set[i];
		for (j=0; j<nc; j++) {
			r=0;
			for (k=0; k<n2; k++) r+=SQR(f[k][i]-cl[k][j]);
			if (r<min) { set[i]=j; min=r; }
		}
		if (ii!=set[i]) bad++;
	}
	
	if (bad==0) break;		// any point that moves between classes means
							// we need to keep iterating
							
	printf("%d changed\n",bad);

	// now move the center of each class to the mean position of it's members
	for (i=0; i<nc; i++) {
		for (j=0; j<n2; j++) cl[j][i]=0;
	
		c=0;
		for (j=0; j<n; j++) {
			if (set[j]!=i) continue;
			for (k=0; k<n2; k++) cl[k][i]+=f[k][j];
			c++;
		}

		// if we get a class with no members, we randomly move it
		// this will make stabilization take longer, but should
		// give a better distribution
		if (!c) { 
			printf("class %d EMPTY!!!!\n",i); 
			k=random()%n;
			for (j=0; j<n2; j++) cl[j][i]=f[j][k];
			continue; 
		}
		for (j=0; j<n2; j++) cl[j][i]/=c;
	
		printf("class %d has %d members\n",i,c);
	}
}


// Ok, the classes are stable, write the individual class files

if (!ff) {
	system("/bin/rm cls???.hed cls???.img");

	dat=new EMData;
	for (i=0; i<n; i++) {
		if (i%10==0) { printf("out -> %d     \r",i); fflush(stdout); }
		dat->readImage(s,i);
		sprintf(fsp,"cls%04d.img",set[i]);
		dat->writeIMAGIC(fsp,-1);
	}
}

// write the classes list file
if (ff) {
	out=fopen("classes.dat","w");
	for (i=0; i<nc; i++) {
		fprintf(out,"%d\t",i);
		for (j=0; j<n; j++) {
			if (set[j]==i) fprintf(out,"%01d ",j);
		}
		fprintf(out,"\n");
	}
	fclose(out);
}
LOG(Ref,NULL,LOG_END,NULL);

}
