// classesbymra.C ?/98  Steve Ludtke

//N classesbymra
//F This program classifies a set of particles based on a set of references (usually projections).
//1
//P <particles>	Raw particle filespec	
//P <refrences>	Reference image filespec
//P [classes=<thr>]	Obsolete
//P [split]	Always use this
//P [frac=<num>/<denom>]	Operate only on a fraction of the raw images
//P [verbose]	More verbose output
//P [mask=<rad>]	Mask the classified particles, also reduces noise for better classification
//P [imask=<rad>]	Inside mask, classified particles are masked for classification, but not in the cls files.
//P [precen]	Particles are assumed to be accurately precentered 
//P [sigfilt]	Apply a real space filter so only high amplitude data is used for classification
//P [tree1=<treefile>]	First stage high speed classification, see <a href=projtree.html>projtree</a>
//P [tree2=<treefile>]	Second stage high speed classification
//P [ctfc]	Projections are CTF corrected
//P [sep=<nspl>]	Puts each raw particle in 'nspl' different classes.
//U  classesbymra start.hed proj.hed split mask=28 ctfc sep=2 tree1=proj.tree
//D This classifies a set of raw particles based on a set of reference projections.
//E The output is a set of cls*.img files, one file for each reference image. If the
//E projections were generated from a CTF corrected model, specify the 'ctfc' option
//E so the projections will be 'decorrected' before classification.
//D The 'tree' options implement 2 part classification for higher speed. Check <a href=projtree.html>projtree</a>
//E details on this.
//D The 'sep' option allows each particle to be placed into several different classes. If the
//E images are very noisy, or if the angular spacing is small, misclassification of some
//E particles will inevitably occur. The sep option makes it very likely that one of the
//E classes each particle ends up in will be the correct one. The classalignall procedure
//E can then be used to obtain sufficient consistency within the set of images actually
//E used in each class average.

#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

// a little class for maintaining an ordered list
class Olist {
public:
float *dot;
int *n;
EMData **ali;
int N;

Olist(int ntokeep) {
	N=ntokeep;
	dot=(float *)malloc(N*sizeof(float));
	n=(int *)malloc(N*sizeof(int));
	ali=(EMData **)malloc(N*sizeof(EMData *));
	for (int i=0; i<N; i++) {
		dot[i]=-1.0e20;
		n[i]=-1;
		ali[i]=NULL;
	}
}

~Olist() {
	free(dot);
	free(n);
	for (int i=0; i<N; i++) delete ali[i];
	free(ali);
}

void insert(float vdot, int vn, EMData *vali) {
	int i,j;
	
	for (i=0; i<N; i++) if (dot[i]<vdot) break;

	if (i==N) { delete vali; return; }

	if (ali[N-1]) delete ali[N-1];
	for (j=N-1; j>i; j--) {
		dot[j]=dot[j-1];
		n[j]=n[j-1];
		ali[j]=ali[j-1];
	}
	dot[i]=vdot;
	n[i]=vn;
	ali[i]=vali;
}

};

main(int argc, char *argv[]) {
int i,j,k,*rn,x,y,nr,ns,ntk=1,split=0,num=0,denom=1,dodot=1,ver=0,sigfilt=0;
int n0,n1,shrink=1,mask=-1,imask=0,precen=0,ctfc=0,maxshift,quiet=0;
List *data,*dataref,*datarefnc,*newref=NULL;
EMData *d,*d2,*df,*dr,*da,*msk;
char s[80];
float dot,classes=-999.0,norm,norm2;
float nullctf[11] = { 0,0,0,0,0,0,0,0,0,0,0 };
float *ctf;
char *tree1=NULL,*tree2=NULL;
FILE *out;
Olist *best;

appinit(argc,argv);

if (argc<3) { printf("classesbymra.c\nUsage: classesbymra <unclassed> <refrences> [classes=<thr>] [split] \
[frac=<num>/<denom>] [verbose] [dot] [mask=<rad>] [imask=<rad>] [precen] [sigfilt] [tree1=<treefile>] [tree2=<treefile>]\n"); exit(0); }

for (i=3; i<argc; i++) {        
        if (strncmp(argv[i],"classes=",8)==0) classes=atof(&argv[i][8]);
        else if (strncmp(argv[i],"split",5)==0) split=1;
        else if (strncmp(argv[i],"tree1=",6)==0) tree1=&argv[i][6];
        else if (strncmp(argv[i],"tree2=",6)==0) tree2=&argv[i][6];
        else if (strncmp(argv[i],"dot",3)==0) dodot=1;		// default now
 		else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
 		else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
		else if (strncmp(argv[i],"num=",4)==0) num=atoi(&argv[i][4]);
		else if (strncmp(argv[i],"denom=",6)==0) denom=atoi(&argv[i][6]);
		else if (strncmp(argv[i],"frac=",5)==0) sscanf(&argv[i][5],"%d/%d",&num,&denom);
		else if (strncmp(argv[i],"shrink=",7)==0) shrink=atoi(&argv[i][7]);
		else if (strncmp(argv[i],"precen",7)==0) precen=1;
		else if (strncmp(argv[i],"sigfilt",7)==0) sigfilt=1;
		else if (strncmp(argv[i],"quiet",5)==0) quiet=1;
		else if (strncmp(argv[i],"ctfc",4)==0) ctfc=1;
		else if (strncmp(argv[i],"sep=",4)==0) ntk=atoi(&argv[i][4]);
		else if (strncmp(argv[i],"verbose",7)==0) ver=1;
        else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

if (tree1 && tree2) { printf("Specify either tree1 or tree2, not both!\n"); exit(1); }
if (num>=denom || num<0) { printf("0 <= num < denom !!!\n"); exit(1); }
if (classes==-999.0 && !split) { printf("Must specify at least 1 of 'classes' or 'split'.\n"); exit(1); }
if (imask>1 && !precen) { printf("imask may only be used in conjunction with precen\n"); exit(0); }

// READ Reference Images
if (tree1) {
	FILE *in = fopen(tree1,"r");
	List *ndr = new List;
	char s[256];
	int cn;
	dataref=new List;

	if (!in) { printf("Cannot open '%s'\n",tree1); exit(1); }

	while (fgets(s,255,in)) {
		if (sscanf(s," %d",&cn)!=1) continue;

		d=new EMData;
		d->readImage(argv[2],cn);
		d->unum=cn;
		dataref->addObject(d);
		
		if (!num) {
			sprintf(s,"tcls%04d.hed",cn);
			if (access(s,F_OK)) continue;
			unlink(s);
			sprintf(s,"tcls%04d.img",cn);
			unlink(s);
		}
	}
	fclose(in);
	nr=dataref->count();
}
else if (tree2) {
	FILE *in = fopen(tree2,"r");
	List *ndr = new List;
	char s[256];
	int cn,cn2,pp[19];
	dataref=new List;

	if (!in) { printf("Cannot open '%s'\n",tree2); exit(1); }

	if (sscanf(argv[1],"tcls%d",&cn2)!=1) {
		printf("tree2 requires 'tcls*' files for input\n"); exit(1);
	}
	while (fgets(s,255,in)) {
		if (sscanf(s," %d",&cn)!=1 || cn!=cn2) continue;

		cn=sscanf(s," %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
			pp,pp+1,pp+2,pp+3,pp+4,pp+5,pp+6,pp+7,pp+8,pp+9,pp+10,pp+11,
			pp+12,pp+13,pp+14,pp+15,pp+16,pp+17,pp+18);

		if (cn<4) { printf("Tree file appears strange, '%s'\n",s); exit(1); }
		
		for (i=0; i<cn; i++) {
			d=new EMData;
			d->readImage(argv[2],pp[i]);
			d->unum=pp[i];
			dataref->addObject(d);
		}
		break;
	}
	fclose(in);
	nr=dataref->count();
}
else {
	nr=fileCount(argv[2]);
	dataref=readImages(argv[2],0,nr);		// read all input images
	if (dataref==NULL) error("Cannot read input",ERR_EXIT);
	nr=dataref->count();
	for (i=0; i<nr; i++) {
		d=(EMData *)dataref->objectAt(i);
		d->unum=i;
	}
}
printf("Read %d references.\n",nr);

rn=(int *)malloc(nr*sizeof(int));
for (i=0; i<nr; i++) rn[i]=((EMData *)dataref->objectAt(i))->unum;

int Ref=0;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[2],LOG_INFILE,NULL);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

for (i=0; i<nr; i++) {
	d=(EMData *)dataref->objectAt(i);
	j=d->unum;
	d->edgeNormalize(1);
	d->applyMask(mask,0);
	d->medianShrink(shrink);
	if (sigfilt) d->realFilter(3,d->Sigma(),-d->Sigma());
	d=d->copy();
	d->unum=j;
	dataref->replaceObjectAt(i,d);
}

x=d->xSize();
y=d->ySize();

// initialize images for classums
if (classes!=-999.0) {
	newref=new List;
	for (i=0; i<nr; i++) {
		d=new EMData;
		d->setSize(x,y,1);
		d->zero();
		newref->addObject(d);
	}
}

msk=((EMData *)dataref->objectAt(0))->copy();
msk->setParent(NULL);
msk->one();
msk->applyMask(mask/shrink,4);
msk->applyMask(imask/shrink,5);

ns=fileCount(argv[1]);
printf("%d images to classify.\n",ns);

d=new EMData;

norm=x*y/1000.0;
norm2=PI/(4.0*x*y);
if (mask<=0) maxshift=y/4; else maxshift=y/2-mask/shrink;

n0=num*ns/denom;
n1=(num+1)*ns/denom;

printf("processing %d - %d of %d\n",n0,n1,ns);
fflush(stdout);
if (mask==-1) mask=-shrink;
if (!quiet) LOG(Ref,"cls000.hed",LOG_OUTFILE,NULL);

if (ctfc) {
	datarefnc=dataref; 
	dataref=new List; 
	for (i=0; i<nr; i++) {
		dataref->addObject(((EMData *)datarefnc->objectAt(i))->copy());
		((EMData *)dataref->objectAt(i))->setCTF(nullctf);
	}
}

// align and classify
for (i=n0; i<n1; i++) {
	best=new Olist(ntk);

	d->readImage(argv[1],i);
	d->edgeNormalize(1);
	d->medianShrink(shrink);
	if (d->xSize()!=x ||d->ySize()!=y) { printf("Bad image size %d,%d != %d,%d\n",d->xSize(),d->ySize(),x,y); exit(1); }

	d2=d->copy();
	df=d->copy();
	df->vFlip();
//	df=df->copy();
	
	for (j=0; j<nr; j++) {
		dr=(EMData *)dataref->objectAt(j);
		if (d->hasCTF() && ctfc && memcmp(d->getCTF(),dr->getCTF(),11*sizeof(float))) {
			EMData *dt;
			// if several images in a row have the same ctf, don't recalculate
			ctf=d->ctfCurve(2);
			dt=((EMData *)datarefnc->objectAt(j))->doFFT()->copy();
			dt->applyRadFn(d->ySize()/2,0,1,ctf,0);
			dr=dt->doIFT();
			dt->gimmeFFT();
			dr->normalize();
			dr->setParent(NULL);
			dr->setCTF(d->getCTF());
			delete ((EMData *)dataref->replaceObjectAt(j,dr));
			delete dt;
			free(ctf);
		}
		if (precen) da=d2->RFAlign(dr,df,imask/shrink,mask/shrink);
		else {
			da=d2->RTFAlign(dr,df,dodot,maxshift);
//			if (fabs(da->Dx())>maxshift || fabs(da->Dy())>maxshift) continue;
		}		
//		da->refineAlign(dr,1);
		da->mult(msk);
//da->writeIMAGIC("abc.hed",-1);
//dr->writeIMAGIC("abc.hed",-1);
		if (!da) continue;
		if (!dodot) dot=(1.0-da->lcmp(dr))/norm2;
		else dot=da->dot(dr);
		
//		da->subtract(dr);
//		da->writeIMAGIC("t.hed",-1);
		
		best->insert(dot,j,da);
		
//		if (ver) { putchar('0'+(int)floor(dot/norm/50)); fflush(stdout); }
		if (ver) printf("%d\t%f\n",j,dot);
	}
//	if (ver) putchar('\n');

	if (!best->ali[0]) { 
		printf("No class for %d\n",i);  
		delete d2;
		delete df;
		continue;
	}

	delete d2;
	delete df;

	// class average for dots better than a threshold	
//	if (classes!=-999.0) {
//		if (bestdot/norm>classes) {
//			d2=(EMData *)newref->objectAt(best);
//			d2->add(dbest);
//		}
//	}

	// split into class files
	if (split) {
		if (shrink>1) d->readImage(argv[1],i);
		printf("%d -> ",i);
		for (j=0; j<ntk; j++) {
			printf("\t%d (%1.2f)",rn[best->n[j]],best->dot[j]);
			sprintf(s,"%scls%04d.hed",tree1?"t":"",rn[best->n[j]]);
			d->writeIMAGIC(s,-1);
		}
		printf("\n");
		fflush(stdout);
	}
	
	delete best;
}



if (classes!=-999.0) {
	for (i=0; i<nr; i++) {
		d2=(EMData *)newref->objectAt(i);
		d2->normalize();
		d2->writeIMAGIC("classes.hed",-1);
	}
	printf("Class averages written\n");
}

if (!quiet) LOG(Ref,NULL,LOG_END,NULL);
		
}



