// classesbymra2.C Copyright 1999 Steve Ludtke
// A modified version of classesbymra which works with multiple reference sets
// in numbered subdirectories
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i,j,k,best,x,y,nr,ns,split=1,num=0,denom=1,dodot=1,ver=0,sigfilt=0;
int n0,n1,shrink=1,mask=-1,imask=0,precen=0,ctfc=0,maxshift,quiet=0,multi=0;
List *data,*dataref,*datarefnc,*newref=NULL,*tlist;
EMData *d,*d2,*df,*dr,*da,*dbest,*msk;
char s[80],ss[1024],*fs;
float dot,bestdot,norm,norm2;
float nullctf[11] = { 0,0,0,0,0,0,0,0,0,0,0 };
float *ctf;
FILE *out;

appinit(argc,argv);

if (argc<4) { printf("classesbymra2\nUsage: classesbymra2 <unclassed> <refrences> <multi #> [split] \
[frac=<0 to d-1>/<d>] [verbose] [dot] [mask=<rad>] [imask=<rad>] [precen] [sigfilt]\n"); exit(0); }

multi=atoi(argv[3]);
if (multi<2) { printf("multi # should be >1\n"); exit(1); }

for (i=4; i<argc; i++) {        
	if (strncmp(argv[i],"split",5)==0) split=1;
    else if (strncmp(argv[i],"dot",3)==0) dodot=1;	// default now
 	else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
 	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp(argv[i],"num=",4)==0) num=atoi(&argv[i][4]);
	else if (strncmp(argv[i],"denom=",6)==0) denom=atoi(&argv[i][6]);
	else if (strncmp(argv[i],"frac=",5)==0) sscanf(&argv[i][5],"%d/%d",&num,&denom);
	else if (strncmp(argv[i],"shrink=",7)==0) shrink=atoi(&argv[i][7]);
	else if (strncmp(argv[i],"precen",7)==0) precen=1;
	else if (strncmp(argv[i],"sigfilt",7)==0) sigfilt=1;
	else if (strncmp(argv[i],"quiet",5)==0) quiet=1;
	else if (strncmp(argv[i],"ctfc",4)==0) ctfc=1;
	else if (strncmp(argv[i],"verbose",7)==0) ver=1;
    else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

if (num>=denom || num<0) { printf("0 <= num < denom !!!\n"); exit(1); }
if (imask>1 && !precen) { printf("imask may only be used in conjunction with precen\n"); exit(0); }

dataref=new List;

// READ Reference Images
for (j=0; j<multi; j++) {
	//delete old refs
	if (split && !num) {
		char ss[100];
		for (i=0; i<1000; i++) {
			sprintf(ss,"%0d/cls%04d.hed",j,i);
			if (access(ss,F_OK)) break;
			unlink(ss);
			sprintf(ss,"%0d/cls%04d.img",j,i);
			unlink(ss);
		}
	}

	// read new refs
	sprintf(ss,"%d/%s",j,argv[2]);
	nr=fileCount(ss);
	tlist=readImages(ss,0,nr);		// read all input images
	if (!tlist) {
		printf("Cannot open '%s'\n",ss);
		exit(1);
	}
	for (i=0; i<tlist->count(); i++) {
		d=(EMData *)tlist->objectAt(i);
		sprintf(ss,"%d/cls%04d.hed",j,i);
		d->setPath(ss);
		d->edgeNormalize();
		d->applyMask(mask,0);
		d->medianShrink(shrink);
		if (split && !num) d->writeIMAGIC(ss,0);		
		if (sigfilt) d->realFilter(3,d->Sigma(),-d->Sigma());
		d=d->copy();
		dataref->addObject(d);
	}
	delete tlist;

}
printf("Read %d references.\n",dataref->count());
nr=dataref->count();
x=d->xSize();
y=d->ySize();

int Ref=0;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[2],LOG_INFILE,NULL);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

msk=((EMData *)dataref->objectAt(0))->copy();
msk->setParent(NULL);
msk->one();
msk->applyMask(mask/shrink,4);
msk->applyMask(imask/shrink,5);

ns=fileCount(argv[1]);
printf("%d images to classify.\n",ns);

d=new EMData;

norm=x*y/1000.0;
norm2=PI/(4.0*x*y);
if (mask<=0) maxshift=y/4; else maxshift=y/2-mask/shrink;

n0=num*ns/denom;
n1=(num+1)*ns/denom;

printf("processing %d - %d of %d\n",n0,n1,ns);
if (mask==-1) mask=-shrink;
if (!quiet) LOG(Ref,"cls0000.hed",LOG_OUTFILE,NULL);

if (ctfc) { 
	datarefnc=dataref; 
	dataref=new List; 
	for (i=0; i<nr; i++) {
		dataref->addObject(((EMData *)datarefnc->objectAt(i))->copy());
		((EMData *)dataref->objectAt(i))->setCTF(nullctf);
	}
}

// align and classify
for (i=n0; i<n1; i++) {
	d->readImage(argv[1],i);
	d->normalize();
	d->medianShrink(shrink);
	if (d->xSize()!=x ||d->ySize()!=y) { printf("Bad image size %d,%d != %d,%d\n",d->xSize(),d->ySize(),x,y); exit(1); }

	d2=d->copy();
	df=d->copy();
	df->vFlip();
//	df=df->copy();
	
	best=0;
	bestdot=-1000.0;
	dbest=NULL;
	for (j=0; j<nr; j++) {
		dr=(EMData *)dataref->objectAt(j);
		if (d->hasCTF() && ctfc && memcmp(d->getCTF(),dr->getCTF(),11*sizeof(float))) {
			EMData *dt;
			// if several images in a row have the same ctf, don't recalculate
			ctf=d->ctfCurve(2);
			dt=((EMData *)datarefnc->objectAt(j))->doFFT()->copy();
			dt->applyRadFn(d->ySize()/2,0,1,ctf,0);
			dr=dt->doIFT();
			dt->gimmeFFT();
			dr->normalize();
			dr->setParent(NULL);
			dr->setCTF(d->getCTF());
			dr->setPath(((EMData *)datarefnc->objectAt(j))->Path());
			delete ((EMData *)dataref->replaceObjectAt(j,dr));
			delete dt;
			free(ctf);
		}
		if (precen) da=d2->RFAlign(dr,df,imask/shrink,mask/shrink);
		else {
			da=d2->RTFAlign(dr,df,dodot,maxshift);
//			if (fabs(da->Dx())>maxshift || fabs(da->Dy())>maxshift) continue;
		}		
//		da->refineAlign(dr,1);
		da->mult(msk);
//da->writeIMAGIC("abc.hed",-1);
//dr->writeIMAGIC("abc.hed",-1);
		if (!da) continue;
		if (!dodot) dot=(1.0-da->lcmp(dr))/norm2;
		else dot=da->dot(dr);
		
//		da->subtract(dr);
//		da->writeIMAGIC("t.hed",-1);
		
		if (dot>bestdot) { 
			bestdot=dot; 
			best=j; 
			if (dbest) delete dbest;
			dbest=da;
			fs=(char *)dr->Path();
		}
		else delete da;

//		if (ver) { putchar('0'+(int)floor(dot/norm/50)); fflush(stdout); }
		if (ver) printf("%d\t%f\n",j,dot);
	}
//	if (ver) putchar('\n');

	if (!dbest) { 
		printf("No class for %d\n",i);  
		delete d2;
		delete df;
		continue;
	}
	printf("%d -> %s ( %f ) %c\n",i,fs,bestdot/norm,'0'+(int)floor(bestdot/norm/50));

	delete d2;
	delete df;

	// class average for dots better than a threshold	

	// split into class files
	if (split) {
		if (shrink>1) d->readImage(argv[1],i);
		d->writeIMAGIC(fs,-1);
	}
	
	delete dbest;
}

if (!quiet) LOG(Ref,NULL,LOG_END,NULL);
		
}



