#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

// a little class for maintaining an ordered list
class Olist {
public:
float *dot;
int *n;
EMData **ali;
int N;

Olist(int ntokeep) {
	N=ntokeep;
	dot=(float *)malloc(N*sizeof(float));
	n=(int *)malloc(N*sizeof(int));
	ali=(EMData **)malloc(N*sizeof(EMData *));
	for (int i=0; i<N; i++) {
		dot[i]=-1.0e20;
		n[i]=-1;
		ali[i]=NULL;
	}
}

~Olist() {
	free(dot);
	free(n);
	for (int i=0; i<N; i++) delete ali[i];
	free(ali);
}

void insert(float vdot, int vn, EMData *vali) {
	int i,j;
	
	for (i=0; i<N; i++) if (dot[i]<vdot) break;

	if (i==N) { delete vali; return; }

	if (ali[N-1]) delete ali[N-1];
	for (j=N-1; j>i; j--) {
		dot[j]=dot[j-1];
		n[j]=n[j-1];
		ali[j]=ali[j-1];
	}
	dot[i]=vdot;
	n[i]=vn;
	ali[i]=vali;
}

};

main(int argc, char *argv[]) {
int i,j,k,x,y,nr,ns,ntk=1,split=0,num=0,denom=1,dodot=1,ver=0,sigfilt=0;
int n0,n1,shrink=1,mask=-1,imask=0,precen=0,ctfc=0,maxshift,quiet=0;
List *data,*dataref,*datarefnc,*newref=NULL;
EMData *d,*d2,*df,*dr,*da,*msk;
char s[80];
float dot,classes=-999.0,norm,norm2;
float nullctf[11] = { 0,0,0,0,0,0,0,0,0,0,0 };
float *ctf;
FILE *out;
Olist *best;

appinit(argc,argv);

if (argc<3) { printf("classesbymra.c\nUsage: classesbymra <unclassed> <refrences> [classes=<thr>] [split] \
[frac=<0 to d-1>/<d>] [verbose] [dot] [mask=<rad>] [imask=<rad>] [precen] [sigfilt]\n"); exit(0); }

for (i=3; i<argc; i++) {        
        if (strncmp(argv[i],"classes=",8)==0) classes=atof(&argv[i][8]);
        else if (strncmp(argv[i],"split",5)==0) split=1;
        else if (strncmp(argv[i],"dot",3)==0) dodot=1;		// default now
 		else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
 		else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
		else if (strncmp(argv[i],"num=",4)==0) num=atoi(&argv[i][4]);
		else if (strncmp(argv[i],"denom=",6)==0) denom=atoi(&argv[i][6]);
		else if (strncmp(argv[i],"frac=",5)==0) sscanf(&argv[i][5],"%d/%d",&num,&denom);
		else if (strncmp(argv[i],"shrink=",7)==0) shrink=atoi(&argv[i][7]);
		else if (strncmp(argv[i],"precen",7)==0) precen=1;
		else if (strncmp(argv[i],"sigfilt",7)==0) sigfilt=1;
		else if (strncmp(argv[i],"quiet",5)==0) quiet=1;
		else if (strncmp(argv[i],"ctfc",4)==0) ctfc=1;
		else if (strncmp(argv[i],"sep=",4)==0) ntk=atoi(&argv[i][4]);
		else if (strncmp(argv[i],"verbose",7)==0) ver=1;
        else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

if (num>=denom || num<0) { printf("0 <= num < denom !!!\n"); exit(1); }
if (classes==-999.0 && !split) { printf("Must specify at least 1 of 'classes' or 'split'.\n"); exit(1); }
if (imask>1 && !precen) { printf("imask may only be used in conjunction with precen\n"); exit(0); }

// READ Reference Images
nr=fileCount(argv[2]);
dataref=readImages(argv[2],0,nr);		// read all input images
if (dataref==NULL) error("Cannot read input",ERR_EXIT);
printf("Read %d references.\n",dataref->count());
nr=dataref->count();

int Ref=0;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[2],LOG_INFILE,NULL);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

// prepare cls files for splitting
if (split && !num) {
	char ss[100];
	for (i=0; i<1000; i++) {
		sprintf(ss,"cls%04d.hed",i);
		if (access(ss,F_OK)) break;
		unlink(ss);
		sprintf(ss,"cls%04d.img",i);
		unlink(ss);
	}
}

for (i=0; i<nr; i++) {
	d=(EMData *)dataref->objectAt(i);
	d->edgeNormalize();
	d->applyMask(mask,0);
	d->medianShrink(shrink);
	if (split && !num) {
		sprintf(s,"cls%04d.hed",i);
		d->writeIMAGIC(s,0);
	}
	if (sigfilt) d->realFilter(3,d->Sigma(),-d->Sigma());
	d=d->copy();
	dataref->replaceObjectAt(i,d);
}
x=d->xSize();
y=d->ySize();

// initialize images for classums
if (classes!=-999.0) {
	newref=new List;
	for (i=0; i<nr; i++) {
		d=new EMData;
		d->setSize(x,y,1);
		d->zero();
		newref->addObject(d);
	}
}

msk=((EMData *)dataref->objectAt(0))->copy();
msk->setParent(NULL);
msk->one();
msk->applyMask(mask/shrink,4);
msk->applyMask(imask/shrink,5);

ns=fileCount(argv[1]);
printf("%d images to classify.\n",ns);

d=new EMData;

norm=x*y/1000.0;
norm2=PI/(4.0*x*y);
if (mask<=0) maxshift=y/4; else maxshift=y/2-mask/shrink;

n0=num*ns/denom;
n1=(num+1)*ns/denom;

printf("processing %d - %d of %d\n",n0,n1,ns);
if (mask==-1) mask=-shrink;
if (!quiet) LOG(Ref,"cls000.hed",LOG_OUTFILE,NULL);

if (ctfc) {
	datarefnc=dataref; 
	dataref=new List; 
	for (i=0; i<nr; i++) {
		dataref->addObject(((EMData *)datarefnc->objectAt(i))->copy());
		((EMData *)dataref->objectAt(i))->setCTF(nullctf);
	}
}

// align and classify
for (i=n0; i<n1; i++) {
	best=new Olist(ntk);

	d->readImage(argv[1],i);
	d->normalize();
	d->medianShrink(shrink);
	if (d->xSize()!=x ||d->ySize()!=y) { printf("Bad image size %d,%d != %d,%d\n",d->xSize(),d->ySize(),x,y); exit(1); }

	d2=d->copy();
	df=d->copy();
	df->vFlip();
//	df=df->copy();
	
	for (j=0; j<nr; j++) {
		dr=(EMData *)dataref->objectAt(j);
		if (d->hasCTF() && ctfc && memcmp(d->getCTF(),dr->getCTF(),11*sizeof(float))) {
			EMData *dt;
			// if several images in a row have the same ctf, don't recalculate
			ctf=d->ctfCurve(2);
			dt=((EMData *)datarefnc->objectAt(j))->doFFT()->copy();
			dt->applyRadFn(d->ySize()/2,0,1,ctf,0);
			dr=dt->doIFT();
			dt->gimmeFFT();
			dr->normalize();
			dr->setParent(NULL);
			dr->setCTF(d->getCTF());
			delete ((EMData *)dataref->replaceObjectAt(j,dr));
			delete dt;
			free(ctf);
		}
		if (precen) da=d2->RFAlign(dr,df,imask/shrink,mask/shrink);
		else {
			da=d2->RTFAlign(dr,df,dodot,maxshift);
//			if (fabs(da->Dx())>maxshift || fabs(da->Dy())>maxshift) continue;
		}		
//		da->refineAlign(dr,1);
		da->mult(msk);
//da->writeIMAGIC("abc.hed",-1);
//dr->writeIMAGIC("abc.hed",-1);
		if (!da) continue;
		if (!dodot) dot=(1.0-da->lcmp(dr))/norm2;
		else dot=da->dot(dr);
		
//		da->subtract(dr);
//		da->writeIMAGIC("t.hed",-1);
		
		best->insert(dot,j,da);
		
//		if (ver) { putchar('0'+(int)floor(dot/norm/50)); fflush(stdout); }
		if (ver) printf("%d\t%f\n",j,dot);
	}
//	if (ver) putchar('\n');

	if (!best->ali[0]) { 
		printf("No class for %d\n",i);  
		delete d2;
		delete df;
		continue;
	}

	delete d2;
	delete df;

	// class average for dots better than a threshold	
//	if (classes!=-999.0) {
//		if (bestdot/norm>classes) {
//			d2=(EMData *)newref->objectAt(best);
//			d2->add(dbest);
//		}
//	}

	// split into class files
	if (split) {
		if (shrink>1) d->readImage(argv[1],i);
		printf("%d -> ",i);
		for (j=0; j<ntk; j++) {
			printf("\t%d (%1.2f)",best->n[j],best->dot[j]);
			sprintf(s,"cls%04d.hed",best->n[j]);
			d->writeIMAGIC(s,-1);
		}
		printf("\n");
	}
	
	delete best;
}



if (classes!=-999.0) {
	for (i=0; i<nr; i++) {
		d2=(EMData *)newref->objectAt(i);
		d2->normalize();
		d2->writeIMAGIC("classes.hed",-1);
	}
	printf("Class averages written\n");
}

if (!quiet) LOG(Ref,NULL,LOG_END,NULL);
		
}



