// clean3d.C 1/00 Steve Ludtke

//N clean3d
//F This program uses make3diter and proc3d to iteratively make models match class averages
//1
//P <classes>	File with class averages
//P <3d model>	Preliminary 3d model to clean up
//P [sym=<sym>]	Symmetry specification
//P [proc=<nproc>]	Number of processors to use
//P [3dit=<n iter>]	Number of 'normal' cleaning iterations
//P [3dit2=<n iter>]	Number of 'zeroing' cleaning iterations
//U clean3d classes.5.img threed.5a.img sym=d7 3dit=2 3dit2=2
//D This program performs iterative real-space refinement to make
//E a 3D model more closely match a set of class-averages. There are 2
//E versions of the routine. The '3dit=' routine simply tries for the
//E 'median' match. The '3dit2=' routine drives the densities to zero
//E as much as is permitted by the class averages.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i,j;
char *sym=NULL;
int it1=0,it2=0,proc=1;
char s[512],*t = tmpnam(NULL);
FILE *out;

appinit(argc,argv);

if (argc<3) { 
	printf("Iteratively improve 3D models.\nUsage: clean3d <classes> <3d model> \
[sym=<sym>] [proc=<nproc>] [3dit=<iter>] [3dit2=<iter>]\n");
	exit(0);
}

for (i=3; i<argc; i++) {	
	if (strncmp("3dit=",argv[i],5)==0) it1=atoi(&argv[i][5]);
	else if (strncmp("3dit2=",argv[i],6)==0) it2=atoi(&argv[i][6]);
	else if (strncmp("proc=",argv[i],5)==0) proc=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"sym=",4)==0) {
		if (tolower(argv[i][4])=='c' || tolower(argv[i][4])=='d' || tolower(argv[i][4])=='i') sym=&argv[i][4];
		else { printf("Unsupported symmetry\n"); exit(1); }
	}
}

if (!sym) sym="i";

for (i=0; i<it2; i++) {
	out=fopen(t,"w");
	for (j=proc-1; j>=0; j--) {
		fprintf(out,"make3diter %s %s %s sym=%s frac=%d/%d tozero\n",argv[2],argv[1],argv[2],sym,j,proc);
	}
	fclose(out);

	sprintf(s,"runpar proc=%d file=%s",proc,t);
//	sprintf(s,"cat %s",t);
	run(s);
	unlink(t);

	if (strlen(sym)>1) {
		sprintf(s,"proc3d %s %s sym=%s",argv[2],argv[2],sym);
		run(s);
	}
}

for (i=0; i<it1; i++) {
	out=fopen(t,"w");
	for (j=proc-1; j>=0; j--) {
		fprintf(out,"make3diter %s %s %s sym=%s frac=%d/%d\n",argv[2],argv[1],argv[2],sym,j,proc);
	}
	fclose(out);

	sprintf(s,"runpar proc=%d file=%s",proc,t);
//	sprintf(s,"cat %s",t);
	run(s);
	unlink(t);

	if (strlen(sym)>1) {
		sprintf(s,"proc3d %s %s sym=%s",argv[2],argv[2],sym);
		run(s);
	}
}

}
