// fftavg.C  1997 Steve Ludtke

//N fftavg
//F This program reads a set of images, transforms them, and generates and averaged power spectrum
//1
//P <image stack>	Reads all images from a single stack file
//2
//P <Image 1>	First image
//P <Image 2>	Second image
//P	etc.
//U fftavg im*.mrc
//U fftavg 2248.img
//D This program reads a set of images (all the same size), does an FFT of each, then
//E averages them to generate an approximate power spectrum. This functionality is already
//E incorporated into <a href=ctfit.html>ctfit</a>, but this program may still be useful
//E for offline calculations. An x/y plot of the power spectrum is dumped to stdout, and
//E the averaged image is output to fftavg.tnf (a complex MRC file)

#include <stdio.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define POINTS 500

main(int argc, char *argv[]) {
int i,j,k,n;
float y[POINTS+1];
EMData *d,*dd,*av;
float *d2;

appinit(argc,argv);

if (argc<2) { printf("fftavg <list>\n"); exit(0); }

n=fileCount(argv[1]);
av=new EMData;
if (av->readImage(argv[1],0)) exit(1);
av->normalize();
av=av->doFFT();
av->ri2ap();

if (n>1) {
	printf("Averaging %d images from one file\n",n);
	for (i=1; i<n; i++) {
		printf("%d\r",i);
		fflush(stdout);
		d=new EMData;
		if (d->readImage(argv[1],i)) { printf("Cannot read %s\n",argv[i]); exit(1); }
		d->normalize();
		dd=d->doFFT();
		dd->ri2ap();
		av->add(dd);
		delete d;
	}
} else {
	printf("Averaging %d images\n",argc-1);
	for (i=2; i<argc; i++) {
		printf("%d\r",i);
		fflush(stdout);
		d=new EMData;
		if (d->readImage(argv[i],0)) { printf("Cannot read %s\n",argv[i]); exit(1); }
		d->normalize();
		dd=d->doFFT();
		dd->ri2ap();
		av->add(dd);
		delete d;
	}
}

av->ap2ri();
av->writeMRC("fftavg.tnf");
av->ri2ap();
av->calcRadDist(POINTS,0,av->xSize()/POINTS/2.0,y);

for (i=0; i<POINTS; i++) printf("%f\t%f\n",i*.416667/av->xSize(),y[i]/(argc-1.0));
}
