// multirefine.C Copyright 1999 Steve Ludtke
// This is a modified version of refine.C which refines several models
// simultaneously from a single set of raw data

//N multirefine
//F This performs refinement on a set of 3D models simultaneously
//1
//P <total iter>	As in refine
//P <# models>	The number of 3D models being refined
//P see refine for others
//D <I>NOTE: This program is still experimental</i>
//D This program operates almost identically to <a href=refine.html>refine</a>. The difference
//E is, this program will simultaneously refine several models from a single set of raw
//E images. That is, each particle will be grouped with one of the N models. In this way,
//E a heterogeneous data set can be separated into multiple models simultaneously. This can also
//E be used in some cases to filter out partially denatured proteins, etc. The options for this 
//E program are almost the same as those for refine, with the addition of the 2nd parameter
//E which specifies the number of models being refined.
//D Rather than putting a single threed.0a.mrc file in the run directory, a subdirectory should
//E be made for each starting model, called '1', '2', etc. Each directory should then contain
//E a different threed.0a.mrc file.


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include "util.h"
#include "EMData.h"

void *cmra(void *fnum);

main(int argc, char *argv[]) {
int i,j,k,n;
int iter=5,mask=-1,imask=-1,pad=0,classiter=8,classfp=0,goodbad=0,precen=0;
int minproc=0,maxproc=0,lp=0,alito=0,median=0,dot=0,mode=2,shrink=1,sigfilt=0,can=0;
int noweight=0,multi=1;
float ang=7.5,keep=1.2,xfiles=0,appix=0,d3s=0,hard=0,range=90.0,ctfc=0,euler2=0.0;
char s[1024],ss[1024],*sym=NULL;
FILE *log;
time_t ti;

appinit(argc,argv);

log=fopen("refine.log","a");

if (argc<3) { printf("Usage:\nPut initial models in '#/threed.0a.mrc'. Particles should\n\
be in 'start.hed/img'.\n\nrefine <total iter> <# multi> [proc=<nproc>] [hard=<maxpr>] [precen] [imask=<rad>]\
[sym=<c2,c4,etc>]\n\t[ang=<dang>] [mask=<rad>] [pad=<size>] [classkeep=<sigma coef>]\n\
\t[classiter=<iter>] [filt3d=<lp rad>] [d3s=<sigmult>] [median]\n\
\t[xfiles=<a/pix>,<volume>,<ali to>] [goodbad]\n\
\t[mode=<1-6>] [dot] [ctfc=<res in A>] [shrink=<n>] [ca3] [sigfilt] [euler2]\n"); exit(0); }

int Ref=LOGbegin(argc,argv);
LOG(Ref,"start.hed",LOG_INFILE,NULL);
LOG(Ref,"refine.log",LOG_OUTFILE,NULL);

ti=time(0);
if (log) fprintf(log,"******** multirefine started %s",ctime(&ti));
for (i=0; i<argc; i++) fprintf(log,"%s ",argv[i]);
fprintf(log,"\n");

iter=atoi(argv[1]);
multi=atoi(argv[2]);
if (multi<2) { printf("Multi count must be >1\n"); exit(1); }

for (i=0; i<multi; i++) {
	sprintf(s,"%d/threed.0a.mrc",i);
	if (access(s,R_OK)) {
		printf("%s does not exist\n",s);
		exit(1);
	}
}

for (i=3; i<argc; i++) {
	if (strncmp("proc=",argv[i],5)==0) sscanf(&argv[i][5],"%d,%d",&minproc,&maxproc);
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("mode=",argv[i],5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp("hard=",argv[i],5)==0) hard=atof(&argv[i][5]);
	else if (strncmp("classkeep=",argv[i],10)==0) keep=atof(&argv[i][10]);
	else if (strncmp("classiter=",argv[i],10)==0) classiter=atoi(&argv[i][10]);
	else if (strncmp("pad=",argv[i],4)==0) pad=atoi(&argv[i][4]);
	else if (strncmp("filt3d=",argv[i],7)==0) lp=atoi(&argv[i][7]);
	else if (strncmp("shrink=",argv[i],7)==0) shrink=atoi(&argv[i][7]);
	else if (strncmp("sigfilt",argv[i],7)==0) sigfilt=1;
	else if (strncmp("euler2",argv[i],6)==0) {
		if (argv[i][6]!='=') euler2=2.0;
		else euler2=atof(&argv[i][7]);
	}
	else if (strncmp("noweight",argv[i],8)==0) noweight=1;
	else if (strncmp("ang=",argv[i],4)==0) ang=atof(&argv[i][4]);
	else if (strncmp("ca3",argv[i],3)==0) can=3;
	else if (strncmp("d3s=",argv[i],4)==0) d3s=atof(&argv[i][4]);
	else if (strncmp("sym=",argv[i],4)==0) sym=&argv[i][4];
//	else if (strncmp("restart",argv[i],7)==0) restart=1;
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("ctfc=",argv[i],5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp("goodbad",argv[i],7)==0) goodbad=1;
	else if (strncmp("classfp=",argv[i],8)==0) classfp=atoi(&argv[i][8]);
	else if (strncmp("dot",argv[i],3)==0) dot=1;
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("xfiles=",argv[i],7)==0) {
		xfiles=atof(&argv[i][7]);
		sscanf(&argv[i][7]," %f , %f , %d",&appix,&xfiles,&alito);
	}
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if (minproc<=0) minproc=1;
if (maxproc<=minproc) maxproc=minproc;

if (xfiles) {
	for (i=0; i<multi; i++) {
		sprintf(s,"cd %d; proc3d threed.0a.mrc x.0.mrc clip=%d,%d,%d mask=%d",i,mask*2,mask*2,mask*2,mask);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		system(s);

		if (xfiles>0) {
			sprintf(s,"cd %d; volume x.0.mrc %f set=%f",i,appix,xfiles);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		
			system(s);
		}
	}
}


for (i=1; i<=iter; i++) {
	ti=time(0);
	if (log) { fprintf(log,"Iteration %d beginning at %s",i,ctime(&ti)); fflush(log); }
	
// project3d, classesbymra, classalignall
	sprintf(s,"0/classes.%d.hed",i);
	if (access(s,R_OK)) {
		// project3d
		for (j=0; j<multi; j++) {
			sprintf(s,"cd %d; project3d threed.%da.mrc prop=%1.3f mask=%d mode=-1",j,i-1,ang,mask);
			if (sym) sprintf(s+strlen(s)," sym=%s",sym);

			if (log) { ti=time(0); fprintf(log,"%s%s\n",ctime(&ti),s); fflush(log); }
			system(s);
		}

		if (classfp) {
			printf("Classfp not supported in multirefine\n"); exit(1);
/*			if (access("start.fp",R_OK)) { 
				if (log) { ti=time(0); fprintf(log,"footprintr start.hed start.fp   %s",ctime(&ti)); fflush(log); } 
				system("footprintr start.hed start.fp");
			}
			if (log) { ti=time(0); fprintf(log,"footprintr proj.hed proj.fp   %s",ctime(&ti)); fflush(log); } 
			system("footprintr proj.hed proj.fp");
			
			sprintf(s,"msa2 proj.fp start.fp proj.msa.fp start.msa.fp %d 100",classfp);
			if (log) { ti=time(0); fprintf(log,"%s    %s",s,ctime(&ti)); fflush(log); } 
			system(s);

			system("classesrefs start.msa.fp proj.msa.fp dot");
			if (log) { ti=time(0); fprintf(log,"classesrefs start.msa.fp proj.msa.fp   %s",ctime(&ti)); fflush(log); } */
		}
		else {
			n=fileCount("start.hed");
			if (n/maxproc<50) n=n/maxproc;
			else n/=50;

			char *t = tmpnam(NULL);
			FILE *out = fopen(t,"w");
			if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
			
			for (j=0; j<n; j++) {
				sprintf(s,"classesbymra2 start.hed proj.hed %d split frac=%d/%d dot mask=%d imask=%d shrink=%d%s%s%s%s%s",
					multi,j,n,mask,imask,shrink,precen?" precen":"",j==0?"":" quiet",can==3?" ca3":"",sigfilt?" sigfilt":"",ctfc?" ctfc":"");
				fprintf(out,"%s\n",s);
				if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
			}
			fclose(out);
			
			sprintf(s,"runpar proc=%d,%d file=%s",minproc,maxproc,t);
			system(s);
 			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			unlink(t);
		}
		
		for (j=0; j<multi; j++) {
			sprintf(s,"cd %d; classalignall ref1 finalref saverefs keep=%1.2f %d proc=%d,%d mask=%d imask=%d %s%s%s",
				j,keep,classiter,minproc,maxproc,mask,imask,median?"median":"",dot?" dot":"",precen?" precen":"");
			if (ctfc!=0) sprintf(s+strlen(s)," ctfc=%f",ctfc);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			system(s);
		
			sprintf(s,"%d/classes.%d.img",j,i);
			sprintf(ss,"%d/classes.img",j);
			rename(ss,s);
			sprintf(s,"%d/classes.%d.hed",j,i);
			sprintf(ss,"%d/classes.hed",j);
			rename(ss,s);
			if (log) fprintf(log,"mv %d/classes.hed/img to %s/img\n",j,s);
		}
	}

// check for 3d files
	sprintf(s,"%d/threed.%da.mrc",multi-1,i);
	if (!access(s,R_OK)) { 
		if (log) { fprintf(log,"Iteration %d already complete\n",i); fflush(log); }
		continue;
	}

// This reassigns all of the euler angles
	if (euler2>0) {
	
		for (k=0; k<multi; k++) {
			// first, reproject the model with finer spacing
			sprintf(s,"cd %d; project3d threed.%da.mrc prop=%1.3f mask=%d mode=-1",k,i-1,ang/euler2,mask);
			if (sym) sprintf(s+strlen(s)," sym=%s",sym);
			system(s);

			char *t = tmpnam(NULL);
			FILE *out = fopen(t,"w");
			if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
			
			for (j=0; j<maxproc; j++) {
				sprintf(s,"cd %d; eulerproj classes.%d.hed proj.hed writeali mask=%d %s %s",k,i,mask,precen?" precen":"",sigfilt?"sigfilt":"");
				fprintf(out,"%s\n",s);
				if (log && !j) { ti=time(0); fprintf(log,"%s > %s  %s",s,t,ctime(&ti)); fflush(log); }
			}
			fclose(out);
			
			sprintf(s,"runpar proc=%d,%d file=%s",minproc,maxproc,t);
			system(s);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			unlink(t);

		}
	}


	for (k=0; k<multi; k++) {
		sprintf(s,"cd %d; make3d classes.%d.hed out=threed.%d.mrc mask=%d",k,i,i,mask);
		if (hard) sprintf(s+strlen(s)," hard=%1.2f",hard);
		if (d3s) sprintf(s+strlen(s)," keep=%1.2f",d3s);
		if (pad) sprintf(s+strlen(s)," pad=%d",pad);
		if (sym) sprintf(s+strlen(s)," sym=%s",sym);
		if (noweight) sprintf(s+strlen(s)," noweight");
		if (mode) sprintf(s+strlen(s)," mode=%d",mode);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		system(s);

// proc3d
		sprintf(s,"cd %d; proc3d threed.%d.mrc threed.%da.mrc norm mask=%d",k,i,i,mask);
		if (lp) sprintf(s+strlen(s)," lp=%d",lp);
		if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
		system(s);
	
		if (xfiles) {
			sprintf(s,"cd %d; proc3d threed.%da.mrc x.%d.mrc clip=%d,%d,%d",k,i,i,mask*2,mask*2,mask*2);
			if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
			system(s);
	
			if (i>alito) {
				sprintf(s,"cd %d; align3d x.%0d.mrc x.%d.mrc x.%d.mrc",k,alito,i,i);
				if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
				system(s);
			}
			else if (log) fprintf(log,"Alignment skipped\n");
			
			if (xfiles>0) {
				sprintf(s,"cd %d; volume x.%d.mrc %f set=%f",k,i,appix,xfiles);
				if (log) { ti=time(0); fprintf(log,"%s   %s",s,ctime(&ti)); fflush(log); }
				system(s);
			}
		}
	}
	
}

if (log) { ti=time(0); fprintf(log,"%s  Refine complete\n",ctime(&ti)); }
fclose(log);
LOG(Ref,NULL,LOG_END,NULL);
}

void *cmra(void *s)
{
char ss[1024];

strcpy(ss,(char *)s);
system(ss);
return NULL;
}
