// pdb2mrc.C  11/99  Steve Ludtke

//N pdb2mrc
//F This program will generate a 3D electron density map from PDB files
//1
//P <pdb input>	The PDB file to convert
//P <mrc output>	The output file for the generated map
//P [info]	If this is specified, information on the PDB file is displayed, no conversion is performed.
//P [apix=<a/voxel>]	Angstroms per voxel in the output, default is 1
//P [res=<resolution>]	Real space resolution, for 1/e width in F space multiply by 2.2
//P [box=<box size>]	Box size in voxels, automatic if unspecified
//P [het]	Include HET atoms in the conversion
//P [center]	Center the model (center of charge)
//P [solvate]	Solvate the model (with water). This is an approximate algorithm.
//U pdb2mrc 1FAQ.pdb FAQ.mrc apix=1 res=1.5 center
//D This will generate a 3D MRC map from a PDB file. This may be used by <a href=align3d.html>
//E align3d</a> or <a href=foldhunter.html>foldhunter</a>, or for other purposes. The solvate
//E option is better than that used by some other similar programs, but is NOT discrete
//E water molecule solvation. It is still an approximate continuous method which still
//E produces artifacts. In addition, without solvation, the program can generate models at any
//E resolution, but with solvation, resolution must be 1.5 A or better. 

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define H2ORAD	1.92		// in A
#define H2ODEN	.334		// in e-/A^3

struct ATOM { float x,y,z; int a; };

main(int argc, char *argv[]) {
float apix=1.0,res=1.5;
int i,j,k,l,box=0,info=0,ires=0,ihelix=0,isheet=0,lres=0,lsheet=0,lhelix=0,het=0;
ATOM *atoms = (ATOM *) malloc(sizeof(ATOM)*32);
int natoms=0,naa=32,solv=0,pipe=0,center=0;
char s[1024],lch=' ';
float totmass=0,totchg=0;
float xr[3] = {0,0,0} ,yr[3] = {0,0,0} ,zr[3] = {0,0,0},rmax=0;

appinit(argc,argv);

if (argc<3) { printf("pdb2mrc - generate an electron density map from a PDB file\n\n\
Usage: pdb2mrc <pdb input> <mrc output> [info] [apix=<a/voxel>] [res=<resolution>] [box=<box size>] [het] [center] [solvate]\n"); exit(0); }

for (i=3; i<argc; i++) {
	if (strncmp(argv[i],"apix=",5)==0) apix=atof(&argv[i][5]);
	else if (strncmp(argv[i],"info",4)==0) info=1;
	else if (strncmp(argv[i],"res=",4)==0) res=atof(&argv[i][4]);
	else if (strncmp(argv[i],"het",3)==0) het=1;
	else if (strncmp(argv[i],"box=",4)==0) box=atoi(&argv[i][4]);
	else if (strncmp(argv[i],"solvate",7)==0) solv=1;
	else if (strncmp(argv[i],"center",6)==0) center=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

//if (res>3.0) { printf("res > 3.0 !  For low resolutions, use proc3d to apply a low-pass filter\n"); exit(1); }
//if (apix>3.0) { printf("apix > 3.0 !   For less sampling, use proc3d with the shrink option\n"); exit(1); }
if (apix>res) printf("Warning, apix should be <= res\n");
if (solv && (apix>1.0 || res>2.5)) { printf("apix<=1.0 and res<=2.5 for solvation to work properly\n"); exit(1); }

FILE *in;
if (strcmp(argv[1]+strlen(argv[1])-4,".bz2")==0) {
	sprintf(s,"bzcat %s",argv[1]);
	in = popen(s,"r");
	pipe=1;
}
else if (strcmp(argv[1]+strlen(argv[1])-3,".gz")==0) {
	sprintf(s,"zcat %s",argv[1]);
	in = popen(s,"r");
	pipe=1;
}
else in = fopen(argv[1],"r");
if (!in) { printf("Cannot open input file.\n"); exit(1); }

while ((fgets(s,200,in)!=NULL)) {
	if (strncmp(s,"ENDMDL",6)==0) break;
	if (ihelix>=0 && strncmp(s,"HELIX",5)==0) {
		if (atoi(s+21)>lhelix || s[19]!=lch) ihelix+=abs(atoi(s+21)-atoi(s+33))+1;
		else { printf("Overlapping helices, invalid \n"); ihelix=-1; }
		lch=s[19];
		lhelix=atoi(s+33);
	}
	if (isheet>=0 &&strncmp(s,"SHEET",5)==0) {
		isheet+=abs(atoi(s+22)-atoi(s+33))+1;
	}
	if (strncmp(s,"ATOM",4)!=0 && (strncmp(s,"HETATM",6)!=0 || !het)) continue;

	if (strncmp(s,"ATOM",4)==0 && atoi(s+22)!=lres) { lres=atoi(s+22); ires++; }

	if (s[13]==' ') s[13]=s[14];
	if (s[13]==' ') s[13]=s[15];
	
	int e=0;
	switch (s[13]) {
	case 'H': e=1.0; totmass+=1.00794; break;
	case 'C': e=6.0; totmass+=12.0107; break;
	case 'A':	// treat 'A'mbiguous atoms as N, not perfect, but good enough
	case 'N': e=7.0; totmass+=14.00674; break;
	case 'O': e=8.0; totmass+=15.9994; break;
	case 'P': e=15.0; totmass+=30.973761; break;
	case 'S': e=16.0; totmass+=32.066; break;
	case 'W': e=18.0; totmass+=1.00794*2+15.9994; break;	// ficticious water 'atom'
	case 'U': 
		if (s[12]=='A') { e=79.0; totmass+=196.96655; }
		break;	// gold
	default: 
		printf("Unknown atom %c\n",s[13]);
		e=0;
	}
	if (e==0) continue;
	totchg+=e;
	
	sscanf(&s[28]," %f %f %f",&atoms[natoms].x,&atoms[natoms].y,&atoms[natoms].z);
	atoms[natoms].a=e;

//	printf("%c\t%1.1f,%1.1f,%1.1f\n",s[13],atoms[natoms].x,atoms[natoms].y,atoms[natoms].z);

	if (natoms==0 || atoms[natoms].x<xr[0]) xr[0]=atoms[natoms].x;
	if (natoms==0 || atoms[natoms].x>xr[1]) xr[1]=atoms[natoms].x;
	if (natoms==0 || atoms[natoms].y<yr[0]) yr[0]=atoms[natoms].y;
	if (natoms==0 || atoms[natoms].y>yr[1]) yr[1]=atoms[natoms].y;
	if (natoms==0 || atoms[natoms].z<zr[0]) zr[0]=atoms[natoms].z;
	if (natoms==0 || atoms[natoms].z>zr[1]) zr[1]=atoms[natoms].z;
	if (hypot3(atoms[natoms].x,atoms[natoms].y,atoms[natoms].z)>rmax) 
		rmax=hypot3(atoms[natoms].x,atoms[natoms].y,atoms[natoms].z);

	xr[2]+=atoms[natoms].x*e;
	yr[2]+=atoms[natoms].y*e;
	zr[2]+=atoms[natoms].z*e;

	natoms++;
	if (natoms>=naa) {
		naa*=1.5;
		atoms=(ATOM *)realloc((void *)atoms,sizeof(ATOM)*naa);
	}
}
if (pipe) pclose(in);
else fclose(in);
if (natoms==0 || ires==0) { printf("no data read!\n"); exit(1); }

xr[2]/=totchg;
yr[2]/=totchg;
zr[2]/=totchg;

printf("%d atoms read\n",natoms);
printf("Total Mass = %1.1f Da, Total Charge = %1.1f e-\n",totmass,totchg);
printf("Limits (in A) %1.1f, %1.1f, %1.1f - %1.1f, %1.1f, %1.1f\n",xr[0],yr[0],zr[0],xr[1],yr[1],zr[1]);
printf("Center of charge = %1.1f,%1.1f,%1.1f\n",xr[2],yr[2],zr[2]);
if (center) {
	printf("Adjust center -> 0,0,0\n");
	
	rmax=0;
	for (i=0; i<natoms; i++) { 
		atoms[i].x-=xr[2]; 
		atoms[i].y-=yr[2]; 
		atoms[i].z-=zr[2]; 
		if (hypot3(atoms[i].x,atoms[i].y,atoms[i].z)>rmax) 
			rmax=hypot3(atoms[i].x,atoms[i].y,atoms[i].z);
	}
	xr[0]-=xr[2];
	xr[1]-=xr[2];
	yr[0]-=yr[2];
	yr[1]-=yr[2];
	zr[0]-=zr[2];
	zr[1]-=zr[2];
	xr[2]=yr[2]=zr[2]=0;
}
rmax=sqrt(rmax);

char *ss=(char *)malloc(strlen(argv[1])+1);
strcpy(ss,argv[1]);
for (i=0; i<strlen(ss); i++) if (ss[i]=='.') ss[i]=0;
printf("%s Max radius = %1.2f\t%d residues\t%1.1f%% sheet\t%1.1f%% helix\n",ss,rmax,ires,isheet*100.0/ires,ihelix*100.0/ires);
if (info) exit(1);

if (box<5) {
/*	box=(int)ceil(fabs(xr[0]));
	if (ceil(fabs(xr[1]))>box) box=(int)ceil(fabs(xr[1]));
	if (ceil(fabs(yr[0]))>box) box=(int)ceil(fabs(yr[0]));
	if (ceil(fabs(yr[1]))>box) box=(int)ceil(fabs(yr[1]));
	if (ceil(fabs(zr[0]))>box) box=(int)ceil(fabs(zr[0]));
	if (ceil(fabs(zr[1]))>box) box=(int)ceil(fabs(zr[1]));*/
	box=ceil(rmax);
	box/=apix;
	box=4*((box-1)/4+2); // round up to nearest 4 then add 4
	box*=2;
}
printf("Box size %d\n",box);

EMData *data,*rdata;
float *d,*rd;

data=new EMData;
data->setSize(box,box,box);
data->zero();

if (solv) {
	rdata=new EMData;
	rdata->setSize(box,box,box);
	rdata->one();
	rdata->multConst(100.0);
	rd=rdata->getData();
}

d=data->getData();

int x[2],y[2],z[2],w;
float xx,yy,zz;
float r,rp,kn;

rp=res/apix;
rp=rp*rp/sqrt(2.0);		// constant for gaussian falloff
kn=1.0/pow(res*res*PI/2,1.5);	// normalization constant
w=round(res*3.0/apix);
if (w<3) { printf("You do not have sufficient sampling for this resolution. Decrease apix.\n"); exit(1); }
for (l=0; l<natoms; l++) {
	xx=(atoms[l].x/apix)+box/2;
	yy=(atoms[l].y/apix)+box/2;
	zz=(atoms[l].z/apix)+box/2;
	x[0]=round(xx-w);
	x[1]=round(xx+w);
	y[0]=round(yy-w);
	y[1]=round(yy+w);
	z[0]=round(zz-w);
	z[1]=round(zz+w);
	
	
	if (x[0]>=box||y[0]>=box||z[0]>=box||x[1]<0||y[1]<0||z[1]<0) continue;
	if (x[0]<0) x[0]=0;
	if (x[1]>=box) x[1]=box-1;
	if (y[0]<0) y[0]=0;
	if (y[1]>=box) y[1]=box-1;
	if (z[0]<0) z[0]=0;
	if (z[1]>=box) z[1]=box-1;
	
	for (i=x[0]; i<x[1]; i++) {
		for (j=y[0]; j<y[1]; j++) {
			for (k=z[0]; k<z[1]; k++) {
				r=hypot3((float)i-xx,(float)j-yy,(float)k-zz);
				if (solv) {
					if (rd[i+j*box+k*box*box]>r) rd[i+j*box+k*box*box]=r;
				}
				d[i+j*box+k*box*box]+=kn*atoms[l].a*exp(-r/rp);
			}
		}
	}
}


if (solv) {
	for (i=0; i<box*box*box; i++) if (rd[i]==100.0) rd[i]=-100.0;
	
	// iteratively locate 'water' regions
	int l,c=1,n,i2,j2,k2;
	while (c) {
		c=0;
		for (i=1; i<box-1; i++) {
			for (j=1; j<box-1; j++) {
				for (k=1; k<box-1; k++) {

					// this counts points in a 3x3x3 cube that are already 'water'
					n=0;
					for (i2=i-1; i2<i+2; i2++) {
						for (j2=j-1; j2<j+2; j2++) {
							for (k2=k-1; k2<k+2; k2++) {
								l=i2+j2*box+k2*box*box;
								if (rd[l]<0) n++;
							}
						}
					}
					
					// if 'water' is in at least 6 neighbors and we have a large radius, we're also water
					l=i+j*box+k*box*box;
					if (rd[l]>H2ORAD/apix && d[l]<H2ODEN && n>6) { rd[l]=-rd[l]; c=1; }
				}
			}
		}
	}
	
	for (i=0; i<box*box*box; i++) {
		rd[i]=-rd[i];
//		if (rd[i]>1.92/apix) d[i]=H2ODEN;		// bulk water
//		else if (rd[i]>.7/apix) d[i]+=exp(-(1.92/apix-rd[i])/rp)*kn*18.0;		//more than 1/2 a C bond dist
		if (rd[i]>SQR(2.0*res/apix)) {
			d[i]=H2ODEN;
//			printf("%d\t%f\n",i,rd[i]);
		}
		else if (rd[i]>SQR(H2ORAD/apix)&&d[i]<H2ODEN) {
//			float rr=(sqrt(rd[i])-H2ORAD/apix)/(2.0*res/apix-H2ORAD/apix);
			d[i]=H2ODEN;
//			d[i]+=H2ODEN*exp(-rr*rr);
//printf("%1.1f\n",rr);
//			rr=exp(-3.0*rr*rr);
//			d[i]=rr*H2ODEN+(1.0-rr)*d[i];
		}
	}
	rdata->doneData();
}

data->doneData();
data->writeMRC(argv[2]);

LOG(Ref,argv[2],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
//if (solv) rdata->writeMRC("rad.mrc");
}
