// startAny.C  1999  Phil Baldwin

//N startAny
//F This program will generate a 3D model from a set of class averages, using cross common lines.
//1
//P <imagefile>	File containing class averages to be used
//P [sym=<c?>]	Cn symmetry if any
//P [proc=<num proc>]	Number of processors to use
//P [rounds=<2-5>]	Rounds of Euler angle determination to use
//P [mask=<rad>]	Mask radius
//P [lp=<radius>]	Lowpass filter radius in Fourier pixels
//U startAny classes.hed sym=c3 mask=42
//D This program uses cross common lines to determine the relative orientations of a set of
//E class averages, then builds a 3d model from them. This will not work well on
//E individual particle images, as they are generally too noisy. Typically 
//E <a href=startnrclasses.html>startnrclasses</a> is used to generate reference free
//E class averages. The user then manually selects 10 or 20 of these class-averages,
//E then runs them through startAny to generate a 3D model.
//D Despite a great deal of time and effort put into this program, it still doesn't
//E work well on some data sets. Usually it will generate results which are
//E adequate for use as a preliminary model for refinement.

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

/****************************************************************/
main(int argc, char *argv[] ) {

appinit(argc,argv);


	if (argc < 2 ) { printf("startAny \n");
		printf("Usage: startAny <imagefile> [sym=<c?>] [proc=<num proc>] \
[rounds=<2-5>] [mask=<rad>] [lp=<radius>] \n"); 
	exit(0); }
	


	int csymnum=1;
	int NumProcs =1;
	int NumRounds =3; 
	char s[1024];
	int mask=-1;
	int filtlp = -1;
	


for (int i=2; i<argc; i++) {
	if (strncmp("sym=c",argv[i],5)==0) {csymnum=atoi(&argv[i][5]);}
	else if (strncmp("proc=",argv[i],5)==0) {NumProcs= atoi(&argv[i][5]);}
	else if (strncmp("rounds=",argv[i],7)==0) {NumRounds= atoi(&argv[i][7]);}
	else if (strncmp("mask=",argv[i],5)==0) {mask= atoi(&argv[i][5]);}
	else if (strncmp("lp=",argv[i],3)==0) {filtlp= atoi(&argv[i][3]);}
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}


int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);



//	List *imageList=readImages(argv[1],0,-1);
//	int N=imageList->count(); 


//	EMData *IM= (EMData*)imageList->objectAt(0);
//	int partSize = IM -> xSize();

	if (csymnum !=1) {
		sprintf(s,"symsort %s sym=c%d mask=%d",argv[1],csymnum,mask);
		system(s);
	}
	
	sprintf(s,"c1startup %s %d fixed=1",argv[1],10);
	system(s);

	
	sprintf(s,"eulerrefine %s %d %d %d",argv[1],csymnum, NumProcs, NumRounds);// That last
														// integer is the number of rounds
	system(s);
	
	
	sprintf(s,"make3dr %s out=threed.0a.mrc mode=2  range=10 hard=30",argv[1]);
	if (csymnum >1) sprintf(s+strlen(s)," sym=c%0d",csymnum);
	system(s);
	

	sprintf(s,"proc3d threed.0a.mrc threed.0a.mrc norm mask=%d",mask);
	if (filtlp != -1 ) sprintf(s+strlen(s)," lp=%d",filtlp);
	system(s);
	
	puts(s);

	
	LOG(Ref,"threed.0a.mrc",LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);

	}
	
