// symsort.C 1999 Steve Ludtke

//N symsort
//F Sorts a set of images in order of symmetry quality
//U symsort start.img sym=c4
//D This program has mostly been superceded by <a href=symbest.html>symbest</a>.

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

struct SORT { float val; EMData *data; };

int cmp(const void *d1, const void *d2) {
float c;
c=((SORT *)d1)->val-((SORT *)d2)->val;
if (c<0) return 1;
if (c>0) return -1;
return 0;
}

main(int argc, char *argv[]) {
int csym=0,n,imask=-1,mask=-1,defo=0;
int i,mirror=0;
float lc;
List *data;
SORT *sort;
EMData *tmp,*msk;

appinit(argc,argv);

if (argc<2) { printf("symsort.C\nUsage: symsort <input file> sym=<c4,icos,etc...> [mirror] [mask=<rad>] [imask=<rad>] [defocus]\n"); exit(0); }

for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"sym=",4)==0) {
		if (tolower(argv[i][4])!='c' && tolower(argv[i][4])!='m') { printf("Only C/M symmetry currently supported!\n"); exit(1); }
		if (tolower(argv[i][4])=='m') csym=-1;
		else csym=atoi(&argv[i][5]);
	}
	else if (strncmp(argv[i],"defocus",7)==0) defo=1;
	else if (strncmp(argv[i],"mirror",6)==0) mirror=1;
	else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

if (csym<=1) { printf("Invalid symmetry\n"); exit(1); }

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

data=readImages(argv[1],-1,-1);
n=data->count();

if (n<2) { printf("Not enough images to sort!\n"); exit(0); }
sort=(SORT *)malloc(n*sizeof(SORT));

msk=((EMData *)data->objectAt(0))->copy();
msk->setParent(NULL);
msk->one();
msk->applyMask(mask,4);
msk->applyMask(imask,5);

if (defo) {
	printf("Resorting data by defocus\n");
	
	if (!((EMData *)data->objectAt(i))->hasCTF()) { printf("This data doesn't have CTF parameters.\n"); exit(1); }

	for (i=0; i<n; i++) {
		sort[i].data=((EMData *)data->objectAt(i));
		sort[i].val=(sort[i].data->getCTF())[0];
	}
}
else {
// do the symmetry comparisons
for (i=0; i<n; i++) {
	sort[i].data=((EMData *)data->objectAt(i));
	sort[i].data->normalize();
	tmp=sort[i].data->copy();
	sort[i].val=0;
	for (float f=2.0*PI/(float)csym; f<(2.0-1.0/(float)csym)*PI; f+=2.0*PI/(float)csym) {
		tmp->setRAlign(f,0,0);
		tmp->rotateAndTranslate();
		tmp->mult(msk);
		sort[i].val+=tmp->dot(sort[i].data);
	}
	if (mirror) {
		tmp->setParent(NULL);
		tmp->vFlip();
		tmp->rotAlign(sort[i].data);
		tmp->rotateAndTranslate();
		sort[i].val=-sort[i].val/(float)(csym-1)+tmp->dot(sort[i].data);
	}
	delete tmp;
	printf("%d. %f\n",i,sort[i].val/(float)(csym-1));
}
}
qsort(sort,n,sizeof(SORT),cmp);

LOG(Ref,argv[1],LOG_OUTFILE,NULL);

for (i=0; i<n; i++) {
	sort[i].data->writeIMAGIC(argv[1],i);
}
LOG(Ref,NULL,LOG_END,NULL);

}
