#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "Column.h"
#include "DataSet.h"

//#define DEBUG11

Column::Column() 
{ 
initMetaObject();                           // initialize meta object
data=NULL; 
min=max=0; 
n=1;
type=COL_RANGE; 
}

Column::~Column() 
{ 
if (data) free(data); 
if (type==COL_EQN) { free(ceqn); delete eqn; }
}

float Column::getMin() { 
	if (eqch) { updateEqn(); eqch=0; } 
	return min; 
}

float Column::getMax() { 
	if (eqch) { updateEqn(); eqch=0; } 
	return max; 
}

void Column::setEqn(const char *neqn,int N,DataSet *par) {
n=N;
parent=par;
if (data) data=(float *)realloc(data,n*sizeof(float));
else data=(float *)malloc(n*sizeof(float));
if (type==COL_EQN) ceqn=(char *)realloc(ceqn,strlen(neqn)+1);
else {
	ceqn=(char *)malloc(strlen(neqn)+1);
	eqn=new Equation;
	QObject::connect(eqn,SIGNAL(changed()),this,SLOT(eqChanged()));
}
strcpy(ceqn,neqn);
eqn->parseEqn(ceqn);
type=COL_EQN;
min=max=0;
eqch=1;

emit changed();
}

void Column::setRepCyc(float Min,float Max,int nr,int nc,int N) {
min=Min;
max=Max;
nrep=nr;
ncyc=nc;
n=N;
type=COL_REPCYC;
emit changed();
}

void Column::setRange(float Min,float Max,int N) {
	min=Min;
	max=Max;
	n=N;
	if (n<=0) n=1;
	if (data) { free(data); data=NULL; }
	else if (type==COL_EQN) { free(ceqn); ceqn=NULL; delete eqn; }
	type=COL_RANGE;
	
	emit changed();
}

// passed array is 'given' to the column
void Column::setData(float *dat,int N) {
	int i;
	
	if (N>=0) n=N;
	if (data && dat && data!=dat) free(data);
	if (type==COL_EQN) { free(ceqn); ceqn=NULL; delete eqn;}
	if (dat) data=dat;
	else {
		if (!data) {
			data=(float *)malloc(N*sizeof(float));
			memset(data,0,N*sizeof(float));
		}
	}
	type=COL_DATA;

	min=max=data[0];
	for (i=1; i<n; i++) {
		if (min>data[i]) min=data[i];
		if (max<data[i]) max=data[i];
	}
	if (min!=min) min=0;
	if (max!=max) max=min+.001;

	emit changed();
}

float Column::dataAt(int N) {
int i,j;

switch (type) {
case COL_RANGE:
	if (n==1) return min;
	return ((float)N/(float)(n-1))*(max-min)+min;
case COL_EQN:
	if (eqch) { updateEqn(); eqch=0; }
case COL_DATA:
	if (N<0||N>=n) return 0;
	return data[N];
case COL_REPCYC:
	j=n/(ncyc*nrep);
	if (j==1) return min;
	i=N%ncyc;
	i/=nrep;
	return ((float)i/(float)(j-1))*(max-min)+min;
}
return 0;
}

void Column::eqChanged() { eqch=1; emit changed(); }

void Column::updateEqn() {
int i,j;
double *row,**alt=NULL;
Column **src=NULL;
char s[5];

if (type!=COL_EQN) return;

//printf("upd %f %f %f\n",eqn->getVar("a"),eqn->getVar("b"),eqn->getVar("c"));

// Other columns can be accessed as xn where n is the column number
// this initializes pointers for fast updates
row=eqn->getVarP("row");
if (parent) {
	alt=(double **)malloc(sizeof(double *)*parent->cols());
	src=(Column **)malloc(sizeof(float *)*parent->cols());
	for (i=0; i<parent->cols(); i++) {
		sprintf(s,"x%1d",i);
		alt[i]=eqn->getVarP(s);
			// forces equation updates on any other accessed columns
		if (alt[i]) {
			parent->dataAt(i,0);
			src[i]=parent->getColumn(i);
		}
	}
}

min=1.0e30;
max=-1.0e30;

for (i=0; i<n; i++) {
	if (row) *row=(float)i;
	for (j=0; j<parent->cols(); j++) {
		if (src[j]&&alt[j]) alt[j][0]=src[j]->dataAt(i);
	}
	data[i]=eqn->eval();

	if (data[i]>max) max=data[i];
	if (data[i]<min) min=data[i];

#ifdef DEBUG1
printf("x %f\t%f\t%f\t%f\n",*xx,data[i],min,max);
#endif
}
if (min!=min) min=0;
if (max!=max) max=min+.001;

if (alt) free(alt);
if (src) free(src);
}
