#include <stdlib.h>
#include <stdio.h>
#include "DataSet.h"

DataSet::DataSet() {
title="new";
initMetaObject();                           // initialize meta object
data=(Column **)malloc(sizeof(Column *));
nx=ny=0;
}

DataSet::~DataSet()
{
free(data);
}

void DataSet::setRC(int x,int y) {
int i;

if (x<0) x=0;
if (y<1) y=1;

data=(Column **)realloc(data,sizeof(Column *)*x);

for (i=0; i<nx; i++) delete data[i];


nx=x;
ny=y;

for (i=0; i<nx; i++) {
	data[i]=new Column;
	QObject::connect(data[i],SIGNAL(changed()),this,SLOT(colChanged()));
	data[i]->setRange(0,1,ny);
}

}

void DataSet::colChanged(void) { emit changed(); }

void DataSet::setVar(char *name,float val) {
int i;

for (i=0; i<nx; i++) data[i]->setVar(name,val);

}

float DataSet::dataAt(int x,int y) {
if (x<0||x>=nx||y<0||y>=ny) return 0;

return data[x]->dataAt(y);
}

void DataSet::setXYfn(float minx, float maxx, int n, char *fofx) {
title=fofx;
setRC(2,n);
data[0]->setRange(minx,maxx,n);
data[1]->setEqn(fofx,n,this);
}

void DataSet::setXYdata(float minx, float maxx, int n) {
title="xydata";
setRC(2,n);
data[0]->setRange(minx,maxx,n);
data[1]->setData(NULL,n);
}

void DataSet::setXYdata(float *xdat, float *ydat, int n) {
title="xydata";
setRC(2,n);
data[0]->setData(xdat,n);
data[1]->setData(ydat,n);
}

void DataSet::setXYparmfn(int n, char *fxofrow, char *fyofrow) {
title=fxofrow;
setRC(2,n);
data[0]->setEqn(fxofrow,n,this);
data[1]->setEqn(fyofrow,n,this);
}

void DataSet::setXYZdata(float *xdat, float *ydat, float *zdat, int n) {
title="XYZdata";
setRC(2,n);
data[0]->setData(xdat,n);
data[1]->setData(ydat,n);
data[2]->setData(zdat,n);
}

void DataSet::setXYZparmfn(int n,char *fxofrow, char *fyofrow, char *fzofrow) {
title=fxofrow;
setRC(2,n);
data[0]->setEqn(fxofrow,n,this);
data[1]->setEqn(fyofrow,n,this);	// Shouldn't really use x, but in case
data[2]->setEqn(fzofrow,n,this);
}


void DataSet::setXYZmxdata(float xmin, float xmax, int nx, float ymin,float ymax, int ny, float *zdata) {
title="3dMx";
setRC(3,nx*ny);
data[0]->setRepCyc(xmin,xmax,ny,1,nx*ny);
data[1]->setRepCyc(ymin,ymax,1,nx,nx*ny);
data[2]->setData(zdata,nx*ny);
}


void DataSet::setXYZmxfn(float xmin, float xmax, int nx, float ymin,float ymax, int ny, char *fofxy) {
title=fofxy;
setRC(3,nx*ny);
data[0]->setRepCyc(xmin,xmax,ny,1,nx*ny);
data[1]->setRepCyc(ymin,ymax,1,nx,nx*ny);
data[2]->setEqn(fofxy,nx*ny,this);
}


// reads data files with up to 10 columns
int DataSet::readFile(const char *path) {
FILE *in;
char s[241],t[30];
int i,n,c,d;
float v;
float **dat;

in=fopen(path,"r");
if (!in) return -1;

title=path;
if (title.findRev('/')!=-1) title.truncate(title.findRev('/'));

// count rows and columns
n=0;
while (fgets(s,240,in)!=NULL) {
	if (s[0]=='#' || s[0]==';') continue;		// skip comments
	if (n==0) c=sscanf(s,"%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g",
		&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v);
	n++;
}
c=(c+1)/2;
//printf("Data file %d x %d\n",n,c);

setRC(c,n);		// make ourselves have the right size
dat=(float **)malloc(11*sizeof(float *));							// space to read data
for (i=0; i<c+1; i++) dat[i]=(float *)malloc(n*sizeof(float));	// allocate 1 extra scratch column
while (i<11) { dat[i]=dat[c]; i++; }					// extra columns point to scratch space

rewind(in);

// actual data read
n=0;
while (fgets(s,240,in)!=NULL) {
	if (s[0]=='#' || s[0]==';') continue;		// skip comments
	d=sscanf(s,"%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g%[^.0-9]%g",
		&dat[0][n],t,&dat[1][n],t,&dat[2][n],t,&dat[3][n],t,&dat[4][n],t,
		&dat[5][n],t,&dat[6][n],t,&dat[7][n],t,&dat[8][n],t,&dat[9][n]);
	if (d!=c*2) printf("Warning row %d has only %d/%d columns.\n",n,d,c);
	n++;
}

//printf("%d actually read\n",n);

for (i=0; i<c; i++) data[i]->setData(dat[i],n);
free(dat[i]);
free(dat);
fclose(in);

return 0;
}

int DataSet::writeFile(const char *path) {
FILE *out;
int i,j;

out=fopen(path,"w");
if (!out) return -1;

for (i=0; i<ny; i++) {
	for (j=0; j<nx; j++) {
		fprintf(out,"%3.7g",dataAt(j,i));
		if (j<nx-1) fprintf(out,"\t"); else fprintf(out,"\n");
	}
}
fclose(out);

return 0;
}
