#include "LSPlot.h"
#include "misc.h"

//#define DEBUG1

LSPlot::LSPlot()
{
initMetaObject();                           // initialize meta object
sym=SYM_PLUS;
symsize=4;
flags=LS_LINE;
color=black;
pen.setColor(color);
brush.setColor(color);
}

LSPlot::~LSPlot()
{
}

void LSPlot::setLine(int ls,int width) {
if (width>=0) { pen.setWidth(width); linewid=width; }
if (ls>=0) linesty=ls;

switch (ls) {
case 0: pen.setStyle(SolidLine); break;
case 1: pen.setStyle(DashLine); break;
case 2: pen.setStyle(DotLine); break;
case 3: pen.setStyle(DashDotLine); break;
case 4: pen.setStyle(DashDotDotLine); break;
}

}

void LSPlot::setColor(int r,int g,int b) {
color.setRgb(r,g,b);
pen.setColor(color);
brush.setColor(color);
}

void LSPlot::setColor(QColor *qc) {
if (!qc) qc=(QColor *)&black;
color=*qc;
pen.setColor(color);
brush.setColor(color);
}

void LSPlot::display(QPainter *qp,Axes2d *axis,int x0,int y0,int x1,int y1) {
int i,j;
int x,y,lx=-1,ly=-1,w;
const int *c;

if (!getData() || cols.size()<2) return;
c=getCols();

#ifdef DEBUG1
printf("LSP disp\n");
#endif

w=pen.width();
if (w==0) w=1;
if (axis->needUpd()==2) pen.setWidth(OVERSMP*4/3*w);	// printing, bug workaround

qp->setPen(pen);

if (flags&LS_SYM) {
	for (i=0; i<getData()->rows(); i++) {
		x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)));
		y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)));
//		if (x<x0||x>x1||y<y0||y>y1) continue;
		drawsym(qp,x,y);
	}
}

if (flags&LS_LINE) {
	QPointArray qpa(getData()->rows());
	
	for (i=0,j=0; i<getData()->rows(); i++) {
		x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)));
		y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)));
/*		if (x<x0||x>x1||y<y0||y>y1) {
			if (j>1) qp->drawPolyline(qpa,0,j);
			j=0;
			continue;
		}*/
		qpa.setPoint(j,x,y);
		if (x!=lx || y!=ly) j++;
		lx=x;
		ly=y;
	}
	
	if (j>1) qp->drawPolyline(qpa,0,j);
}

if (flags&LS_XERROR) {
	QPointArray qpa(getData()->rows());
	
	for (i=0,j=0; i<getData()->rows(); i++) {
		x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)-getData()->dataAt(c[2],i)/2.0));
		y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)));
		qpa.setPoint(j,x,y); j++;
		x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)+getData()->dataAt(c[2],i)/2.0));
		qpa.setPoint(j,x,y); j++;
	}	
	if (j>1) qp->drawLineSegments(qpa,0,-1);
}

if (flags&LS_YERROR) {
	QPointArray qpa(getData()->rows());
	
	for (i=0,j=0; i<getData()->rows(); i++) {
		x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)));
		y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)-getData()->dataAt(c[3],i)/2.0));
		qpa.setPoint(j,x,y); j++;
		y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)+getData()->dataAt(c[3],i)/2.0));
		qpa.setPoint(j,x,y); j++;
	}	
	if (j>1) qp->drawLineSegments(qpa,0,-1);
}

if (flags&LS_VECXY || flags&LS_VECRT) {
	QPointArray qpa(getData()->rows());
	float m0=0,m1=0;
	if (flags&LS_VECRT) {
		m0=getData()->getColumn(c[3])->getMin();
		m1=2.0*PI/(getData()->getColumn(c[3])->getMax()-m0);
	}		
	
	for (i=0,j=0; i<getData()->rows(); i++) {
		x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)));
		y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)));
		qpa.setPoint(j,x,y); j++;
		if (flags&LS_VECXY) {
			x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)+getData()->dataAt(c[2],i)));
			y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)+getData()->dataAt(c[3],i)));
		}
		else {
			float r=getData()->dataAt(c[2],i);
			float t=(getData()->dataAt(c[3],i)-m0)*m1;
			x=rnd(axis->xdtor2(getData()->dataAt(c[0],i)+r*cos(t)));
			y=rnd(axis->ydtor2(getData()->dataAt(c[1],i)+r*sin(t)));
		}
		qpa.setPoint(j,x,y); j++;
	}	
	if (j>1) qp->drawLineSegments(qpa,0,-1);
}

if (axis->needUpd()==2) pen.setWidth(w);
}

int LSPlot::legend(QPainter *qp,Axes2d *axis,int x,int y) {

//printf("legend '%s' %d %d\n",label(),x,y);
if (strlen(getLabel())==0) return 0;

qp->setPen(pen);

if (flags&LS_SYM) {
	drawsym(qp,x,y+lmet->descent()-lmet->height()/2);
}

if (flags&LS_LINE) {
	qp->drawLine(x-10,y+lmet->descent()-lmet->height()/2,x+10,y+lmet->descent()-lmet->height()/2);
}

qp->setFont(*lfont);
qp->drawText(x+14,y,getLabel());

return(lmet->height()+3.0);
}

void LSPlot::drawsym(QPainter *qp,int x,int y) {

switch(sym) {
case SYM_X:
	qp->drawLine(rnd(-symsize/1.4+x),rnd(-symsize/1.4+y),rnd(symsize/1.4+x),rnd(symsize/1.4+y));
	qp->drawLine(rnd(-symsize/1.4+x),rnd(symsize/1.4+y),rnd(symsize/1.4+x),rnd(-symsize/1.4+y));
	break;
case SYM_PLUS:
	qp->drawLine(rnd(-symsize+x),rnd(0+y),rnd(symsize+x),rnd(0+y));
	qp->drawLine(rnd(0+x),rnd(-symsize+y),rnd(0+x),rnd(symsize+y));
	break;
case SYM_CIRCLE:
	qp->drawEllipse(rnd(x-symsize/2),rnd(y-symsize/2),rnd(symsize),rnd(symsize));
	break;
case SYM_BOX:
	qp->drawRect(rnd(x-symsize/2),rnd(y-symsize/2),rnd(symsize),rnd(symsize));
	break;
case SYM_DIAM:
case SYM_FDIAM: {
	QPointArray qpa(4);
	qpa.putPoints(0,4,rnd(x-symsize),rnd(y), rnd(x),rnd(y-symsize), rnd(x+symsize),rnd(y), rnd(x),rnd(y+symsize));
	if (sym==SYM_DIAM) qp->drawPolyline(qpa);
	else qp->drawPolygon(qpa);
	break; }
case SYM_STAR: {
	QPointArray qpa(7);
	qpa.putPoints(0,7,rnd(x-symsize/1.2),rnd(y-symsize/1.2),
		rnd(x+symsize/1.2),rnd(y+symsize/1.2), rnd(x-symsize/1.2),rnd(y+symsize/1.2),
		rnd(x+symsize/1.2),rnd(y-symsize/1.2), rnd(x-symsize),rnd(y),
		rnd(x+symsize),rnd(y), rnd(x),rnd(y-symsize), rnd(x),rnd(y+symsize));
	qp->drawPolyline(qpa);
	break; }
case SYM_FBOX:
	qp->fillRect(rnd(x-symsize/2),rnd(y-symsize/2),rnd(symsize),rnd(symsize),brush);
	break;
}

return;
}
